/*** Following doxy comment is for the file description ***/
/*!
 *
 * Project: Bus Firmware
 * (C)    : KUNBUS GmbH, Robert-Bosch-Strasse 12a, D-76275 Ettlingen
 *
 *
 * \file comm_interface_errors.h
 *
 *
 * \brief 
 * - Containes communication error defines.
 *
 * \author SF\BS
 *
 */

#ifndef COMM_INTERFACE_ERRORS_H
#define COMM_INTERFACE_ERRORS_H

/*****************************************************************************
 ****  DEFINES                                                            ****
 *****************************************************************************/

/*! @name Error Class constants
    Defines different levels of possible error sources 
    - Application
    - Driver
    - Embedded Software
    - Profinet Protocol stack
*/
//@{
#define ERROR_CLASS_NO_ERROR                        0 //!< Indicates overall no error occurred */
#define ERROR_CLASS_APP_SYSTEM                      1 //!< The error is located in application level and caused by the system, e.g. Dioctl returns 0 */
#define ERROR_CLASS_DLL                             2 //!< The error is located in DLL level, e.g. */
#define ERROR_CLASS_DRV                             3 //!< The error is located in OS driver level */
#define ERROR_CLASS_FW                              4 //!< The error is located in Firmware level */
#define ERROR_CLASS_PNSTACK                         5 //!< The error is located in Profinet IO stack level */
#define ERROR_CLASS_NIVISA                          6 //!< The error is located in NI-VISA level */
#define ERROR_CLASS_LVDRV                           7 //!< The error is located in LabVIEW driver level */
#define ERROR_CLASS_APP_USER                        8 //!< The error is located in the user application */
#define ERROR_CLASS_EXT_COMPMONENT                  9 //!< The error is located in an externel helper dll or application */

#define ERROR_CLASS_LABVIEW_TEXT                    "LabView class specified"
#define ERROR_CLASS_UNKNOWN_TEXT                    "Unknown error class specified"
#define ERROR_CLASS_APP_USER_TEXT                   "User defined error. No Error Text available."
//@}


/*! @name DLL Errors constants
    Defines errors, located in the windows DLL level and in application level
    (when a win32 API call returns an error)
*/
//@{
// new error numbers and text constants also needs to be programmed in the DLL Module "ErrorHandler.cpp" 
// for text output with GetDLLErrorText(). Deleted values needs to be acordingly deleted there.
#define RET_SUCCESSFULL                             0          //!< No error occurred */
#define RET_DLL_SLOT_SUBSLOT_NOT_FOUND              0x80000001 //!< The requested slot/subslot combination does not exist */
#define RET_DLL_MEM_ALLOCATION_FAILED               0x80000002 //!< The DLL could not allocate enough memory for the current command */
#define RET_DLL_HANDLE_INVALID                      0x80000003 //!< The DLL could create a handle of the driver, which may be not yet started */
#define RET_DLL_ASYNC_TIMEOUT                       0x80000004 //!< The user specified timeout for the asynchron service call is expired before data returned */
#define RET_DLL_ASYNC_WAIT_ERROR                    0x80000005 //!< The wait function for the asynchron service call failed */
#define RET_DLL_CONTROLLER_INDEX_INVALID            0x80000006 //!< The specified zero based index for the controller board is too high */
#define RET_DLL_DEVICE_INDEX_INVALID                0x80000007 //!< The specified index for the device on a controller board is invalid */
#define RET_DLL_CYCLIC_DATA_NOT_INITIALIZED         0x80000008 //!< The cyclic data are not yet initialized; InitStartProfinet() was not yet called */
#define RET_DLL_IDENT_ALL_NOT_INITIALIZED           0x80000009 //!< The command IdentifyAll was not yet issued */
#define RET_DLL_IDENTIFY_ALL_WRONG_LENGTH           0x8000000A //!< The DCP command IdentifyAll returned with a wrong length */
#define RET_DLL_SET_NAME_LENGTH_NULL                0x8000000B //!< The new name for DCP command SetName is empty */
#define RET_DLL_NO_CONTROLLER_ACTIVE                0x8000000C //!< No controller available; hardware or driver is missing */
#define RET_DLL_PNIO_OUT_BUFFER_INVALID             0x8000000D //!< The PNIO output buffer is invalid or too small */
#define RET_DLL_PNIO_IN_BUFFER_INVALID              0x8000000E //!< The PNIO input buffer is invalid or too small */
#define RET_DLL_DEVICE_HAS_NO_OUTPUT_DATA           0x8000000F //!< The specified device is configured witout output data */
#define RET_DLL_USER_POINTER_INVALID                0x80000010 //!< The user given pointer is invalid */
#define RET_DLL_CONTROLLER_NOT_CONFIGURED           0x80000011 //!< The controller is not configured with any device */
#define RET_DLL_DEVICE_HAS_NO_INPUT_DATA            0x80000012 //!< The specified device is configured witout input data */
#define RET_DLL_SET_IP_INCORRECT                    0x80000013 //!< The new IP or the SubnetMask for DCP command SetIP is incorrect */
#define RET_DLL_UNKNOWN_CHANNEL                     0x80000014 //!< The requested Command can't be routed to the correct receiver */
#define RET_DLL_BOARD_NOT_SUPPORTED                 0x80000015 //!< The Firmware revision of the selected board is to old for this dll. */
#define RET_DLL_CMD_NOT_SUPPORTED_VIA_TCP           0x80000016 //!< The Command is not supported while Controller is connected via TCP/IP.
#define RET_DLL_TCP_SOCKET_ERROR                    0x80000017 //!< A error happend with the TCP/IP socket.

#define APP_SYSTEM_LEVEL_ERROR_TEXT                 "API level : "
#define APP_SYSTEM_UNKNOWN_ERROR_TEXT               "SYSTEM ERROR ID caused by APPLICATION is unknown"
#define RET_DLL_LEVEL_ERROR_TEXT                    "DLL level : "
#define RET_DLL_SUCCESSFULL_BUT_CLASS_TEXT          "No error occurred, although DLL error class is specified."
#define RET_DLL_SLOT_SUBSLOT_NOT_FOUND_TEXT         "The requested slot and subslot combination does not exist."
#define RET_DLL_MEM_ALLOCATION_FAILED_TEXT          "The DLL could not allocate enough memory for the current command."
#define RET_DLL_HANDLE_INVALID_TEXT                 "The DLL could create a handle of the driver, which may be not yet started."
#define RET_DLL_ASYNC_TIMEOUT_TEXT                  "The user specified timeout for the asynchron service call expired before data returned."
#define RET_DLL_ASYNC_WAIT_ERROR_TEXT               "The wait function for the asynchron service call failed."
#define RET_DLL_CONTROLLER_INDEX_INVALID_TEXT       "The specified zero based index for the controller board is too high."
#define RET_DLL_DEVICE_INDEX_INVALID_TEXT           "The specified index for the device on a controller board is invalid."
#define RET_DLL_CYCLIC_DATA_NOT_INITIALIZED_TEXT    "The cyclic data are not yet initialized."
#define RET_DLL_IDENT_ALL_NOT_INITIALIZED_TEXT      "The command IdentifyAll was not yet issued, or 0 devices were found"
#define RET_DLL_IDENTIFY_ALL_WRONG_LENGTH_TEXT      "The DCP command IdentifyAll returned a wrong length."
#define RET_DLL_SET_NAME_LENGTH_NULL_TEXT           "The new name for DCP command SetName is empty."
#define RET_DLL_NO_CONTROLLER_ACTIVE_TEXT           "No controller available; hardware or driver is missing."
#define RET_DLL_PNIO_OUT_BUFFER_INVALID_TEXT        "The PNIO output buffer is invalid or too small."
#define RET_DLL_PNIO_IN_BUFFER_INVALID_TEXT         "The PNIO input buffer is invalid or too small."
#define RET_DLL_DEVICE_HAS_NO_OUTPUT_DATA_TEXT      "The specified device is configured witout output data."
#define RET_DLL_UNKNOWN_ERROR_TEXT                  "An unknown error code on DLL level was received."
#define RET_DLL_USER_POINTER_INVALID_TEXT           "An user given pointer is invalid."
#define RET_DLL_CONTROLLER_NOT_CONFIGURED_TEXT      "The controller is not configured with any device."
#define RET_DLL_DEVICE_HAS_NO_INPUT_DATA_TEXT       "The specified device is configured witout input data."
#define RET_DLL_SET_IP_INCORRECT_TEXT               "The new IP or the SubnetMask for DCP command SetIP is incorrect."
#define RET_DLL_UNKNOWN_CHANNEL_TEXT                "The requested Command can't be routed to the correct receiver"
#define RET_DLL_BOARD_NOT_SUPPORTED_TEXT            "The Firmware revision of the selected board is too old for this DLL."
#define RET_DLL_CMD_NOT_SUPPORTED_VIA_TCP_TEXT      "The Command is not supported while Controller is connected via TCP/IP."
#define RET_DLL_TCP_SOCKET_ERROR_TEXT               "A error happend with the TCP/IP socket."

//@}


/*! @name Driver Errors constants
    Defines errors, located in the OS driver level
*/
//@{
// new error numbers and text constants also needs to be programmed in the DLL Module "ErrorHandler.cpp" 
// for text output with GetDLLErrorText(). Deleted values needs to be acordingly deleted there.
#define RET_SUCCESSFULL                             0          //!< No error occurred */
#define RET_FW_NOT_STARTED                          0x80000001 //!< The firmware is not running */
#define RET_PN_NOT_STARTED                          0x80000002 //!< The Profinet is not running */
#define RET_DMA_ERROR                               0x80000005 //!< The driver has detected an error associated with the DMA. */
#define RET_ACYCLIC_SERVICE_TIMED_OUT               0x80000007 //!< The wait on manual IRQ to PC trigger timed-out. (only for LabVIEW driver) */
#define RET_ACYCLIC_SERVICE_IS_PENDING              0x80000008 //!< A different acyclic service is still active. */
#define RET_COMMAND_HANG_OR_RESET                   0x8000000A //!< The controller does not respond any more or had an internal reset. */
#define RET_INVALID_IN_BUFFER_LENGTH                0x8000000B //!< The input buffer is too small for the acyclic command data */
#define RET_INVALID_OUT_BUFFER_LENGTH               0x8000000C //!< The output buffer is too small for the acyclic command data */
#define RET_INVALID_IN_PROC_IMAGE_LENGTH            0x8000000D //!< The input buffer is too small for the cyclic process image input data */
#define RET_INVALID_OUT_PROC_IMAGE_LENGTH           0x8000000E //!< The output buffer is too small for the cyclic process image output data */
#define RET_INVALID_CYCLIC_SUB_COMMAND              0x8000000F //!< The user given subcommand is invalid */
#define RET_IN_RESET                                0x80000010 //!< The reset command is still active */
#define RET_RESOURCE_NAME_CANNOT_BE_USED            0x80000011 //!< It is not allowed to call the same VI (clone) with different resource names! Please duplicate VI. */
#define RET_RESOURCE_LOCKED                         0x80000012 //!< Multiple access to the same resource is not allowed */

#define RET_DRV_LEVEL_ERROR_TEXT                    "Driver level : "
#define RET_SUCCESSFULL_TEXT                        "No error occurred."
#define RET_DRV_SUCCESSFULL_BUT_CLASS_TEXT          "No error occurred, although driver error class is specified."
#define RET_FW_NOT_STARTED_TEXT                     "The firmware is not running."
#define RET_PN_NOT_STARTED_TEXT                     "The Profinet is not running."
#define RET_DMA_ERROR_TEXT                          "The driver has detected an error associated with the DMA."
#define RET_ACYCLIC_SERVICE_TIMED_OUT_TEXT          "The wait on manual IRQ to PC trigger timed-out."
#define RET_ACYCLIC_SERVICE_IS_PENDING_TEXT         "A different acyclic service is still active."
#define RET_COMMAND_HANG_OR_RESET_TEXT              "The controller does not respond any more or had an internal reset."
#define RET_INVALID_IN_BUFFER_LENGTH_TEXT           "The input buffer is too small for the acyclic command data."
#define RET_INVALID_OUT_BUFFER_LENGTH_TEXT          "The output buffer is too small for the acyclic command data."
#define RET_INVALID_IN_PROC_IMAGE_LENGTH_TEXT       "The input buffer is too small for the cyclic process image input data."
#define RET_INVALID_OUT_PROC_IMAGE_LENGTH_TEXT      "The output buffer is too small for the cyclic process image output data."
#define RET_INVALID_CYCLIC_SUB_COMMAND_TEXT         "The user given subcommand is invalid."
#define RET_DRV_UNKNOWN_ERROR_TEXT                  "An unknown error code on driver level was received."
#define RET_IN_RESET_TEXT                           "The reset command is still active."
#define RET_RESOURCE_NAME_CANNOT_BE_USED_TEXT       "It is not allowed to call the same VI (clone) with different resource names! Please duplicate VI."
#define RET_RESOURCE_LOCKED_TEXT                    "Multiple access to the same resource is not allowed"

//@}


/*! @name Firmware Errors constants (ERROR_CLASS_FW)
    Defines errors, located in the firmware level
*/
//@{
// new error numbers and text constants also needs to be programmed in the DLL Module "ErrorHandler.cpp" 
// for text output with GetDLLErrorText(). Deleted values needs to be acordingly deleted there.
#define RET_SUCCESSFULL                               0  
#define RET_ERR_INVALID_COMMAND                       0x01         //1  
#define RET_ERR_INVALID_SUB_COMMAND                   0x02         //2  
#define RET_ERR_STACK_ERROR                           0x03         //3  
#define RET_ERR_STACK_TIMEOUT                         0x04         //4  
#define RET_ERR_UNHANDLED_OS_ERROR                    0x05         //5  
#define RET_ERR_BUS_RUNNING                           0x06         //6  
#define RET_ERR_DEVICE_COUNT_EXCEEDED                 0x07         //7  
#define RET_ERR_AR_DEV_NO_INVALID                     0x08         //8  
#define RET_ERR_ALARM_TYPE_INVALID                    0x09         //9  
#define RET_ERR_NULL_POINTER                          0x0A         //10 
#define RET_ERR_OFFSET_TABLE_SIZE_EXCEEDED            0x0B         //11 
#define RET_ERR_BUS_NOT_RUNNING                       0x0C         //12 
#define RET_ERR_INVALID_SUB_COMMAND_BUFFER_SIZE       0x0D         //13 
#define RET_ERR_DCP_SEND_BUFFER_OVERFLOW              0x1E        //30 
#define RET_ERR_READ_RECORD_OVERFLOW_ERROR            0x1F        //31 
#define RET_ERR_PACKET_NO_MISMATCH                    0x32        //50 
#define RET_ERR_FILE_OPEN_FAILED                      0x3C        //60 
#define RET_ERR_SYSFLASHGET_FAILED                    0x3D        //61 
#define RET_ERR_SYSFLASHSET_FAILED                    0x3E        //62 
#define RET_ERR_FILE_WRITE_FAILED                     0x3F        //63 
#define RET_ERR_FILE_DOES_NOT_EXIST                   0x40        //64 
#define RET_ERR_CREATING_DIRECTORY_LIST               0x41        //65 
#define RET_ERR_CSCOMPRESS_FAILED                     0x46        //70 
#define RET_ERR_DEBUG_LEVEL_NOT_ALLOWED               0x47        //71 
#define RET_ERR_CSUNCOMPRESS_FAILED                   0x48        //72 
#define RET_ERR_INVALID_FILE_LOCATION                 0x49        //73 
#define RET_ERR_INVALID_FILE_FORMAT                   0x4A        //74 
#define RET_ERR_INVALID_FILE_NAME                     0x4B        //75 
#define RET_ERR_FLASH_SET_PLAUSABILITY_FAILED         0x4C        //76 
#define RET_ERR_TASK_WD_TRIGGERED                     0x50        //80 
#define RET_ERR_NO_LOG_FILE_AVAILABLE                 0x55        //85 
#define RET_ERR_SET_TIME_FAILED                       0x5A        //90
#define RET_ERR_BZIP2_CS_COMPRESS_TIMEOUT             0x5B        //91
#define RET_ERR_BZIP2_CS_UN_COMPRESS_TIMEOUT          0x5C        //92
#define RET_ERR_BZIP2_UNKNOWN_COMMPRESS_DIRECTION     0x5D        //93
#define RET_ERR_BZIP2_UNKNOWN_EVENT                   0x5E        //94
#define RET_ERR_BZIP2_OUTPUT_FILE_ALREADY_EXISTS      0x5F        //95
#define RET_ERR_BZIP2_CANT_CREATE_OUTPUT_FILE         0x60        //96
#define RET_ERR_BZIP2_CANT_OPEN_INPUT_FILE            0x61        //97
#define RET_ERR_BZIP2_NO_BZIP2_FILE_FORMAT            0x62        //98
#define RET_ERR_BESTUECKERTEST_CRIO_FAILED            0x63        //99
#define RET_ERR_DEVICE_NO_NOT_VALID                   0x64        //100
#define RET_ERR_DUPLICATE_IP_OR_NETWORK_FAILURE       0x65        //101
#define RET_ERR_TCP_CYCLIC_TRANSFER_TIMEOT            0x66        //102
#define RET_ERR_TCP_CONCURRENT_DRIVER                 0x67        //103
#define RET_ERR_PARAMETER_FAULT                       0x68        //104
#define RET_ERR_DCP_SET_RESULT_DEVICE_FAILED          0x69        //105
#define RET_ERR_HAS_TO_BE_IMPLEMENTED                 0xFF        //255

#define RET_FW_LEVEL_ERROR_TEXT                       "FW level : "
#define RET_FW_SUCCESSFULL_BUT_CLASS_TEXT             "No error occurred, although firmware error class is specified."
#define RET_ERR_INVALID_COMMAND_TEXT                  "Firmware received an invalid command."
#define RET_ERR_INVALID_SUB_COMMAND_TEXT              "Firmware received an invalid sub command."
#define RET_ERR_STACK_ERROR_TEXT                      "Firmware had a stack error."
#define RET_ERR_STACK_TIMEOUT_TEXT                    "Firmware had a stack timeout error."
#define RET_ERR_UNHANDLED_OS_ERROR_TEXT               "Firmware had an unhandled OS error."
#define RET_ERR_BUS_RUNNING_TEXT                      "The Profinet has already started."
#define RET_ERR_DEVICE_COUNT_EXCEEDED_TEXT            "Too many devices are configured."
#define RET_ERR_AR_DEV_NO_INVALID_TEXT                "The configured Device Number = AR USER ID is invalid."
#define RET_ERR_ALARM_TYPE_INVALID_TEXT               "The Alarm type is invalid."
#define RET_ERR_NULL_POINTER_TEXT                     "Null pointer."
#define RET_ERR_OFFSET_TABLE_SIZE_EXCEEDED_TEXT       "The offset table exeeds the limit."
#define RET_ERR_BUS_NOT_RUNNING_TEXT                  "The Profinet has not been started."
#define RET_ERR_INVALID_SUB_COMMAND_BUFFER_SIZE_TEXT  "The request buffer size does not fit to the subcommand."
#define RET_ERR_DCP_SEND_BUFFER_OVERFLOW_TEXT         "The send buffer in a DCP call is too small."
#define RET_ERR_READ_RECORD_OVERFLOW_ERROR_TEXT       "The read buffer is too small."
#define RET_ERR_PACKET_NO_MISMATCH_TEXT               "Packet processing order mismatch."
#define RET_ERR_FILE_OPEN_FAILED_TEXT                 "File open failed."
#define RET_ERR_SYSFLASHGET_FAILED_TEXT               "Get system flash failed."
#define RET_ERR_SYSFLASHSET_FAILED_TEXT               "Set system flash failed."
#define RET_ERR_FILE_WRITE_FAILED_TEXT                "The write operation to a file failed. Drive might be full?"
#define RET_ERR_FILE_DOES_NOT_EXIST_TEXT              "The file specified for the read operation does not exist on this drive."
#define RET_ERR_CREATING_DIRECTORY_LIST_TEXT          "Error while creating a file list of the flash drive."
#define RET_ERR_CSCOMPRESS_FAILED_TEXT                "Compress algorithm failed."
#define RET_ERR_DEBUG_LEVEL_NOT_ALLOWED_TEXT          "The given debug level is not allowed."
#define RET_ERR_CSUNCOMPRESS_FAILED_TEXT              "Uncompress algorithm failed."
#define RET_ERR_INVALID_FILE_LOCATION_TEXT            "The location parameter is incorrect set or does not fit to the subcommand." 
#define RET_ERR_INVALID_FILE_FORMAT_TEXT              "The CompressionMode parameter is incorrect set."
#define RET_ERR_INVALID_FILE_NAME_TEXT                "The File name does contain spaces."
#define RET_ERR_FLASH_SET_PLAUSABILITY_FAILED_TEXT    "The size of the transfered file is not plausible."
#define RET_ERR_TASK_WD_TRIGGERED_TEXT                "Task control watchdog has expired."
#define RET_ERR_NO_LOG_FILE_AVAILABLE_TEXT            "The Logfile was not exisiting in the ramdisk."
#define RET_ERR_SET_TIME_FAILED_TEXT                  "The system time could not been set!"
#define RET_ERR_BZIP2_CS_COMPRESS_TIMEOUT_TEXT        "The Compress subfunction had a timeout."
#define RET_ERR_BZIP2_CS_UN_COMPRESS_TIMEOUT_TEXT     "The UnCompress subfunction had a timeout."     
#define RET_ERR_BZIP2_UNKNOWN_COMMPRESS_DIRECTION_TEXT "The compression option has an unknown value."     
#define RET_ERR_BZIP2_UNKNOWN_EVENT_TEXT              "The Compression subfunction received an unknown event."     
#define RET_ERR_BZIP2_OUTPUT_FILE_ALREADY_EXISTS_TEXT "The destination file for the uncompress action does already exist and the force overwright option is set to false."     
#define RET_ERR_BZIP2_CANT_CREATE_OUTPUT_FILE_TEXT    "The destination file for the uncompress action could not be created due an OS error (drive full ?)."         
#define RET_ERR_BZIP2_CANT_OPEN_INPUT_FILE_TEXT       "The specified file for the compression action could not be opened."    
#define RET_ERR_BZIP2_NO_BZIP2_FILE_FORMAT_TEXT       "The specified file to uncompress has no BZ2 format."       
#define RET_ERR_BESTUECKERTEST_CRIO_FAILED_TEXT       "The Bestuertest failed. Check if a cRIO PN with device name crio-pn-1 in mirror mode is at the bus available."
#define RET_ERR_DEVICE_NO_NOT_VALID_TEXT              "Device No not valid!"
#define RET_ERR_DUPLICATE_IP_OR_NETWORK_FAILURE_TEXT  "A duplicate IP address conflict, invalid IP configuration or an unknown network failure prevents card from network communication."
#define RET_ERR_TCP_CYCLIC_TRANSFER_TIMEOT_TEXT       "Received an TCP timeout!"
#define RET_ERR_TCP_CONCURRENT_DRIVER_TEXT            "Concurrent Driver is connected.\r\nPlease deactivate concurrent connection!"        //103
#define RET_ERR_PARAMETER_FAULT_TEXT                  "Invalid Parameter for this command."
#define RET_ERR_DCP_SET_RESULT_DEVICE_FAILED_TEXT     "DCP SET command failed on Device."

#define RET_ERR_HAS_TO_BE_IMPLEMENTED_TEXT            "Not yet implemented specific error code."
#define RET_FW_UNKNOWN_ERROR_TEXT                     "An unknown error code on firmware level was received."

//@}


/*! @name Profinet stack Errors
    PN Stack Errors are defined in PN_LibMain.h
*/
//@{
// in case of a new PNIO stack release and new error codes: 
// new error numbers and text constants also needs to be programmed in the DLL Module "ErrorHandler.cpp" 
// for text output with GetDLLErrorText(). Deleted values needs to be acordingly deleted there.
#define RET_PN_ERROR_TEXT                             "Error from Profinet stack level"

/* Text Definitions for Error Classes */
#define E_MONA_CLASS_COP_TEXT          " in class COP"
#define E_MONA_CLASS_OS_TEXT           " in class OSP"
#define E_MONA_CLASS_DDI_TEXT          " in class DDI"
#define E_MONA_CLASS_BOOT_LOADER_TEXT  " in class BOOT_LOADER"
#define E_MONA_CLASS_VEN_TEXT          " in class VEN"
#define E_MONA_CLASS_BASMOD_TEXT       " in class BASMOD"
#define E_MONA_CLASS_DDMOD_TEXT        " in class DDMOD"
#define E_MONA_CLASS_API_TEXT          " in class API"
#define E_MONA_CLASS_APP_TEXT          " in class APP"

#define E_MONA_CLASS_UNKNOWN_TEXT      " in unknown class"

/* Text Definitions for Error Types */
#define E_MONA_TYPE_NONE_TEXT          " type NONE"
#define E_MONA_TYPE_USER_TEXT          " type USER"
#define E_MONA_TYPE_WARNING_TEXT       " type WARNING"
#define E_MONA_TYPE_RUNTIME_TEXT       " type RUNTIME"

#define E_MONA_TYPE_UNKNOWN_TEXT       " unknown type"

/* Text Definitions for Module IDs */
#define SDDI_MODUL_ID_TEXT             " SDDI module"
#define LLDP_MODUL_ID_TEXT             " LLDP module"
#define ETC_MODUL_ID_TEXT              " ETC module"
#define CTM_MODUL_ID_TEXT              " CTM module"
#define PNIO_IOAR_MODUL_ID_TEXT        " PNIO IOAR module"
#define PNIO_RPC_MODUL_ID_TEXT         " PNIO RPC module"
#define PNIO_CMCTL_MODUL_ID_TEXT       " PNIO CMCTL module"
#define PNIO_CMDEV_MODUL_ID_TEXT       " PNIO CMDEV module"
#define DCP_ARP_MODUL_ID_TEXT          " DCP ARP module"
#define PN_MODUL_ID_TEXT               " PN module"
#define SNMP_MODUL_ID_TEXT             " SNMP module"
#define PNDIAG_MODUL_ID_TEXT           " PNDIAG module"

#define VE_API_ID_TEXT                 " VE API"
#define MH_API_ID_TEXT                 " MH API"
#define ES_API_ID_TEXT                 " ES API"
#define DDI_API_ID_TEXT                " DDI API"
#define CM_API_ID_TEXT                 " CM API"
#define SMEM_API_ID_TEXT               " SMEM API"
#define DH_API_ID_TEXT                 " DH API"
#define DEV_API_ID_TEXT                " DEV API"
#define FILE_API_ID_TEXT               " FILE API"
#define L2_API_ID_TEXT                 " L2 API"
#define MSGQ_API_ID_TEXT               " MSGQ API"
#define PNIO_IOAR_API_ID_TEXT          " PNIO IOAR API"
#define PNIO_RPC_API_ID_TEXT           " PNIO RPC API"
#define PNIO_CMCTL_API_ID_TEXT         " PNIO CMCTL API"
#define PNIO_CMDEV_API_ID_TEXT         " PNIO CMDEV API"
#define SNMP_API_ID_TEXT               " SNMP API"
#define PN_API_ID_TEXT                 " PN API"

#define MODUL_ID_UNKNOWN_TEXT          " unknown module or api"

/* Text Definitions for Error Codes */
#define E_MONA_NO_ERROR_TEXT           " : no error" 
#define E_MONA_PASSED_TEXT             " : passed" 

#define E_MONA_PROC_FATAL_ERROR_TEXT      " : PROC fatal error" 
#define E_MONA_PROC_STATE_CONFLICT_TEXT   " : PROC state conflict" 
#define E_MONA_PROC_STATE_UNDEFINED_TEXT  " : PROC state undefined" 
#define E_MONA_PROC_OUT_OF_SYNC_TEXT      " : PROC out of sync" 
#define E_MONA_PROC_STACK_OVERFLOW_TEXT   " : PROC stack overflow" 
#define E_MONA_PROC_PLC_STOPPED_TEXT      " : PROC PLC stopped" 

#define E_MONA_ACCESS_FAILED_TEXT         " : access failed" 
#define E_MONA_ACCESS_DENIED_TEXT         " : access denied" 
#define E_MONA_ACCESS_CONFLICT_TEXT       " : access conflict" 
#define E_MONA_ACCESS_LOCKED_TEXT         " : access locked" 
#define E_MONA_ACCESS_OUT_OF_RANGE_TEXT   " : access out of range" 

#define E_MONA_PARAM_INVLD_TEXT              " : param is invalid"
#define E_MONA_PARAM_INVLD_HANDLE_TEXT       " : param is invalid handle"
#define E_MONA_PARAM_INVLD_ID_TEXT           " : param is invalid ID"
#define E_MONA_PARAM_INVLD_POINTER_TEXT      " : param is invalid pointer"
#define E_MONA_PARAM_INVLD_PATH_TEXT         " : param is invalid path"
#define E_MONA_PARAM_INVLD_NAME_TEXT         " : param is invalid name"
#define E_MONA_PARAM_NAME_LENGTH_EXCEED_TEXT " : param name length exceeds"
#define E_MONA_PARAM_INVLD_OPTION_TEXT       " : param is invalid option"
#define E_MONA_PARAM_NULL_BUFFER_TEXT        " : param buffer is NULL"
#define E_MONA_PARAM_INCONSISTENT_TEXT       " : param is inconsistent"
#define E_MONA_PARAM_OUT_OF_RANGE_TEXT       " : param is out of range"
#define E_MONA_PARAM_INVLD_USERNAME_TEXT     " : param username is invalid"
#define E_MONA_PARAM_INVLD_PASSWORD_TEXT     " : param password is invalid"
#define E_MONA_PARAM_INVLD_DIRECTORY_TEXT    " : param directory is invalid"
#define E_MONA_PARAM_INVLD_ACCESS_TEXT       " : param invalid access"
#define E_MONA_PARAM_INVLD_USER_TEXT         " : param user text is invalid"
#define E_MONA_PARAM_NO_MORE_FILES_TEXT      " : param is out of range"

#define E_MONA_MSG_UNKNOWN_CODE_TEXT         " : message has unknown code"
#define E_MONA_MSG_UNSUPPORTED_CODE_TEXT     " : message has unsupported code"
#define E_MONA_MSG_INVLD_PARA_COUNT_TEXT     " : message has invalid parameter count"
#define E_MONA_MSG_INVLD_PARA_BYTE_TEXT      " : message has invalid parameter byte"
#define E_MONA_MSG_INVLD_PARA_WORD_TEXT      " : message has invalid parameter word"
#define E_MONA_MSG_INVLD_PARA_LONG_TEXT      " : message has invalid parameter long"
#define E_MONA_MSG_UNKNOWN_SUB_CODE_TEXT     " : message has unknown sub code"
#define E_MONA_MSG_UNSUPPORTED_SUB_CODE_TEXT " : message has unsupported sub code"
#define E_MONA_MSG_INVLD_SUB_PARA_COUNT_TEXT " : message has invalid sub parameter count"
#define E_MONA_MSG_INVLD_SUB_PARA_BYTE_TEXT  " : message has invalid sub parameter byte"
#define E_MONA_MSG_INVLD_SUB_PARA_WORD_TEXT  " : message has invalid sub parameter word"
#define E_MONA_MSG_INVLD_SUB_PARA_LONG_TEXT  " : message has invalid sub parameter long"
#define E_MONA_MSG_INCONS_PARA_COUNTS_TEXT   " : message has inconsistent parameter counts"
#define E_MONA_MSG_INCONS_LENGTH_TEXT        " : message has inconsistent length"
#define E_MONA_MSG_SIZE_EXCEED_TEXT          " : message size exceeded"
#define E_MONA_MSG_INV_CHECKSUM_TEXT         " : message checksum wrong"
                                            
#define E_MONA_RESRC_EXHAUSTED_TEXT          " : resource exhausted"
#define E_MONA_RESRC_ALREADY_EXISTS_TEXT     " : resource already exists"
#define E_MONA_RESRC_NOT_EXISTS_TEXT         " : resource does not exist"
#define E_MONA_RESRC_DEFECT_TEXT             " : resource is defect"
#define E_MONA_RESRC_OPERATION_FAILED_TEXT   " : resource operation failed"
#define E_MONA_RESRC_OPERATION_FATAL_TEXT    " : resource fatal operation"
#define E_MONA_RESRC_WAIT_TIMEOUT_TEXT       " : resource wait timeout"
#define E_MONA_RESRC_WAIT_FAILED_TEXT        " : resource wait failed"
#define E_MONA_RESRC_EOF_TEXT                " : resource end of file"
#define E_MONA_RESRC_NAME_LENGTH_EXCEED_TEXT " : resource name length exceeded"
#define E_MONA_RESRC_CANCELED_TEXT           " : resource canceled"
#define E_MONA_RESRC_NO_PDD_TEXT             " : resource no PDD"
#define E_MONA_RESRC_DISC_FULL_TEXT          " : resource disc is full"

#define E_MONA_CONN_EXHAUSTED_TEXT           " : connection exhausted"
#define E_MONA_CONN_LOST_TEXT                " : connection lost"
#define E_MONA_CONN_ABORT_TEXT               " : connection aborted"
#define E_MONA_CONN_REC_TIMEOUT_TEXT         " : connection receive timeout"
#define E_MONA_CONN_TARGET_NOT_FOUND_TEXT    " : connection target not found"
#define E_MONA_CONN_OUT_OF_RES_TEXT          " : connection out of resources"
                                            
#define E_MONA_INI_BLOCK_INVLD_NAME_TEXT     " : ini block has invalid name"
#define E_MONA_INI_BLOCK_NOT_EXISTS_TEXT     " : ini block does not exist"
#define E_MONA_INI_BLOCK_INVLD_SYNTAX_TEXT   " : ini block has invalid syntax"
#define E_MONA_INI_LINE_LENGTH_EXCEED_TEXT   " : ini line length exceeded"
#define E_MONA_INI_VAR_INVLD_NAME_TEXT       " : ini var has invalid name"
#define E_MONA_INI_VAR_NOT_EXISTS_TEXT       " : ini var does not exist"
#define E_MONA_INI_VAR_INVLD_SYNTAX_TEXT     " : ini var has invalid syntax"
#define E_MONA_INI_VAR_INVLD_VALUE_TEXT      " : ini var has invalid value"
#define E_MONA_INI_CF_CARD_NOT_PRESENT_TEXT  " : ini CF card not present"
#define E_MONA_INI_INVALID_CTRL_TYPE_TEXT    " : ini CTRL type is invalid"
#define E_MONA_INI_INVALID_CHECKSUM_TEXT     " : ini invalid checksum"
#define E_MONA_INI_INVALID_IP_CONFIG_TEXT    " : ini invalid IP configuration"

#define E_MONA_UNKNOWN_ERROR_TEXT            " : unknown error"
//@}

#endif // COMM_INTERFACE_ERRORS_H
