/*****************************************************************************/
/*                                                                           */
/*    --------------------------------------------------------               */
/*    PROFINET IO Controller Device Driver unter Linux                       */
/*    Version 1.0                                                            */
/*    --------------------------------------------------------               */
/*                                                                           */
/*    ---------------------------------                                      */
/*    KUNBUS  GmbH                                                           */
/*    ---------------------------------                                      */
/*                                                                           */
/*    ------------------------------------------------                       */
/*    Datum : 09.01.2015  Weber, Wolfgang                                    */
/*    ------------------------------------------------                       */
/*                                                                           */
/*    ------------------------------------------------------------           */
/*    Geaendert : 24.04.2015        Name : Weber, Wolfgang                   */
/*    Grund     :                                                            */
/*    ------------------------------------------------------------           */
/*                                                                           */
/*****************************************************************************/


/*---------------------------------------------------------------------------*/


#define __module__smp
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/kdev_t.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <asm/signal.h>
#include <linux/wait.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/compat.h>
#include <asm/dma.h>

#include <linux/pci.h>
#include <linux/poll.h>

#include <linux/vmalloc.h>

#include <asm/delay.h>
#include <asm/uaccess.h>
#include <asm/io.h>

#include <linux/cdev.h>
#include <linux/slab.h>
#include <linux/mm.h>

// DMA includes
#include <asm/dma.h>
#include <linux/dma-mapping.h>

#include "df_profinet.h"

#define DF_IOC_MAGIC  'C'
#define  DF_SPECIAL       _IO(DF_IOC_MAGIC, 10 )

#ifndef  DMA_BIT_MASK
#define DMA_BIT_MASK(n) (((n) == 64) ? ~0ULL : ((1ULL<<(n))-1))
#endif

MODULE_AUTHOR("Wolfgang M. Weber");
MODULE_DESCRIPTION("KUNBUS ProfiNET board driver");
MODULE_LICENSE( "Proprietary" );

int debug = 0;
module_param( debug, int, 0 );
MODULE_PARM_DESC(debug, "Debug output level");

#define NUM_OF_MSI_IRQ 1

#define VERSION_STR   "Linux V0.0.5"
char   *df_prnet_version = VERSION_STR;    

#define BUILD_VERSION_STR "Driver built on " __DATE__ " at " __TIME__
char   *df_prnet_build_version = BUILD_VERSION_STR;

/*---------------------------------------------------------------------------*/

struct df_pci_dev
{
    struct pci_dev *pci_dev;
    struct cdev *cdev;
    DESC_PCI *descpci;
    void *dfinstance;

    unsigned int df_prnet_we_ctl_acyclic_available;
    unsigned int df_prnet_we_dev_acyclic_available;

    struct semaphore  df_prnet_sem_ctl_acyclic;
    struct semaphore  df_prnet_sem_dev_acyclic;
    struct semaphore  df_prnet_sem_ctl_cyclic ;
    struct semaphore  df_prnet_sem_dev_cyclic ;

    wait_queue_head_t df_prnet_we_ctl_acyclic ;
    wait_queue_head_t df_prnet_we_dev_acyclic ;

    struct tasklet_struct tasklet;
};

static int     df_prnet_init_pci( struct pci_dev *pdev, const struct pci_device_id *id );
static void    df_prnet_remove_pci(struct pci_dev *pdev);
static int     df_prnet_open    ( struct inode * inode, struct file * file );
static int     df_prnet_release ( struct inode * inode, struct file * file );

#ifdef CONFIG_COMPAT
static long df_prnet_compatioctl( struct file *file, unsigned int prg_nr, unsigned long usr_addr );
#endif

#ifdef HAVE_UNLOCKED_IOCTL
static long    df_prnet_ioctl       ( struct file *filp, unsigned int prg_nr, unsigned long usr_addr );
#else
static int  df_prnet_ioctl ( struct inode * inode, struct file * filp, unsigned int prg_nr, unsigned long usr_addr );
#endif

/*---------------------------------------------------------------------------*/

static struct df_pci_dev df_boards[MAX_BOARDS];
static int df_prnet_major;
static dev_t df_prnet_dev;

/*---------------------------------------------------------------------------*/

static struct file_operations df_prnet_fops = {
        owner:                   THIS_MODULE,
#ifdef CONFIG_COMPAT
        compat_ioctl:            df_prnet_compatioctl,
#endif
#ifdef HAVE_UNLOCKED_IOCTL
        unlocked_ioctl:          df_prnet_ioctl,
#else
        ioctl:                   df_prnet_ioctl,
#endif
        open:                    df_prnet_open,
        release:                 df_prnet_release
};

static struct pci_device_id df_prnet_pci_tbl[]  = {
        { 0x1957, 0x0050, 0x4353, 0x434f, },
        { 0x1957, 0x0051, 0x4353, 0x434f, },
        { 0, }, /* terminate list */
};

MODULE_DEVICE_TABLE(pci, df_prnet_pci_tbl);

static struct pci_driver df_prnet_driver = {
        name:           "df_prnet-pci",
        id_table:       df_prnet_pci_tbl,
        probe:          df_prnet_init_pci,
        remove:         df_prnet_remove_pci,
};


/*---------------------------------------------------------------------------*/

void df_prnet_wake_up_we_ctl_acyclic( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    pdfdev->df_prnet_we_ctl_acyclic_available = 1;
    wake_up( &pdfdev->df_prnet_we_ctl_acyclic );
}

void df_prnet_wake_up_we_dev_acyclic( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    pdfdev->df_prnet_we_dev_acyclic_available = 1;
    wake_up( &pdfdev->df_prnet_we_dev_acyclic );
}

/*---------------------------------------------------------------------------*/
int df_prnet_wait_event_irp_we_ctl_acyclic( void *maininstance, long timeout )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;
    int ret;

    pdfdev->df_prnet_we_ctl_acyclic_available = 0;
    ret = wait_event_interruptible_timeout(pdfdev->df_prnet_we_ctl_acyclic, pdfdev->df_prnet_we_ctl_acyclic_available, msecs_to_jiffies(timeout) );

    return ( ret );
}

int df_prnet_wait_event_irp_we_dev_acyclic( void *maininstance, long timeout )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;
    int ret;

    pdfdev->df_prnet_we_dev_acyclic_available = 0;
    ret = wait_event_interruptible_timeout(pdfdev->df_prnet_we_dev_acyclic, pdfdev->df_prnet_we_dev_acyclic_available, msecs_to_jiffies(timeout) );

    return ( ret );
}
/*---------------------------------------------------------------------------*/

void df_prnet_ctl_acyc_up( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    up( &pdfdev->df_prnet_sem_ctl_acyclic );
}

void df_prnet_ctl_cyc_up( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    up( &pdfdev->df_prnet_sem_ctl_cyclic );
}

void df_prnet_dev_acyc_up( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    up( &pdfdev->df_prnet_sem_dev_acyclic );
}

void df_prnet_dev_cyc_up( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    up( &pdfdev->df_prnet_sem_dev_cyclic );
}

/*---------------------------------------------------------------------------*/

int df_prnet_ctl_acyc_down_int( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    if ( down_interruptible( &pdfdev->df_prnet_sem_ctl_acyclic ) == -EINTR)
        return(-EINTR);
    else
        return(0);
}

int df_prnet_ctl_cyc_down_int( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    if ( down_interruptible( &pdfdev->df_prnet_sem_ctl_cyclic ) == -EINTR)
        return(-EINTR);
    else
        return(0);
}

int df_prnet_dev_acyc_down_int( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    if ( down_interruptible( &pdfdev->df_prnet_sem_dev_acyclic ) == -EINTR)
        return(-EINTR);
    else
        return(0);
}

int df_prnet_dev_cyc_down_int( void *maininstance )
{
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)maininstance;

    if ( down_interruptible( &pdfdev->df_prnet_sem_dev_cyclic ) == -EINTR)
        return(-EINTR);
    else
        return(0);
}

/*---------------------------------------------------------------------------*/

int copyin( void *dest, const void  *source, unsigned long len )
{
    if ( copy_from_user( (void *)dest, (const void __user *)source, len ))
        return( -EFAULT );
    else
        return( 0 );
}

int copyout( void *dest, const void  *source, unsigned long len )
{
    if ( copy_to_user( (void *)dest, (const void __user *)source, len ))
        return( -EFAULT );
    else
        return( 0 );
}

void * df_prnet_calloc(unsigned long length)
{
    return kcalloc(1, length, GFP_KERNEL);
}

void df_prnet_free(void *addr)
{
    kfree(addr);
}


/*****************************************************************************/
/*       interrupt routine for shared IRQ                                    */
/*****************************************************************************/

static irqreturn_t df_prnet_interrupt( int level, void *pinst)
{	
    struct df_pci_dev *pdfdev = (struct df_pci_dev *)pinst;

    if ( debug ) printk( "--> DF_PRNET_ISR\n" );

    if ( df_prnet_irq (pdfdev->dfinstance) == 1 )
    {
        tasklet_schedule(&pdfdev->tasklet);     // Tasklet (Interruptnachbearbeitung) aufrufen
    }
    else
    {
        return IRQ_NONE;
    }

    return IRQ_HANDLED;
}

/*****************************************************************************/
/*              O P E N   function                                           */
/*****************************************************************************/
static int df_prnet_open( struct inode *inode, struct file *file )
{

    int            board;
    struct df_pci_dev *pdfdev;

    board = MINOR(inode->i_rdev);
    if (board > MAX_BOARDS)
        return -EFAULT;

    pdfdev = &df_boards[board];
    file->private_data = (void *)pdfdev;

    if ( debug ) printk("--> df_prnet OPEN board=%d\n",  board );

    if ( ! pdfdev->descpci->init )
        return E_100;

    init_waitqueue_head(&pdfdev->df_prnet_we_ctl_acyclic );
    init_waitqueue_head(&pdfdev->df_prnet_we_dev_acyclic );

    if ( debug ) printk("<-- df_prnet OPEN-Ende board=%d\n", board );


    return( 0 );
}

/*****************************************************************************/
/*       C L O S E   function                                                */
/*****************************************************************************/
static int df_prnet_release( struct inode *inode, struct file  *file )
{
    struct df_pci_dev *pdfdev;

    pdfdev = (struct df_pci_dev *)file->private_data;

    if ( debug ) printk("CLOSE Dev\n");

    return( 0 );

}

/*****************************************************************************/
/*       I O C T L  - Funktion                                               */
/*       Verzweigt in die verschiedenen Aufrufe                              */
/*****************************************************************************/
#ifdef CONFIG_COMPAT
static long df_prnet_compatioctl( struct file *file, unsigned int prg_nr, unsigned long usr_addr )
{
    struct df_pci_dev *pdfdev;

    pdfdev = (struct df_pci_dev *)file->private_data;

    if (debug) printk("IOCTL: prg_nr=%x user_addr=%lx\n", prg_nr, usr_addr);

    if ( prg_nr != DF_SPECIAL )  return( E_111 );

    return( df_prnet_ioctl_switch( pdfdev->dfinstance, compat_ptr(usr_addr)));
}
#endif

#ifdef HAVE_UNLOCKED_IOCTL
static long df_prnet_ioctl( struct file *file, unsigned int prg_nr, unsigned long usr_addr )
#else
static int  df_prnet_ioctl ( struct inode * inode, struct file * file, unsigned int prg_nr, unsigned long usr_addr )
#endif

{
    struct df_pci_dev *pdfdev;

    pdfdev = (struct df_pci_dev *)file->private_data;

    if (debug) printk("IOCTL: prg_nr=%x user_addr=%lx\n", prg_nr, usr_addr);

    if ( prg_nr != DF_SPECIAL )  return( E_111 );

    return( df_prnet_ioctl_switch( pdfdev->dfinstance, __user (caddr_t)usr_addr ));
}

void KeStallExecutionProcessor(unsigned long dma_wait_stall_time_us)
{

    udelay(dma_wait_stall_time_us);
}

/*****************************************************************************/
/*        df_prnet - INIT - Routine                                              */
/*****************************************************************************/

static int df_prnet_init_pci( struct pci_dev *pdev, const struct pci_device_id *id )
{
    int irq;
    int minor;
    int ret = -EIO;
    struct df_pci_dev *pdfdev = NULL;
    struct cdev *cdev = NULL;
    DESC_PCI *pdescpci = NULL;

    unsigned char  *dest_ptr;

    if (debug) printk( "df_prnet_init_pci Board:\n" );

    if ( !pdev )
    {
        printk( "PROFINET IO Controller: ERROR pdev=NULL \n" );
        return( ret );
    }

    for (minor=0; minor<MAX_BOARDS; minor++)
    {
        if (df_boards[minor].pci_dev == NULL)
        {
            pdfdev = &df_boards[minor];
            break;
        }
    }

    if (pdfdev == NULL)
    {
        dev_err(&(pdev->dev),"PROFINET IO Controller: No Resources available" );
        return ret;
    }

    pdfdev->pci_dev = pdev;

    /* allocate struct cdev */
    cdev = cdev_alloc();

    /* initialise struct cdev */
    cdev_init(cdev, &df_prnet_fops);
    cdev->owner = THIS_MODULE;

    /* register cdev */
    ret = cdev_add(cdev, MKDEV(df_prnet_major, minor), 1);
    if (ret < 0) {
        dev_err(&(pdev->dev), "Can't register character device\n");
        goto error;
    }
    pdfdev->cdev = cdev;

    if ( pci_enable_device( pdev ) != 0 )
    {
        goto error;
    }

    printk( "Board %d\n", minor );

    pdescpci = kcalloc(1, sizeof(DESC_PCI), GFP_KERNEL);
    if (pdescpci == NULL)
    {
        dev_err(&(pdev->dev), "Can't allocate memory\n");
        goto error;
    }
    pdfdev->descpci = pdescpci;

    pdescpci->pci_ioaddr    = pci_resource_start( pdev, 0 );
    pdescpci->pci_io_size   = pci_resource_len(   pdev, 0 );
    pdescpci->pci_memaddr   = pci_resource_start( pdev, 1 );
    pdescpci->pci_mem_size  = pci_resource_len(   pdev, 1 );

    printk(  "IO=%x[%x], DPR=%x[%lx]\n",
             pdescpci->pci_ioaddr, pdescpci->pci_io_size, pdescpci->pci_memaddr, pdescpci->pci_mem_size );

    if ( request_mem_region( pdescpci->pci_ioaddr, pdescpci->pci_io_size, "df_prnet" ) == NULL )
    {
        printk( " IO-addr 0x%x could not be mapped !!!\n", pdescpci->pci_ioaddr );
        goto error;
    }
    else
        printk( " IO-addr 0x%x mapped !!!\n", pdescpci->pci_ioaddr );

    if ( request_mem_region( pdescpci->pci_memaddr, pdescpci->pci_mem_size, "df_prnet" ) == NULL )
    {
        printk( " MEM-addr 0x%x could not be mapped !!!\n", pdescpci->pci_memaddr );
        goto error;
    }
    else
        printk( " MEM-addr 0x%x mapped !!!\n", pdescpci->pci_memaddr );


    pdescpci->virt_io_addr    = ioremap( pdescpci->pci_ioaddr, pdescpci->pci_io_size );
    printk( " DF ProfiNET - alloc io address %px\n ", pdescpci->virt_io_addr );
    pdescpci->virt_board_addr = ioremap( pdescpci->pci_memaddr, pdescpci->pci_mem_size );
    printk( " DF ProfiNET - alloc board address. %px\n ", pdescpci->virt_board_addr );

    pdescpci->virt_base_addr  = pdescpci->virt_board_addr;
    dest_ptr = pdescpci->virt_base_addr;

    printk( " DF ProfiNET - trying to use 32Bit DMA ..." );

    if ( !pci_set_dma_mask( pdev, DMA_BIT_MASK(32)) )
    {
        printk( " - we are using 32Bit DMA\n" );
        pci_set_consistent_dma_mask( pdev, DMA_BIT_MASK(32));
    }
    else
    {
        printk( " - we can't use 32Bit DMA trying 24Bit DMA\n" );
        if ( !pci_set_dma_mask( pdev, DMA_BIT_MASK(24)) )
        {
            printk( " - we are using 24Bit DMA\n" );
            pci_set_consistent_dma_mask( pdev, DMA_BIT_MASK(24));
        }
        else
        {
            printk( " - we can't use 24Bit DMA --> abort\n" );
            return -EIO;
        }
    }
    printk( " DF ProfiNET - trying to allocate DMA buffer ...\n" );

    // We use DMA 1MB
    pdescpci->dma_mem_size = 1024*1024;

    pdescpci->cpu_addr = pci_alloc_consistent( pdev, pdescpci->dma_mem_size, &pdescpci->dma_handle );

    if ( pdescpci->cpu_addr == 0 )
    {
        printk( "  failed !!!\n" );
        goto error;
    }
    else
        printk( "  OK !!!\n" );


    printk( " DF ProfiNET - allocated DMA buffer at %px\n",  pdescpci->cpu_addr );

    pci_set_master(pdev);

    //--------------------------------------------------------------------------------
    //initiate DF PROFINET registers

    pdfdev->dfinstance = df_prnet_initiate_register( (void *)pdfdev, pdfdev->descpci );

    if (pdfdev->dfinstance == NULL)
    {
        printk( "  df_prnet_initiate_register failed\n" );
        goto error;
    }

    irq = pci_alloc_irq_vectors(pdev, NUM_OF_MSI_IRQ, NUM_OF_MSI_IRQ, PCI_IRQ_ALL_TYPES);
    if (irq < 0)
    {
        dev_err(&(pdev->dev), "failed to get interrupts");
        goto error;
    }

    pdescpci->pci_irq_line = pci_irq_vector(pdev, 0);

#ifdef SA_INTERRUPT
    if ( request_irq( pdescpci->pci_irq_line, (void *)df_prnet_interrupt, SA_INTERRUPT | SA_SHIRQ, "df_prnet", pdfdev ))
#else
#ifdef IRQF_DISABLED
    if ( request_irq( pdescpci->pci_irq_line, (void *)df_prnet_interrupt, IRQF_DISABLED | IRQF_SHARED, "df_prnet", pdfdev ))
#else
    if (request_irq( pdescpci->pci_irq_line, (void *)df_prnet_interrupt, IRQF_SHARED, "df_prnet", pdfdev))
#endif
#endif
    {
        printk( "DF ProfiNET - IRQ %d is not free\n", pdescpci->pci_irq_line );
        goto error;
    }

    sema_init( &pdfdev->df_prnet_sem_ctl_acyclic, 1 );
    sema_init( &pdfdev->df_prnet_sem_dev_acyclic, 1 );
    sema_init( &pdfdev->df_prnet_sem_ctl_cyclic,  1 );
    sema_init( &pdfdev->df_prnet_sem_dev_cyclic,  1 );

    tasklet_init(&pdfdev->tasklet, df_prnet_do_tasklet, (unsigned long)pdfdev->dfinstance);

    pdescpci->init = TRUE;

    printk( "DF ProfiNET -sizeof DESC_PCI=%zu\n", sizeof(DESC_PCI) );

    return( 0 );

error:
    if (pdfdev->dfinstance != NULL)
    {
        kfree(pdfdev->dfinstance);
    }

    if (pdescpci != NULL)
    {
        if (pdescpci->virt_board_addr)
        {
            iounmap( pdescpci->virt_board_addr );
        }
        if (pdescpci->virt_io_addr)
        {
            iounmap( pdescpci->virt_io_addr );
        }
        if (pdescpci->cpu_addr)
        {
            pci_free_consistent(pdev, pdescpci->dma_mem_size, pdescpci->cpu_addr, pdescpci->dma_handle);
        }
        kfree(pdescpci);
    }

    if (cdev != NULL)
    {
        cdev_del(cdev);
    }

    if (pdfdev != NULL)
    {
        memset(pdfdev, 0, sizeof(struct df_pci_dev));
    }

    return -EIO;
}

/*****************************************************************************/
/*        df_prnet - REMOVE - Routine                                              */
/*****************************************************************************/
static void df_prnet_remove_pci(struct pci_dev *pdev)
{
    int minor;
    struct df_pci_dev *pdfdev = NULL;
    DESC_PCI *pdescpci = NULL;

    if (debug) printk( "df_prnet_release_pci Board:\n" );


    for (minor=0; minor<MAX_BOARDS; minor++)
    {
        if (df_boards[minor].pci_dev == pdev)
        {
            pdfdev = &df_boards[minor];
            break;
        }
    }

    if (pdfdev == NULL)
    {
        dev_err(&pdev->dev, "Resource not found");
        return;
    }
    pdescpci = pdfdev->descpci;

    tasklet_kill(&pdfdev->tasklet);

    cdev_del(pdfdev->cdev);

    df_prnet_initiate_unregister(pdfdev->dfinstance);

    if ( pdescpci->init == TRUE )
    {
        free_irq( pci_irq_vector(pdev, 0), pdfdev ) ;
        pci_free_irq_vectors(pdev);

        printk( "DF ProfiNET - free board address. %px\n ", pdescpci->virt_board_addr );
        iounmap( pdescpci->virt_board_addr );
        printk( "DF ProfiNET - free io address %px\n ", pdescpci->virt_io_addr );
        iounmap( pdescpci->virt_io_addr    );
        printk( "DF ProfiNET - free irq %d\n ", pdescpci->pci_irq_line );

        if ( pdescpci->pci_memaddr ) release_mem_region( pdescpci->pci_memaddr, pdescpci->pci_mem_size );
        if ( pdescpci->pci_ioaddr )  release_mem_region( pdescpci->pci_ioaddr,  pdescpci->pci_io_size  );

        printk( "DF ProfiNET - free dma memory\n " );

        pci_free_consistent( pdev, pdescpci->dma_mem_size, pdescpci->cpu_addr, pdescpci->dma_handle );

        kfree(pdescpci);
    }
    
    kfree(pdfdev->dfinstance);

    printk("Remove Board %u\n", minor);
}

/*****************************************************************************/
/*       Loadabel driver routine init_module                                 */
/*****************************************************************************/

static int __init ProfiNET_Init( void )
{
    int ret;

    memset(df_boards, 0, sizeof(df_boards));

    printk( "DF ProfiNET device driver (c) Copyright KUNBUS GmbH Karlsruhe %s (%s)\n",
            df_prnet_version, df_prnet_build_version );

    /* allocate (several) major number */
    ret = alloc_chrdev_region(&df_prnet_dev, 0, MAX_BOARDS, "df_profinet");
    if (ret < 0) {
        printk( "ProfiNET board driver not registered\n" );
        return ret;
    }
    df_prnet_major = MAJOR(df_prnet_dev);

    ret = pci_register_driver (&df_prnet_driver);

    printk( "MAJOR-No. : %d\n", df_prnet_major );

    return 0;
}

module_init( ProfiNET_Init );

/*****************************************************************************/
/*       Loadabel driver routine cleanup_module                              */
/*****************************************************************************/

static void __exit ProfiNET_Exit( void )
{
    pci_unregister_driver (&df_prnet_driver);

    unregister_chrdev_region(df_prnet_dev, MAX_BOARDS);

    printk( "DF ProfiNET - driver stopped with MAJOR-No. %d\n ", df_prnet_major );
}

module_exit( ProfiNET_Exit );


