/*---------------------------------------------------------------------------*/

//-----------------------------

//*

#include <linux/errno.h>
#include <linux/string.h>

//*/
//-------------------------------


#include "common_define.h"
#include "PN_LibMain.h"
#include "pnio_ctl.h"
#include "comm_interface.h"
#include "comm_interface_errors.h"
#include "dfprnet_mod.h"

extern int debug;

static void           df_prnet_set_in_out_bound_values( void *instance, INT32U set );

extern int printk(const char *fmt, ...);

/*---------------------------------------------------------------------------*/


static unsigned int SWITCH_ENDIAN_32_BIT( unsigned int swap_value )
{   
    unsigned int ret;

    //   if ( debug ) ret = debug;

    ret = (((swap_value>>24)&0xff) | ((swap_value<<8)&0xff0000) | ((swap_value>>8)&0xff00) | ((swap_value<<24)&0xff000000 ));
    return(ret);
}

void df_prnet_memory_dump( unsigned char *ptr, unsigned int len, char* buffer )
{
    int i;

    if ( debug == 0 ) return;

    printk("\n\nDF ProfiNET  Memory Dump %s\n\n", buffer);
    printk("\n %px:", ptr );

    for ( i=0; i<len; i++ )
    {
        if ( i%8 == 0 && i != 0)
        {
            printk(" - ");
        }
        if ( i%16 == 0 && i != 0 )
        {
            printk("\n %px: ", ptr );
        }
        else
        {
            printk(" ");
        }
        printk("%02x", *ptr++);
    }
    printk(" -\n\n");

}

void df_prnet_memory_dump_reverse( unsigned char *ptr, unsigned int len )
{
    int i;

    if ( debug == 0 ) return;

    ptr = ptr+len-1;
    printk("\n\nDF ProfiNET Memory Dump reverse\n\n");
    printk("\n %px:", ptr );

    for ( i=0; i<len; i++ )
    {
        if ( i%8 == 0 && i != 0)
        {
            printk(" - ");
        }
        if ( i%16 == 0 && i != 0 )
        {
            printk("\n %px: ", ptr );
        }
        else
        {
            printk(" ");
        }
        printk("%02x", *ptr--);
    }
    printk(" -\n\n");

}

/*!
 * Routine Description:
 *   - This routine initializes the inbound window with the necessary values, the
 *     driver must tell the firmware after FW_READY_INDICATION was detected.
 *     The values are:
 *     - driver type
 *     - version info 
 *     - interrupts to PC enabled
 *     - outbound window is ready
 * 
 * Arguments:
 *   - DEVICE_EXTENSION* pDevExt   Address of the device extension with all kind of informations
 * 
 * Return Value:
 *    - none
 * 
 * Details and remarks for this function:
 */
/*****************************************************************************/
/*        set_in_out_bound_values                                            */
/*****************************************************************************/
VOID df_prnet_set_in_out_bound_values(void * instance, DWORD set)
{
    DESC_DFPRNET *pdfdesc = (DESC_DFPRNET *)instance;

    if ( debug ) printk( "--> df_prnet_set_in_out_bound_values\n" );

    // copy pci relevant variables into local desc_prnet
    //RtlCopyMemory(desc_prnet[board], desc_pci[board], sizeof(DESC_PCI));
    //memcpy((void*)&desc_prnet[board], (const void*)&desc_pci[board], sizeof(DESC_PCI));

    if (set)
    {
        //in addition to the values in the Outbound window, we reinit the PCI registers in the processor.
        //they might be deleted after a hardware reset of the processor. Hardware resets are triggered manually
        //or by the HW watchdog.

        //df_prnet_memory_dump( (unsigned char *)pdfdesc->pinbound_global, 128,  "pinbound_global" );
        //df_prnet_memory_dump( (unsigned char *)pdfdesc->poutbound_global, 128,  "poutbound_global" );

        //if ( debug ) printk( "--> FwNeedsReadyCheck = 0x%x\n", pdfdesc->FwNeedsReadyCheck );
        //if ( debug ) printk( "--> pinbound_global->InterruptToPC_Enable = 0x%x\n", pdfdesc->pinbound_global->InterruptToPC_Enable );

        pdfdesc->FwNeedsReadyCheck = FALSE;

        pdfdesc->pinbound_global->DriverType               = LINUX_DRIVER;
        //pdfdesc->pinbound_global->DriverVersion1         = FILE_MAJVER;
        //pdfdesc->pinbound_global->DriverVersion2         = FILE_MINVER;
        //pdfdesc->pinbound_global->DriverVersion3         = FILE_MAJBLD;
        //pdfdesc->pinbound_global->DriverVersion4         = FILE_BUILD;
        pdfdesc->pinbound_global->InterruptToPC_Enable   = INTERRUPT_TO_PC_ENABLED;
        pdfdesc->pinbound_global->Outbound_Ready         = OUTBOUND_READY_INDICATION;

        pdfdesc->poutbound_global->ErrorClass = ERROR_CLASS_NO_ERROR;
        pdfdesc->poutbound_global->Error      = RET_SUCCESSFULL;

        //pdfdesc->poutbound_devglobal->ErrorClass = ERROR_CLASS_NO_ERROR;
        //pdfdesc->poutbound_devglobal->Error      = RET_SUCCESSFULL;
    }
    else  // reset; only on driver deactivation, stop or BSOD
    {
        pdfdesc->FwNeedsReadyCheck = TRUE;

        pdfdesc->pinbound_global->DriverType             = 0;
        pdfdesc->pinbound_global->DriverVersion1         = 0;
        pdfdesc->pinbound_global->DriverVersion2         = 0;
        pdfdesc->pinbound_global->DriverVersion3         = 0;
        pdfdesc->pinbound_global->DriverVersion4         = 0;
        pdfdesc->pinbound_global->InterruptToPC_Enable   = 0;
        pdfdesc->pinbound_global->Outbound_Ready         = 0;

        // on driver deactivation, stop or BSOD it makes no sense
        // to write on the outbound window
    }
    if ( debug ) printk( "<-- dfctrl_set_in_out_bound_values\n" );
}
//--------------------------------------------------------------------------------

void * df_prnet_initiate_register( void *maininstance, DESC_PCI *pdesc )
{

    VREG_32 reg_val;
    unsigned int *preg_val;
    DWORD offset;
    DESC_DFPRNET *pdfdesc = NULL;
    int isPCI = 1;  // To decide if Hardware is PCI or PCIe

    PMPC8536E_DMA_REGS  pdma;

    // This type is only used as pointer to an already existing memory area, allocated in
    // DRAM of the hardware component.
    P_INBOUND_WINDOW_TYPE pinbound;

    // This type is only used as pointer to an already existing memory area, allocated in
    // contiguous PC memory.
    P_OUTBOUND_WINDOW_TYPE poutbound;

    pdfdesc = df_prnet_calloc(sizeof(DESC_DFPRNET));
    if (pdfdesc == NULL)
    {
        printk( "--> no memory\n" );
        return NULL;
    }

    if ( debug ) printk( "--> df_prnet_initiate_register\n" );

    pdfdesc->pdescpci = pdesc;
    pdfdesc->maininstance = maininstance;

    pinbound     = (P_INBOUND_WINDOW_TYPE)pdesc->virt_base_addr;
    poutbound   = (P_OUTBOUND_WINDOW_TYPE)pdesc->cpu_addr;

    if ( debug ) printk( " -- df_prnet_initiate_register: pinbound=%px, poutbound=%px\n", pinbound, poutbound );

    /// - Set some register for DMA and outbound window usage
    /// - Enable the Outbound PCI Window with the approriate offset and size

    // Check on runtime which board is in System.
    // If Register PEXITAR1 is 0x0 than the hardware is a PCI hardware, otherwise it's a PCIe card.
    reg_val = SWITCH_ENDIAN_32_BIT((*((VREG_32*)(pdesc->virt_io_addr + PCIE_PEXITAR1))));
    if(reg_val != 0)
    {
        isPCI = 0;
    }


    reg_val = SWITCH_ENDIAN_32_BIT(PCI_OW_BASE >> 12);


    if(isPCI == 1)
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCI_OW1_BASE_ADDR);
    else
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCIE_OW1_BASE_ADDR);

    *preg_val = reg_val;

    if ( debug ) printk( "\ndf_prnet: PCI_OW1_BASE_ADDR  preg_val=%px *preg_val=%x reg_val=%u\n", preg_val, *preg_val, reg_val );
    // dma_handle ist 32Bit Wert auch bei 64 Bit System, deshalb reicht shift 12


    reg_val = SWITCH_ENDIAN_32_BIT(pdesc->dma_handle >> 12);

    if(isPCI == 1)
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCI_OW1_ADDR_OFS);
    else
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCIE_OW1_ADDR_OFS);

    *preg_val = reg_val;

    if ( debug ) printk( "\ndf_prnet: PCI_OW1_ADDR_OFS preg_val=%px *preg_val=%x reg_val=%u\n", preg_val, *preg_val, reg_val );


    reg_val = 0;


    if(isPCI == 1)
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCI_OW1_EXT_ADDR_OFS);
    else
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCIE_OW1_EXT_ADDR_OFS);

    *preg_val = reg_val;


    reg_val = SWITCH_ENDIAN_32_BIT(PCI_OW_ATTR_ENABLE | PCI_OW_ATTR_R_MEM | PCI_OW_ATTR_W_MEM | PCI_OW_WIN_SIZE_1M);


    if(isPCI == 1)
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCI_OW1_ATTRIBUTES);
    else
        preg_val = (unsigned int *)(pdesc->virt_io_addr + PCIE_OW1_ATTRIBUTES);

    *preg_val = reg_val;

    /// Initialize the DMA registers
    pdfdesc->pdma     = (PMPC8536E_DMA_REGS)((VREG_32*)(pdesc->virt_io_addr + MPC8536E_DMA1));

    pdma = pdfdesc->pdma;

    if ( debug ) printk( "\ndf_prnet: pdma %px len=%ld\n",  pdma, sizeof(MPC8536E_DMA_REGS) );

    pdfdesc->pdma_sts = ((VREG_32*)(pdesc->virt_io_addr + MPC8536E_DMA_STAT));

    pdma->MR   = DMA_REG_CLEAR;
    pdma->MR   = DMA_BW_CONTROLL | DMA_DIRECT_MODE | DMA_SRW_ENABLE; // write to destination address register starts DMA
    pdma->SATR = DMA_ATTR_SNOOP;
    pdma->DATR = DMA_ATTR_SNOOP;

    /// Initialize the DMA registers
    pdfdesc->pdev_dma     = (PMPC8536E_DMA_REGS)((VREG_32*)(pdesc->virt_io_addr + MPC8536E_DMA3));
    pdfdesc->pdev_dma_sts = ((VREG_32*)(pdesc->virt_io_addr + MPC8536E_DMA_STAT));
    pdfdesc->pdev_dma->MR   = DMA_REG_CLEAR;
    pdfdesc->pdev_dma->MR   = DMA_BW_CONTROLL | DMA_DIRECT_MODE | DMA_SRW_ENABLE; // write to destination address register starts DMA
    pdfdesc->pdev_dma->SATR = DMA_ATTR_SNOOP;
    pdfdesc->pdev_dma->DATR = DMA_ATTR_SNOOP;

    // set source address; this address will not change during operation of the driver and is
    // determined by the structure layed over the outbound window rsp contiguous memory
    offset  = FIELD_OFFSET(struct TAG_P_OUTBOUND_WINDOW_TYPE, CyclicData) + FIELD_OFFSET(OUTBOUND_CYCLIC_DATA_TYPE, OutputData) + FIELD_OFFSET(PNIO_PROCESSIMAGE_TYPE, ProcessImage);
    reg_val = SWITCH_ENDIAN_32_BIT(DEFAULT_OUTBOUND_WINDOW_BASE + offset);
    //SWITCH_ENDIAN_32_BIT(reg_val);
    pdfdesc->dma_source_addr = reg_val;

    if ( debug ) printk( "\ndf_prnet: DEFAULT_OUTBOUND_WINDOW_BASE - offset %x reg_val=%x pdesc->dma_source_addr=%x\n", offset, reg_val,  pdfdesc->dma_source_addr );

    // set virtual source address (for copy from user buffer to contiguous memory source area
    // as DMA source address for output data )
    pdfdesc->contig_dma_src_addr = (PVOID)&poutbound->CyclicData.OutputData.ProcessImage;

    // set destination address base in firmware notation;
    // the variing pair/index offset is added in each DMA then
    if(isPCI == 1)
        reg_val = SWITCH_ENDIAN_32_BIT((*((VREG_32*)(pdesc->virt_io_addr + PITAR1))));
    else
        reg_val = SWITCH_ENDIAN_32_BIT((*((VREG_32*)(pdesc->virt_io_addr + PCIE_PEXITAR1))));
	

    //SWITCH_ENDIAN_32_BIT(reg_val);
    pdfdesc->dma_simpson_addr = ((reg_val << 12) + FIELD_OFFSET(struct TAG_P_INBOUND_WINDOW_TYPE, CyclicData));


    // set source address; this address will not change during operation of the driver and is
    // determined by the structure layed over the outbound window rsp contiguous memory
    offset  = FIELD_OFFSET(struct TAG_P_OUTBOUND_WINDOW_TYPE, CyclicDataDev) + FIELD_OFFSET(OUTBOUND_DEV_CYCLIC_DATA_TYPE, InputData);
    reg_val = SWITCH_ENDIAN_32_BIT(DEFAULT_OUTBOUND_WINDOW_BASE + offset);
    //SWITCH_ENDIAN_32_BIT(reg_val);
    pdfdesc->dev_dma_source_addr = reg_val;

    // set virtual source address (for copy from user buffer to contiguous memory source area
    // as DMA source address for output data )
    pdfdesc->dev_contig_dma_src_addr = (PVOID)&poutbound->CyclicDataDev.InputData;

    // set destination address base in firmware notation;
    // the variing pair/index offset is added in each DMA then
    if(isPCI == 1)
        reg_val = SWITCH_ENDIAN_32_BIT((*((VREG_32*)(pdesc->virt_io_addr + PITAR1))));
    else
        reg_val = SWITCH_ENDIAN_32_BIT((*((VREG_32*)(pdesc->virt_io_addr + PCIE_PEXITAR1))));

    //SWITCH_ENDIAN_32_BIT(reg_val);
    pdfdesc->dev_dma_simpson_addr = ((reg_val << 12) + FIELD_OFFSET(struct TAG_P_INBOUND_WINDOW_TYPE, CyclicDataDev));

    // set virtual dest address for acyclic jobs
    pdfdesc->interface_addr = &pinbound->Interface;
    pdfdesc->dev_interface_addr = &pinbound->InterfaceDev;

    // set virtual address for global info, inbound and outbound data
    pdfdesc->pinbound_global  = &pinbound->Globals;
    pdfdesc->pinbound_cyclic  = &pinbound->CyclicData;
    pdfdesc->pinbound_devcyclic  = &pinbound->CyclicDataDev;
    pdfdesc->poutbound_global = &poutbound->Globals;
    pdfdesc->poutbound_cyclic = &poutbound->CyclicData;
    pdfdesc->poutbound_devcyclic = &poutbound->CyclicDataDev;

    df_prnet_set_in_out_bound_values(pdfdesc, INBOUND_VALUES_SET);

    if ( debug ) printk( "<-- df_prnet_initiate_register\n" );

    return pdfdesc;
}
//--------------------------------------------------------------------------------

unsigned int df_prnet_initiate_unregister (void *instance) 
{ 
    DESC_DFPRNET *pdfdesc = (DESC_DFPRNET *)instance;    
    unsigned int *preg_val;
    VREG_32 reg_val;
    int isPCI = 1;  // To decide if Hardware is PCI or PCIe

    /// - Tell the controller firmware, that the driver is going by reset 
    ///   some flags in the inbound window e.g. the Interrupts to the PC etc.
    df_prnet_set_in_out_bound_values(pdfdesc, INBOUND_VALUES_RESET);

    // Check on runtime which board is in System.
    // If Register PEXITAR1 is 0x0 than the hardware is a PCI hardware, otherwise it's a PCIe card.
    reg_val = SWITCH_ENDIAN_32_BIT((*((VREG_32*)(pdfdesc->pdescpci->virt_io_addr + PCIE_PEXITAR1))));
    if(reg_val != 0)
    {
        isPCI = 0;
    }

    /// - Disable the Outbound PCI Window
    if(isPCI == 1)
        preg_val = (unsigned int *)(pdfdesc->pdescpci->virt_io_addr + PCI_OW1_ATTRIBUTES);
    else
        preg_val = (unsigned int *)(pdfdesc->pdescpci->virt_io_addr + PCIE_OW1_ATTRIBUTES);

    
    reg_val = SWITCH_ENDIAN_32_BIT(*preg_val);
    reg_val &= ~PCI_OW_ATTR_ENABLE;
    
    *preg_val = SWITCH_ENDIAN_32_BIT(reg_val);

    return 0;
}


/*****************************************************************************/
/*       ISR Bottom Halves Tasklet                                           */
/*****************************************************************************/

unsigned int df_prnet_irq (void *instance)  //--> Uebergabe IntToHandle !!!!
{ 
    int    i;
    int    ret;
    PUCHAR pregs;
    VREG_32       IRQ_reg_val;
    DESC_DFPRNET *pdfdesc = (DESC_DFPRNET *)instance;

    if ( debug ) printk("--> DF_PRNET_ISR(2) IRQ \n" );

    pregs = (unsigned char *)(pdfdesc->pdescpci->virt_io_addr); //RegsBase;
    /// - Check if DF Board has caused IRQ. If not, bail out with FALSE#ifdef DFPROFI_SIMULATION
    IRQ_reg_val = *((VREG_32*)(pregs + IRQ_STATUS_SUMMARY));  // could also be detected by  IRQ_MSI_STATUS

    if (IRQ_reg_val)  // MSI 0-7 are routed to IRQ_OUT and one of them triggered. Now check which ones
    {
        for (i = 0; i < 8; i++)
        {
            if (IRQ_reg_val & (1 << (7 - i)))
            {
                /// - Quit the IRQ reason on the PPC by reading MSIR(i);
                ///   this resets the bit(s) and thus the IRQ (level to 1: System is level triggerd, low active)
                pdfdesc->MSI_Reg[i] = *((VREG_32*)(pregs + IRQ_RESET_MSIR(i)));
            }
        }

        pdfdesc->MSI_StatusReg |= IRQ_reg_val;
        ret = 1;
    }
    else
    {
        ret = 0xff;
    }

    if ( debug ) printk("--> DF_PRNET_ISR(2) IRQ  ret=%d\n", ret );

    return( ret );
}


/*****************************************************************************/
/*       ISR Bottom Halves Tasklet                                           */
/*****************************************************************************/

void df_prnet_do_tasklet (unsigned long instance)  //--> Uebergabe IntToHandle !!!!
{
    //WDFREQUEST                 Request           = NULL;  /*! The current write job on the specific channel */
    //PDEVICE_EXTENSION          pDevExt;

    TBOOL                    complete;
    UCHAR                      i;
    DWORD                      msi;
    DWORD                      msg;
    DWORD                      mask;
    P_COMMON_INTERFACE_TYPE    interface_addr    = 0;
    DESC_DFPRNET *pdfdesc = (DESC_DFPRNET *)instance;

    size_t length;
    //size_t information;
    INTERFACE_STD_HEADER_TYPE p_header;



    if ( debug ) printk("--> IRQ - Halves Bottom DF_PRNET_DO_TASKLET\n" );

    msi = pdfdesc->MSI_StatusReg;

    // clear the local interrupt flags. We can do this first, because the MSI will not be reloaded.
    pdfdesc->MSI_StatusReg = 0;

    mask = 0x80;
    for (i = 0; i < 8; i++, mask >>= 1)
    {
        if (msi & mask)
        {
            msg = SWITCH_ENDIAN_32_BIT(pdfdesc->MSI_Reg[i]);
            //SWITCH_ENDIAN_32_BIT(msg);

#if 0 // msg is at the moment not used and the coding is not defined
            for (j = 0; j < 32; j++)
            {
                if (msg & (1 << j))
                {
                    break;
                }
            }
#endif

            if ( debug ) printk("--> IRQ - Halves Bottom DF_PRNET_DO_TASKLET i=%d\n", i );


            switch (i)
            {
            case 0:   // from controller only with (so far dummy) message 7
                //Request = InterlockedExchangePointer(&pDpcContext->Request, NULL);
                interface_addr = pdfdesc->interface_addr;
                break;
            case 1:  // from device mode (sent in IRQ_TRIGGER_1 reg with msi and message from header.Reserved[0]
                //Request = InterlockedExchangePointer(&pDpcContext->DevRequest, NULL);
                interface_addr = pdfdesc->dev_interface_addr;
                break;
            case 2:  // not used
            case 3:
            case 4:
            case 5:
            case 6:
            case 7:
            default:
                //Request = NULL;
                break;
            }

            if ( debug ) printk( " -- DF_PRNET_DO_TASKLET MSI %d : MSG 0x%x\n", i, msg);

            complete = TRUE;

            length = INTERFACE_STD_HEADER_TYPE_SIZE + interface_addr->StdHeader.Length;


            // copy acyclic command header + data from the controller DRAM interface buffer to the user space

            if ( debug ) printk( " -- DF_PRNET_DO_TASKLET vor RtlCopyMemory interface_addr=%px length=%ld\n", interface_addr, length );

            memcpy(&p_header, interface_addr, INTERFACE_STD_HEADER_TYPE_SIZE );

            // in case of a new profinet start we must check for current process image length, since it could
            // have changed due to a different xml configuration file used now.
            // if we would use different shared message interrupts, we wouldn't need the if checks

            if ( p_header.Command == CMD_PROFINET            && p_header.SubCommand == SUBCMD_PROFINET_START &&
                    ((p_header.ErrorClass == ERROR_CLASS_NO_ERROR && p_header.Error == RET_SUCCESSFULL))          &&
                    pdfdesc->pinbound_global->PNBusRunning != 0 )
            {
                VREG_32 reg_val;
                // set input and output length for cyclic data exchange.  This makes sense only
                // after PN was started and thus the xml config file parsed. Means after IRQ in DPC (= here)
                reg_val = SWITCH_ENDIAN_32_BIT(pdfdesc->pinbound_global->InputDataProcessImageLen);
                //SWITCH_ENDIAN_32_BIT(reg_val);
                pdfdesc->proc_image_input_len = reg_val;
                reg_val = SWITCH_ENDIAN_32_BIT(pdfdesc->pinbound_global->OutputDataProcessImageLen);
                //SWITCH_ENDIAN_32_BIT(reg_val);
                pdfdesc->proc_image_output_len = reg_val;
            }

            // check, whether there was a command issued, which leads to a FW reset like CMD_RESET
            // or CMD_HW_INFO with SUBCMD_HW_INFO_SET
            // It is presumed, that all commands with a following reset will first reset the FW_Ready
            // flag and only after that issue the interrupt. Otherwise we could be here too early and
            // complete the job here instead in the timer function.
            // The completion is delayed to keep the application hanging, until the FW reset is done.
            // Completing in the here DPC would lead to the behaviour that the application can react immediately 
            // and send new commands, which were rejected in the ioctl dispatcher.
            if (pdfdesc->pinbound_global->FW_Ready != FW_READY_INDICATION)
            {
                //if (pdfdesc->RegistryData.WaitOnReset)
                {
                    complete = FALSE;
                    pdfdesc->FwNeedsReadyCheck = TRUE;
                    pdfdesc->poutbound_global->ErrorClass = ERROR_CLASS_DRV;
                    pdfdesc->poutbound_global->Error      = RET_IN_RESET;
                    //WdfRequestMarkCancelable(Request, DfctrlEvtRequestCancel);
                    // complete the request in the watchdog timer
                    {

                        // TODO: df_prnet_timer is obsolete - needs to be refactored
                        //if ( ! timer_pending( &df_prnet_timer ))
                        //{
                        //
                        //}

                        //mod_timer(&df_prnet_timer, jiffies + jiffies_from_ms(RESET_TIMEOUT_MS)  );

                        //current->state = TASK_INTERRUPTIBLE;
                        //schedule_timeout( jiffies_from_ms(RESET_TIMEOUT_MS) );

                    }
                }
            }

            // It is not allowed to complete the request with information of size grater then the
            // framework has seen in the Irp. This size is the size of a structure. The only way to
            // inform the user application about the number of bytes transferred is ReturnBuffer->size
            // Otherwise BSOD 0x10D_6_4 issues (at least under FX verifier. No certification possible.

            if (complete)
            {

                switch (i)
                {
                case 0:

                    df_prnet_ctl_acyc_up( (void *)pdfdesc->maininstance );

                    df_prnet_wake_up_we_ctl_acyclic((void *)pdfdesc->maininstance);
                    break;
                case 1:

                    df_prnet_dev_acyc_up( (void *)pdfdesc->maininstance );

                    df_prnet_wake_up_we_dev_acyclic( (void *)pdfdesc->maininstance );

                    break;
                case 2:  // not used
                case 3:
                case 4:
                case 5:
                case 6:
                case 7:
                default:
                    break;
                }
            }
        }
    }

    if ( debug ) printk( "<-- DF_PRNET_DO_TASKLET\n");


}


/*****************************************************************************/
/*       IOCTL Routine                                           */
/*****************************************************************************/

int df_prnet_ioctl_switch( void *instance, const void  * usr_addr )

{

    int      status;
    DESC_DFPRNET *pdfdesc = (DESC_DFPRNET *)instance;

    status = STATUS_SUCCESS;

    if ( debug ) printk(" --- df_prnet IOCTL -> usr_addr=%px \n" , usr_addr );

    // WDFDPC Dpc = pdfdesc->DpcObject;
    // PDFCTRL_DPC_CONTEXT DpcContext = DfctrlGetDpcContext(Dpc);
    // size_t length;
    // P_INTERFACE_STD_HEADER_TYPE pdfdesc->inout_header, pdfdesc->inout_header;

    if ( debug ) printk( "--> vor copy_from_user inout_header]" );

    //status = WdfRequestRetrieveInputBuffer(Request, INTERFACE_STD_HEADER_TYPE_SIZE, (PVOID*)&pdfdesc->inout_header, &input_information);

    if(copyin( (void *)&pdfdesc->inout_header, usr_addr, INTERFACE_STD_HEADER_TYPE_SIZE) != 0 )
    {
        if ( debug ) printk( "\ncopy_from_user failed: %d", status);
        return -EFAULT;
    }


    df_prnet_memory_dump( (unsigned char *)&pdfdesc->inout_header, INTERFACE_STD_HEADER_TYPE_SIZE,  "pdfdesc->inout_header(1)" );

    if ( debug ) printk( "--> vor Command aus inout_header holen" );

    if (pdfdesc->inout_header.Command == CMD_PN_CYCLIC_DATA_TRANSFER) // issue no interrupt and complete here in this function
    {

        DWORD pi_input_len  = pdfdesc->proc_image_input_len;
        DWORD pi_output_len = pdfdesc->proc_image_output_len;

        //P_CYCLIC_PROCESSIMAGE_TYPE  pcyclic_in_transfer;
        //P_CYCLIC_PROCESSIMAGE_TYPE  pcyclic_out_transfer;

        P_GLOBALS_DATA_TYPE         pinbound_global  = (P_GLOBALS_DATA_TYPE)pdfdesc->pinbound_global;
        P_INBOUND_CYCLIC_DATA_TYPE  pinbound_cyclic  = (P_INBOUND_CYCLIC_DATA_TYPE)pdfdesc->pinbound_cyclic;
        P_OUTBOUND_CYCLIC_DATA_TYPE poutbound_cyclic = (P_OUTBOUND_CYCLIC_DATA_TYPE)pdfdesc->poutbound_cyclic;
        // in case of error, the values will be set according the error
        DWORD ErrorClass = ERROR_CLASS_NO_ERROR;
        DWORD Error      = RET_SUCCESSFULL;

        if ( debug ) printk( " -- down_interruptible\n" );
        //down_interruptible( &df_prnet_sem_ctl_cyclic[board]);
        df_prnet_ctl_cyc_down_int( pdfdesc->maininstance );

        //if ( debug ) printk( " -- pdfdesc->pinbound_cyclic=%x, pdfdesc->poutbound_cyclic\n", pdfdesc->pinbound_cyclic, pdfdesc->poutbound_cyclic);
        //if ( debug ) printk( " -- pinbound_cyclic=%x, poutbound_cyclic\n", pinbound_cyclic, poutbound_cyclic);

        if ( debug ) printk( " -- vor Command CMD_PN_CYCLIC_DATA_TRANSFER\n" );

        if (pdfdesc->poutbound_global->ErrorClass) // this is set only during a reset by command
        {
            // driver issued a reset command
            //if ( debug ) printk( "Firmware is on reset command and cyclic data transfer");
            ErrorClass = pdfdesc->poutbound_global->ErrorClass;
            Error      = pdfdesc->poutbound_global->Error;
            pdfdesc->FwNeedsReadyCheck = TRUE;
        }
        else if (pdfdesc->pinbound_global->FW_Ready != FW_READY_INDICATION)
        {  // firmware has an internal reset
            //if ( debug ) printk( "Firmware is on internal reset during cyclic data transfer");
            ErrorClass = ERROR_CLASS_DRV;
            Error      = RET_FW_NOT_STARTED;
        }

        if (!Error)
        {
            switch(pdfdesc->inout_header.SubCommand)
            {
            case SUBCMD_PN_SET_OUTPUT_DATA:
            {
                //SIZE_T equal_length;

                if ( debug ) printk( "--> vor SubCommand CMD_PN_SET_OUTPUT_DATA\n" );

                // test, whether data have changed. If not, nothing is to do
                // this test saves factor 8-10 of time in case of no new data
                // but costs additional 10%-12% for new data sent to the device
                // equal_length = RtlCompareMemory(pdfdesc->contig_dma_src_addr, (PVOID)&pcyclic_in_transfer->CyclicData.ProcessImage, pi_output_len);
                // if (equal_length != pi_output_len)
                {
                    if (pdfdesc->pinbound_global->PNBusRunning != 0)
                    {

                        int ret;
                        // copy the new output data from the user space to contiguous memory OutputData area
                        // to transfer it then by DMA to the controllers simpson buffer in DRAM

                        ret = copyin( (void *)pdfdesc->contig_dma_src_addr,
                                (void *)usr_addr+2*MAX_DEVICES_COUNT+INTERFACE_STD_HEADER_TYPE_SIZE,
                                INTERFACE_STD_HEADER_TYPE_SIZE + pi_output_len + 2*MAX_DEVICES_COUNT );

                        if ( ret )
                        {
                            if ( debug ) printk( " -- copy_from_user for cyclic data transfer subcmd 0x%X failed\n", pdfdesc->inout_header.SubCommand );
                            ErrorClass = ERROR_CLASS_DRV;
                            Error      = RET_INVALID_OUT_PROC_IMAGE_LENGTH;
                        }

                        // do DMA to inbound Window at offset for current simpson write area
                        {
                            // initialization of values, which do not change, is already done in df_init_PCI_register
                            PMPC8536E_DMA_REGS  pdma          = pdfdesc->pdma;
                            VREG_32*            pdma_sts      = pdfdesc->pdma_sts;
                            DWORD               DMA_wait_loop = 0;
                            DWORD               /*reg_val,*/ offset;
                            XDWORD              reg_val;

                            unsigned char       pair, index;

                            // set DMA byte counter; must be set for each DMA new, because the DMA counts it down to 0
                            reg_val = SWITCH_ENDIAN_32_BIT(pi_output_len);

                            pdma->BCR = reg_val;


                            if ( debug ) printk( " -- pdma->BCR = %x (reg_val=%x[pi_output_len=%d])\n", pdma->BCR,  reg_val, pi_output_len );

                            // source address is const and already detected in df_init_PCI_register()
                            //pdfdesc->pdma->SAR = pdfdesc->dma_source_addr;
                            reg_val = (DWORD)pdfdesc->dma_source_addr;

                            pdma->SAR = reg_val;

                            //if ( debug ) printk( " -- pdfdesc->dma_source_addr = %lx\n", pdfdesc->dma_source_addr );
                            if ( debug ) printk( " -- pdma->SAR = %x (reg_val=%x)\n", pdma->SAR,  reg_val );

                            // set destination address dependent of the simpson status
                            pair    = !(pinbound_cyclic->outp_reading);
                            index   = !(pinbound_cyclic->outp_slot[pair]);
                            offset  =  FIELD_OFFSET(INBOUND_CYCLIC_DATA_TYPE, OutputSimpsonData[pair][index].ProcessImage);

                            //df_prnet_memory_dump( (unsigned char *)pdma, MPC8536E_DMA_REGS_LEN, "pdma" );

                            reg_val = SWITCH_ENDIAN_32_BIT((pdfdesc->dma_simpson_addr + offset) & 0xFFFFFFFF);

                            if ( debug ) printk( " -- pdma->DAR(reg_val=%x)=%x, index=%d, pair=%d, offset=%x\n", reg_val, pdma->DAR, index, pair, offset );

                            pdma->DAR = reg_val; // writing to the reg starts the DMA, because SRW is set and SWSM is not set
                            if ( debug ) printk( " -- pdma->DAR(reg_val=%x)=%x, index=%d, pair=%d, offset=%x\n", reg_val, pdma->DAR, index, pair, offset );
                            //if ( debug ) printk( " -- pdma->DAR(reg_val=%x)=%x, index=%d, pair=%d, offset=%lx\n", reg_val, pdma->DAR, index, pair, offset );
                            //if ( debug ) printk( " -- pdma->DAR(reg_val=%x)=%x, index=%d, pair=%d, offset=%lx\n", reg_val, pdma->DAR, index, pair, offset );

                            // waiting until busy bit is 0 costs 20 µs here after
                            // BCR is already zero, but the DMA busy bit is still signaled
                            // For simpson write, we need to wait, before it is possible to switch the "slot" and "latest"
                            while (*pdma_sts & DMA_STATUS_GENERAL_CB1)
                            {
                                // use time control and indicate the catastrophic fault RET_DMA_ERROR, if the busy flag does not vanish in DMA_TIMEOUT_MS
                                if (++DMA_wait_loop * DMA_WAIT_STALL_TIME_US >= DMA_TIMEOUT_MS * 1000 )
                                {
                                    if ( debug ) printk( " -- DMA Error Timeout\n" );
                                    ErrorClass = ERROR_CLASS_DRV;
                                    Error      = RET_DMA_ERROR;
                                    break;
                                }
                                KeStallExecutionProcessor(DMA_WAIT_STALL_TIME_US);
                            }
                            pinbound_cyclic->outp_slot[pair] = index;
                            pinbound_cyclic->outp_latest     = pair;

                            df_prnet_memory_dump( (unsigned char *)&pinbound_cyclic->OutputSimpsonData[pair][index], 512, "&pinbound_cyclic->OutputSimpsonData[pair][index]" );

                            // trigger application watchdog
                            pinbound_global->AppAliveCtl++;
                        }
                    }
                    else // profinet is still not running (thus no xml file was parsed and no
                        // info about the process image and input and output length are available
                    {
                        // write response data
                        ErrorClass = ERROR_CLASS_DRV;
                        Error      = RET_PN_NOT_STARTED;
                    }
                }
            }
            break;

            case SUBCMD_PN_GET_INPUT_DATA:
            {
                VREG_32 reg_val;

                if ( debug ) printk( "--> vor SubCommand CMD_PN_GET_INPUT_DATA\n" );
                // simpson read
                if (pinbound_global->PNBusRunning != 0)
                {

                    unsigned char pair, index;

                    pair  = poutbound_cyclic->inp_latest;
                    index = poutbound_cyclic->inp_slot[pair];
                    poutbound_cyclic->inp_reading = pair;

                    //RtlCopyMemory((PVOID)&pcyclic_out_transfer->CyclicData, (PVOID)&poutbound_cyclic->InputSimpsonData[pair][index], pi_input_len);

                    // set input and output length for cyclic data exchange.  This makes sense only
                    // after PN was started and thus the xml config file parsed. Means after IRQ in DPC (= here)
                    reg_val = SWITCH_ENDIAN_32_BIT(pdfdesc->pinbound_global->InputDataProcessImageLen);

                    pdfdesc->proc_image_input_len = pi_input_len = reg_val;
                    reg_val = SWITCH_ENDIAN_32_BIT(pdfdesc->pinbound_global->OutputDataProcessImageLen);

                    pdfdesc->proc_image_output_len = pi_output_len = reg_val;

                    //--> copyOut
                    if ( debug ) printk( " -- vor copy_to_user pair=%d, index=%d len=%d\n -- poutbound_cyclic=%px, &poutbound_cyclic->InputSimpsonData[pair][index]=%px\n",
                            pair, index, pi_input_len, poutbound_cyclic, &poutbound_cyclic->InputSimpsonData[pair][index] );

                    df_prnet_memory_dump( (unsigned char *)&poutbound_cyclic->InputSimpsonData[pair][index], 128, "&poutbound_cyclic->InputSimpsonData[pair][index]" );

                    if(copyout( (void *)usr_addr+INTERFACE_STD_HEADER_TYPE_SIZE, (void *)&poutbound_cyclic->InputSimpsonData[pair][index], pi_input_len ))
                    {
                        if ( debug ) printk( " -- copy_to_user failed" );
                        return -EFAULT;
                    }


                    //pcyclic_out_transfer->StdHeader.Length = pi_input_len;
                    pdfdesc->inout_header.Length = pi_input_len;
                }
                else // profinet is still not running (thus no xml file was parsed and no
                    // info about the process image and input and output length are available
                {
                    // write response data
                    ErrorClass = ERROR_CLASS_DRV;
                    Error      = RET_PN_NOT_STARTED;
                }
            }
            break;

            default:
            {
                ErrorClass = ERROR_CLASS_DRV;
                Error      = RET_INVALID_CYCLIC_SUB_COMMAND;
            }
            break;
            }
        }
        pdfdesc->inout_header.ErrorClass = ErrorClass;
        pdfdesc->inout_header.Error      = Error;

        //KeReleaseSemaphore(&pdfdesc->sem_ctl_cyclic, HIGH_PRIORITY,  1, FALSE);
        df_prnet_ctl_cyc_up( pdfdesc->maininstance );
    }

    else if (pdfdesc->inout_header.Command == CMD_PN_DEV_CYCLIC_DATA_TRANSFER) // issue no interrupt and complete here in this function
    {
        DWORD pi_input_len;
        DWORD pi_output_len;

        //P_CYCLIC_PROCESSIMAGE_DEV_TYPE  pdevcyclic_in_transfer;
        //P_CYCLIC_PROCESSIMAGE_DEV_TYPE  pdevcyclic_out_transfer;

        P_GLOBALS_DATA_TYPE             pinbound_global     = (P_GLOBALS_DATA_TYPE)pdfdesc->pinbound_global;
        P_INBOUND_DEV_CYCLIC_DATA_TYPE  pinbound_devcyclic  = (P_INBOUND_DEV_CYCLIC_DATA_TYPE)pdfdesc->pinbound_devcyclic;
        P_OUTBOUND_DEV_CYCLIC_DATA_TYPE poutbound_devcyclic = (P_OUTBOUND_DEV_CYCLIC_DATA_TYPE)pdfdesc->poutbound_devcyclic;
        // in case of error, the values will be set according the error
        DWORD ErrorClass = ERROR_CLASS_NO_ERROR;
        DWORD Error      = RET_SUCCESSFULL;

        unsigned char pair, index;

        if ( debug ) printk( "--> vor Command CMD_PN_DEV_CYCLIC_DATA_TRANSFER" );
        //down_interruptible( &df_prnet_sem_dev_cyclic[board] );
        df_prnet_dev_cyc_down_int( pdfdesc->maininstance );

        // refresh the processimage length if the AR has changed
        pair  = poutbound_devcyclic->outp_latest;
        index = poutbound_devcyclic->outp_slot[(unsigned char)pair];
        poutbound_devcyclic->outp_reading = pair;

        df_prnet_memory_dump( (unsigned char *)pdfdesc->pinbound_global, 128, "pdfdesc->pinbound_global" );
        df_prnet_memory_dump( (unsigned char *)pdfdesc->poutbound_global, 128, "pdfdesc->poutbound_global" );

        if (pdfdesc->dev_picounter != pdfdesc->poutbound_devcyclic->OutputSimpsonData[pair][index].ProcessImageCounter)
        {
            VREG_32 reg_val;
            // set input and output length for cyclic data exchange.  This makes sense only
            // after PN was started and thus the xml config file parsed. Means after IRQ in DPC (= here)


            df_prnet_memory_dump( (unsigned char *)pdfdesc->pinbound_global, 128, "pdfdesc->pinbound_global" );


            reg_val = SWITCH_ENDIAN_32_BIT(pdfdesc->pinbound_global->InputDataDevProcessImageLen);

            pdfdesc->dev_proc_image_input_len = reg_val;
            reg_val = SWITCH_ENDIAN_32_BIT(pdfdesc->pinbound_global->OutputDataDevProcessImageLen);

            pdfdesc->dev_proc_image_output_len = reg_val;

            pdfdesc->dev_picounter = pdfdesc->poutbound_devcyclic->OutputSimpsonData[pair][index].ProcessImageCounter;

            if ( debug ) printk( "--> vor memset((VOID *)(&pdfdesc->poutbound_devcyclic->InputData)...\n" );
            memset((VOID *)(&pdfdesc->poutbound_devcyclic->InputData), 0, PNIO_PROCESSIMAGE_DEV_TYPE_SIZE);
        }

        // set the in and out data length
        if (pdfdesc->inout_header.SubCommand == SUBCMD_PN_DEV_GET_STATUS)
        {
            pi_input_len  = MAX_SIZE_OF_DEVPROCESSIMAGE_HEADER;
            pi_output_len = MAX_SIZE_OF_DEVPROCESSIMAGE_HEADER;
        }
        else
        {
            pi_input_len  = pdfdesc->dev_proc_image_input_len;
            pi_output_len = pdfdesc->dev_proc_image_output_len;
        }

        if (pdfdesc->poutbound_global->ErrorClass) // this is set only during a reset by command
        {
            // driver issued a reset command
            //if ( debug ) printk( "Firmware is on reset command and cyclic data transfer");
            ErrorClass = pdfdesc->poutbound_global->ErrorClass;
            Error      = pdfdesc->poutbound_global->Error;
            pdfdesc->FwNeedsReadyCheck = TRUE;
        }
        else if (pdfdesc->pinbound_global->FW_Ready != FW_READY_INDICATION)
        {  // firmware has an internal reset
            //if ( debug ) printk( "Firmware is on internal reset during cyclic data transfer");
            ErrorClass = ERROR_CLASS_DRV;
            Error      = RET_FW_NOT_STARTED;
        }

        if (!Error)
        {
            switch(pdfdesc->inout_header.SubCommand)
            {
            case SUBCMD_PN_DEV_SET_INPUT_DATA:
            {
                //SIZE_T equal_length;

                if ( debug ) printk( "--> vor SubCommand SUBCMD_PN_DEV_SET_INPUT_DATA\n" );

                // test, whether data have changed. If not, nothing is to do
                // this test saves factor 8-10 of time in case of no new data
                // but costs additional 10%-12% for new data sent to the device
                //equal_length = RtlCompareMemory(pdfdesc->dev_contig_dma_src_addr, (PVOID)&pdevcyclic_in_transfer->CyclicData, pi_input_len);
                //if (equal_length != pi_input_len)
                {
                    if (pinbound_global->PNDeviceRunning != 0)
                    {
                        int ret;

                        // copy the new output data from the user space to contiguous memory OutputData area
                        // to transfer it then by DMA to the controllers simpson buffer in DRAM
                        // RtlCopyMemory(pdfdesc->dev_contig_dma_src_addr, (PVOID)&pdevcyclic_in_transfer->CyclicData, pi_input_len);

                        // --> copyin
                        ret = copyin( (void *)pdfdesc->dev_contig_dma_src_addr,
                                (void *)(usr_addr + INTERFACE_STD_HEADER_TYPE_SIZE), pi_input_len );

                        if ( ret )
                        {
                            if ( debug ) printk( " -- copy_from_user for devcyclic data transfer subcmd 0x%X failed\n", pdfdesc->inout_header.SubCommand );
                            ErrorClass = ERROR_CLASS_DRV;
                            Error      = RET_INVALID_OUT_PROC_IMAGE_LENGTH;
                        }


                        // do DMA to inbound Window at offset for current simpson write area
                        {
                            // initialization of values, which do not change, is already done in df_init_PCI_register
                            PMPC8536E_DMA_REGS pdma = pdfdesc->pdev_dma;
                            VREG_32* pdma_sts       = pdfdesc->pdev_dma_sts;
                            DWORD DMA_wait_loop     = 0;
                            DWORD reg_val;
                            unsigned int offset;
                            BYTE /*pair,*/ index;
                            //unsigned char pair;
                            BYTE pair;

                            // set DMA byte counter; must be set for each DMA new, because the DMA counts it down to 0
                            reg_val = SWITCH_ENDIAN_32_BIT(pi_input_len);

                            pdma->BCR = reg_val;

                            // source address is const and already detected in df_init_PCI_register()
                            //pdfdesc->pdev_dma->SAR = pdfdesc->dev_dma_source_addr;
                            pdma->SAR = (unsigned int)(pdfdesc->dev_dma_source_addr);

                            // set destination address dependent of the simpson status
                            pair    = !(pinbound_devcyclic->inp_reading);
                            index   = !(pinbound_devcyclic->inp_slot[pair]);
                            offset  =  FIELD_OFFSET(INBOUND_DEV_CYCLIC_DATA_TYPE, InputSimpsonData[pair][index]);

                            reg_val = SWITCH_ENDIAN_32_BIT((pdfdesc->dev_dma_simpson_addr + offset) & 0xFFFFFFFF);

                            if ( debug ) printk( " -- SUBCMD_PN_DEV_SET_INPUT_DATA pair=%d, index=%d offset=%x len=%d\n -- pinbound_devcyclic=%px \n",
                                    pair, index, offset, pi_input_len, pinbound_devcyclic );

                            pdma->DAR = reg_val; // writing to the reg starts the DMA, because SRW is set and SWSM is not set

                            // waiting until busy bit is 0 costs 20 µs here after
                            // BCR is already zero, but the DMA busy bit is still signaled
                            // For simpson write, we need to wait, before it is possible to switch the "slot" and "latest"
                            while (*pdma_sts & DMA_STATUS_GENERAL_CB3)
                            {
                                // use time control and indicate the catastrophic fault RET_DMA_ERROR, if the busy flag does not vanish in DMA_TIMEOUT_MS
                                if (++DMA_wait_loop * DMA_WAIT_STALL_TIME_US >= DMA_TIMEOUT_MS * 1000 )
                                {
                                    ErrorClass = ERROR_CLASS_DRV;
                                    Error      = RET_DMA_ERROR;
                                    break;
                                }
                                KeStallExecutionProcessor(DMA_WAIT_STALL_TIME_US);
                            }
                            pinbound_devcyclic->inp_slot[pair] = index;
                            pinbound_devcyclic->inp_latest     = pair;
                        }

                        // trigger application watchdog
                        pdfdesc->pinbound_global->AppAlive++;
                    }
                    else // profinet is still not running (thus no xml file was parsed and no
                        // info about the process image and input and output length are available
                    {
                        // write response data
                        ErrorClass = ERROR_CLASS_DRV;
                        Error      = RET_PN_NOT_STARTED;
                    }
                }
            }
            break;
            case SUBCMD_PN_DEV_GET_INPUT_DATA:
            {
                int ret;

                // read back PNIO input data from contiguous mem
                if ( debug ) printk( "--> vor SubCommand SUBCMD_PN_DEV_GET_INPUT_DATA\n" );

                //  --> copyOut

                ret = copyout( (void *)(usr_addr + INTERFACE_STD_HEADER_TYPE_SIZE), (void *)pdfdesc->dev_contig_dma_src_addr, pi_input_len );

                if ( ret )
                {
                    if ( debug ) printk( " -- copy_from_user for cyclic data transfer subcmd 0x%X failed\n", pdfdesc->inout_header.SubCommand );
                    ErrorClass = ERROR_CLASS_DRV;
                    Error      = RET_INVALID_OUT_PROC_IMAGE_LENGTH;
                    return -EFAULT;
                }


                pdfdesc->inout_header.Length     = pi_input_len;
                pdfdesc->inout_header.ErrorClass = ErrorClass;
                pdfdesc->inout_header.Error      = Error;
            }
            break;
            case SUBCMD_PN_DEV_GET_STATUS: // is the same as SUBCMD_PN_DEV_GET_OUTPUT_DATA but with different length
            case SUBCMD_PN_DEV_GET_OUTPUT_DATA:
            {
                // simpson read
                if ( debug ) printk( "--> vorSubCommand SUBCMD_PN_DEV_GET_STATUS oder SubCommand SUBCMD_PN_DEV_GET_OUTPUT_DATA\n" );

                if (pinbound_global->PNDeviceRunning != 0)
                {

                    unsigned char pair, index;

                    int   ret;

                    pair  = poutbound_devcyclic->outp_latest;
                    index = poutbound_devcyclic->outp_slot[pair];
                    poutbound_devcyclic->outp_reading = pair;
                    //RtlCopyMemory((PVOID)&pdevcyclic_out_transfer->CyclicData, (PVOID)&poutbound_devcyclic->OutputSimpsonData[pair][index], pi_output_len);

                    //  --> copyOut
                    if ( debug ) printk( " -- vor copy_to_user pair=%d, index=%d len=%d\n -- poutbound_devcyclic=%px, &poutbound_devcyclic->OutputSimpsonData[pair][index]=%px\n",
                            pair, index, pi_input_len, poutbound_devcyclic, &poutbound_devcyclic->OutputSimpsonData[pair][index] );

                    //df_prnet_memory_dump( (unsigned char *)&poutbound_devcyclic->OutputSimpsonData[pair][index], 128, "&poutbound_devcyclic->OutputSimpsonData[pair][index]" );


                    ret = copyout( (void *)(usr_addr + INTERFACE_STD_HEADER_TYPE_SIZE), (void *)&poutbound_devcyclic->OutputSimpsonData[pair][index], pi_output_len );

                    if ( ret )
                    {
                        if ( debug ) printk( " -- copy_to_user for cyclic data transfer subcmd 0x%X failed\n", pdfdesc->inout_header.SubCommand );
                        ErrorClass = ERROR_CLASS_DRV;
                        Error      = RET_INVALID_OUT_PROC_IMAGE_LENGTH;
                        return -EFAULT;
                    }



                    //pdevcyclic_out_transfer->StdHeader.Length = pi_output_len;
                    pdfdesc->inout_header.Length = pi_output_len;
                }
                else // profinet is still not running (thus no xml file was parsed and no
                    // info about the process image and input and output length are available
                {
                    // write response data
                    ErrorClass = ERROR_CLASS_DRV;
                    Error      = RET_PN_NOT_STARTED;
                }
            }
            break;

            default:
            {
                ErrorClass = ERROR_CLASS_DRV;
                Error      = RET_INVALID_CYCLIC_SUB_COMMAND;
            }
            break;
            }
        }
        pdfdesc->inout_header.ErrorClass = ErrorClass;
        pdfdesc->inout_header.Error      = Error;

        df_prnet_dev_cyc_up( pdfdesc->maininstance );
    }
    else
    {
        INT32U length;

        //if ( debug ) printk( "--> vor pinbound_global->FW_Ready == FW_READY_INDICATION\n" );
        //if ( debug ) printk( "--> pinbound_global->FW_Ready = 0x%x, poutbound_global->Error = 0x%x \n", pdfdesc->pinbound_global->FW_Ready, pdfdesc->poutbound_global->Error );

        //df_prnet_memory_dump( (unsigned char *)pdfdesc->pinbound_global, 128,  "pinbound_global" );

        if ( debug ) printk( "--> FW_READY_INDICATION=%x, pinbound_global->FW_Ready=%x \n", FW_READY_INDICATION, pdfdesc->pinbound_global->FW_Ready );

        if ( pdfdesc->pinbound_global->FW_Ready == FW_READY_INDICATION && !pdfdesc->poutbound_global->Error)
        {
            //if ( debug ) printk( "--> (2)FW_READY_INDICATION=%x, pinbound_global->FW_Ready=%x \n", FW_READY_INDICATION, pdfdesc->pinbound_global->FW_Ready );

            if (LINUX_DRIVER != pdfdesc->pinbound_global->DriverType)
            {
                df_prnet_set_in_out_bound_values(pdfdesc, INBOUND_VALUES_SET);
            }

            df_prnet_memory_dump( (unsigned char *)&pdfdesc->inout_header, 128,  "inout_header" );

            length = INTERFACE_STD_HEADER_TYPE_SIZE + pdfdesc->inout_header.Length;
            if ( debug ) printk( " -- pdfdesc->inout_header.Length=%u, INTERFACE_STD_HEADER_TYPE_SIZE=%zu\n", pdfdesc->inout_header.Length, INTERFACE_STD_HEADER_TYPE_SIZE);
            //#pragma warning(disable:4090) // different volatile qualifiers
            //#pragma warning(default:4090)
            {
                //DWORD msg = 0;
                //DWORD *pmsg;

                VREG_32 msg = 0;
                VREG_32 *pmsg;

                if (pdfdesc->inout_header.Interrupt == 0) // default = controller
                {
                    if ( debug ) printk( " -- vor down_interruptible (df_prnet_ctl_acyc_down_int)\n" );
                    //down_interruptible( &df_prnet_sem_ctl_acyclic[board] );
                    df_prnet_ctl_acyc_down_int( pdfdesc->maininstance );

                    {
                        if ( debug ) printk( " -- vor RtlCopyMemory( interface_addr, pdfdesc->inout_header\n" );
                        // copy acyclic command header + data from the user space to the controller DRAM interface buffer

                        if(copyin( (void *)pdfdesc->interface_addr, (void *)usr_addr, length ) )
                        {
                            if ( debug ) printk( " -- copyin failed: %d", status);
                            return -EFAULT;
                        }


                        //if ( debug ) printk( "--> vor msg auf esc_pci[board].virt_io_addr + IRQ_TRIGGER_0 setzen\n" );
                        //trigger interrupt (FW does not support messages for this controller IRQ)

                        pmsg  = ((VREG_32*)(pdfdesc->pdescpci->virt_io_addr + IRQ_TRIGGER_0));

                        if ( debug ) printk( "--> msg=%x pmsg=%px Addr: desc_prnet.virt_io_addr + IRQ_TRIGGER_0= %px \n",
                                msg, pmsg, pdfdesc->pdescpci->virt_io_addr + IRQ_TRIGGER_0 );

                        *pmsg = msg;

                        // Im Gegensatz zu Windows muss hier auf IRQ gewartet werden

                        if ( df_prnet_wait_event_irp_we_ctl_acyclic( pdfdesc->maininstance, 1000*1000 ) == 0 )
                        {
                            printk( " -- wait event ctl timed out" );
                            pdfdesc->inout_header.ErrorClass = ERROR_CLASS_DRV;
                            pdfdesc->inout_header.Error      = RET_FW_NOT_STARTED;
                            goto NO_COMPLETE;
                        }

                        // Request completion without the controllers answer IRQ/DPC, takes place in
                        // the timer routine or the cancel routine, in case of the application is killed

                        if ( debug ) printk( " --> vor copy_to_user\n" );
                        length =  pdfdesc->interface_addr->StdHeader.Length;
                        if(copyout( (void *)usr_addr, (void *)pdfdesc->interface_addr,  INTERFACE_STD_HEADER_TYPE_SIZE + length ))
                        {
                            if ( debug ) printk( " -- copy_to_user failed" );
                            return -EFAULT;
                        }

                        goto COMPLETE_ACYCLIC;
                    }
                }
                else // special purpose Message IRQs for PN device mode
                {

                    df_prnet_dev_acyc_down_int( pdfdesc->maininstance );

                    {
                        // copy acyclic command header + data from the user space to the controller DRAM interface buffer
                        if ( debug ) printk( "--> (2)vor RtlCopyMemory( devinterface_addr, pdfdesc->inout_header\n" );
                        if ( debug ) printk("dev_interface_addr=%px, usr_addr=%px, length=%u", pdfdesc->dev_interface_addr, usr_addr, length );
                        if(copyin( (void *)pdfdesc->dev_interface_addr, (void *)usr_addr, length ) )
                        {
                            if ( debug ) printk( " -- copy_from_user failed dest=%px src=%px length=%u\n", pdfdesc->dev_interface_addr, usr_addr, length );

                            //df_prnet_memory_dump( (unsigned char *)pdfdesc->dev_interface_addr, 256,  "dev_interface_addr" );

                            df_prnet_dev_acyc_up( pdfdesc->maininstance );
                            return -EFAULT;
                        }

                        msg   = SWITCH_ENDIAN_32_BIT(pdfdesc->inout_header.Interrupt); // invalid values from app for MSI (>7 or 0) and MSG (>31) are corrected in FW
                        pmsg  = ((VREG_32*)(pdfdesc->pdescpci->virt_io_addr + IRQ_TRIGGER_1));
                        *pmsg = msg;

                        // Im Gegensatz zu Windows muss hier auf IRQ gewartet werden

                        if ( df_prnet_wait_event_irp_we_dev_acyclic( pdfdesc->maininstance, 1000*1000 ) == 0 )
                        {
                            printk( " -- wait event ctl timed out" );
                            pdfdesc->inout_header.ErrorClass = ERROR_CLASS_DRV;
                            pdfdesc->inout_header.Error      = RET_FW_NOT_STARTED;
                            goto NO_COMPLETE;
                        }

                        // Request completion without the controllers answer IRQ/DPC, takes place in
                        // the timer routine or the cancel routine, in case of the application is killed

                        length =  pdfdesc->dev_interface_addr->StdHeader.Length;
                        if(copyout( (void *)usr_addr, (void *)pdfdesc->dev_interface_addr,  INTERFACE_STD_HEADER_TYPE_SIZE + length ))
                            //if(copyout( (void *)usr_addr, (void *)pdfdesc->dev_interface_addr,  INTERFACE_STD_HEADER_TYPE_SIZE + 128 ))
                        {
                            if ( debug ) printk( " -- copy_to_user failed" );
                            return -EFAULT;
                        }


                        goto COMPLETE_ACYCLIC;
                    }
                }
            }
        }
        else // NO command must be issued, as long as FW is not ready!
        {
            if (pdfdesc->poutbound_global->ErrorClass)
            {
                if ( debug ) printk( " -- Firmware is on reset command and a new command was sent");
                pdfdesc->inout_header.ErrorClass = pdfdesc->poutbound_global->ErrorClass;
                pdfdesc->inout_header.Error      = pdfdesc->poutbound_global->Error;
            }
            else
            {
                if ( debug ) printk( " --Firmware is not ready");
                pdfdesc->inout_header.ErrorClass = ERROR_CLASS_DRV;
                pdfdesc->inout_header.Error      = RET_FW_NOT_STARTED;
            }
            pdfdesc->FwNeedsReadyCheck = TRUE;
        }
    }

    NO_COMPLETE:

    if ( debug ) printk( " -- DfctrlEvtIoDeviceControl: not completed\n");


    if ( copyout( (void *)(usr_addr), (void *)&pdfdesc->inout_header ,INTERFACE_STD_HEADER_TYPE_SIZE) )
    {
        if ( debug ) printk( " -- copy_to_user for cyclic data transfer subcmd 0x%X failed\n", pdfdesc->inout_header.SubCommand );
        return -EFAULT;
    }

    COMPLETE_ACYCLIC:

    if ( debug ) printk( "<-- DfctrlEvtIoDeviceControl\n");

    return( 0 );
}

