#!/bin/sh
###############################################################################
##
##  S 9 0 d f p r o f i n e t
##
##-----------------------------------------------------------------------------
##
## File    : /etc/init.d/DFProfiNET
##
##---- Description ------------------------------------------------------------
##
## Shell script to start DFPROFINET-PCI driver and generating devices
##
##---- History ----------------------------------------------------------------
##
## 19.12.00 / Wolfgang M. Weber  initial version
## 26.07.05 / Wolfgang M. Weber  modified for chkconfig and SuSE Yast
## 29.04.15 / Wolfgang M. Weber  modified
##
###############################################################################

# S90dfprofinet    This shell script takes care of starting and stopping
#                  the DFPROFINET board driver
#
# chkconfig: 35 90 20
# description:     This shell script takes care of starting and stopping \
#                  the DFPROFINET board driver

### BEGIN INIT INFO
# Provides:       DFPROFINET
# Required-Start: $syslog $local_fs
# Required-Stop:
# Default-Start:  3 5
# Default-Stop:   0 6
# Description:    Start/Stop the DFPROFINET board driver.
### END INIT INFO

 
DEVICE_PATH=/dev/dfprnet
DRIVER_NAME=df_profinet
DRIVER_FILE=dfprnet
DRIVER_COUNT=10

case "$1" in
'start')
	SCRIPT_PATH="`dirname \"$0\"`" 
	SCRIPT_PATH="`( cd \"$SCRIPT_PATH\" && pwd )`" 

        if [ -e /lib/modules/`uname -r`/kernel/drivers/misc/$DRIVER_FILE.ko  ]
        then
		echo "Driver existing"
        else
		echo "Driver not existing --> compiling"
		make -C $SCRIPT_PATH || exit 1
		make -C $SCRIPT_PATH install || exit 1
        fi

        major=`awk -v dname="$DRIVER_NAME" '$2 == dname { print $1 }' /proc/devices`

        if [ -z $major  ]
        then
		/sbin/insmod /lib/modules/`uname -r`/kernel/drivers/misc/$DRIVER_FILE.ko || exit 1
        	major=`awk -v dname="$DRIVER_NAME" '$2 == dname { print $1 }' /proc/devices`
		sleep 1
        fi

        if [ -z $major  ]
        then
           echo "DFPROFINET IO driver $DRIVER_NAME not found"
           exit
        else
           echo "DFPROFINET IO driver $DRIVER_NAME found with MAJOR=$major"
        fi

	rm -rf $DEVICE_PATH || exit 1
	mkdir -p $DEVICE_PATH || exit 1

	for i in $(seq 0 $(($DRIVER_COUNT-1)))
	do
		mknod $DEVICE_PATH/df$i c $major $i || exit 1
	done

	chmod 777 $DEVICE_PATH/* || exit 1

        ;;
'stop')
        echo "removing driver $DRIVER_FILE"
        rmmod $DRIVER_FILE.ko
        ;;
*)
        echo "USAGE=$0 { start | stop }"
        ;;
esac
                                                                                                                                                                
