/*** Following doxy comment is for the file description ***/
/*!
 *
 * Project: 700361_Profinet_IO_Controller
 * (C)    : KUNBUS GmbH, Robert-Bosch-Strasse 12a, D-76275 Ettlingen
 *
 *
 * \file dfpnio.h
 *
 *
 * \brief 
 * -  Include file for all exported functions.
 *    This header is included in user applications projects (together with
 *    the lib file for the linker information). An Application then has an 
 *    easy access to the PNIO functionality of the controller board via DLL
 *    function calls.
 *
 * \author CH
 *
 */

#ifndef _DFPNIO_H_EDB10C66_8034_4f4f_9090_7FE899314E4A_
#define _DFPNIO_H_EDB10C66_8034_4f4f_9090_7FE899314E4A_

#ifdef __cplusplus
extern "C"
{
#endif

#include "common_define.h"

#define PNKW_NOINTERNAL_STRUCTURES // needed for Pn_LibMain include

#pragma pack(push, Kunbus, 4)

/******************************************************************************/
/*********************************  Defines  **********************************/
/******************************************************************************/
#define MAX_CTRL_HANDLES 256    //!< Maximum number of supported controller boards.
#define MAX_TEXT_LEN     256    //!< Maximum length of joint error texts. 
#define INTERN_DEV_OFFSET 0x100  //!< DeviceNo for accessing features in device role


/*! \typedef CONTROLLER_INFO"
 *  \brief Info structure for the user about a controller.
 *
 * The typedef provides the logical index, the name and the physical PCI location information 
 * in the form of a number which is 100*bus + 10*device + function.
 */
typedef struct CONTROLLER_INFO
{
   INT32U  Index;               //!< PNIO-controllers logical number for application access
   INT32U  Active;              //!< PNIO-controllers current state (active or deactivated)
   INT32U  PCIslot;             //!< PCI slot indicator, the physical place where the controller resides in the PC
   CHAR8*  Name;                //!< PNIO-controller driver internal name with PCI slot suffix
} CONTROLLER_INFO, *PCONTROLLER_INFO;
#define CONTROLLER_INFO_SIZE sizeof(CONTROLLER_INFO)

          
/*! \typedef DEVICE_INFO"
 *  \brief Info structure for the user to access the IO data of a certain device.
 *
 * The typedef provides pointer and length information for a device pure IO data and stati.
 * This is the APDU status, the Device status and the ored stati values of all input elements 
 * and the ored value of all output element stati.
 * Two length values for input and for output data as well as pointer to the DLL managed
 * memory area of the pure input and output data. These data areas contain no IOCS and IOPS
 * status values any more, they are ored in the status values of this structure.
 *
 */
typedef struct DEVICE_INFO
{
   INT8U  APDUStatus;    //!< APDU status of the device
   INT8U  DeviceStatus;  //!< Device status of the device
   INT8U  InputsStatus;  //!< Sum status on a device of all input value related stati. A value != 0x80 indicates an error.
   INT8U  OutputsStatus; //!< Sum status on a device of all output value related stati. A value != 0x80 indicates an error.
   INT16U InputsLength;  //!< Length of all input values on a device without status values
   INT16U OutputsLength; //!< Length of all output values on a device without status values
   INT8U* pInputData;    //!< Pointer to the input data of a device without status values
   INT8U* pOutputData;   //!< Pointer to the output data of a device without status values
} DEVICE_INFO, *PDEVICE_INFO;



/* Alarm Channel Properties */
/* Alarm Channel Properties Type */
#define DFPN_ALARM_CP_TYPE_NONE                              0x0000
#define DFPN_ALARM_CP_TYPE_1_BIT                             0x0001
#define DFPN_ALARM_CP_TYPE_2_BIT                             0x0002
#define DFPN_ALARM_CP_TYPE_4_BIT                             0x0003
#define DFPN_ALARM_CP_TYPE_8_BIT                             0x0004
#define DFPN_ALARM_CP_TYPE_16_BIT                            0x0005
#define DFPN_ALARM_CP_TYPE_32_BIT                            0x0006
#define DFPN_ALARM_CP_TYPE_64_BIT                            0x0007
/* 0x0008 to 0x00FF Reserved */

/* Alarm Channel Properties Accumulative */
#define DFPN_ALARM_CP_ACCUMULATIVE_NONE                      0x0000  /*!< 0x00, no accumulative diagnosis */
#define DFPN_ALARM_CP_ACCUMULATIVE_PRESENT                   0x0100  /*!< 0x01, accumulative diagnosis from more then one channel */

/* Alarm Channel Properties Maintenance */
#define DFPN_ALARM_CP_MAINTENANCE_NONE                       0x0000  /*!< 0x00, no maintance required or demanded */
#define DFPN_ALARM_CP_MAINTENANCE_REQUIRED                   0x0200  /*!< 0x01, maintance required */
#define DFPN_ALARM_CP_MAINTENANCE_DEMANDED                   0x0400  /*!< 0x02, maintance demanded */
#define DFPN_ALARM_CP_MAINTENANCE_REQUIRED_DEMANDED          0x0600  /*!< 0x03, maintance required and demanded */

/* Alarm Channel Properties Specifier */
#define DFPN_ALARM_CP_SPECIFIER_ALL_DISAPPEARS               0x0000  /*!< 0x00, all subsequent disappears */
#define DFPN_ALARM_CP_SPECIFIER_APPEARS                      0x0800  /*!< 0x01, an event appears and/or exists further */
#define DFPN_ALARM_CP_SPECIFIER_DISAPPEARS                   0x1000  /*!< 0x02, an event disappears and/or exists no longer */
#define DFPN_ALARM_CP_SPECIFIER_DISAPPEARS_REMAIN            0x1800  /*!< 0x03, an event disappears but there are other */

/* Alarm Channel Properties Direction */
#define DFPN_ALARM_CP_DIRECTION_MANUFACTURER_SPECIFIC        0x0000  /*!< 0x00, manufacturer specific direction */
#define DFPN_ALARM_CP_DIRECTION_INPUT                        0x2000  /*!< 0x01, input direction */
#define DFPN_ALARM_CP_DIRECTION_OUTPUT                       0x4000  /*!< 0x02, output direction */
#define DFPN_ALARM_CP_DIRECTION_INPUT_OUTPUT                 0x6000  /*!< 0x03, input and output direction */
/* 0x04 to 0x07 Reserved */

/* Alarm Channel Error Types */
#define DFPN_ALARM_CET_RESERVED                              0x0000  /*!< Unknown error */
#define DFPN_ALARM_CET_SHORT_CIRCUIT                         0x0001  /*!< Short circuit */
#define DFPN_ALARM_CET_UNDERVOLTAGE                          0x0002  /*!< Undervoltage */
#define DFPN_ALARM_CET_OVERVOLTAGE                           0x0003  /*!< Overvoltage */
#define DFPN_ALARM_CET_OVERLOAD                              0x0004  /*!< Overload */
#define DFPN_ALARM_CET_OVERTEMPERATURE                       0x0005  /*!< Overtemperature */
#define DFPN_ALARM_CET_LINE_BREAK                            0x0006  /*!< Line break */
#define DFPN_ALARM_CET_UPPER_LIMIT_VALUE_EXCEEDED            0x0007  /*!< Upper limit value exceeded */
#define DFPN_ALARM_CET_LOWER_LIMIT_VALUE_EXCEEDED            0x0008  /*!< Lower limit value exceeded */
#define DFPN_ALARM_CET_ERROR                                 0x0009  /*!< Error */
#define DFPN_ALARM_CET_SIMULATION ACTIVE                     0x000A  /*!< Simulation active */
#define DFPN_ALARM_CET_RESERVED_1                            0x000B  /*!< Unknown error */
#define DFPN_ALARM_CET_RESERVED_2                            0x000C  /*!< Unknown error */
#define DFPN_ALARM_CET_RESERVED_3                            0x000D  /*!< Unknown error */
#define DFPN_ALARM_CET_RESERVED_4                            0x000E  /*!< Unknown error */
#define DFPN_ALARM_CET_PARAMETER_MISSING                     0x000F  /*!< Manufacturer specific, recommended for "parameter missing" */
                                                                /* The channel needs (additional) parameters. No or to less parameters are written */
#define DFPN_ALARM_CET_PARAMETRIZATION_FAULT                 0x0010  /*!< Manufacturer specific, recommended for "parametrization fault" */
                                                                /* Parametrization fault. Wrong or to many parameters are written */
#define DFPN_ALARM_CET_POWER_SUPPLY_FAULT                    0x0011  /*!< Manufacturer specific, recommended for "power supply fault" */
                                                                /* Power supply fault */
#define DFPN_ALARM_CET_FUSE_BLOWN_OR_OPEN                    0x0012  /*!< Manufacturer specific, recommended for "fuse blown / open" */
                                                                /* Fuse blown / open */
#define DFPN_ALARM_CET_COMMUNICATION_FAULT                   0x0013  /*!< Manufacturer specific, recommended for "communication fault" */
                                                                /* Communication fault. Sequence number wrong / sequence wrong */
#define DFPN_ALARM_CET_GROUND_FAULT                          0x0014  /*!< Manufacturer specific, recommended for "ground fault" */
                                                                /* Ground fault */
#define DFPN_ALARM_CET_REFERENCE_POINT_LOST                  0x0015  /*!< Manufacturer specific, recommended for "reference point lost" */
                                                                /* Reference point lost */
#define DFPN_ALARM_CET_PROCESS_EVENT_LOST                    0x0016  /*!< Manufacturer specific, recommended for "process event lost / sampling error" */
                                                                /* Process event lost / sampling error */
#define DFPN_ALARM_CET_THRESHOLD_WARNING                     0x0017  /*!< Manufacturer specific, recommended for "threshold warning" */
                                                                /* Threshold warning */
#define DFPN_ALARM_CET_OUTPUT_DISABLED                       0x0018  /*!< Manufacturer specific, recommended for "output disabled" */
                                                                /* Output disabled */
#define DFPN_ALARM_CET_SAFETY_EVENT                          0x0019  /*!< Manufacturer specific, recommended for "safety event" */
                                                                /* Safety event */
#define DFPN_ALARM_CET_EXTERNAL_FAULT                        0x001A  /*!< Manufacturer specific, recommended for "external fault" */
                                                                /* External fault */
#define DFPN_ALARM_CET_MANUFACTURER_SPECIFIC_1               0x001B  /*!< Manufacturer specific */
#define DFPN_ALARM_CET_MANUFACTURER_SPECIFIC_2               0x001C  /*!< Manufacturer specific */
#define DFPN_ALARM_CET_MANUFACTURER_SPECIFIC_3               0x001D  /*!< Manufacturer specific */
#define DFPN_ALARM_CET_MANUFACTURER_SPECIFIC_4               0x001E  /*!< Manufacturer specific */
#define DFPN_ALARM_CET_TEMPORARY_FAULT                       0x001F  /*!< Manufacturer specific, recommended for "temporary fault" */
                                                                /* Temporary fault */
/* 0x0020 to 0x00FF Reserved for common profiles */             /* Central administrative number to unambiguous distinguish between common profiles */
/* 0x0100 to 0x7FFF Manufacturer specific */                    /* Manufacturer specific */
#define DFPN_ALARM_CET_DATA_TRANSMISSION_IMPOSSIBLE          0x8000  /*!< Data Transmission Impossible */
#define DFPN_ALARM_CET_REMOTE_MISMATCH                       0x8001  /*!< Remote Mismatch */
#define DFPN_ALARM_CET_MEDIA_REDUNDANCY_MISMATCh             0x8002  /*!< Media Redundancy Mismatch */
#define DFPN_ALARM_CET_SYNC_MISMATCH                         0x8003  /*!< Sync Mismatch */
#define DFPN_ALARM_CET_ISOCHRONOUSMODE_MISMATCH              0x8004  /*!< IsochronousMode Mismatch */
#define DFPN_ALARM_CET_MULTICAST_CR_MISMATCH                 0x8005  /*!< Multicast CR Mismatch */
#define DFPN_ALARM_CET_RESERVED_5                            0x8006  /*!< Reserved */
#define DFPN_ALARM_CET_FIBER_OPTIC_MISMATCH                  0x8007  /*!< Information for fiber optic link */
#define DFPN_ALARM_CET_NETWORK_COMPONENT_FUNCTION_MISMATCH   0x8008  /*!< Network functionality problems occur */
#define DFPN_ALARM_CET_TIME_MISMATCH                         0x8009  /*!< Time master not existent or precision problems */
#define DFPN_ALARM_CET_DFP_FUNC_MISMATCH                     0x800A  /*!< Dynamic frame packing function mismatch */
#define DFPN_ALARM_CET_MRPD_MISMATCH                         0x800B  /*!< Media redundancy with planned duplication mismatch */
#define DFPN_ALARM_CET_SYSTEM_REDUNDANCY_MISMATCH            0x800C  /*!< System redundancy mismatch */
#define DFPN_ALARM_CET_MULTIPLE_INTERFACE_MISMATCH           0x800D  /*!< Information about multiple interface problems */
#define DFPN_ALARM_CET_NESTED_DIAGNOSIS_INDICATION           0x800E  /*!< Used for propagation of diagnosis in hierachical structures */
/* 0x800F to 0x8FFF Reserved */                                 /* Unknown error */
/* 0x9000 to 0x9FFF Reserved for profiles */                    /* Profile specific */
/* 0xA000 to 0xFFFF Reserved */                                 /* Unknown error */


#define DFPN_DIAG_CHAN_DIAG            0x8000   /*!< Channel Diagnosis Identifier */
#define DFPN_DIAG_EXT_CHAN_DIAG        0x8002   /*!< Ext Channel Diagnosis Identifier */
#define DFPN_DIAG_QUA_CHAN_DIAG        0x8003   /*!< Qualified Channel Diagnosis Identifier */

#define DFPN_ALARM_NO_RTA_TRIGGER      0x00     /*!< No alarm is triggered, only ASE is updated */
#define DFPN_ALARM_LOW_RTA_TRIGGER     0x01     /*!< An alarm handling with low priority is triggered */
#define DFPN_ALARM_HIGH_RTA_TRIGGER    0x02     /*!< An alarm handling with high priority is triggered */

/*! \typedef DFPN_DIAG_INFO"
 *  \brief Info structure for diagnosis records.
 *
 * The typedef provides diagnosis information. Depending on the UserStructureIdentifier
 * the other variables will be used. 0
 */
typedef struct DFPN_DIAG_INFO_TYPE
{
   INT16U    ChannelProperties; /*!< Properties of this channel 
                                         Allowed Values:
                                         - DFPN_ALARM_CP_... (Multiple Values can be ored)
                                         */
   INT16U    UserStructureIdentifier;
                                    /*!< User structure identifier (USI) describing the diagnosis type
                                        
                                        Allowed Values:
                                        - DFPN_DIAG_CHAN_DIAG (0x8000, Channel Diagnosis)
                                        - DFPN_DIAG_EXT_CHAN_DIAG (0x8002, Ext Channel Diagnosis)
                                        - DFPN_DIAG_QUA_CHAN_DIAG (0x8003, Qualified Channel Diagnosis)
                                    */
   INT16U    ChannelErrorType;  /*!< Error type of the channel. This value is valid for all USI
                                         Allowed Values:
                                         - DFPN_ALARM_CET_...
                                         */
   INT16U    ExtChannelErrorType;
                                    /*!< Extended error type of the channel
                                         This value is valid for USI PNDIAG_EXT_CHAN_DIAG and PNDIAG_QUA_CHAN_DIAG only
                                    */
   INT32U     ExtChannelAddValue;
                                    /*!< Additional value in case of extended channel  diagnosis
                                         This value is valid for USI PNDIAG_EXT_CHAN_DIAG and PNDIAG_QUA_CHAN_DIAG only
                                    */
   INT32U     QualifiedChannelQualifier;
                                    /*!< Qualified channel qualifier
                                         This value ist valid for USI PNDIAG_QUA_CHAN_DIAG only.
                                    */
   INT8U     RTATrigger;
                                    /*!< With this parameter the client module can choose the RTA handling mode of the channel diagnosis entry. If an alarm handling is triggered (low or high priority), the applicable acknowledge of the remote IOC is given back to the triggered module, here the DiagAgent.
                                        
                                        Allowed Values: 
                                        - DFPN_ALARM_NO_RTA_TRIGGER: No Alarm handling is triggered, only the Diagnosis ASU will be updated 
                                        - DFPN_ALARM_LOW_RTA_TRIGGER: An Alarm handling with low priority is triggered (RTA frame with low priority is sent)
                                        - DFPN_ALARM_HIGH_RTA_TRIGGER: An Alarm handling with high priority is triggered (RTA frame with high priority is sent)
                                    */
} DFPN_DIAG_INFO;


/*! \typedef DFPN_AR_INFO"
 *  \brief Info structure for diagnosis records.
 *
 * The typedef provides diagnosis information. Depending on the UserStructureIdentifier
 * the other variables will be used. 0
 */
typedef struct DFPN_AR_INFO_TYPE
{
   INT8U  CMInitName[256];                    /*!< Station name*/
   INT8U  CMInitIPAddr[4];                    /*!< IP address */
   INT8U  CMInitMacAddr[6];                   /*!< MAC address (only Device) */
   INT8U  DAPConfigured;                      /*!< TRUE if this AR has a configured DAP (only Device) */
   INT8U  Reserved8;
   INT16U ARID;                               /*!< Internal ARID */
   INT16U ARUID;                              /*!< DeviceNo to access the device */
   INT16U TxSendClockFactor;                  /*!< Sendclock Factor for tx direction */
   INT16U RxSendClockFactor;                  /*!< Sendclock Factor for rx direction */
   INT16U TxReductionRatio;                   /*!< Reduction Ratio for tx direction */
   INT16U RxReductionRatio;                   /*!< Reduction Ratio for rx direction */
   INT16U MaxAlarmDataLength;                 /*!< Maximum Alarm Data Length */
   INT16U Reserved16;
} DFPN_AR_INFO;


/*! \typedef DFPN_STD_ALARM"
 *  \brief Std Info structure for Alarm Items.
 *
 * The typedef is used for a standard alarm items. The meaning of the fields is the same as in DFPN_DIAG_INFO
 */
typedef struct DFPN_STD_ALARM_TYPE
{
   INT16U UserStructureIdentifier;
                                    /*!< User structure identifier (USI) describing the diagnosis type
                                        
                                        Allowed Values:
                                        - DFPN_DIAG_CHAN_DIAG (0x8000, Channel Diagnosis) 
                                             Structure length is 8 Byte
                                        - DFPN_DIAG_EXT_CHAN_DIAG (0x8002, Ext Channel Diagnosis)
                                             Structure length is 14 Byte
                                        - DFPN_DIAG_QUA_CHAN_DIAG (0x8003, Qualified Channel Diagnosis)
                                             Structure length is 18 Byte
                                    */
   INT16U ChannelNumber;       
                                    /*!< Logical channel of the occoured alarm. e.g. the pin number */
   INT16U ChannelProperties;   
                                    /*!< Properties of this channel 
                                         Allowed Values:
                                         - DFPN_ALARM_CP_... (Multiple Values can be ored)
                                    */
   INT16U ChannelErrorType;
                                    /*!< Error type of the channel. This value is valid for all USI
                                         Allowed Values:
                                         - DFPN_ALARM_CET_...
                                    */
   INT16U ExtChannelErrorType;
                                    /*!< Extended error type of the channel
                                         This value is valid for USI PNDIAG_EXT_CHAN_DIAG and PNDIAG_QUA_CHAN_DIAG only
                                    */
   INT32U  ExtChannelAddValue;
                                    /*!< Additional value in case of extended channel  diagnosis
                                         This value is valid for USI PNDIAG_EXT_CHAN_DIAG and PNDIAG_QUA_CHAN_DIAG only
                                    */
   INT32U  QualifiedChannelQualifier;
                                    /*!< Qualified channel qualifier
                                         This value ist valid for USI PNDIAG_QUA_CHAN_DIAG only.
                                    */
} DFPN_STD_ALARM;


/*! \typedef DFPN_RECORD_RESULT"
 *  \brief Std Info structure for Record Results.
 *
 * The typedef is used for results of read and wirte records
 */
typedef struct DFPN_RECORD_RESULT_TYPE
{
   INT8U  ErrorCode;
   INT8U  ErrorDecode;
   INT8U  ErrorCode1;
   INT8U  ErrorCode2;
   INT16U AddData1;
   INT16U AddData2;
   INT32U  DataLength;    /*!< writen or read number of bytes */
} DFPN_RECORD_RESULT;


/*! \typedef DFPN_ALARM_NOTIFICATION"
 *  \brief Std Info structure for Alarm Results.
 *
 * The typedef is used for results of alarm get requests
 */
typedef struct DFPN_ALARM_NOTIFICATION_TYPE{
       INT16U        BlockType;	           /*!< Type of this Alarm CR Block */
       INT16U        BlockVersion;	        /*!< Version of this Alarm CR Block
                                                    High Byte = Block Version High
                                                    Low Byte = Block Version Low */
       INT16U        BlockLength;	           /*!< Length of this Alarm CR Block */
       INT16U        AlarmType;              /*!< Type of the Alarm
                                                    Allowed Values:
                                                    DFPN_ALARM_TYPE_DIAG_APPEARS
                                                    DFPN_ALARM_TYPE_PROCESS
                                                    DFPN_ALARM_TYPE_PULL
                                                    DFPN_ALARM_TYPE_PLUG
                                                    DFPN_ALARM_TYPE_STATUS
                                                    DFPN_ALARM_TYPE_UPDATE
                                                    DFPN_ALARM_TYPE_REDUNDANCY
                                                    DFPN_ALARM_TYPE_CONTR_BY_SUPERVISOR
                                                    DFPN_ALARM_TYPE_RELEASED
                                                    DFPN_ALARM_TYPE_PLUG_WRONG_SUBMODULE
                                                    DFPN_ALARM_TYPE_RETURN_OF_SUBMODULE
                                                    DFPN_ALARM_TYPE_DIAG_DISAPPEARS */
       INT32U         API;                   /*!< API identifier */
       INT16U        SlotNumber;             /*!< This parameter contains the value of
                                                    the attribute Slot Number.
                                                    Allowed values:
                                                    0 to 0x7FFF */
       INT16U        SubslotNumber;          /*!< This attribute defines to which slot
                                                    the object belongs.
                                                    Allowed values:
                                                    1 to 0x7FFF */
       INT32U         ModuleIdentNumber;     /*!< This parameter contains the value
                                                    of the attribute Module Ident Number */
       INT32U         SubmoduleIdentNumber;  /*!< This attribute defines the
                                                    identification of the subslot.
                                                    Allowed values:
                                                    0 to 0xFFFFFFFF */
       INT16U        AlarmSpecifier;         /*!< Alarm Specifier is needed to identify
                                                    the acknowledge by the sender
                                                    of this alarm. */
       INT16U        LengthOfAlarmItem;      /*!< Length of the AlarmItem in bytes */
       INT8U         *pAlarmItem;           /*!< Pointer to AlarmItem is valid until next acyclic call*/
} DFPN_ALARM_NOTIFICATION;


/*! \typedef DFPN_MODULE_DIFF_BLOCK_SUBSLOT"
 *  \brief Std Info structure for module diff block on subslot level.
 *
 * The typedef is used for results of module diff block
 */
typedef struct DFPN_MODULE_DIFF_BLOCK_SUBSLOT_TYPE
{
   INT32U SubslotNumber;
   INT32U SubmoduleIdentNumber;
   INT32U SubmoduleState;
} DFPN_MODULE_DIFF_BLOCK_SUBSLOT;


/*! \typedef DFPN_MODULE_DIFF_BLOCK_SLOT"
 *  \brief Std Info structure for module diff block on slot level.
 *
 * The typedef is used for results of module diff block
 */
typedef struct DFPN_MODULE_DIFF_BLOCK_SLOT_TYPE
{
   INT32U SlotNumber;
   INT32U ModuleIdentNumber;
   INT32U ModuleState;
   INT32U NumberOfSubmodules;
      #define MODULE_DIFF_BLOCK_MAX_NUMBER_OF_SUBMODULES 25
   DFPN_MODULE_DIFF_BLOCK_SUBSLOT ListOfSubmodules[MODULE_DIFF_BLOCK_MAX_NUMBER_OF_SUBMODULES];
} DFPN_MODULE_DIFF_BLOCK_SLOT;


/*! \typedef DFPN_MODULE_DIFF_BLOCK"
 *  \brief Std Info structure for module diff block on device level.
 *
 * The typedef is used for results of module diff block
 */
typedef struct DFPN_MODULE_DIFF_BLOCK_TYPE
{
   INT32U ARID;                                   /* Application Releation ID of the Module Diff Block */
   CHAR8 StationName[256];                                /* Station Name of the Device */
   INT32U NumberOfModules;                        /* Number of Modules in the following array */
      #define MODULE_DIFF_BLOCK_MAX_NUMBER_OF_MODULES    25      
   DFPN_MODULE_DIFF_BLOCK_SLOT ListOfModules[MODULE_DIFF_BLOCK_MAX_NUMBER_OF_MODULES];
} DFPN_MODULE_DIFF_BLOCK;


/*! \typedef DFPN_DCP_RESULT"
 *  \brief Std Info structure for identify all 
 *
 * The typedef is used for identify all result
 */
typedef struct DFPN_DCP_RESULT_TYPE
{
    INT32U    XID;                                /* Transaction identifier             */
    INT8U    ServiceID;                          /* This variable contains the DCP service number
                                                            which issued this confirmation.
                                                            Allowed values: DCP_GET, DCP_SET, DCP_IDENTIFY*/
    INT8U    MACAddr[6];                         /* This variable receives the MAC address of the DCP request target.*/
    INT8U    IPAddr[4];                          /* This attribute receives the IP address of the DCP request target.*/
    INT8U    Gateway[4];                         /* This attribute receives the standard gateway address of the DCP request target.*/
    INT8U    NetworkMask[4];                     /* This attribute receives the network mask of the DCP request target.*/
    INT8U    StationNameLength;                  /* This attribute receives the length of the station name.*/
    CHAR8             StationName[256];                   /* This attribute receives a pointer to a buffer with size MAX_STATION_NAME_SIZE
                                                            containing the station name.*/
    INT8U    AliasNameLength;                    /* This attribute receives the length of the alias name.*/
    CHAR8             AliasName[256];                     /* This attribute receives a pointer to a buffer with size MAX_STATION_NAME_SIZE
                                                            containing the alias name.*/
    INT8U    DevTypeLength;                      /* This attribute receives the length of the device type.*/
    CHAR8             DevType[256];                       /* This attribute receives a pointer to a buffer with size MAX_STATION_NAME_SIZE
                                                            containing the device type.*/
    INT16U   VendorID;                           /* This attribute receives the vendor ID.*/
    INT16U   DeviceID;                           /* This attribute receives the device ID.*/
    INT8U    DeviceRole;                         /* This attribute receives the device role.*/
    INT8U    Result;                             /* This variable contains the result of the DCP operation.*/
} DFPN_DCP_RESULT;


/* AlarmClass for AlarmGet() and Flags in DeviceStatus Byte */
#define DFPN_DIAGNOSIS_ALARM                       0x01  /*!< AlarmGet(): Get Diagnosis Alarm / DeviceStatus: Diagnosis Alarm is available */
#define DFPN_PROCESS_ALARM                         0x02  /*!< AlarmGet(): Get Process Alarm / DeviceStatus: Process Alarm is available */ 
#define DFPN_MODULE_DIFF_BLOCK_FLAG                0x04  /*!< DeviceStatus: Module Diff Block is available. Read */

/* Alarm Handling Modes */
#define DFPN_AUTOMATIC_ALARM_ACKNOWLEDGE           0x1   /*!< Acknowledge after receive */
#define DFPN_MANUAL_ALARM_ACKNOWLEDGE              0x2   /*!< Acknowledge with acyclic request */
#define DFPN_READ_ALARM_ACKNOWLEDGE                0x3   /*!< Acknowledge after alarm read it with ModuleDiffBlockGet() */


/* Alarm Types */
#define DFPN_ALARM_TYPE_DIAG_APPEARS                                      0x0001 /*!< Device Mode: will be sent by DeviceSetDiag() */
#define DFPN_ALARM_TYPE_PROCESS                                           0x0002 /*!< Device Mode: can be sent by DeviceSendProcessAlarm() */
#define DFPN_ALARM_TYPE_PULL                                              0x0003 /*!< Device Mode: will be sent by DeviceChangeModState() */
#define DFPN_ALARM_TYPE_PLUG                                              0x0004 /*!< Device Mode: will be sent by DeviceChangeModState() */
#define DFPN_ALARM_TYPE_RELEASED                                          0x0009 /*!< Device Mode: will be sent by StopProfinetDevice() during active communication */
#define DFPN_ALARM_TYPE_PLUG_WRONG_SUBMODULE                              0x000A /*!< Device Mode: Alarm will never occour, because the DFPNIO Card prevent this */
#define DFPN_ALARM_TYPE_RETURN_OF_SUBMODULE                               0x000B /*!< Device Mode: will be sent by setting IOxS from BAD to GOOD */
#define DFPN_ALARM_TYPE_DIAG_DISAPPEARS                                   0x000C /*!< Device Mode: will be sent by DeviceSetDiag() */

/* ModuleIds for fixed module configuration - DeviceSetFixedModules() */
#define DFPN_MODULE_ID_IO_1               0x20  /*!< 1 Byte Input / 1 Byte Output */
#define DFPN_MODULE_ID_IO_2               0x21  /*!< 2 Byte Input / 2 Byte Output */
#define DFPN_MODULE_ID_IO_4               0x22  /*!< 4 Byte Input / 4 Byte Output */
#define DFPN_MODULE_ID_IO_8               0x23  /*!< 8 Byte Input / 8 Byte Output */
#define DFPN_MODULE_ID_IO_16              0x24  /*!< 16 Byte Input / 16 Byte Output */
#define DFPN_MODULE_ID_IO_32              0x25  /*!< 32 Byte Input / 32 Byte Output */
#define DFPN_MODULE_ID_IO_64              0x26  /*!< 64 Byte Input / 64 Byte Output */
#define DFPN_MODULE_ID_IO_128             0x27  /*!< 128 Byte Input / 128 Byte Output */
#define DFPN_MODULE_ID_IO_256             0x28  /*!< 256 Byte Input / 256 Byte Output */
#define DFPN_MODULE_ID_IN_1               0x30  /*!< 1 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_2               0x31  /*!< 2 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_4               0x32  /*!< 4 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_8               0x33  /*!< 8 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_16              0x34  /*!< 16 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_32              0x35  /*!< 32 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_64              0x36  /*!< 64 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_128             0x37  /*!< 128 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_IN_256             0x38  /*!< 256 Byte Input / 0 Byte Output */
#define DFPN_MODULE_ID_OUT_1              0x40  /*!< 0 Byte Input / 1 Byte Output */
#define DFPN_MODULE_ID_OUT_2              0x41  /*!< 0 Byte Input / 2 Byte Output */
#define DFPN_MODULE_ID_OUT_4              0x42  /*!< 0 Byte Input / 4 Byte Output */
#define DFPN_MODULE_ID_OUT_8              0x43  /*!< 0 Byte Input / 8 Byte Output */
#define DFPN_MODULE_ID_OUT_16             0x44  /*!< 0 Byte Input / 16 Byte Output */
#define DFPN_MODULE_ID_OUT_32             0x45  /*!< 0 Byte Input / 32 Byte Output */
#define DFPN_MODULE_ID_OUT_64             0x46  /*!< 0 Byte Input / 64 Byte Output */
#define DFPN_MODULE_ID_OUT_128            0x47  /*!< 0 Byte Input / 128 Byte Output */
#define DFPN_MODULE_ID_OUT_256            0x48  /*!< 0 Byte Input / 256 Byte Output */


#ifdef __linux__
#define DFPN_DLL_EXPORT
#else // windows
#define DFPN_DLL_EXPORT __declspec(dllexport)
#endif

/* This is the list of exported functions, which contains all PNIO functionality and DLL management */

/*! @name DLL function interface
    Exported functions related to DLL management and driver access 
*/
//@{

/***********************************************************************************/
/*!
Routine Description:
   - GetDllVersion() provides the DLL version string of this instance of the DLL.
     

@param[out]   ppVersionString Pointer to an address where the DLL version string is stored
                              in DLL managed memory. The application can just access this
                              string by pointer usage

@return       INT32U  An error_class, here always ERROR_CLASS_NO_ERROR.

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetDllVersion(CHAR8** ppVersionString);

/***********************************************************************************/
/*!
Routine Description:
   - ReInitDLLInstance() Initializes the DLL like at the first start. All dynamic
     allocated structures and memory as well as lists are cleaned rsp. freed.
     A call to this function makes sense e.g. when during the user application is
     active, another controller is activated by demand of the service control manager.
     After this function call the new situation (number of active controller boards) 
     is represented by the DLL internal structures.
     

@param[out]   pBoardCount Pointer to a value, that contains the number of active controllers then.

@return       INT32U      An error_class, in case of problems during DLL reinitialization

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioReInitDLLInstance(INT32U* pBoardCount);

/***********************************************************************************/
/*!
Routine Description:
   - ReleaseDLL() Release the DLL allocated memory areas and all Handles to the driver, 
     so that the driver can be unloaded without to quit the application.
     After this function call it is necessary, that the application calls ReInitDLLInstance 
     again.
     

@param[out]   -

@return       INT32U      An error_class, in case of problems during DLL release

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioReleaseDLL(void);

/***********************************************************************************/
/*!
Routine Description:
   - GetDriverHandle() Looks in the DLL managed controller collection and get the handle
     from the CONTROLLER_FULL_INFO structure that holds the driver handle for the 
     synchronous and asynchronous device driver handle to a controller board interface.
     

@param[in]    BoardNo       The zero based index of the controller board
@param[out]   phandle       Pointer to a handle value, that contains the device driver 
                            handle to this controller boards interface then. The handle was generated
                            during the initialization of this instance of the DLL 
                            The synchronous handle is usable only in synchronous calls of Win32 API functions.
                            The asynchronous handle is usable only in asynchronous calls of Win32 API functions.
@param[in]    flags         Determines the kind of handle (synchronous or asynchronous) to give back
                            0 : then the synchronous handle is given back.
                            FILE_FLAG_OVERLAPPED : then the asynchronous handle is given back.

@return       INT32U  An error_class, in case of problems during access to the DLL internal
                            array of device driver handles. E.g. no controller present or all 
                            controllers deactivated, or the controller number of consideration
                            is too high.
                            In case of an error_class > 0, additional information is found in the 
                            error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetDriverHandle(INT32U BoardNo,
                                                   void**       phandle,
                                                   INT32U flags);
//@}


/*! @name Profinet-management interface
    Exported functions related to Profinet management 
*/
//@{
/***********************************************************************************/
/*!
Routine Description:
   - GetControllerInfo() Gives the user application access to the DLL managed controller
     information structure, if available at all, rsp. the controller count is not zero.
     That structure is generated and filled dynamically during DLL initialization and 
     reinitialization.
     

@param[out]   pBoardCount  Pointer to a value, that contains the number of active controllers then.
@param[out]   ppInfo       Pointer to a pointer value, that contains the address of the controller 
                           information data then or NULL, when no controller was detected as active.

@return       INT32U       An error_class. In case of no controller was detected as active, additional
                           information is found in the error value, which can be read by a call to 
                           GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetControllerInfo(INT32U* pBoardCount, CONTROLLER_INFO** ppInfo);
/***********************************************************************************/
/*!
Routine Description:
   - InitStartProfinet() initializes a controller board and starts the profinet traffic.
     It performs the following steps by call to DLL internal functions
     to set the profinet into a state where cyclic datatransfer is available.
      - starts the Profinet (which uses the configuration file ippnio.xml)
      - parses the firmware built configuration offset table to get
         - Process image input and output length
         - Offset information and a list of all elements
         - Allocates necessary memory for the internal structures
      - get first time the cyclic input and output data and fills the allocated memory
      - sets an DLL internal initialization flag as indicator for other functions, which
        reley on this function to be called befor one time.

@param[in]   BoardNo       The zero based index of the controller board.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioInitStartProfinet(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - StopProfinet() just stopps the profinet traffic on a controller board.

@param[in]   BoardNo       The zero based index of the controller board.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioStopProfinet(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - IdentifyAllDevices() is the DCP function that gives back the information of Profinet
     of which devices are available in the network. The user has access to a DLL provided
     and filled memory area which contains a DFPN_DCP_RESULT structure for each found device.
     All [out] values are preset to 0. If the function was successful, these values will 
     contain the number of found devices, the size of the DLL managed buffer and a pointer 
     to that buffer. 
     

@param[in]   BoardNo       The zero based index of the controller board
@param[out]  pDevices      Pointer to an user supplied value, which returns the number of found devices  
@param[out]  pSize         Pointer to an user supplied value, which returns the size of device information.
                           This is the number of found devices * size of the structure DFPN_DCP_RESULT 
@param[out]  ppInfo        Pointer to an address of a DLL supplied memory area, which contains the device
                           information, or NULL, when BoardNo is a invalid number.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioIdentifyAllDevices(INT32U  BoardNo,
                                                      INT32U* pDevices, 
                                                      INT32U* pSize, 
                                                      CHAR8**        ppInfo);

/***********************************************************************************/
/*!
Routine Description:
   - SetDeviceName() is the DCP function that sets a device a new name.
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   IdAllDevice   The one based device number, which should get a new name assigned.
                           This number must be > 0 and <= the number of found devices.  
@param[in]   pName         Pointer to an user supplied string with the new name.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/                                                                                  
DFPN_DLL_EXPORT INT32U CsPnioSetDeviceName(INT32U BoardNo,
                                                 INT32U IdAllDevice, 
                                                 CHAR8*        pName);


/***********************************************************************************/
/*!
Routine Description:
   - SetDeviceIP() is the DCP function that sets a device a new IP.
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   IdAllDevice   The one based device number, which should get a new IP assigned.
                           This number must be > 0 and <= the number of found devices.  
@param[in]   newIP         IP which should be set to the device.
@param[in]   newSubnetMask SubnetMask which should be set to the device.
@param[in]   newGateway    Gateway which should be set to the device.
@param[in]   permanent     0 = IP is stored volatile, else IP is stored permanent,.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/                                                                                  
DFPN_DLL_EXPORT INT32U CsPnioSetDeviceIP(INT32U BoardNo,
                                               INT32U IdAllDevice, 
                                               INT8U newIP[4],
                                               INT8U newSubnetMask[4],
                                               INT8U newGateway[4],
                                               INT8U permanent); // 1 = true = permanent   0 = false = volatile   


/***********************************************************************************/
/*!
Routine Description:
   - SetStationName() sets the stationname of the controller
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   pName         Pointer to an user supplied, zero terminated, string with the new name.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioSetStationName(INT32U BoardNo,
                                                  CHAR8* pName);

/***********************************************************************************/
/*!
Routine Description:
   - WriteRecord() writes a record to a specific device
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   DeviceNo      Number of the device
@param[in]   Slot          Slot Number which have to be accessed
@param[in]   SubSlot       SubSlot Number which have to be accessed
@param[in]   Index         Record Index
@param[in]   pData         Pointer to user provided data, which have to be written
@param[in]   Length        Length of pData
@param[in]   pResult       Pointer to a user provided structure with detailed results, filled by DLL

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioWriteRecord(INT32U BoardNo,
                                              INT32U DeviceNo, 
                                              INT16U Slot, 
                                              INT32U SubSlot,
                                              INT16U Index, 
                                              INT8U *pData, 
                                              INT32U Length, 
                                              DFPN_RECORD_RESULT *pResult);

/***********************************************************************************/
/*!
Routine Description:
   - ReadRecord() reads a record from a specific device
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   DeviceNo      Number of the device
@param[in]   Slot          Slot Number which have to be accessed
@param[in]   SubSlot       SubSlot Number which have to be accessed
@param[in]   Index         Record Index
@param[in]   pData         Pointer to a user provided read memmory
@param[in]   Length        Length of pData
@param[in]   pResult       Pointer to a user provided structure with detailed results, filled by DLL

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioReadRecord(INT32U BoardNo,
                                             INT32U DeviceNo, 
                                             INT16U Slot, 
                                             INT32U SubSlot,
                                             INT16U Index, 
                                             INT8U *pData, 
                                             INT32U Length, 
                                             DFPN_RECORD_RESULT *pResult);

/***********************************************************************************/
/*!
Routine Description:
   - ModuleDiffBlockGet() gets the module diff block from a specific device
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   DeviceNo      Number of the device
@param[out]  ppModDiffBlock  Pointer to result structure which is filled with the module
                           diff block. The data is valid until the next acyclic call.


@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioModuleDiffBlockGet(INT32U BoardNo,
                                                      INT32U DeviceNo, 
                                                      DFPN_MODULE_DIFF_BLOCK **ppModDiffBlock);

/***********************************************************************************/
/*!
Routine Description:
   - AlarmGet() gets the alarms of a specific device
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   DeviceNo      Number of the device
@param[in]   AlarmClass    Type of Alarm which have to be read 
                           - DFPN_DIAGNOSIS_ALARM
                           - DFPN_PROCESS_ALARM
@param[out]  pAlarmNotification  Pointer too result structure which is filled


@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioAlarmGet(INT32U BoardNo,
                                            INT32U DeviceNo, 
                                            INT32U AlarmClass,
                                            DFPN_ALARM_NOTIFICATION *pAlarmNotification);

/***********************************************************************************/
/*!
Routine Description:
   - AlarmSetMode() set the alarm handling
     

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   Mode          Target Mode
                           - DFPN_AUTOMATIC_ALARM_ACKNOWLEDGE
                           - DFPN_MANUAL_ALARM_ACKNOWLEDGE
                           - DFPN_READ_ALARM_ACKNOWLEDGE


@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioAlarmSetMode(INT32U BoardNo, INT32U Mode);

/***********************************************************************************/
/*!
Routine Description:
   - AlarmAck() acknowleges the last alarm in DFPN_MANUAL_ALARM_ACKNOWLEDGE Mode
     

@param[in]   BoardNo       The zero based index of the controller board.


@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioAlarmAck(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
- CsPnioEnableWatchdog() Enables the application triggered watchdog. Retrigger is
done by writing the processdata.

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   TimeoutMs     Timeout in ms until the Watchdog expires

@return      INT32U  An error_class, in case of the command was not processed correctly.
In case of a error_class > 0 additional information is found in
the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioEnableWatchdog(INT32U BoardNo, INT32U TimeoutMs);

/***********************************************************************************/
/*!
Routine Description:
- GetArInfo() reads the available Ar information of the current running configuration

@param[in]   BoardNo       The zero based index of the controller board.
@param[out]  pNumberOfAr   Return the Number of Available Ars
@param[out]  ppArInfo      Points to an Array of Objects. Pointer is valid until next acyclic call

@return      INT32U  An error_class, in case of the command was not processed correctly.
In case of a error_class > 0 additional information is found in
the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetArInfo(INT32U BoardNo, INT16U *pNumberOfAr, DFPN_AR_INFO  **ppArInfo);


/***********************************************************************************/
/*!
Routine Description:
- CsPnioDownloadConfig() downloads a new bus configuration to the controller.


@param[in]   BoardNo            The zero based index of the controller board.
@param[in]   pBusConfigFileName path and name of the configuration file

@return      INT32U  An error_class, in case of the command was not processed correctly.
In case of a error_class > 0 additional information is found in
the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDownloadConfig(INT32U BoardNo, CHAR8* pBusConfigFileName);


/***********************************************************************************/
/*!
Routine Description:
- CsPnioFirmwareDownload() downloads a Firmware file to the controller.
  Than the Firmware will be updated and the board will be rebooted.


@param[in]   BoardNo            The zero based index of the controller board.
@param[in]   pFirmwareFileName  path and name of the firmware file

@return      INT32U  An error_class, in case of the command was not processed correctly.
In case of a error_class > 0 additional information is found in
the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioFirmwareDownload(INT32U BoardNo, CHAR8* pFirmwareFileName);


/***********************************************************************************/
/*!
Routine Description:
   - StopProfinetDevice() just starts the profinet traffic on a device board.

@param[in]   BoardNo       The zero based index of the controller board.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioStartProfinetDevice(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - StopProfinetDevice() just stopps the profinet traffic on a device board.

@param[in]   BoardNo       The zero based index of the controller board.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioStopProfinetDevice(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - DeviceChangeModState() pull or plug a submodule

@param[in]   BoardNo       The zero based index of the device board.
@param[in]   Slot          Slot Number
@param[in]   PlugState     0 = pull module / 1 = plug module

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceChangeModState(INT32U BoardNo,
                                                        INT32U Slot, 
                                                        INT32U PlugState);

/***********************************************************************************/
/*!
Routine Description:
   - DeviceSetFixedModules() sets a fixed module configuration. This can only be
     done if the device is not running. The fixed configuration is stored volatile
     and have to be set after each reset. If "Number of Slots" is 0, the fixed configuration
     will be disabled. 

@param[in]   BoardNo       The zero based index of the device board.
@param[out]  pModuleIds    Pointer to an Array of ModuleIds
@param[out]  NumberOfSlots  Length of pModuleIds Array

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceSetFixedModules(INT32U BoardNo,
                                                         INT32U *pModuleIds,
                                                         INT32U NumberOfSlots);

/***********************************************************************************/
/*!
Routine Description:
   - DeviceSendAlarm() sends an alarm

@param[in]   BoardNo       The zero based index of the device board.
@param[in]   Slot          Slot Number
@param[in]   AlarmType     Type of Alarm accoding the PNIO specification
@param[in]   pAlarmItem    Additional Alarm information (max 200 Byte) 
@param[in]   ItemLength    Length of pAlarmItem

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceSendAlarm(INT32U BoardNo,
                                                   INT16U Slot,   
                                                   INT16U AlarmType, 
                                                   INT8U *pAlarmItem, 
                                                   /* for pAlarmItem can DFPN_STD_ALARM_TYPE be used (cast to INT8U *)
                                                      the first two bytes represent a Structure identifier. This is used by
                                                      the controller to identfiy the following data. For more information refer
                                                      the ProfinetIO specification documents. 
                                                      The AlarmItem is optional and can also be NULL*/
                                                   INT16U ItemLength); 

#define DeviceSendProcessAlarm(BoardNo, Slot, pAlarmItem, ItemLength) CsPnioDeviceSendAlarm(BoardNo, Slot, DFPN_ALARM_TYPE_PROCESS, pAlarmType, ItemLength) /*!< Macro for sending a process alarm */

/***********************************************************************************/
/*!
Routine Description:
   - DeviceGetArInfo() reads the information about active application relations.

@param[in]   BoardNo       The zero based index of the device board.
@param[out]  pNumberOfAr   Return the Number of Available Ars
@param[out]  ppArInfo      Points to an Array of Objects. Pointer is valid until next acyclic call

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceGetArInfo(INT32U BoardNo,
                                                   INT16U *pNumberOfAr,
                                                   DFPN_AR_INFO  **ppArInfo);

/***********************************************************************************/
/*!
Routine Description:
   - DeviceSetDiag() sets and removes diagnostic data

@param[in]   BoardNo       The zero based index of the device board.
@param[in]   Slot          Slot Number
@param[in]   pDiagItem     Pointer to object of type DFPN_DIAG_INFO

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceSetDiag(INT32U BoardNo,
                                                 INT16U Slot, 
                                                 DFPN_DIAG_INFO  *pDiagItem);

/***********************************************************************************/
/*!
Routine Description:
   - DeviceDiagReset() Reset the whole diagnosis module (all slots)

@param[in]   BoardNo       The zero based index of the device board.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceDiagReset(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - DeviceSetRecordData() sets the recorddata of a specific slot on device

@param[in]   BoardNo       The zero based index of the device board.
@param[in]   Slot          Slot Number
@param[in]   Index         Index of the Record. The first is 0 and is mapped to Index 0x1000
@param[in]   ReadAccess    if 0 the Record is not readable
@param[in]   WriteAccess   if 0 the Record is not writeable
@param[in]   pData         Data for the Record
@param[in]   Length        Length of pData. Maximum is 1440

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceSetRecordData(INT32U BoardNo,
                                                       INT16U Slot, 
                                                       INT16U Index, 
                                                       INT8U ReadAccess, 
                                                       INT8U WriteAccess, 
                                                       INT8U *pData,
                                                       INT16U Length) ;

/***********************************************************************************/
/*!
Routine Description:
   - DeviceGetRecordData() gets the recorddata of a specific slot

@param[in]    BoardNo      The zero based index of the device board.
@param[in]    Slot         Slot Number
@param[in]    Index        Index of the Record. The first is 0 and is mapped to Index 0x1000
@param[out]   ReadAccess   if 0 the Record is not readable
@param[out]   WriteAccess  if 0 the Record is not writeable
@param[out]   pData        Data for the Record
@param[inout]   Length     Length of pData. Maximum is 1440

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceGetRecordData(INT32U BoardNo,
                                                       INT16U Slot, 
                                                       INT16U Index,  
                                                       INT8U *ReadAccess, 
                                                       INT8U *WriteAccess, 
                                                       INT8U *pData, 
                                                       INT16U *Length) ;

/***********************************************************************************/
/*!
Routine Description:
   - DeviceEnableWatchdog() Enables the application triggered watchdog. Retrigger is
     done by writing the processdata.

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   TimeoutMs     Timeout in ms until the Watchdog expires

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceEnableWatchdog(INT32U BoardNo,
                                                        INT32U TimeoutMs) ;

//@}

/*! @name Profinet-communication interface
    Exported functions related to Profinet communication 
*/
//@{   
/***********************************************************************************/
/*!
Routine Description:
   - GetInputPointer() provides accesses to the internal data structure that contains 
     the latest PNIO input values for a controllers devices slots subslot. 
     The data are the most recent data after a preceeding call to RW_ProcessData. 
     The data- and APDU stati access-pointer are given to the user application by the [out] values.
     A call to this function is possible only after the DLL internal structures are 
     allocated and prepared by a call to InitStartProfinet.
     Once the user application retrieved the pointer to the data and status values, the
     application can reuse this pointer and length value without a new call to this function
     unless the DLL was re-initialized.

     Note: In case of accessing ProcessData in DeviceMode, the parameter Device is always INTERN_DEV_OFFSET.
           If the function RW_DeviceProcessData returns the Error RET_DLL_CYCLIC_DATA_NOT_INITIALIZED,
           the application has to recall this function, because the pointers maybe have changed.

@param[in]   BoardNo       The zero based index of the controller/device board.
@param[in]   Device        The one based index of the device of interest.
@param[in]   SlotNumber    The slot number on the device of interest, according to the device configuration.
@param[in]   SubslotNumber The subslot number on the slot of the device of interest, according to the device configuration.
@param[out]  pSubModulType Pointer to a value that indicates the IO type (NO_IO, INPUT, OUTPUT or IO) of the submodule the subslot belongs to.
@param[out]  plength       Pointer to a value that contains the length of the IO data of the subslot of request.
@param[out]  ppdata        Pointer to an address of a DLL managed buffer that contains the IO data of the subslot of request.
@param[out]  ppiops        Pointer to an address of a DLL managed buffer that contains the provider status of the subslot of request.
@param[out]  ppiocs        Pointer to an address of a DLL managed buffer that contains the consumer status of the subslot of request.
@param[out]  ppApduStatus  Pointer to an address of a DLL managed buffer that contains the APDU stati of all configured devices.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetInputPointer(INT32U    BoardNo,
                                                   INT32U    Device,
                                                   INT16U  SlotNumber, 
                                                   INT16U  SubslotNumber, 
                                                   INT32U*   pSubModulType,
                                                   INT16U* plength,
                                                   INT8U** ppdata,
                                                   INT8U** ppiops,
                                                   INT8U** ppiocs,
                                                   INT8U** ppApduStatus);

/***********************************************************************************/
/*!
Routine Description:
   - GetOutputPointer() provides accesses to the internal data structure that contains 
     the latest PNIO output values for a controllers devices slots subslot. These data 
     will be written to the controller at the next update by a call to RW_ProcessData.
     The user application writes new PNIO output data and APDU status values to the DLL 
     allocated data areas by use of the pointers returned in the [out] values.

     Once the user application retrieved the pointer to the data and status values, the
     application can reuse this pointer and length value without a new call to this function 
     unless the DLL was re-initialized.

     A call to this function is possible only after the DLL internal structures are 
     allocated and prepared by a call to InitStartProfinet.
     
     Note: In case of accessing ProcessData in DeviceMode, the parameter Device is always INTERN_DEV_OFFSET.
           If the function RW_DeviceProcessData returns the Error RET_DLL_CYCLIC_DATA_NOT_INITIALIZED,
           the application has to recall this function, because the pointers maybe have changed.

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   Device        The one based index of the device of interest.
@param[in]   SlotNumber    The slot number on the device of interest, according to the device configuration.
@param[in]   SubslotNumber The subslot number on the slot of the device of interest, according to the device configuration.
@param[out]  pSubModulType Pointer to a value that indicates the IO type (NO_IO, INPUT, OUTPUT or IO) of the submodule the subslot belongs to.
@param[out]  plength       Pointer to a value that contains the length of the IO data of the subslot of request.
@param[out]  ppdata        Pointer to an address of a DLL managed buffer that contains the IO data of the subslot of request.
@param[out]  ppiops        Pointer to an address of a DLL managed buffer that contains the provider status of the subslot of request.
@param[out]  ppiocs        Pointer to an address of a DLL managed buffer that contains the consumer status of the subslot of request.
@param[out]  ppApduStatus  Pointer to an address of a DLL managed buffer that contains the APDU stati of all configured devices.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetOutputPointer(INT32U    BoardNo,
                                                    INT32U    Device,
                                                    INT16U  SlotNumber, 
                                                    INT16U  SubslotNumber, 
                                                    INT32U*   pSubModulType,
                                                    INT16U* plength,
                                                    INT8U** ppdata,
                                                    INT8U** ppiops,
                                                    INT8U** ppiocs,
                                                    INT8U** ppApduStatus);

/***********************************************************************************/
/*!
Routine Description:
   - GetDeviceData() provides accesses to the DLL managed internal data structures that  
     contains the latest pure PNIO input and output values (without status values) for a device. 
     The data are the most recent data after a preceeding call to RW_ProcessData.
     The data access pointer, the data length, a sum status over all subslots and the 
     APDU status are given to the user application by the filled DEVICE_INFO structure.
     Once the user application retrieved the pointer to the DEVICE_INFO structure, the
     application can reuse this pointer without a new call to this function unless the
     DLL was re-initialized.

     A call to this function is possible only after the DLL internal structures are 
     allocated and prepared by a call to InitStartProfinet.
     
     Note: In case of accessing ProcessData in DeviceMode, the parameter Device is always INTERN_DEV_OFFSET.
           If the function RW_DeviceProcessData returns the Error RET_DLL_CYCLIC_DATA_NOT_INITIALIZED,
           the application has to recall this function, because the pointers maybe have changed.

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   Device        The one based index of the device of interest.
@param[in]   pDeviceInfo   Pointer to a user supplied DEVICE_INFO structure.
                           This structure provides the APDU status, the Device status and
                           the ored stati values of all input elements and the ored value
                           of all output elements as well as the length values of input
                           and of output data and pointer to the DLL managed memory area
                           of the pure (without ststus values in between) input and output data.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetDeviceData(INT32U BoardNo,
                                                 INT32U Device,
                                                 DEVICE_INFO* pDeviceInfo);

/***********************************************************************************/
/*!
Routine Description:
   - SetDeviceData() provides accesses to the DLL managed internal data structures that  
     contains the latest pure PNIO input and output values (without status values) for a device. 
     The output data are set to new values. The PNIO input data remain unchanged.
     These DLL internal new data are sent to the controller with a call to RW_ProcessData.
     The user application provides the new output data and device status value by a filled   
     DEVICE_INFO structure.

     A call to this function is possible only after the DLL internal structures are 
     allocated and prepared by a call to InitStartProfinet.

     Note: In case of accessing ProcessData in DeviceMode, the parameter Device is always INTERN_DEV_OFFSET.
           If the function RW_DeviceProcessData returns the Error RET_DLL_CYCLIC_DATA_NOT_INITIALIZED,
           the application has to recall this function, because the pointers maybe have changed.

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   Device        The one based index of the device of interest.
@param[in]   pDeviceInfo   Pointer to a user supplied DEVICE_INFO structure.
                           This structure provides the APDU status, the Device status and
                           the ored stati values of all input elements and the ored value
                           of all output elements as well as the length values of input
                           and of output data and pointer to the DLL managed memory area
                           of the pure (without ststus values in between) input and output data.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).
                           Or full analyzed error status in text form by a call to
                           GetDLLErrorText( error_class, GetControllerError(#) )

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioSetDeviceData(INT32U BoardNo,
                                                 INT32U Device,
                                                 DEVICE_INFO* pDeviceInfo);


/***********************************************************************************/
/*!
Routine Description:
- CsPnioRemappingSet() is used to setup the firmware processdata remapping
    This reduces the processing time of CsPnioGetDeviceData and CsPnioSetDeviceData,
    but decreases the maximum number of configured devices/modules.

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   Enable        0 for disable / other values to enable

@return      INT32U  An error_class, in case of the command was not processed correctly.
In case of a error_class > 0 additional information is found in
the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioRemappingSet(INT32U BoardNo, INT8U Enable);

/***********************************************************************************/
/*!
Routine Description:
- CsPnioDeviceRemappingSet() is used to setup the firmware processdata remapping of the
    device instance. This reduces the processing time of CsPnioGetDeviceData and CsPnioSetDeviceData,
    but decreases the maximum number of configured devices/modules.

@param[in]   BoardNo       The zero based index of the controller board.
@param[in]   Enable        0 for disable / other values to enable

@return      INT32U  An error_class, in case of the command was not processed correctly.
In case of a error_class > 0 additional information is found in
the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeviceRemappingSet(INT32U BoardNo, INT8U Enable);

/***********************************************************************************/
/*!
Routine Description:
   - RW_ProcessData() renews the content of the DLL internal data structures with the
     complete process image. A call to this function is possible only after the DLL
     internal structures are allocated and prepared by a call to InitStartProfinet.
     The function writes new PNIO output values to the driver and reads the latest 
     PNIO input values from the driver. 
     This function is the only way to exchange data with the controller.
     The new PNIO output values for a slot/subslot are supplied by a call to GetOutputPointer.
     The new PNIO input values for a slot/subslot are accessable by a call to GetInputPointer.

@param[in]   BoardNo       The zero based index of the controller board.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioRW_ProcessData(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - RW_DeviceProcessData() renews the content of the DLL internal data structures with the
     complete process image. 
     The function writes new PNIO input values to the driver and reads the latest 
     PNIO output values from the driver. 
     This function is the only way to exchange data with the controller.
     The new PNIO output values for a slot/subslot are supplied by a call to GetOutputPointer.
     The new PNIO input values for a slot/subslot are supplied by a call to GetInputPointer.

@param[in]   BoardNo       The zero based index of the device board.

@return      INT32U  An error_class, in case of the command was not processed correctly.
                           In case of a error_class > 0 additional information is found in
                           the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioRW_DeviceProcessData(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - PerformService() is a low level call, where the user is responsible to fill a StdHeader
     structure with meaningful data for various parameters (at least Command, SubCommand and ID)
     and is responsible to provide enough memory to hold the PNIO input and output data of 
     the specified length in the StdHeader.Length field.
     

@param[in]   pOutpData       Pointer to a byte array that contains the PNIO output data.
@param[in]   OutpLength      Value that determines the number of data to write. 
@param[in]   pInpData        Pointer to a byte array that contains the PNIO input data then.
@param[in]   InpLength       Value that determines the number of data to read. 

@return      INT32U    An error_class, in case of the command was not processed correctly.
                             In case of a error_class > 0 additional information is found in
                             the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioPerformService(INT8U pOutpData[],
                                                  INT32U  OutpLength, 
                                                  INT8U pInpData[],  
                                                  INT32U  InpLength);
//@}


/*! @name UDP functions
Exported functions related to udp handling
*/
//@{
/***********************************************************************************/
/*!
Routine Description:
- CsPnioCreateUdpSocket()  creates and opens a socket for a given UDP-Port

@param[in]     BoardNo           The zero based index of the controller board.
@param[in]     Port              Port on which should be listen for UDP telegrams.
@param[out]    pSocketIndex      Socket Index for the user.

@return        INT32U            An error_class, in case of the command was not processed correctly.
                                 In case of a error_class > 0 additional information is found in
                                 the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioCreateUdpSocket(INT32U BoardNo,
   const INT16U Port,
   INT8S * const pSocketIndex);


/***********************************************************************************/
/*!
Routine Description:
- CsPnioDeleteUdpSocket()  deletes a socket of a given Socket-Index

@param[in]     BoardNo           The zero based index of the controller board.
@param[in]     SocketIndex       Socket Index for the user.

@return        INT32U            An error_class, in case of the command was not processed correctly.
                                 In case of a error_class > 0 additional information is found in
                                 the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioDeleteUdpSocket(INT32U BoardNo, const INT8S SocketIndex);


/***********************************************************************************/
/*!
Routine Description:
- CsPnioSendUdpData()  sends a UDP telegram with the given message to a special IP address and port

@param[in]     BoardNo           The zero based index of the controller board.
@param[in]     SocketIndex       Socket Index for the user. 
@param[in]     IpAddress         IP address where the data should be send to
@param[in]     Port              Port where the data should be send to
@param[in]     MsgData           Data which should be send
@param[out]    MsgLen            Length of message which should be send
                                 out: Length of sent message
                                 
                                 
@return        INT32U            An error_class, in case of the command was not processed correctly.
                                 In case of a error_class > 0 additional information is found in
                                 the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioSendUdpData(INT32U BoardNo,
   const INT8S  SocketIndex,
   const INT32U IpAddress,
   const INT16U Port,
   const INT8U  MsgData[],
   const INT32U MsgLen);


/***********************************************************************************/
/*!
Routine Description:
- udpUserReceive()  gets the data from the socket and checks if more data is available

@param[in]     BoardNo           The zero based index of the controller board.
@param[in]     SocketIndex       Socket Index for the user.
@param[out]    pIpAddress        IP address of sender
@param[out]    pPort             Port of sender
@param[out]    MsgData           received data
@param[out]    pMsgLen           Length of received message
@param[out]    pDataAvailable    More data available for read.
                                 - User should call this function with same SocketIndex again

@return        INT32U            An error_class, in case of the command was not processed correctly.
                                 In case of a error_class > 0 additional information is found in
                                 the error value, which can be read by a call to GetControllerError(#).

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioReceiveUdpData(INT32U BoardNo,
   INT8S  SocketIndex,
   INT32U * const pIpAddress,
   INT16U * const pPort,
   INT8U    MsgData[],
   INT32U * const pMsgLen,
   INT8U  * const pDataAvailable);


//@}


/*! @name Error handling functions
    Exported functions related to error handling and text information about error sources 
*/
//@{
/***********************************************************************************/
/*!
Routine Description:
   - SetDLLTextOutput() controls the output of DLL internal informations. The DLL
     sets this Flag "DLL_text_output" to false during initialization.
     In the debug version of the DLL the Flag is true by default whereas in
     release versions it is false by default. 
     This feature is only supported on windows systems.

@param[in]           show_text    Flag to show DLL internal Informations or not. In console
                                  applications the info-text is shown in the console window,
                                  in window applications the information is shown in a popup
                                  dialog window.

@return              INT32S          The value which was active before the function was called.
                                  With that value the application can restore the former setting.

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32S          CsPnioSetDLLTextOutput(INT32S show_text);

/***********************************************************************************/
/*!
Routine Description:
   - GetControllerError() just returns the global error value, set by the DLL in case of an error.
     It is called from aplications to get more detailed information when an error class was set.


@param[in]           BoardNo  The zero based index of the controller board.

@return              INT32U   The error value, which was last set by the DLL

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT INT32U CsPnioGetControllerError(INT32U BoardNo);

/***********************************************************************************/
/*!
Routine Description:
   - GetDLLErrorText() converts the information of error_class and error into readable
     text in CHAR8 format and returns a pointer to the text.


@param[in]           error_class  Specifies the level in which an error occurred
@param[in]           error        More detailed information to the specific fault reason

@return              CHAR8*        Pointer to DLL allocated memory which holds the error
                                  reason in CHAR8 text format up to a length of MAX_TEXT_LEN

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT CHAR8*        CsPnioGetDLLErrorText(INT32U error_class,
                                                   INT32U error);

/***********************************************************************************/
/*!
Routine Description:
   - GetDLLErrorTextW() converts the information of error_class and error into readable
     text in wide format and returns a pointer to the wide text.


@param[in]           error_class  Specifies the level in which an error occurred
@param[in]           error        More detailed information to the specific fault reason

@return              wchar_t*     Pointer to DLL allocated memory which holds the error
                                  reason in wide text format up to a length of MAX_TEXT_LEN

Details and remarks for this function:
************************************************************************************/
DFPN_DLL_EXPORT wchar_t*     CsPnioGetDLLErrorTextW(INT32U error_class,
                                                    INT32U error);
//@}

#pragma pack(pop, Kunbus)

#ifdef __cplusplus
}
#endif

#endif // _DFPNIO_H_EDB10C66_8034_4f4f_9090_7FE899314E4A_
