/*****************************************************************************/
/*                                                                           */
/*    -----------------------------------------------                        */
/*    dfprnet_mod.h  -  Includefile fuer df_prnet Board-Treiber              */
/*    unter Linux                                                            */
/*    Version 1.0                                                            */
/*    -----------------------------------------------                        */
/*                                                                           */
/*    ---------------------------------                                      */
/*    COMSOFT  GmbH                                                          */
/*    Wachhausstr. 5a                                                        */
/*    7500 Karlsruhe 41                                                      */
/*    Tel.  0721 / 40908-0 ( Zentrale )                                      */
/*    ---------------------------------                                      */
/*                                                                           */
/*    ------------------------------------                                   */
/*    Datum : 27.01.2015  Weber, Wolfgang                                    */
/*    ------------------------------------                                   */
/*                                                                           */
/*    ------------------------------------------------------------           */
/*    Geaendert : 21.04.2015        Name : Weber, Wolfgang                   */
/*    Grund     :                                                            */
/*    ------------------------------------------------------------           */
/*                                                                           */
/*****************************************************************************/

#include <linux/types.h>
#include "df_profinet.h"

#define vremap(a,b) ioremap((a),(b))

#define DF_PRINT printk
#define MAX_BAUGRUPPE   5
#define  FALSE          0

#ifndef TRUE
#define  TRUE         1
#endif
#define  NOT_INIT         1



typedef uint8_t UINT_8;
typedef uint16_t UINT_16;
typedef uint32_t UINT_32;
typedef uint8_t BYTE;

typedef INT32U DWORD;

typedef long LONG;
typedef unsigned char UCHAR;
typedef unsigned short USHORT;
typedef INT32U ULONG;

typedef size_t SIZE_T;
typedef unsigned char * PUCHAR;

//#define NTSTATUS int

// WWe 27.04.2015
//#define FIELD_OFFSET(type, field)    ((unsigned int)&(((type *)0)->field))
#define FIELD_OFFSET(type, field)    ((unsigned long)&(((type *)0)->field))

#define df_delay_high_IRQL(a)        msecs_to_jiffies(a)


#define jiffies_from_ms(a)    msecs_to_jiffies(a)

//typedef volatile DWORD VREG_32;
typedef volatile uint32_t VREG_32;

#define STATUS_NOT_SUPPORTED  -1


#define TEST_VAL            8


#define INTERRUPT_TIMEOUT_60S    60000 /*!< Default timeout for waiting on a controller command to return, if not specified different in the registry */

#define RESET_TIMEOUT_MS           500 /*!< ms ganularity for checking the reset result FW_READY_INDICATION in inbound window globals FW_Ready and whether the firmware is still alive */
#define ALIVE_TIMEOUT_S              3 /*!< s to wait before a command is canceled in case of a stopped firmware (but still FW_READY_INDICATION set) */
#define RESET_TIMEOUT_TOTAL_S       25 /*!< s total wait time for a reset command to block an application */
#define DMA_WAIT_STALL_TIME_US       4 /*!< µs to stall the CPU during wait on DMA status bit reset */
#define DMA_TIMEOUT_MS               1 /*!< time in ms to wait on the DMA status bit reset until a DMA controller error is assumed */

#define INBOUND_VALUES_RESET 0     /*!< Flag to control the behaviour of the inbound window value manage function */
#define INBOUND_VALUES_SET   1     /*!< Flag to control the behaviour of the inbound window value manage function */

typedef struct _DESC_dfprnet_
{
    DESC_PCI *pdescpci;
    void *maininstance;

    DWORD          MSI_StatusReg;                          /*!< indicate message shared interrupt MSIR 0-7 to check in the DPC */
    DWORD          MSI_Reg[8];                             /*!< indicate message shared interrupt source 0-31 to check in the DPC */

    // DMA related informations
    PMPC8536E_DMA_REGS           pdma;                     /*!< DMA convenient pointer to register structure; volatile */
    VREG_32*                     pdma_sts;                 /*!< DMA status register; volatile */
    UINT_32                      dma_simpson_addr;         /*!< DMA destination address in controller DRAM in controller notation */
    UINT_32                      dma_source_addr;          /*!< DMA source address in contiguous memory in controller notation */
    PVOID                        contig_dma_src_addr;      /*!< DMA source address in contiguous memory in driver notation */
    P_COMMON_INTERFACE_TYPE      interface_addr;           /*!< Virtual destination address for acyclic commands and data */
    VREG_32                      proc_image_input_len;     /*!< Size of the process image input data; the driver reads this info from the controller during initialization */
    VREG_32                      proc_image_output_len;    /*!< Size of the process image output data; the driver reads this info from the controller during initialization */

    PMPC8536E_DMA_REGS        pdev_dma;                    /*!< DMA convenient pointer to register structure; volatile */
    VREG_32*                  pdev_dma_sts;                /*!< DMA status register; volatile */
    UINT_32                   dev_dma_simpson_addr;        /*!< DMA destination address in controller DRAM in controller notation */
    UINT_32                   dev_dma_source_addr;         /*!< DMA source address in contiguous memory in controller notation */
    PVOID                     dev_contig_dma_src_addr;     /*!< DMA source address in contiguous memory in driver notation */
    P_COMMON_INTERFACE_TYPE   dev_interface_addr;          /*!< Virtual destination address for acyclic commands and data */
    VREG_32                   dev_proc_image_input_len;    /*!< Size of the process image input data; the driver reads this info from the controller during initialization */
    VREG_32                   dev_proc_image_output_len;   /*!< Size of the process image output data; the driver reads this info from the controller during initialization */
    DWORD                     dev_picounter;               /*!< last known processimage number. is used to detect a changed processimage */

    P_GLOBALS_DATA_TYPE              pinbound_global;      /*!< Virtual address of global informations in the inbound window */
    P_INBOUND_CYCLIC_DATA_TYPE       pinbound_cyclic;      /*!< Virtual address for cyclic data access to PN input data from simpson buffer by DMA */
    P_INBOUND_DEV_CYCLIC_DATA_TYPE   pinbound_devcyclic;   /*!< Virtual address for device cyclic data access to pn output data from simposn buffer by DMA */
    P_OUTBOUND_GLOBALS_DATA_TYPE     poutbound_global;     /*!< Virtual address of global informations in the outbound window */
    P_OUTBOUND_CYCLIC_DATA_TYPE      poutbound_cyclic;     /*!< Virtual address for cyclic data access from PN output data from simpson buffer by DMA */
    P_OUTBOUND_DEV_CYCLIC_DATA_TYPE  poutbound_devcyclic;  /*!< Virtual address for device cyclic data access from PN input data from simpson buffer by DMA */

    P_OUTBOUND_GLOBALS_DEV_DATA_TYPE poutbound_devglobal;    /*!< Virtual address of global informations in the outbound window */

    DWORD                   FwNeedsReadyCheck;        /*!< Flag to control the check of the inbound window */
    /* Test values */
    DWORD                   TestLevel;                     /*!< Value for runtime changes of a certain behavior via ioctl from an application */
    DWORD                   TestValue[TEST_VAL];           /*!< Value for runtime changes of a certain behavior via ioctl from an application */
    DWORD                   TestCounter[TEST_VAL];         /*!< Value for count purposes */



    INTERFACE_STD_HEADER_TYPE inout_header;
}  DESC_DFPRNET;



#define  DEV    (dev-1)
#define  SS_NR  1

#define E_100           -100
#define E_101           -101
#define E_102           -102
#define E_103           -103
#define E_104           -104
#define E_105           -105
#define E_106           -106
#define E_107           -107
#define E_108           -108
#define E_109           -109
#define E_110           -110
#define E_111           -111
#define E_112           -112
#define E_113           -113
#define E_114           -114
#define E_120           -120
#define E_121           -121
#define E_122           -122
#define E_123           -123
#define E_124           -124
#define E_125           -125
#define E_126           -126
#define E_127           -127
#define E_130           -130
#define E_131           -131
#define E_132           -132
#define E_140           -140
#define E_141           -141
#define E_150           -150

#define STATUS_SUCCESS  0

///-----------------------------------------------------------------------------------
