/*
 *
 * Project: 700361_Profinet_IO_Controller
 * (C)    : COMSOFT GmbH, Wachhausstr. 5a, D-76227 Karlsruhe
 *
 * ---- doxygen info ----------------------------------------------------------
 *
 * \file pnio_ctl.h
 *
 *
 */
   
#pragma pack(push)

#ifndef _PNIO_CTL_H_5D62B1E6_4C6B_4d6e_A67F_C3BB628D0BEB_
#define _PNIO_CTL_H_5D62B1E6_4C6B_4d6e_A67F_C3BB628D0BEB_

/*! @name PNIO-controller related
    Defines for PNIO-controller board access
*/
//@{
/// PNIO_CTL-controller addresses used by PNIO controller and
/// expected sizes of the board resources
#define PNIO_CTL_INTREQ        (0x00002000)  /*!< Interrupt request value. Write to interrupt register */

#define CTRL_DDRRAM_SIZE       (0x01000000)  /*!< DDR memory size on a DF-ProfiNet Controller */
#define CTRL_REGS_SIZE         (0x00100000)  /*!< Register size on a DF-ProfiNet Controller */
//@}

//-----------------------------------------------------------------------------   
/*! @name PCI Device/Vendor Ids */
//@{
#define PNIO_CTL_PCI_VENDOR_ID               0x1957  // PNIO-controller
#define PNIO_CTL_PCI_DEVICE_ID               0x0050  // PNIO-controller
//@}


/*************************** Register defines ***********************************/
/*! @name Register constants
    Freescale PPC Powerquick III E500 Registers 
    - IRQ areas and PIC address offsets
    - DMA related registers
    - PCI Addressmapping for outbound Window
*/
//@{
#define IRQ_GLOBAL_PIC_REGS   0x00040000 // Global PIC registers
#define IRQ_INT_SRC_CFG_REGS  0x00050000 // Interrupt source configuration registers
#define IRQ_PER_CPU_REGS      0x00060000 // Per-CPU registers

#define IRQ_SET_MSG_ENABLE   ((IRQ_GLOBAL_PIC_REGS)  | 0x1500)
#define IRQ_SET_MSG_VECTOR   ((IRQ_INT_SRC_CFG_REGS) | 0x1600)
#define IRQ_CLEAR_CTPRO      ((IRQ_PER_CPU_REGS)     | 0x0080)
#define IRQ_SET_GCR          ((IRQ_GLOBAL_PIC_REGS)  | 0x1020)
#define IRQ_TRIGGER_0        ((IRQ_GLOBAL_PIC_REGS)  | 0x1400)
#define IRQ_TRIGGER_1        ((IRQ_GLOBAL_PIC_REGS)  | 0x1410)
#define IRQ_STATUS_SUMMARY   ((IRQ_GLOBAL_PIC_REGS)  | 0x1310)
#define IRQ_MSI_STATUS       ((IRQ_GLOBAL_PIC_REGS)  | 0x1720)
#define IRQ_RESET_MSIR0      ((IRQ_GLOBAL_PIC_REGS)  | 0x1600)
//#define IRQ_RESET_MSIR(n)    ((IRQ_GLOBAL_PIC_REGS)  | (0x1600 + 0x10 * n##))    //WWe 18.02.2015 Linux Compiler Error
#define IRQ_RESET_MSIR(n)    ((IRQ_GLOBAL_PIC_REGS)  | (0x1600 + 0x10 * n))


#define DMA_REGS              0x00021000L // DMA registers base offset
#define DMA0_BASE             ((DMA_REGS)  | 0x100);
#define DMA1_BASE             ((DMA_REGS)  | 0x180);
#define DMA2_BASE             ((DMA_REGS)  | 0x200);
#define DMA3_BASE             ((DMA_REGS)  | 0x280); 
#define DMA_GENERAL_STATUS    ((DMA_REGS)  | 0x300); 


#define PCI_REGS              0x00008000L // PCI registers base offset
#define PCI_OW1_ATTRIBUTES   ((PCI_REGS)  | 0x0C30)
#define PCI_OW1_ADDR_OFS     ((PCI_REGS)  | 0x0C20)
#define PCI_OW1_EXT_ADDR_OFS ((PCI_REGS)  | 0x0C24)
#define PCI_OW1_BASE_ADDR    ((PCI_REGS)  | 0x0C28)

// The following values are the same for PCI and PCIe
#define PCI_OW_BASE          0x80000000L
#define PCI_OW_ATTR_R_MEM    ((DWORD)1 << 18)
#define PCI_OW_ATTR_R_IO     ((DWORD)1 << 19)
#define PCI_OW_ATTR_W_MEM    ((DWORD)1 << 14)
#define PCI_OW_ATTR_W_IO     ((DWORD)1 << 15)
#define PCI_OW_ATTR_ENABLE   ((DWORD)1 << 31)
#define PCI_OW_WIN_SIZE_4K   11
#define PCI_OW_WIN_SIZE_128K 16
#define PCI_OW_WIN_SIZE_256K 17
#define PCI_OW_WIN_SIZE_512K 18
#define PCI_OW_WIN_SIZE_1M   19
//#define PCI_OW_ATTR_WIN_SIZE(N) )(DWORD)(2^(N## + 1)))   //WWe 18.02.2015 Linux Compiler Error
#define PCI_OW_ATTR_WIN_SIZE(N) )(DWORD)(2^(N + 1)))
#define PITAR1         (0x00008DE0)
//@}

// PCIe Registers offsets
#define PCIE_REGS              0x0000A000L // PCIE registers base offset
#define PCIE_OW1_ADDR_OFS     ((PCIE_REGS)  | 0x0C20)
#define PCIE_OW1_EXT_ADDR_OFS ((PCIE_REGS)  | 0x0C24)
#define PCIE_OW1_BASE_ADDR    ((PCIE_REGS)  | 0x0C28)
#define PCIE_OW1_ATTRIBUTES   ((PCIE_REGS)  | 0x0C30)
#define PCIE_PEXITAR1         ((PCIE_REGS)  | 0x0DE0)



/*****************************************************************************
 ****  DEFINES                                                            ****
 *****************************************************************************/
//Communication
#define COMM_RESET                     0x00000000
#define COMM_PCI                       0x01000000
#define COMM_TCP                       0x02000000
#define COMM_UDP                       0x03000000

#define MPC8536E_DMA0     (0x00021100)
#define MPC8536E_DMA1     (0x00021180)
#define MPC8536E_DMA2     (0x00021200)
#define MPC8536E_DMA3     (0x00021280)
#define MPC8536E_DMA_STAT (0x00021300)

#define DMA_REG_CLEAR           0x00000000             
#define DMA_STATUS_CHANNEL_BUSY 0x04000000             
#define DMA_STATUS_GENERAL_CB0  0x00000004             
#define DMA_STATUS_GENERAL_CB1  0x00000400             
#define DMA_STATUS_GENERAL_CB2  0x00040000             
#define DMA_STATUS_GENERAL_CB3  0x04000000             
#define DMA_ATTR_DONT_SNOOP     0x00000400  
#define DMA_ATTR_SNOOP          0x00000500  
#define DMA_STRIDE_SIZE         0x00040000  
#define DMA_STRIDE_DIST         0x04000000  
#define DMA_STRIDE_ENABLED      0x00000001  
#define DMA_BW_CONTROLL         0x0000000F  
#define DMA_DA_HOLD_SIZE_1B     0x00000000  
#define DMA_DA_HOLD_SIZE_4B     0x00000200  
#define DMA_SA_HOLD_SIZE_1B     0x00000000  
#define DMA_SA_HOLD_SIZE_4B     0x00800000  
#define DMA_DA_HOLD_ENABLE      0x00200000  
#define DMA_SRW_ENABLE          0x00040000  
#define DMA_CDSM_ENABLE         0x10000000
#define DMA_SA_HOLD_ENABLE      0x00100000  
#define DMA_EXTENDSIONS_ENABLED 0x40000000  
#define DMA_CHANNEL_ABORT       0x08000000  
#define DMA_DIRECT_MODE         0x04000000  
#define DMA_START               0x01000000  
#define SYSCLK_66MHZ            66  

#define POWER_ON_RESET (0x000E0000) 
#define PMC0_LOWER     (0x000E1018) 
#define PMC0_UPPER     (0x000E101C) 


#define IRQ_MSIIR      (0x00041740) 

#define XDWORD unsigned int

/// This structure contains information for the DMA registers.
typedef volatile struct _MPC8536E_DMA_REGS
{
   XDWORD MR;         //mode register
   XDWORD SR;         //status register
   
   XDWORD ECLNDAR;    //current link descriptor extended address register
   XDWORD CLNDAR;     //current link descriptor address register
   
   XDWORD SATR;       //source attributes register
   XDWORD SAR;        //source address register
   
   XDWORD DATR;       //destination attributes register
   XDWORD DAR;        //destination address register
   
   XDWORD BCR;        //byte count register
   
   XDWORD ENLNDAR;    //next link descriptor extended address register
   XDWORD NLNDAR;     //next link descriptor address register

   XDWORD ECLSDAR;    //current list descriptor extended address register
   XDWORD CLSDAR;     //current list descriptor address register

   XDWORD ENLSDAR;    //next list descriptor extended address register
   XDWORD NLSDAR;     //next list descriptor address register

   XDWORD SSR;        //source stride register
   XDWORD DSR;        //destination stride register   
} MPC8536E_DMA_REGS, *PMPC8536E_DMA_REGS; 
#define MPC8536E_DMA_REGS_LEN sizeof(MPC8536E_DMA_REGS)

typedef struct _MPC8536E_DMA_LINK_DESCR {
    volatile INT32U SATR;       //source attributes register
    volatile INT32U SAR;        //source address register

    volatile INT32U DATR;       //destination attributes register
    volatile INT32U DAR;        //destination address register

    volatile INT32U ENLNDAR;  //next link descriptor extended address register
    volatile INT32U NLNDAR;     //next link descriptor address register

    volatile INT32U BCR;        //byte count register
    volatile INT32U Reserved;
} MPC8536E_DMA_LINK_DESCR;

#endif /* _PNIO_CTL_H_5D62B1E6_4C6B_4d6e_A67F_C3BB628D0BEB_ */
   
#pragma pack(pop)
