/*
+------------------------------------------------------------------------------+
| ******************************* PN_LibMain.h ******************************* |
+------------------------------------------------------------------------------+
|                  Copyright by PHOENIX CONTACT Software GmbH                  |
|                                                                              |
|                        PHOENIX CONTACT Software GmbH                         |
|                        Langenbruch 6                                         |
|                        D-32657 Lemgo                                         |
|                        Germany                                               |
|                                                                              |
+------------------------------------------------------------------------------+
| Author:      Andreas Toensfeuerborn                                          |
| Description: Public Header of PN Stack Library                               |
+------------------------------------------------------------------------------+
*/

#ifndef INC_PNLIBMAIN
#define INC_PNLIBMAIN

#ifdef __cplusplus
extern "C" {
#endif /* #ifdef __cplusplus */

/*****************************************************************************
****  TYPE DEFINITIONS                                                    ****
*****************************************************************************/

#define VOID        void
#define PVOID       void*
#define INT8        INT8S
#define INT16       INT16S
#define INT32       INT32S
#define USIGN8      INT8U
#define USIGN16     INT16U
#define USIGN32     INT32U
#define CHAR        char
#ifndef _MSC_VER
#define BOOL        unsigned char
#endif
#define PHO_BOOL    unsigned char

#define ERR_OK      0
#define ERR_ERROR   1
#define FWAPIRET_OK ERR_OK

#ifndef NULL
#define NULL        (PVOID)0L
#endif
#ifndef TRUE
#define TRUE        1
#endif
#ifndef FALSE
#define FALSE       0
#endif

#define PHO_TRUE    TRUE
#define PHO_FALSE   FALSE

typedef USIGN32 FWAPIRET;
typedef PVOID H_ES;

/*****************************************************************************
****  CONSTANT DEFINITIONS                                                ****
*****************************************************************************/

#define CONTROLLER_INSTANCE_ID              1
#define FIRST_DEVICE_INSTANCE_ID            2

#define MAX_SRT_PACKET_SIZE                 1500

#define MAX_STATION_NAME_SIZE               256
#define MAX_STATION_NAME_LENGTH             240
#define DEVICE_TYPE_LEN                     25
#define OUI_SIZE                            3

#define IOCAR_SINGLE                        0x0001
#define IOCAR_CIR                           0x0003
#define IOCAR_CONTROLLER_REDUNDANT          0x0004
#define IOCAR_DEVICE_REDUNDANT              0x0005
#define IOSAR                               0x0006
#define IOCAR_SR                            0x0020

#define BACKUP                              0x00
#define CMCTL_PRIMARY                       0x01
#define NOT_ALLOWED                         0x00
#define ALLOWED                             0x01
#define EXTERNAL_PRM_SERVER                 0x00
#define CM_INITIATOR                        0x01
#define EXT_PRM_SERVER                      0x00
#define DR_AT_LEAST_100_MBIT                0x00
#define DR_100_MBIT                         0x01
#define DR_1_GBIT                           0x02
#define DR_10_GBIT                          0x03
#define DA_CONTR_BY_EXP_SUBMOD_BLOCK        0x00
#define DA_CONTR_BY_IO_DEVICE_APP           0x01
#define CAR_SINGLE_OR_SECOND_AR             0x00
#define CAR_FIRST_AR_COMPANION_AR_FOLLOWS   0x01
#define CAR_COMPANION_AR                    0x02
#define ALLOW_ALARM_TYPE_PULL               0x00
#define ALLOW_ALARM_TYPE_PULL_MODULE        0x01
#define UDP_TRANSPORT                       0x00
#define SRT_TRANSPORT                       0x01
#define PNIO_STARTUP_MODE_LEGACY            0x00
#define PNIO_STARTUP_MODE_ADVANCED          0x01

#define NORMAL_STARTUP_MODE                 0x00000000
#define FAST_STARTUP_MODE_TIMEOUT_MIN       0x0000001E
#define FAST_STARTUP_MODE_TIMEOUT_MAX       0x00003E80

#define INPUT_CR                            1
#define OUTPUT_CR                           2
#define MC_PROV_CR                          3
#define MC_CONS_CR                          4

#define RT_CLASS1_RTC                       0x01
#define RT_CLASS2_RTC                       0x02
#define RT_CLASS3_RTC                       0x03
#define RT_CLASS_UDP                        0x04

#define BEST_EFFORT                         0xFFFFFFFFL

#define IOCR_PRIORITY                       6

#define NO_VLANID                           0x00
#define DEFAULT_VLANID                      0x01

#define CMCTL_NORMAL_ADDR                   0x00
#define CMCTL_TOPOLOGICAL_ADDR              0x01

#define CMCTL_NO_IO                         0x00
#define CMCTL_INPUT                         0x01
#define CMCTL_OUTPUT                        0x02
#define CMCTL_IO                            0x03
#define PNIO_CMDEV_NO_IO                    0x00
#define PNIO_CMDEV_INPUT                    0x01
#define PNIO_CMDEV_OUTPUT                   0x02
#define PNIO_CMDEV_IO                       0x03

#define CMCTL_IOC                           0x00
#define CMCTL_IOC_SHARED                    0x01
#define PNIO_CMDEV_IOC                      0x00
#define PNIO_CMDEV_IOC_SHARED               0x01
#define INVALID                             0x02

#define CMCTL_EXPECTED                      0x00
#define CMCTL_ZERO                          0x01
#define PNIO_CMDEV_EXPECTED                 0x00
#define PNIO_CMDEV_ZERO                     0x01

#define CMCTL_DATA_INPUT                    0x01
#define CMCTL_DATA_OUTPUT                   0x02
#define PNIO_CMDEV_DATA_INPUT               0x01
#define PNIO_CMDEV_DATA_OUTPUT              0x02

#define CMCTL_DATA                          0x01
#define CMCTL_IOPS                          0x02
#define CMCTL_IOCS                          0x03
#define PNIO_CMDEV_DATA                     0x01
#define PNIO_CMDEV_IOPS                     0x02
#define PNIO_CMDEV_IOCS                     0x03

#define DATA_RTA_PDU                        0x00
#define UDP_RTA_PDU                         0x01

#define NULL_VALUE                          0x00
#define LAST_VALUE                          0x01
#define SUBSTITUTION_VALUE                  0x02

#define PNIO_USER_PRIORITY                  0x0000
#define PNIO_LOW_PRIORITY                   0x0001

#define PNIO_CMCTL_SND_CLK_FACT_FOR_1ms     32
#define PNIO_CMCTL_SND_CLK_FACT_FOR_2ms     64
#define PNIO_CMCTL_SND_CLK_FACT_FOR_4ms     128
#define PNIO_CMDEV_SND_CLK_FACT_FOR_1ms     32
#define PNIO_CMDEV_SND_CLK_FACT_FOR_2ms     64
#define PNIO_CMDEV_SND_CLK_FACT_FOR_4ms     128

/* startup types for starting an AR */
#define CMCTL_AR_NORMAL                     0x00
#define CMCTL_AR_DISCOVER_DEVICE            0x01
#define CMCTL_AR_FINISH_STARTUP             0x02

/* AR ID of the first connection to the local PNIO device */
#define CMDEV_FIRST_ARID                    0xAB01

#define IND_ABORT                           1
#define IND_APP_READY                       2
#define IND_NBH_CHANGED                     4
#define IND_CMDEV_ABORT                     100
#define IND_CMDEV_AR_INFO                   101
#define IND_CMDEV_APP_READY                 102
#define IND_CMDEV_AR_CHECK                  103
#define IND_CMDEV_AR_INDATA                 104
#define IND_CMDEV_PRM_BEGIN                 105
#define IND_CMDEV_PRM_END                   106
#define IND_CMDEV_STATE                     107
#define IND_DIAG_STATE_CHANGED              200
#define IND_AR_DIAG_STATE_CHANGED           201

#define FIND_BY_STATION_NAME                0x01
#define FIND_BY_ARID                        0x02
#define FIND_BY_AR_USER_ID                  0x03
#define FIND_AR_LIST_WITH_DIAGNOSIS         0x04

#define AR_STATUS_ADDED                                  1
#define AR_STATUS_STARTED                                2
#define AR_STATUS_WAIT_FOR_DCP_HELLO                     3
#define AR_STATUS_DNS_STARTED                            4
#define AR_STATUS_DCP_IDENTIFY_STARTED                   5
#define AR_STATUS_WAIT_FOR_DNS_IND                       6
#define AR_STATUS_WAIT_FOR_DCP_IDENTIFY_IND              7
#define AR_STATUS_WAIT_FOR_DCP_IDENTIFY_ALIAS_IND        8
#define AR_STATUS_WAIT_FOR_DCP_SET_NAME_IND              9
#define AR_STATUS_WAIT_FOR_DCP_SET_IP_IND                10
#define AR_STATUS_WAIT_FOR_ARP_CNF                       11
#define AR_STATUS_DCP_SET_IP                             12
#define AR_STATUS_WAIT_FOR_DCP_SET_IP_CNF                13
#define AR_STATUS_RPC_CONNECT_REQ_WAIT                   14
#define AR_STATUS_RPC_CONNECT_REQ                        15
#define AR_STATUS_WAIT_FOR_RPC_CONNECT_CNF               16
#define AR_STATUS_AR_ACTIVATE                            17
#define AR_STATUS_WRITE_PRM_REQ                          19
#define AR_STATUS_WAIT_FOR_WRITE_PRM_CNF                 20
#define AR_STATUS_PRM_END_REQ                            21
#define AR_STATUS_WAIT_FOR_PRM_END_CNF                   22
#define AR_STATUS_WAIT_FOR_APP_READY_IND                 23
#define AR_STATUS_APP_READY_RES                          24
#define AR_STATUS_APP_READY                              25
#define AR_STATUS_WAIT_FOR_READ_CNF                      26
#define AR_STATUS_WAIT_FOR_WRITE_CNF                     27
#define AR_STATUS_RPC_CANCEL                             28
#define AR_STATUS_WAIT_FOR_RPC_CANCEL_CNF                29
#define AR_STATUS_RELEASE                                30
#define AR_STATUS_WAIT_FOR_RELEASE_CNF                   31
#define AR_STATUS_AR_DEACTIVATE                          32
#define AR_STATUS_STOP                                   33
#define AR_STATUS_PROV_ERROR_IND                         34
#define AR_STATUS_CONS_ERROR_IND                         35
#define AR_STATUS_ALARM_ERROR_IND                        36
#define AR_STATUS_PRM_BEGIN_REQ                          37
#define AR_STATUS_WAIT_FOR_PRM_BEGIN_CNF                 38

#define AR_INFO_NONE                                      0
#define AR_INFO_CON_ERR_TRANS                             1
#define AR_INFO_CON_ERR_ADR_DNS                           2
#define AR_INFO_CON_ERR_ADR_DCP_NAME                      3
#define AR_INFO_CON_ERR_ADR_DCP_ALIAS                     4
#define AR_INFO_CON_ERR_MTP_NAME                          5
#define AR_INFO_CON_ERR_MTP_ALIAS                         6
#define AR_INFO_CON_ERR_MTP_IP_ADDRESS                    7
#define AR_INFO_CON_ERR_REFUSED_TO                        8
#define AR_INFO_CON_ERR_REFUSED_NEGRES                    9
#define AR_INFO_CON_ERR_DUPL_NBH_INFO                    10
#define AR_INFO_CON_ERR_NAME_ALREADY_USED                11
#define AR_INFO_CON_ERR_NBH_NOT_UNIQUE                   12
#define AR_INFO_UNINITIALIZED                            0xffff

#define DEV_AR_STATUS_OFFLINE                             0
#define DEV_AR_STATUS_CONNECT_IND                         1
#define DEV_AR_STATUS_WAIT_FOR_CONNECT_RSP_FROM_APP       2
#define DEV_AR_STATUS_PPM_ACTIVATE                        3
#define DEV_AR_STATUS_ALPM_ACTIVATE                       4
#define DEV_AR_STATUS_WAIT_FOR_PRM                        5
#define DEV_AR_STATUS_WAIT_FOR_PRM_WRITE_CNF_FROM_APP     6
#define DEV_AR_STATUS_WAIT_FOR_APPL_READY_FROM_APP        7
#define DEV_AR_STATUS_WAIT_FOR_APPL_READY_CNF             8
#define DEV_AR_STATUS_APPL_READY                          9
#define DEV_AR_STATUS_REMOVE                             10

#define PNIO_STATUS_NO_ERROR                0x00
#define PNIO_IOAR_NO_NEW_DATA               0x00000001L

#define RTA_ERROR                           0xCF
#define ALARM_ACK                           0xDA
#define IOD_CONNECT_RES                     0xDB
#define IOD_RELEASE_RES                     0xDC
#define IOX_CONTROL_RES                     0xDD
#define IOD_READ_RES                        0xDE
#define IOD_WRITE_RES                       0xDF

#define PNIORW                              0x80
#define PNIO                                0x81

#define PNIORW_READ_ERROR                           0xA0
#define PNIORW_WRITE_ERROR                          0xA1
#define PNIORW_MODULE_FAILURE                       0xA2
#define PNIORW_BUSY                                 0xA7
#define PNIORW_VERSION_CONFLICT                     0xA8
#define PNIORW_FEATURE_NOT_SUPPORTED                0xA9
#define PNIORW_INVALID_INDEX                        0xB0
#define PNIORW_WRITE_LENGTH_ERROR                   0xB1
#define PNIORW_INVALID_SLOT_SUBSLOT                 0xB2
#define PNIORW_TYPE_CONFLICT                        0xB3
#define PNIORW_INVALID_AREA                         0xB4
#define PNIORW_STATE_CONFLICT                       0xB5
#define PNIORW_ACCESS_DENIED                        0xB6
#define PNIORW_INVALID_RANGE                        0xB7
#define PNIORW_INVALID_PARAMETER                    0xB8
#define PNIORW_INVALID_TYPE                         0xB9
#define PNIORW_BACKUP                               0xBA
#define PNIORW_READ_CONSTRAIN_CONFLICT              0xC0
#define PNIORW_WRITE_CONSTRAIN_CONFLICT             0xC1
#define PNIORW_RESOURCE_BUSY                        0xC2
#define PNIORW_RESOURCE_UNAVAILABLE                 0xC3
#define PNIORW_READ_TIMEOUT                         0xD0
#define PNIORW_WRITE_TIMEOUT                        0xE0

#define PNIO_FAULTY_AR_BLOCK_REQ                    1
#define PNIO_FAULTY_IOCR_BLOCK_REQ                  2
#define PNIO_FAULTY_EXP_SUBMODULE_BLOCK_REQ         3
#define PNIO_FAULTY_ALARM_CR_BLOCK_REQ              4
#define PNIO_FAULTY_PRM_SERVER_LOCK_REQ             5
#define PNIO_FAULTY_MCR_BLOCK_REQ                   6
#define PNIO_FAULTY_CONTROL_BLOCK_CONNECT           20
#define PNIO_FAULTY_CONTROL_BLOCK_PLUG              21
#define PNIO_FAULTY_CONTROL_BLOCK_AFTER_CON_EST     22
#define PNIO_FAULTY_CONTROL_BLOCK_PLUG_ALARM        23
#define PNIO_FAULTY_RELEASE_BLOCK                   40
#define PNIO_ALARM_ACK_ERROR_CODES                  60
#define PNIO_RTA_ERR_CLS_PROTOCOL                   253

#define BLOCK_TYPE_IM0                                               0x0020
#define BLOCK_TYPE_IM1                                               0x0021
#define BLOCK_TYPE_IM2                                               0x0022
#define BLOCK_TYPE_IM3                                               0x0023
#define BLOCK_TYPE_IM4                                               0x0024
#define BLOCK_TYPE_IM5                                               0x0025
#define BLOCK_TYPE_IM6                                               0x0026
#define BLOCK_TYPE_IM7                                               0x0027
#define BLOCK_TYPE_IM8                                               0x0028
#define BLOCK_TYPE_IM9                                               0x0029
#define BLOCK_TYPE_IM10                                              0x002A
#define BLOCK_TYPE_IM11                                              0x002B
#define BLOCK_TYPE_IM12                                              0x002C
#define BLOCK_TYPE_IM13                                              0x002D
#define BLOCK_TYPE_IM14                                              0x002E
#define BLOCK_TYPE_IM15                                              0x002F

/* Coding of the field ModuleState */
#define MODULE_STATE_NO_MODULE                                       0x0000
#define MODULE_STATE_WRONG_MODULE                                    0x0001
#define MODULE_STATE_PROPER_MODULE                                   0x0002
#define MODULE_STATE_SUBSTITUTE                                      0x0003

/* Coding of the field SubmoduleState */
/* Bit 15: SubmoduleState.FormatIndicator */
#define SUBMODULE_STATE_FORMAT_INDICATOR                             0x8000

/* Coding of the field SubmoduleState with SubmoduleState.FormatIndicator == 1 */
/* Bit 0 - 2: SubmoduleState.AddInfo */
#define SUBMODULE_STATE_ADD_INFO_NONE                                0x0000
#define SUBMODULE_STATE_ADD_INFO_TAKEOVER_NOT_ALLOWED                0x0001
/* Bit 3: SubmoduleState.Reserved */
#define SUBMODULE_STATE_RESERVED                                     0x0008 /* SUBMODULE_STATE_QUALIFIED_INFO_AVAILABLE */
/* Bit 4: SubmoduleState.MaintenanceRequired */
#define SUBMODULE_STATE_MAINTENANCE_REQUIRED_AVAILABLE               0x0010
/* Bit 5: SubmoduleState.MaintenanceDemanded */
#define SUBMODULE_STATE_MAINTENANCE_DEMANDED_AVAILABLE               0x0020
/* Bit 6: SubmoduleState.Fault */
#define SUBMODULE_STATE_FAULT                                        0x0040 /* SUBMODULE_STATE_DIAGNOSIS_DATA_AVAILABLE */
/* Bit 7 - 10: SubmoduleState.ARInfo */
#define SUBMODULE_STATE_AR_INFO_OWN                                  0x0000
#define SUBMODULE_STATE_AR_INFO_APPLICATION_READY_PENDING            0x0080
#define SUBMODULE_STATE_AR_INFO_SUPERORDINATION_LOCKED               0x0100
#define SUBMODULE_STATE_AR_INFO_LOCKED_BY_IO_CONTROLLER              0x0180
#define SUBMODULE_STATE_AR_INFO_LOCKED_BY_IO_SUPERVISOR              0x0200
/* Bit 11 - 14: SubmoduleState.IdentInfo */
#define SUBMODULE_STATE_IDENT_INFO_OK                                0x0000
#define SUBMODULE_STATE_IDENT_INFO_SUBSTITUTE                        0x0800
#define SUBMODULE_STATE_IDENT_INFO_WRONG                             0x1000
#define SUBMODULE_STATE_IDENT_INFO_NO_SUBMODULE                      0x1800

/* Coding of the field SubmoduleState with SubmoduleState.FormatIndicator == 0 */
/* Bit 0 - 14: SubmoduleState.Detail */
#define SUBMODULE_STATE_DETAIL_NO_SUBMODULE                          0x0000
#define SUBMODULE_STATE_DETAIL_WRONG_SUBMODULE                       0x0001
#define SUBMODULE_STATE_DETAIL_LOCKED_BY_IO_CONTROLLER               0x0002
#define SUBMODULE_STATE_DETAIL_APPLICATION_READY_PENDING             0x0004
#define SUBMODULE_STATE_DETAIL_SUBSTITUTE                            0x0007

/* Mask defintions for field SubmoduleState */
#define SUBMODULE_STATE_ADD_INFO_MASK                                0x0007
#define SUBMODULE_STATE_AR_INFO_MASK                                 0x0780
#define SUBMODULE_STATE_IDENT_INFO_MASK                              0x7800
#define SUBMODULE_STATE_DETAIL_MASK                                  0x7FFF

#define PNIO_IOAR_BLOCKTYPE_ALARM_NOTIF_HIGH                         0x0001
#define PNIO_IOAR_BLOCKTYPE_ALARM_NOTIF_LOW                          0x0002
#define BLOCK_TYPE_NESTED_DIAGNOSIS_INFO                             0x0A02

#define PNIO_IOAR_BLOCK_VERSION_HIGH                                 0x01
#define PNIO_IOAR_BLOCK_VERSION_LOW                                  0x00

/* Alarm Types */
#define ALARM_TYPE_DIAG_APPEARS                                      0x0001
#define ALARM_TYPE_PROCESS                                           0x0002
#define ALARM_TYPE_PULL                                              0x0003
#define ALARM_TYPE_PLUG                                              0x0004
#define ALARM_TYPE_STATUS                                            0x0005
#define ALARM_TYPE_UPDATE                                            0x0006
#define ALARM_TYPE_REDUNDANCY                                        0x0007
#define ALARM_TYPE_CONTR_BY_SUPERVISOR                               0x0008
#define ALARM_TYPE_RELEASED                                          0x0009
#define ALARM_TYPE_PLUG_WRONG_SUBMODULE                              0x000A
#define ALARM_TYPE_RETURN_OF_SUBMODULE                               0x000B
#define ALARM_TYPE_DIAG_DISAPPEARS                                   0x000C
#define ALARM_TYPE_MC_COMM_MISMATCH                                  0x000D
#define ALARM_TYPE_PORT_DATA_CHANGED_NOTIF                           0x000E
#define ALARM_TYPE_SYNC_DATA_CHANGED_NOTIF                           0x000F
#define ALARM_TYPE_ISO_MODE_PROBLEM_NOTIF                            0x0010

/* Alarm Specifier */
#define ALARM_SEQUENCE_NUMBER_MASK                      0x07FF
#define ALARM_CHANNEL_DIAGNOSIS                         0x0800
#define ALARM_MANUFACTURER_SPECIFIC_DIAGNOSIS           0x1000
#define ALARM_SUBMODULE_DIAGNOSIS_STATE                 0x2000
#define ALARM_SPECIFIER_RESERVED                        0x4000
#define ALARM_AR_DIAGNOSIS_STATE                        0x8000

/* Alarm User Structure Identifiers */
/* 0x0000 to 0x7FFF ManufacturerSpecific */
#define ALARM_USI_CHANNEL_DIAGNOSIS                     0x8000
#define ALARM_USI_MULTIPLE                              0x8001
#define ALARM_USI_EXT_CHANNEL_DIAGNOSIS                 0x8002
#define ALARM_USI_QUALIFIED_CHANNEL_DIAGNOSIS           0x8003
/* 0x8004 to 0x80FF Reserved */
#define ALARM_USI_MAINTENANCE                           0x8100
/* 0x8101 to 0x81FF Reserved */
#define ALARM_USI_UPLOAD_AND_RETRIEVAL                  0x8200
#define ALARM_USI_IPARAMETER                            0x8201
/* 0x8202 to 0x8FFF Reserved */
/* 0x9000 to 0x9FFF Reserved for profiles */
/* 0xA000 to 0xFFFF Reserved */

/* Alarm Channel Numbers */
/* 0x0000 to 0x7FFF ManufacturerSpecific */
#define ALARM_CN_SUBMODULE                              0x8000
/* 0x8001 to 0x8FFF Reserved */

/* Alarm Channel Properties */
/* Alarm Channel Properties Type */
#define ALARM_CP_TYPE_MASK                              0x00FF
#define ALARM_CP_TYPE_NONE                              0x0000
#define ALARM_CP_TYPE_1_BIT                             0x0001
#define ALARM_CP_TYPE_2_BIT                             0x0002
#define ALARM_CP_TYPE_4_BIT                             0x0003
#define ALARM_CP_TYPE_8_BIT                             0x0004
#define ALARM_CP_TYPE_16_BIT                            0x0005
#define ALARM_CP_TYPE_32_BIT                            0x0006
#define ALARM_CP_TYPE_64_BIT                            0x0007
/* 0x0008 to 0x00FF Reserved */

/* Alarm Channel Properties Accumulative */
#define ALARM_CP_ACCUMULATIVE_MASK                      0x0100
#define ALARM_CP_ACCUMULATIVE_NONE                      0x0000  /* 0x00, no accumulative diagnosis */
#define ALARM_CP_ACCUMULATIVE_PRESENT                   0x0100  /* 0x01, accumulative diagnosis from more then one channel */

/* Alarm Channel Properties Maintenance */
#define ALARM_CP_MAINTENANCE_MASK                       0x0600
#define ALARM_CP_MAINTENANCE_NONE                       0x0000  /* 0x00, no maintance required or demanded */
#define ALARM_CP_MAINTENANCE_REQUIRED                   0x0200  /* 0x01, maintance required */
#define ALARM_CP_MAINTENANCE_DEMANDED                   0x0400  /* 0x02, maintance demanded */
#define ALARM_CP_MAINTENANCE_REQUIRED_DEMANDED          0x0600  /* 0x03, maintance required and demanded */

/* Alarm Channel Properties Specifier */
#define ALARM_CP_SPECIFIER_MASK                         0x1800
#define ALARM_CP_SPECIFIER_ALL_DISAPPEARS               0x0000  /* 0x00, all subsequent disappears */
#define ALARM_CP_SPECIFIER_APPEARS                      0x0800  /* 0x01, an event appears and/or exists further */
#define ALARM_CP_SPECIFIER_DISAPPEARS                   0x1000  /* 0x02, an event disappears and/or exists no longer */
#define ALARM_CP_SPECIFIER_DISAPPEARS_REMAIN            0x1800  /* 0x03, an event disappears but there are other */

/* Alarm Channel Properties Direction */
#define ALARM_CP_DIRECTION_MASK                         0xE000
#define ALARM_CP_DIRECTION_MANUFACTURER_SPECIFIC        0x0000  /* 0x00, manufacturer specific direction */
#define ALARM_CP_DIRECTION_INPUT                        0x2000  /* 0x01, input direction */
#define ALARM_CP_DIRECTION_OUTPUT                       0x4000  /* 0x02, output direction */
#define ALARM_CP_DIRECTION_INPUT_OUTPUT                 0x6000  /* 0x03, input and output direction */
/* 0x04 to 0x07 Reserved */

/* Alarm Channel Error Types */
#define ALARM_CET_RESERVED                              0x0000  /* Unknown error */
#define ALARM_CET_SHORT_CIRCUIT                         0x0001  /* Short circuit */
#define ALARM_CET_UNDERVOLTAGE                          0x0002  /* Undervoltage */
#define ALARM_CET_OVERVOLTAGE                           0x0003  /* Overvoltage */
#define ALARM_CET_OVERLOAD                              0x0004  /* Overload */
#define ALARM_CET_OVERTEMPERATURE                       0x0005  /* Overtemperature */
#define ALARM_CET_LINE_BREAK                            0x0006  /* Line break */
#define ALARM_CET_UPPER_LIMIT_VALUE_EXCEEDED            0x0007  /* Upper limit value exceeded */
#define ALARM_CET_LOWER_LIMIT_VALUE_EXCEEDED            0x0008  /* Lower limit value exceeded */
#define ALARM_CET_ERROR                                 0x0009  /* Error */
#define ALARM_CET_SIMULATION ACTIVE                     0x000A  /* Simulation active */
#define ALARM_CET_RESERVED_1                            0x000B  /* Unknown error */
#define ALARM_CET_RESERVED_2                            0x000C  /* Unknown error */
#define ALARM_CET_RESERVED_3                            0x000D  /* Unknown error */
#define ALARM_CET_RESERVED_4                            0x000E  /* Unknown error */
#define ALARM_CET_PARAMETER_MISSING                     0x000F  /* Manufacturer specific, recommended for "parameter missing" */
                                                                /* The channel needs (additional) parameters. No or to less parameters are written */
#define ALARM_CET_PARAMETRIZATION_FAULT                 0x0010  /* Manufacturer specific, recommended for "parametrization fault" */
                                                                /* Parametrization fault. Wrong or to many parameters are written */
#define ALARM_CET_POWER_SUPPLY_FAULT                    0x0011  /* Manufacturer specific, recommended for "power supply fault" */
                                                                /* Power supply fault */
#define ALARM_CET_FUSE_BLOWN_OR_OPEN                    0x0012  /* Manufacturer specific, recommended for "fuse blown / open" */
                                                                /* Fuse blown / open */
#define ALARM_CET_COMMUNICATION_FAULT                   0x0013  /* Manufacturer specific, recommended for "communication fault" */
                                                                /* Communication fault. Sequence number wrong / sequence wrong */
#define ALARM_CET_GROUND_FAULT                          0x0014  /* Manufacturer specific, recommended for "ground fault" */
                                                                /* Ground fault */
#define ALARM_CET_REFERENCE_POINT_LOST                  0x0015  /* Manufacturer specific, recommended for "reference point lost" */
                                                                /* Reference point lost */
#define ALARM_CET_PROCESS_EVENT_LOST                    0x0016  /* Manufacturer specific, recommended for "process event lost / sampling error" */
                                                                /* Process event lost / sampling error */
#define ALARM_CET_THRESHOLD_WARNING                     0x0017  /* Manufacturer specific, recommended for "threshold warning" */
                                                                /* Threshold warning */
#define ALARM_CET_OUTPUT_DISABLED                       0x0018  /* Manufacturer specific, recommended for "output disabled" */
                                                                /* Output disabled */
#define ALARM_CET_SAFETY_EVENT                          0x0019  /* Manufacturer specific, recommended for "safety event" */
                                                                /* Safety event */
#define ALARM_CET_EXTERNAL_FAULT                        0x001A  /* Manufacturer specific, recommended for "external fault" */
                                                                /* External fault */
#define ALARM_CET_MANUFACTURER_SPECIFIC_1               0x001B  /* Manufacturer specific */
#define ALARM_CET_MANUFACTURER_SPECIFIC_2               0x001C  /* Manufacturer specific */
#define ALARM_CET_MANUFACTURER_SPECIFIC_3               0x001D  /* Manufacturer specific */
#define ALARM_CET_MANUFACTURER_SPECIFIC_4               0x001E  /* Manufacturer specific */
#define ALARM_CET_TEMPORARY_FAULT                       0x001F  /* Manufacturer specific, recommended for "temporary fault" */
                                                                /* Temporary fault */
/* 0x0020 to 0x00FF Reserved for common profiles */             /* Central administrative number to unambiguous distinguish between common profiles */
/* 0x0100 to 0x7FFF Manufacturer specific */                    /* Manufacturer specific */
#define ALARM_CET_DATA_TRANSMISSION_IMPOSSIBLE          0x8000  /* Data Transmission Impossible */
#define ALARM_CET_REMOTE_MISMATCH                       0x8001  /* Remote Mismatch */
#define ALARM_CET_MEDIA_REDUNDANCY_MISMATCh             0x8002  /* Media Redundancy Mismatch */
#define ALARM_CET_SYNC_MISMATCH                         0x8003  /* Sync Mismatch */
#define ALARM_CET_ISOCHRONOUSMODE_MISMATCH              0x8004  /* IsochronousMode Mismatch */
#define ALARM_CET_MULTICAST_CR_MISMATCH                 0x8005  /* Multicast CR Mismatch */
#define ALARM_CET_RESERVED_5                            0x8006  /* Reserved */
#define ALARM_CET_FIBER_OPTIC_MISMATCH                  0x8007  /* Information for fiber optic link */
#define ALARM_CET_NETWORK_COMPONENT_FUNCTION_MISMATCH   0x8008  /* Network functionality problems occur */
#define ALARM_CET_TIME_MISMATCH                         0x8009  /* Time master not existent or precision problems */
#define ALARM_CET_DFP_FUNC_MISMATCH                     0x800A  /* Dynamic frame packing function mismatch */
#define ALARM_CET_MRPD_MISMATCH                         0x800B  /* Media redundancy with planned duplication mismatch */
#define ALARM_CET_SYSTEM_REDUNDANCY_MISMATCH            0x800C  /* System redundancy mismatch */
#define ALARM_CET_MULTIPLE_INTERFACE_MISMATCH           0x800D  /* Information about multiple interface problems */
#define ALARM_CET_NESTED_DIAGNOSIS_INDICATION           0x800E  /* Used for propagation of diagnosis in hierachical structures */
/* 0x800F to 0x8FFF Reserved */                                 /* Unknown error */
/* 0x9000 to 0x9FFF Reserved for profiles */                    /* Profile specific */
/* 0xA000 to 0xFFFF Reserved */                                 /* Unknown error */


#define ALARM_DATA_LENGTH_MIN                   200
#define ALARM_DATA_LENGTH_MAX                   1432
#define ALARM_NOTIFICATION_RTA_SDU_BLOCK_LENGTH 22
#define ALARM_SEQUENCE_NUMBER_MAX               0x07FF

#define ALARM_NO_RTA_TRIGGER                    0x00            /* No alarm is triggered, only ASE is updated */
#define ALARM_LOW_RTA_TRIGGER                   0x01            /* An alarm handling with low priority is triggered */
#define ALARM_HIGH_RTA_TRIGGER                  0x02            /* An alarm handling with high priority is triggered */

#define PNDIAG_STATE_GOOD                       1               /* No diagnosis available */
#define PNDIAG_STATE_MR                         2               /* Maintenance required   */
#define PNDIAG_STATE_MD                         3               /* Maintenance demanded   */
#define PNDIAG_STATE_DIAGNOSIS                  4               /* Diagnosis available    */

#define AR_DIAG_STATE_MULTIPLE_STATION_NAMES    1               /* More than one device with the same station name was detected */
#define AR_DIAG_STATE_MULTIPLE_IP_ADDRESSES     2               /* More than one device with the same IP address was detected */
#define AR_DIAG_STATE_AR_DISABLED               4               /* AR was closed and controller is ready for another connect */
#define AR_DIAG_STATE_NEIGHBOR_NOT_AVAILABLE    8               /* No neighbor/peer information was found */
#define AR_DIAG_STATE_NEIGHBOR_NOT_UNIQUE       16              /* There is more than one neighbor/peer on the same port */
#define AR_DIAG_STATE_CONNECT_ERROR             32              /* The connect request was respond with an error */

#define IP_ADDR_SIZE                        4
#define MAC_ADDR_SIZE                       6

#define FW_USR_MSG_LEN_BYTE                 1540
#define FW_USR_MSG_LEN_WORD                 (FW_USR_MSG_LEN_BYTE / 2)

#define EA_STR_LEN                          300

#define EA_TYPE_INFORMATION                 0x0001
#define EA_TYPE_WARNING                     0x0002
#define EA_TYPE_ERROR                       0x0004

#define EA_ZONE_NONE                        0x0000
#define EA_ZONE_RETAIL                      0x8000
#define EA_ZONE_DEBUG                       0xC000
#define EA_ZONE_TEST                        0xE000

#define EA_FLAG_NONE                        0

/* device roles */
#define DCP_ROLE_IO_DEVICE                  0x01
#define DCP_ROLE_IO_CONTROLLER              0x02
#define DCP_ROLE_IO_MULTIDEVICE             0x04
#define DCP_ROLE_IO_SUPERVISOR              0x08

/* DCP Parameters Bits */
#define DCP_PARAM_ENA_IDENTIFY              0x00000001
#define DCP_PARAM_ENA_SET_NAME              0x00000002
#define DCP_PARAM_ENA_SET_IP                0x00000004
#define DCP_PARAM_ENA_FLASH                 0x00000008
#define DCP_PARAM_ENA_FACTORY_RESET         0x00000010
#define DCP_PARAM_ENA_START_TRANS           0x00000020
#define DCP_PARAM_ENA_STOP_TRANS            0x00000040

/* DCP Service ID definitions */
#define DCP_GET                             3
#define DCP_SET                             4
#define DCP_IDENTIFY                        5

/* DCP Option definitions */
#define DCP_TCP_IP_OPTIONS                  0x01
#define DCP_DEVICE_PROP_OPTIONS             0x02
#define DCP_DHCP_OPTIONS                    0x03
#define DCP_LLDP_OPTIONS                    0x04
#define DCP_CONTROL                         0x05
#define DCP_MFR_SPECIFIC_OPTIONS_START      0x80
#define DCP_MFR_SPECIFIC_OPTIONS_END        0xFE
#define DCP_DEV_IDENT_ALL_SELECTOR          0xff

/* DCP Suboption definitions for option DCP_DEVICE_PROP_OPTIONS */
#define DCP_DEV_PROP_DEVICE_TYPE            0x01
#define DCP_DEV_PROP_NAME_OF_STATION        0x02
#define DCP_DEV_PROP_DEVICE_ID              0x03
#define DCP_DEV_PROP_DEVICE_ROLE            0x04
#define DCP_DEV_PROP_DEVICE_OPTIONS         0x05
#define DCP_DEV_PROP_ALIAS_NAME_OF_STATION  0x06
#define DCP_DEV_IDENT_ALL_SELECTOR          0xff

/* DCP Suboption definitions for option DCP_TCP_IP_OPTIONS */
#define DCP_TCP_IP_MAC_ADDRESS              0x01
#define DCP_TCP_IP_PARAMETER                0x02

/* DCP Suboption definitions for option  DCP_CONTROL */
#define DCP_CONTROL_START_TRANS             0x01
#define DCP_CONTROL_STOP_TRANS              0x02
#define DCP_CONTROL_SIGNAL                  0x03
#define DCP_CONTROL_RESPONSE                0x04
#define DCP_CONTROL_FACTORY_RESET           0x05
#define DCP_CONTROL_RESET_TO_FACTORY        0x06

/* DCP Reset To Factory Block Qualifier options */
#define DCP_BLOCK_QUALIFIER_RESET_APP_DATA  0x0002
#define DCP_BLOCK_QUALIFIER_RESET_COM_PARAM 0x0004
#define DCP_BLOCK_QUALIFIER_RESET_ENG_DATA  0x0006
#define DCP_BLOCK_QUALIFIER_RESET_ALL_DATA  0x0008

/* DCP Suboption definitions for option DCP_MFR_SPECIFIC_OPTIONS_X */
#define DCP_MFR_SPECIFIC_SUBOPTION_START    0x00
#define DCP_MFR_SPECIFIC_SUBOPTION_STOP     0xFF

/* DCP Status definitions */
#define DCP_STORE_VOLATILE                  0x0000 /* Store DCP values temporarily */
#define DCP_STORE_NON_VOLATILE              0x0001 /* Store DCP values permanently */

/* Definitions for Provider set status */
#define SET_BY_APPLICATION                  0x00
#define SET_BY_PROVIDER                     0x01

/* APDU DataStatus */
#define DS_STATE_MASK                       0x01
#define DS_STATE_BACKUP                     0x00
#define DS_STATE_PRIMARY                    0x01

#define DS_REDUNDANCY_MASK                  0x02
#define DS_PRIMARY_AR_PRESENT               0x00
#define DS_PRIMARY_AR_NOT_PRESENT           0x02

#define DS_DATAVALID_MASK                   0x04
#define DS_DATA_INVALID                     0x00
#define DS_DATA_VALID                       0x04

#define DS_RESERVED_1                       0x08

#define DS_PROVIDERSTATE_MASK               0x10
#define DS_PROV_STATE_STOP                  0x00
#define DS_PROV_STATE_RUN                   0x10

#define DS_STATIONPROBLEMINDICATOR_MASK     0x20
#define DS_STATION_PROBLEM                  0x00
#define DS_STATION_NO_PROBLEM               0x20

#define DS_RESERVED_2                       0x40

#define DS_IGNORE_MASK                      0x80
#define DS_EVALUATE_DATASTATUS              0x00
#define DS_IGNORE_DATASTATUS                0x80

/* IOxS */
#define BAD_BY_SUBSLOT                      0x00
#define BAD_BY_SLOT                         0x20
#define BAD_BY_DEVICE                       0x40
#define BAD_BY_CONTROLLER                   0x60
#define GOOD                                0x80

/* PROFINET Record Indices */
/* subslot specific */
#define INDEX_EXPECTED_IDENTIFICATION_DATA_FOR_ONE_SUBSLOT          0x8000
#define INDEX_REAL_IDENTIFICATION_DATA_FOR_ONE_SUBSLOT              0x8001
#define INDEX_DIAG_IN_CHANNEL_CODING_FOR_ONE_SUBSLOT                0x800A
#define INDEX_DIAG_IN_ALL_CODINGS_FOR_ONE_SUBSLOT                   0x800B
#define INDEX_DIAG_MQS_FOR_ONE_SUBSLOT                              0x800C
#define INDEX_MAINT_REQ_IN_CHANNEL_CODING_FOR_ONE_SUBSLOT           0x8010
#define INDEX_MAINT_DEM_IN_CHANNEL_CODING_FOR_ONE_SUBSLOT           0x8011
#define INDEX_MAINT_REQ_IN_ALL_CODINGS_FOR_ONE_SUBSLOT              0x8012
#define INDEX_MAINT_DEM_IN_ALL_CODINGS_FOR_ONE_SUBSLOT              0x8013
#define INDEX_INPUT_DATA_OBJECT_ELEMENT_FOR_ONE_SUBSLOT             0x8028
#define INDEX_OUTPUT_DATA_OBJECT_ELEMENT_FOR_ONE_SUBSLOT            0x8029
#define INDEX_PD_PORT_DATA_REAL_FOR_ONE_SUBSLOT                     0x802A
#define INDEX_PD_PORT_DATA_CHECK_FOR_ONE_SUBSLOT                    0x802B
#define INDEX_PD_PORTDATA_ADJUST_FOR_ONE_SUBSLOT                    0x802F
#define INDEX_PD_PORTDATA_CHECK_FOR_ONE_SUBSLOT                     0x802B
#define INDEX_PD_INTERFACE_DATA_REAL_FOR_ONE_SUBSLOT                0x8080
#define INDEX_EXPECTED_PD_INTERFACE_FSU_DATA_ADJUST_FOR_ONE_SUBSLOT 0x8090
#define INDEX_PROFILES_COVERING_ENERGY_SAVING_RECORD_0              0x80A0
#define INDEX_IM0_DATA                                              0xAFF0
#define INDEX_IM1_DATA                                              0xAFF1
#define INDEX_IM2_DATA                                              0xAFF2
#define INDEX_IM3_DATA                                              0xAFF3
#define INDEX_IM4_DATA                                              0xAFF4
#define INDEX_IM5_DATA                                              0xAFF5
#define INDEX_IM6_DATA                                              0xAFF6
#define INDEX_IM7_DATA                                              0xAFF7
#define INDEX_IM8_DATA                                              0xAFF8
#define INDEX_IM9_DATA                                              0xAFF9
#define INDEX_IM10_DATA                                             0xAFFA
#define INDEX_IM11_DATA                                             0xAFFB
#define INDEX_IM12_DATA                                             0xAFFC
#define INDEX_IM13_DATA                                             0xAFFD
#define INDEX_IM14_DATA                                             0xAFFE
#define INDEX_IM15_DATA                                             0xAFFF

/* slot specific */
#define INDEX_EXPECTED_IDENTIFICATION_DATA_FOR_ONE_SLOT             0xC000
#define INDEX_REAL_IDENTIFICATION_DATA_FOR_ONE_SLOT                 0xC001
#define INDEX_DIAG_IN_CHANNEL_CODING_FOR_ONE_SLOT                   0xC00A
#define INDEX_DIAG_IN_ALL_CODINGS_FOR_ONE_SLOT                      0xC00B
#define INDEX_DIAG_MQS_FOR_ONE_SLOT                                 0xC00C
#define INDEX_MAINT_REQ_IN_CHANNEL_CODING_FOR_ONE_SLOT              0xC010
#define INDEX_MAINT_DEM_IN_CHANNEL_CODING_FOR_ONE_SLOT              0xC011
#define INDEX_MAINT_REQ_IN_ALL_CODINGS_FOR_ONE_SLOT                 0xC012
#define INDEX_MAINT_DEM_IN_ALL_CODINGS_FOR_ONE_SLOT                 0xC013

/* AR specific */
#define INDEX_EXPECTED_IDENTIFICATION_DATA_FOR_ONE_AR               0xE000
#define INDEX_REAL_IDENTIFICATION_DATA_FOR_ONE_AR                   0xE001
#define INDEX_MODULEDIFFBLOCK_FOR_ONE_AR                            0xE002
#define INDEX_DIAG_IN_CHANNEL_CODING_FOR_ONE_AR                     0xE00A
#define INDEX_DIAG_IN_ALL_CODINGS_FOR_ONE_AR                        0xE00B
#define INDEX_DIAG_MQS_FOR_ONE_AR                                   0xE00C
#define INDEX_MAINT_REQ_IN_CHANNEL_CODING_FOR_ONE_AR                0xE010
#define INDEX_MAINT_DEM_IN_CHANNEL_CODING_FOR_ONE_AR                0xE011
#define INDEX_MAINT_REQ_IN_ALL_CODINGS_FOR_ONE_AR                   0xE012
#define INDEX_MAINT_DEM_IN_ALL_CODINGS_FOR_ONE_AR                   0xE013
#define INDEX_MULTIPLE_WRITE_FOR_ONE_AR                             0xE040
#define INDEX_AR_FSU_DATA_ADJUST_FOR_ONE_AR                         0xE050

/* API specific */
#define INDEX_REAL_IDENTIFICATION_DATA_FOR_ONE_API                  0xF000
#define INDEX_DIAG_IN_CHANNEL_CODING_FOR_ONE_API                    0xF00A
#define INDEX_DIAG_IN_ALL_CODINGS_FOR_ONE_API                       0xF00B
#define INDEX_DIAG_MQS_FOR_ONE_API                                  0xF00C
#define INDEX_MAINT_REQ_IN_CHANNEL_CODING_FOR_ONE_API               0xF010
#define INDEX_MAINT_DEM_IN_CHANNEL_CODING_FOR_ONE_API               0xF011
#define INDEX_MAINT_REQ_IN_ALL_CODINGS_FOR_ONE_API                  0xF012
#define INDEX_MAINT_DEM_IN_ALL_CODINGS_FOR_ONE_API                  0xF013
#define INDEX_AR_DATA_FOR_ONE_API                                   0xF020

/* device specific */
#define INDEX_DIAG_MQS_FOR_ONE_DEVICE                               0xF80C
#define INDEX_AR_DATA_FOR_ONE_DEVICE                                0xF820
#define INDEX_API_DATA_FOR_ONE_DEVICE                               0xF821
#define INDEX_LOG_DATA_FOR_ONE_DEVICE                               0xF830
#define INDEX_PDEV_DATA_FOR_ONE_DEVICE                              0xF831
#define INDEX_IM0_FILTER_DATA_FOR_ONE_DEVICE                        0xF840
#define INDEX_PD_REAL_DATA_FOR_ONE_DEVICE                           0xF841
#define INDEX_PD_EXPECTED_DATA_FOR_ONE_DEVICE                       0xF842
#define INDEX_AUTO_CONFIGURATION_FOR_ONE_DEVICE                     0xF850

/* specified lengths of I&M0 entries */
#define IM0_ORDERID_LEN                     20
#define IM0_SERIALNUMBER_LEN                16

/* specified lengths of I&M1 entries */
#define IM1_TAG_FUNCTION_LEN                32
#define IM1_TAG_LOCATION_LEN                22

/* specified length of I&M2 entry */
#define IM2_INSTALLATION_DATE_LEN           16

/* specified length of I&M3 entry */
#define IM3_DESCRIPTOR_LEN                  54

/* specified length of I&M4 entry */
#define IM4_SIGNATURE_LEN                   54

/* I&M support flags */
#define PROFILE_SPECIFIC_IM_SUPPORTED       0x0001
#define IM1_SUPPORTED                       0x0002
#define IM2_SUPPORTED                       0x0004
#define IM3_SUPPORTED                       0x0008
#define IM4_SUPPORTED                       0x0010
#define IM5_SUPPORTED                       0x0020
#define IM6_SUPPORTED                       0x0040
#define IM7_SUPPORTED                       0x0080
#define IM8_SUPPORTED                       0x0100
#define IM9_SUPPORTED                       0x0200
#define IM10_SUPPORTED                      0x0400
#define IM11_SUPPORTED                      0x0800
#define IM12_SUPPORTED                      0x1000
#define IM13_SUPPORTED                      0x2000
#define IM14_SUPPORTED                      0x4000
#define IM15_SUPPORTED                      0x8000

/* MAU types */
#define MAUTYPE_10BASET                     0x0005
#define MAUTYPE_10BASETXHD                  0x000A
#define MAUTYPE_10BASETXFD                  0x000B
#define MAUTYPE_10BASEFLHD                  0x000C
#define MAUTYPE_10BASEFLFD                  0x000D
#define MAUTYPE_100BASETXHD                 0x000F
#define MAUTYPE_100BASETXFD                 0x0010
#define MAUTYPE_100BASEFXHD                 0x0011
#define MAUTYPE_100BASEFXFD                 0x0012
#define MAUTYPE_1000BASEXHD                 0x0015
#define MAUTYPE_1000BASEXFD                 0x0016
#define MAUTYPE_1000BASELXHD                0x0017
#define MAUTYPE_1000BASELXFD                0x0018
#define MAUTYPE_1000BASESXHD                0x0019
#define MAUTYPE_1000BASESXFD                0x001A
#define MAUTYPE_1000BASETHD                 0x001D
#define MAUTYPE_1000BASETFD                 0x001E
#define MAUTYPE_10GIGBASEFX                 0x001F
#define MAUTYPE_100BASELX10                 0x002E
#define MAUTYPE_100BASEPXFD                 0x0036

/* media types */
#define MEDIATYPE_UNKNOWN                   0x00
#define MEDIATYPE_COPPER                    0x01
#define MEDIATYPE_FIBEROPTIC                0x02
#define MEDIATYPE_RADIO                     0x03

/* Auto negotiation capabilities */
#define AUTO_NEGOTIATION_SUPPORT            0x01
#define AUTO_NEGOTIATION_ENABLED            0x02

/* Auto negotiation advertised capabilities */
#define AUTO_NEG_CAP_UNKNOWN                0x8000 /* other or unknown */
#define AUTO_NEG_CAP_B10BASET               0x4000 /* 10BASE-T  half duplex mode */
#define AUTO_NEG_CAP_B10BASETFD             0x2000 /* 10BASE-T  full duplex mode */
#define AUTO_NEG_CAP_B100BASET4             0x1000 /* 100BASE-T4 */
#define AUTO_NEG_CAP_B100BASETX             0x0800 /* 100BASE-TX half duplex mode */
#define AUTO_NEG_CAP_B100BASETXFD           0x0400 /* 100BASE-TX full duplex mode */
#define AUTO_NEG_CAP_B100BASET2             0x0200 /* 100BASE-T2 half duplex mode */
#define AUTO_NEG_CAP_B100BASET2FD           0x0100 /* 100BASE-T2 full duplex mode */
#define AUTO_NEG_CAP_BFDXPAUSE              0x0080 /* PAUSE for full-duplex links */
#define AUTO_NEG_CAP_BFDXAPAUSE             0x0040 /* Asymmetric PAUSE for full-duplex links */
#define AUTO_NEG_CAP_BFDXSPAUSE             0x0020 /* Symmetric PAUSE for full-duplex links */
#define AUTO_NEG_CAP_BFDXBPAUSE             0x0010 /* Asymmetric and Symmetric PAUSE for full-duplex links */
#define AUTO_NEG_CAP_B1000BASEX             0x0008 /* 1000BASE-X, -LX, -SX, -CX half duplex mode */
#define AUTO_NEG_CAP_B1000BASEXFD           0x0004 /* 1000BASE-X, -LX, -SX, -CX full duplex mode */
#define AUTO_NEG_CAP_B1000BASET             0x0002 /* 1000BASE-T half duplex mode */
#define AUTO_NEG_CAP_B1000BASETFD           0x0001 /* 1000BASE-T full duplex mode */

/* defines for input and output data object elements */
#define INPUT_DATA_OBJECT_ELEMENT           0x01
#define OUTPUT_DATA_OBJECT_ELEMENT          0x02
#define OUTPUT_DATA_OPERATION               0x0000
#define OUTPUT_DATA_SUBSTITUTE              0x0001
#define SUBSTITUTION_MODE_ZERO              0x0000
#define SUBSTITUTION_MODE_LAST_VALUE        0x0001
#define SUBSTITUTION_MODE_REPLACEMENT_VALUE 0x0002

/* debug options */
#define DEBUG_OPTION_CONSOLE                0x00000001 /* debug output is sent to console instead of event archive */
#define DEBUG_OPTION_NON_VERBOSE            0x00000002 /* non verbose debug output */

/* configuration file definitions */
#define INI_SYSTEM_BLOCK                        "[SYSTEM]"
#define INI_SYSTEM_MAXNUMOFPNIOIPADAPTERS       "MaxNumOfPnioIpAdapters"

#define INI_CTM_BLOCK                           "[CTM]"
#define INI_CTM_PROFINET_IP_ADAPTER             "PnioIpAdapter"

#define INI_PNIO_CMDEV_BLOCK                    "[CMDEV]"
#define INI_PNIO_CMDEV_IDX                      "[CMDEV%d]"
#define INI_PNIO_CMDEV_PNIO_CMDEV_IP_ADAPTER    "PnioIpAdapter"

#define INI_IP_CONFIG1_BLOCK                    "[IPCONFIG1]"
#define INI_IP_CONFIG_IDX                       "[IPCONFIG%d]"

#define INI_MAC_ADDRESS                         "MACAddress"
#define INI_IP_ADDRESS                          "IPAddress"
#define INI_SUBNET_MASK                         "SubnetMask"
#define INI_DEFAULT_GATEWAY                     "DefaultGateway"
#define INI_PNIO_STATION_NAME_LENGTH            "StationNameLen"
#define INI_PNIO_STATION_NAME                   "StationName"
#define INI_PNIO_DEVICE_ID                      "DeviceID"
#define INI_PNIO_VENDOR_ID                      "VendorID"
#define INI_PNIO_DEVICE_TYPE                    "DeviceType"

#ifndef PNKW_NOINTERNAL_STRUCTURES
/*****************************************************************************
****  STRUCTURE DEFINITIONS                                               ****
*****************************************************************************/

/*! This structure contains one AR user id together with its diagnosis information. It is build about all consumer of one AR. */
typedef struct
{
  USIGN16       wARUserID;  /*!< User specific identifier of this AR. */
  USIGN16       wARDiag;    /*!< Special status of this AR, bitwise coded.
                                 - Bit 0: set if not connected
                                 - Bit 1: set if data invalid
                                 - Bit 2: set if diagnosis is available
                                 - Bit 3: set if module difference is available in case of configuration difference
                                 - Bit 4: set  if AR is deactivated
                                 - Bit 5: set if neighborhood information is not available
                                 - Bit 6: neighborhood information not unique
                                 - Bit 7 - Identify.cnf on alias received, but name of the device is already configured in another AR
                                 - Bit 8: Maintenance Required Information is available
                                 - Bit 9: Maintenance Demanded Information is available
                                 - Bit 10: Channel or Manufacturer Specific Diagnosis Information are available
                                 - Bit 11..15: reserved (shall be 0)
                             */
} T_AR_DIAG;

/*! This structure contains all ARs only with diagnosis summarized within a list. */
typedef struct
{
  USIGN16       wNumOfARs;  /*!< Number of entries with diagnosis entries within the following list. */
  T_AR_DIAG *   poAREntry;  /*!< This list contains the appropriate ARs with its status.

                                 \note
                                 The memory for the entries has to be allocated and freed by the caller and should be calculated as: 2 + AmountOfARs * 4.
                             */
} T_AR_LIST_WITH_DIAGNOSIS;


/*! This structure comprises an entry for the alias name list for one device */
typedef struct
{
   USIGN8            byIODeviceAliasNameLen;
                                       /*!< This attribute defines the length of the alias name. */
   CHAR              *pcIODeviceAliasName;
                                       /*!< This attribute is a pointer to the alias name of the Station.

                                            - This field shall be coded as data type OctetString with 1 to 255 octets.
                                            - Shall be set to NULL, if byIODeviceAliasNameLen is set to 0.
                                            - IODeviceAliasName := LLDP_PortID + "." + LLDP_ChassisID
                                            - LLDP_ChassisID is coded like IODeviceStationName
                                            - LLDP_PortID shall be "port-yxz" or "port-xyz-rstuv" with xyz from 001 to 255 and rstuv from 0000 to 65535
                                        */
} T_PNIO_CMCTL_ALIAS_NAME_LIST;

/*! This structure comprises the properties of one AR. */
typedef struct {
/*in*/  USIGN8            byState;     /*!< Reserved for future use.

                                            Allowed value:
                                            = CMCTL_PRIMARY
                                       */
/*in*/  USIGN8            bySupVisTakeOver;
                                       /*!< Reserved for future use.

                                            This attribute is used to enable or disable the supervisor
                                            takeover function for all submodules used by the AR.

                                            Allowed values:
                                            = ALLOWED
                                            = NOT_ALLOWED
                                       */
/*in*/  USIGN8            byParamServer;
                                       /*!< Reserved for future use.

                                            Allowed values:
                                            = CM_INITIATOR
                                            = EXT_PRM_SERVER
                                       */
/*in*/  USIGN8            byDataRate;  /*!< This attribute contains the data rate for this AR.

                                            Allowed values:
                                            = DR_AT_LEAST_100_MBIT
                                            = DR_100_MBIT
                                            = DR_1_GBIT
                                            = DR_10_GBIT
                                       */
/*in*/  USIGN8            byDeviceAccess;
                                       /*!< This parameter defines the access to the submodules which belong to the AR

                                            Allowed Values:
                                            = DA_CONTR_BY_EXP_SUBMOD_BLOCK = Only the submodules from the ExpectedSubmoduleBlock are accessible
                                            = DA_CONTR_BY_IO_DEVICE_APP = Submodule access is controlled by IO device application
                                       */
/*in*/  USIGN8            byCompanionAR;
                                       /*!< Reserved for future use.

                                            Allowed Values:
                                            = CAR_SINGLE_OR_SECOND_AR
                                            = CAR_FIRST_AR_COMPANION_AR_FOLLOWS
                                            = CAR_COMPANION_AR
                                       */
/*in*/  USIGN8            byPullModAlarmAllowed;
                                       /*!< Reserved for future use.

                                            Set value of the Alarm Type after pulling of a module.

                                            Allowed Values:
                                            = ALLOW_ALARM_TYPE_PULL (mandatory)
                                              = The Alarm Type(=Pull) shall signal pulling of submodule and module
                                              = The subslot number zero shall code pulling of module in conjunction with Alarm Type(=Pull).
                                            = ALLOW_ALARM_TYPE_PULL_MODULE
                                              = The Alarm Type(=Pull) shall signal pulling of submodule
                                              = The Alarm Type(=Pull module) shall signal pulling of module.
                                              = The subslot number 0 = 0x8FFF shall code pulling of submodule in conjunction with Alarm Type(=Pull).
                                       */
/*in*/  USIGN8            byCR_Transport;
                                       /*!< This attribute is used to select the transport protocol for the IO and Alarm CR.

                                            Allowed values:
                                            = SRT_TRANSPORT
                                            = UDP_TRANSPORT

                                            \note
                                            This attribute is no longer compliant to the specification and is only provided for compatibility reasons.
                                       */
/*in*/  USIGN8            byStartupMode;
                                       /*!< Startup mode during connection establishment

                                            Allowed values:
                                            = PNIO_STARTUP_MODE_LEGACY
                                            = PNIO_STARTUP_MODE_ADVANCED
                                       */
} T_AR_PROPERTIES;

/*! This structure comprises the attributes for the VLAN Tag.  */
typedef struct {
/*in*/  USIGN16           wVLANID;     /*!< This attribute contains the IEEE 802.1Q VLAN ID.
                                            The default value is NO_VLANID that indicates that the tag header contains only priority information.

                                            Allowed values:
                                            - NO_VLANID
                                            - DEFAULT_VLANID
                                        */
/*in*/  USIGN16           wIOCRPriority;
                                       /*!< This attribute contains the IEEE 802.1Q VLAN ID priority.
                                            The default value is IOCR_PRIORITY.

                                            Allowed values:
                                            - IOCR_PRIORITY = higher priority than IP traffic
                                        */
} T_IOCR_TAG_HEADER;

/*! This structure comprises related IO objects. */
typedef struct {
/*in*/  USIGN16           wSlotNumber; /*!< This attribute defines to which slot the object belongs.
                                            Allowed values:
                                            - 0 to 0x7FFF
                                        */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which subslot the object belongs.

                                            Allowed values:
                                            - 1 to 0x7FFF
                                        */
} T_RELATED_IO_OBJECT;

/*! This structure comprises the attributes for the related API of IOCR. */
typedef struct {
/*in*/  USIGN32           dwAPI;       /*!< API Identifier */
/*in*/  USIGN16           wNumberOfRelatedIODataObj;
                                       /*!< This attribute contains the number of IO Data objects that are related to the CR. */
/*in*/  T_RELATED_IO_OBJECT  *poRelatedIOData;
                                       /*!< Pointer to a dynamic array that contains structures with parameters of the IO Data object. */
/*in*/  USIGN16           wNumberOfRelatedIOCS;
                                       /*!< This attribute contains the number of IOCS that are related to the CR. */
/*in*/  T_RELATED_IO_OBJECT  *poRelatedIOCS;
                                       /*!< Pointer to a dynamic array that contains structures with parameters of the IOCS object. */
} T_RELATED_API_IOCR;

/*! This structure comprises the parameters for one input or output CR. */
typedef struct {
        USIGN16           wIOCRBlockVersion; /*!< Version of this CR Block

                                                  - High Byte = Block Version High
                                                  - Low Byte = Block Version Low
                                              */
/*in*/  USIGN16           wIOCRID;      /*!< Unique identifier of this IOCR.

                                             Allowed values:
                                             - 1 to ARCountMax * 4
                                        */
/*in*/  USIGN8            byRTClass;    /*!< Defines the realtime class

                                             Allowed values:
                                             - RT_CLASS1_RTC
                                             - RT_CLASS2_RTC
                                             - RT_CLASS3_RTC
                                             - RT_CLASS_UDP
                                        */
/*in*/  PHO_BOOL          bMediaRedundancy; /*!< Reserved for future use.

                                                 Switch for media redundant frame transfer

                                                 - PHO_FALSE = No media redundant frame transfer
                                                 - PHO_TRUE = Media redundant frame transfer
                                            */
/*in*/  USIGN16           wIOCRType;   /*!< This attribute indicates the type of this CR.

                                            Allowed values:
                                            - INPUT_CR
                                            - OUTPUT_CR
                                            - MULTI_PROV_CR
                                            - MULTI_CONS_CR
                                       */
/*in*/  USIGN16           wDataLength; /*!< This attribute contains the length of the elements in the data field (DATA, IOPS, IOCS).

                                            Allowed values:
                                            - 40 to 1440 for RT_CLASS1_RTC and RT_CLASS2_RTC
                                            - 0 to 1440 for RT_CLASS3_RTC
                                            - 12-1440 for RT_CLASS_UDP
                                       */
/*in*/  USIGN16           wFrameID;    /*!< This attribute contains the identifier of the data within the frame.
                                            The attribute value is provided by the consumer for Input CRs.
                                            It is provided by project planning tool. */
/*in*/  USIGN16           wSendClockFactor;
                                       /*!< This attribute contains the Send Clock Factor with a time base of 31,25 microseconds.

                                            Allowed values:
                                            - PNIO_CMCTL_SND_CLK_FACT_FOR_1ms
                                            - PNIO_CMCTL_SND_CLK_FACT_FOR_4ms
                                            - PNIO_CMCTL_SND_CLK_FACT_FOR_8ms
                                       */
/*in*/  USIGN16           wReductionRatio;
                                       /*!< This attribute contains the reduction ratio of the send clock of the device.
                                            The actual send cycle of the data set is Reduction Ratio multiplied with the send clock.
                                            It defines a macro cycle for this particular CR.

                                            Allowed value:
                                            - The following values shall used for SRT_TRANSPORT or UDP_TRANSPORT:
                                              - 1, 2, 4, 8, 16, 32, 64,128, 256, 512, 1024, 2048, 4096, 8192, 16384.
                                            - The values 1024 up to 16384 shall be used for UDP_TRANSPORT only.
                                       */
/*in*/  USIGN16           wPhase;      /*!< This attribute indicates in which particular send cycle the data has to be sent.
                                            This parameter depends from the parameter ReductionRatio allowed values are between 1 and selected ReductionRatio.
                                       */
/*in*/  USIGN16           wSequence;   /*!< This attribute contains the position of the data frame in the sending queue.
                                            It shall be less or equal to the related Reduction Ratio.
                                            Default value of wSequence is zero.
                                       */
/*in*/  USIGN32           dwFrameSendOffset;
                                       /*!< This attribute contains the relative send offset to the start of the related Reduction Ratio and phase.
                                            The time base is 250 nanoseconds.

                                            Allowed Value:
                                            - BEST_EFFORT (mandatory): as soon as possible
                                       */
/*in*/  USIGN16           wWatchDogFactor;
                                       /*!< This attribute contains factor for the watchdog, which is used to calculate the Watchdog Time.
                                            The interval for the Watchdog Time is the product of the attribute values of WatchDogFactor,
                                            SendClockFactor, ReductionRatio and the time base of 31,25 microseconds related to the monitored consumer.
                                            The Watchdog Time shall equal or less 1,92 seconds if the value of the AR attribute CR Transport is
                                            SRT_TRANSPORT and equal or less 49,152 seconds if the value of the AR attribute CR Transport is
                                            UDP_TRANSPORT.

                                            Allowed values:
                                            - 0x0003 to 0x1E00

                                            \note
                                            The watchdog factor must be divisible by 3
                                       */
/*in*/  USIGN16           wDataHoldFactor;
                                       /*!< This attribute contains a factor, which is used to calculate the DataHoldTime.
                                            This time is the time interval for holding last output data if no IO data have been received.
                                            For non-redundant ARs this value should be equal to the value of the attribute WatchDogFactor.
                                            The interval for the Data Hold Time is the product of the attribute values of DataHoldFactor,
                                            SendClockFactor, ReductionRatio and the time base of 31,25 microseconds related to the monitored consumer.
                                            The DataHoldTime shall equal or less 1,92 seconds if the value of the AR attribute CR Transport is
                                            SRT_TRANSPORT and equal or less 49,152 seconds if the value of the AR attribute CR Transport is
                                            UDP_TRANSPORT.

                                            Allowed values:
                                            - 0x0003 to 0x1E00

                                            \note
                                            The datahold factor must be divisible by 3
                                       */
/*in*/  T_IOCR_TAG_HEADER *poIOCRTagHeader;
                                       /*!< This structure contains the VLAN_ID and the Priority for this IOCR. */
/*in*/  USIGN8            pbyMulticastAddr[MAC_ADDR_SIZE];
                                       /*!< This attribute contains the multicast address according IEEE802.3 in case of multicast transmission.
                                            The value shall be set to zero for unicast connections.

                                            Example:
                                            - MAC-Address: 001122334455
                                              - pbyMulticastAddr[0] = 0x00;
                                              - pbyMulticastAddr[1] = 0x11;
                                              - pbyMulticastAddr[2] = 0x22;
                                              - pbyMulticastAddr[3] = 0x33;
                                              - pbyMulticastAddr[4] = 0x44;
                                              - pbyMulticastAddr[5] = 0x55;
                                       */
/*in*/  USIGN16           wNumberOfRelatedAPIs;
                                       /*!< Number of related APIs */
/*in*/  T_RELATED_API_IOCR  *poRelatedAPI;
                                       /*!< Pointer to a dynamic array which contains structures with parameters of the APIs. */
} T_IOCR_PARAM_BLOCK;

/*! This structure comprises the description of the elements for one Submodule. */
typedef struct {
/*in*/  USIGN16           wSlotNumber;  /*!< Related slot number of this element */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< Related subslot number of this element */
/*in*/  USIGN16           wOffset;     /*!< Offset of the element in the Data Field */
/*in*/  USIGN8            byInterpretation;
                                       /*!< Interpretation of the element in the Data Field

                                            Allowed values:
                                            - CMCTL_DATA
                                            - CMCTL_IOPS
                                            - CMCTL_IOCS
                                        */
/*in*/  USIGN16           wDataDescription;
                                       /*!< Description of the data for this Submodule

                                            Allowed values:
                                            - CMCTL_DATA_INPUT
                                            - CMCTL_DATA_OUTPUT
                                        */
/*in*/  USIGN16           wLength;
                                       /*!< Length of this one element in byte */
/*in*/  USIGN8            bySubstitutionMode;
                                       /*!< Mode for Substitution Value

                                            Allowed Values:
                                            - NULL_VALUE
                                            - LAST_VALUE
                                            - SUBSTITUTION_VALUE
                                        */
/*in*/  USIGN8            *pbySubstitutionValue;
                                       /*!< Shall be a NULL-pointer in case of NULL_VALUE or LAST_VALUE.
                                            Shall contain a pointer to a specific replacement value in case of SUBSTITUTION_VALUE.
                                            \note
                                            The length of the specific replacement value must have the length of parameter wLength of this structure.
                                        */

} T_SUBMODULE_ELEMENT_DESCRIPTION;

/*! This structure comprises the properties of one Submodule. */
typedef struct {
/*in*/  USIGN16           wSubmoduleType;
                                       /*!< This attribute defines the type of the subslot.

                                            Allowed values:
                                            - CMCTL_NO_IO
                                            - CMCTL_INPUT
                                            - CMCTL_OUTPUT
                                            - CMCTL_IO
                                        */
/*in*/  USIGN8            bySharedInput;
                                       /*!< This attribute defines the properties of the subslot.
                                            The value CMCTL_IOC_SHARED shall only be used if the attribute Submodule Type
                                            contains the value CMCTL_INPUT or CMCTL_IO.
                                            In the other cases the value of this attribute will be ignored.

                                            Allowed values:
                                            - CMCTL_IOC
                                            - CMCTL_IOC_SHARED
                                        */
/*in*/ USIGN8             byRedInpSubmodLength;
                                       /*!< (Bit 3 of SubmoduleProperties)

                                            Allowed values:
                                            - CMCTL_EXPECTED = Use expected input SubmoduleDataLength for INPUT-CR
                                            - CMCTL_ZERO = Reduce input SubmoduleDataLength to zero for INPUT-CR
                                        */
/*in*/ USIGN8             byRedOutSubmodLength;
                                       /*!< (Bit 4 of SubmoduleProperties)

                                            Allowed values:
                                            - CMCTL_EXPECTED = Use expected input SubmoduleDataLength for OUTPUT-CR
                                            - CMCTL_ZERO = Reduce input SubmoduleDataLength to zero for OUTPUT -CR
                                        */
/*in*/ USIGN8             byDiscardIOXS;
                                       /*!< (Bit 5 of SubmoduleProperties)

                                            Allowed values:
                                            - CMCTL_EXPECTED = Merge all expected IOXS for this submodule in the frames of the corresponding CRs
                                            - CMCTL_ZERO = Discard all expected IOXS for this submodule in the frames of the corresponding CRs
                                        */
} T_SUBMODULE_PROPERTIES;

/*! This structure comprises the description of one Submodule. */
typedef struct {
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 1 to 0x7FFF
                                        */
/*in*/  USIGN32           dwSubmoduleIdentNum;
                                       /*!< This attribute defines the identification of the subslot.

                                            Allowed values:
                                            - 0 to 0xFFFFFFFF
                                        */
/*in*/  USIGN32           dwNodeId;
                                       /*!< This parameter is not used. */
/*in*/  T_SUBMODULE_PROPERTIES
                          *poSubmoduleProperties;
                                       /*!< This structure includes properties for one Submodule. */
/*in*/  USIGN16           wNumberOfElements;
                                       /*!< Number of elements for this Submodule*/
/*in*/  T_SUBMODULE_ELEMENT_DESCRIPTION *poSubmoduleElementDes;
                                       /*!< Pointer to a dynamic array which contains structures with detailed descriptions for every element of this submodule. */
} T_SUBMODULE_DESCRIPTION;


/*! This structure comprises parameters for the related API of Submodules. */
typedef struct {
/*in*/  USIGN32           dwAPI;       /*!< API Identifier */
/*in*/  USIGN16           wSlotNumber; /*!< This parameter contains the value of the attribute Slot Number.

                                            Allowed values:
                                            - 0 to 0x7FFF
                                        */
/*in*/  USIGN32           dwModuleIdentNumber;
                                       /*!< This parameter contains the value of the attribute Module Ident Number.

                                            Not Allowed:
                                            - 0
                                        */
/*in*/  USIGN16           wModuleProperties;
                                       /*!< This attribute is reserved for future use.

                                            Allowed value:
                                            - 0
                                        */
/*in*/  USIGN16           wNumberOfSubmodules;
                                       /*!< This attribute defines the lower limit of the subslot number of the slot.

                                            Not Allowed:
                                            - 0
                                        */
/*in*/  T_SUBMODULE_DESCRIPTION *poSubmoduleDescription;
                                       /*!< Pointer to a dynamic array that contains structures with detailed descriptions for the Submodules. */
} T_RELATED_API_SUBMODULE;

/*! This structure comprises parameters for the Submodules. */
typedef struct {
       USIGN16           wSubModBlockVersion; /*!<  Version of this Submodule Parameter Block

                                                    - High Byte = Block Version High
                                                    - Low Byte = Block Version Low
                                               */
/*in*/  USIGN16           wNumberOfRelatedAPIs;
                                       /*!< Number of related APIs */
/*in*/  T_RELATED_API_SUBMODULE *poRelatedAPI;
                                       /*!< Pointer to a dynamic array that contains structures with parameters of the API. */
} T_SUBMODULE_PARAM_BLOCK;

/*! This structure comprises parameters for one Alarm CR */
typedef struct {
       USIGN16            wAlarmCRBlockVersion;
                                       /*!< Version of this Alarm CR Block

                                            - High Byte = Block Version High
                                            - Low Byte = Block Version Low
                                        */
       USIGN8             byAlarmCRProtType;
                                       /*!< This attribute is used to select the transport protocol type for the Alarm CR.

                                            Allowed values:
                                            - DATA_RTA_PDU
                                            - UDP_RTA_PDU
                                        */
/*in*/  USIGN16           wPriority;   /*!< This attribute contains the priority of the alarm.

                                            Allowed Values:
                                            - USER_PRIORITY (default) = the priority given by the user is used and two alarm resources are available
                                            - LOW_PRIORITY = user priority is ignored and only one alarm resource is available
                                        */
/*in*/  USIGN16           wRTATimeout; /*!< This attribute indicates the timeout value of the alarm source with a base of 100 milliseconds.
                                            The default value is 100 milliseconds.

                                            Allowed values:
                                            - 100 up to 10000 (steps 100)
                                        */
/*in*/  USIGN16           wRetries;    /*!< This attribute contains the number of retries.

                                            Allowed values:
                                            - 3 to 15
                                        */
/*in*/  USIGN16           wLocalAlarmReference;
                                       /*!< This attribute contains the alarm identifier of the CM Initiator (e.g. IO controller). */
/*in*/  USIGN16           wMaxAlarmDataLength;
                                       /*!< This attribute contains the length of the alarm data.

                                            Allowed values:
                                            - 200 to 1432
                                        */
} T_ALARM_CR_PARAM_BLOCK;

/*! This structure comprises parameters for one IO Device parameter block */
typedef struct {
/*in*/  USIGN16           wDeviceID;   /*!< DeviceID */
/*in*/  USIGN16           wVendorID;   /*!< VendorID */
/*in*/  USIGN16           wDeviceInstance;
                                       /*!< Device Instance

                                            Allowed values:
                                            - 1
                                        */
/*in*/  USIGN8            byIODeviceStationNameLen;
                                       /*!< This attribute defines the length of the Station Name. */
/*in*/  CHAR              *pcIODeviceStationName;
                                       /*!< This attribute is a pointer to the name of the Station.
                                            This field shall be coded as data type String with 1 to 240 octets.

                                            The following syntax applies:
                                            - 1 or more labels, separated by [.]
                                            - total length is 1-240
                                            - label length is 1-63
                                            - labels consist of [a-z0-9-]
                                            - labels do not start with [-]
                                            - labels do not end with [-]
                                            - the first label does not start with "port-xyz" or "port-xyz-abcde" with a,b,c,d,e, x, y, z = 0...9
                                            - station-names do not have the form n.n.n.n with n = 0...999
                                            - labels do only start with 'xn-' if RFC 3490 is applied
                                            - Furthermore, the definition of RFC 3490 shall be applied.

                                            EXAMPLE: "device-1.machine-1.plant-1.vendor"

                                            \note
                                            The field pcIODeviceStationName is not terminated by zero.
                                            Each pcIODeviceStationName shall be unique in case a device is equipped with more than one interface module.
                                         */
/*in*/  USIGN8            byNumOfAliasNames;
                                        /*!< This attribute defines the amount of alias names for the device. */
/*in*/  T_PNIO_CMCTL_ALIAS_NAME_LIST   *poIODeviceAliasNameList;
                                        /*!< This attribute points to a list of alias names for this device. */
        USIGN8            byAddressingMode; /*!< This attribute defines the addressing mode of this AR.
                                                In case of topological addressing only the alias name is used for identification of the device.
                                                Otherwise the station name is used first.

                                                Allowed values:
                                                - CMCTL_NORMAL_ADDR
                                                - CMCTL_TOPOLOGICAL_ADDR
                                             */
        PHO_BOOL          boIsDeviceReplacementAllowed;
                                        /*!< With this flag it is possible to control the PNIO device name assignment after PNIO Device Replacement.

                                            Allowed values:
                                            - PHO_TRUE = Replacement is allowed
                                            - PHO_FALSE = Replacement is not allowed
                                         */
/*in*/  USIGN8            pbyIPAddr[IP_ADDR_SIZE];
                                       /*!< IP address of the IO Device.

                                            Example:
                                            - IP-Address: 192.168.10.2
                                              - pbyIPAddr[0] = 192;
                                              - pbyIPAddr[1] = 168;
                                              - pbyIPAddr[2] = 10;
                                              - pbyIPAddr[3] = 2;

                                            \note
                                            The IP address is checked according to RFC 791 and RFC 6890
                                            and according the coding rules of the PROFINET specification.
                                        */
/*in*/  USIGN8            pbySubnetMask[IP_ADDR_SIZE];
                                       /*!< Subnet mask of the IO Device.

                                            Example:
                                            - subnet mask: 255.255.255.0
                                              - pbySubnetMask [0] = 255;
                                              - pbySubnetMask [1] = 255;
                                              - pbySubnetMask [2] = 255;
                                              - pbySubnetMask [3] = 0;

                                            \note
                                            The subnet mask is checked according to RFC 791 and RFC 6890
                                            and according the coding rules of the PROFINET specification.
                                        */
/*in*/  USIGN8            pbyStandardGateway[IP_ADDR_SIZE];
                                       /*!< IP address of the StandardGateway.

                                            Example:
                                            - IP-Address: 192.168.10.2
                                              - pbyStandardGateway[0] = 192;
                                              - pbyStandardGateway[1] = 168;
                                              - pbyStandardGateway[2] = 10;
                                              - pbyStandardGateway[3] = 2;

                                            \note
                                            The standard gateway is checked according to RFC 791 and RFC 6890
                                            and according the coding rules of the PROFINET specification.
                                        */
/*in*/  PHO_BOOL          bFlagIPoverDNS;
                                       /*!< With this Flag it is possible to set the source of the IP Address.

                                            Allowed value:
                                            - PHO_TRUE = IO Device gets the IP Address from DNS
                                            - PHO_FALSE = IP Address of the IO Device is set by the parameter pbyIPAddr
                                        */
       USIGN16            wDapSlotnum;
                                       /*!< Slotnumber of the Device Access Point (DAP) described within the GSDML. */
       USIGN16            wDapSubslotnum;
                                       /*!< Subslotnumber of the Device Access Point (DAP) described within the GSDML. */
} T_DEVICE_PARAM_BLOCK;

/*! This structure stores a globally unique identifier (GUID).
    GUIDs uniquely identify objects such as interfaces, manager entry-point vectors and class objects. */
typedef struct
{
  USIGN32 dwData1;      /*!< first eight hexadecimal digits */
  USIGN16 wData2;       /*!< first group of four hexadecimal digits */
  USIGN16 wData3;       /*!< second group of four hexadecimal digits */
  USIGN8  pbyData4[8];  /*!< Array of eight bytes. The first two bytes contain the third group of four hexadecimal digits.
                             The remaining six bytes contain the final 12 hexadecimal digits
                         */
} T_GUID;

/* <-- System 4.3 */
/*! This structure contains the parameters for system redundancy. */
typedef struct
{
   USIGN16 wRedDataHoldFact;  /*!< RedundancyDataHoldFactor for the RedundancyDataHoldTime with a time base of 1ms. The expiration leads to an AR termination

                                   Allowed values:
                                   - 0x0000 to 0x0002 are reserved
                                   - 0x0003 to 0xFFFF
                               */
   USIGN32 dwSRProps;         /*!< System Redundancy Properties

                                    Allowed values:
                                    - Bit 0 (InputValidOnBackupAR)
                                      - 0x00: The device may deliver valid input data
                                      - 0x01: The device shall deliver valid input data
                                    - Bit 1 (ActivateRedundancyAlarm)
                                      - 0x00: The device shall not send a redundancy alarm
                                      - 0x01: The device shall send a redundancy alarm
                                    - Bits 2 to 31 are reserved
                               */
} T_SR_INFO_BLOCK;

/*! This structure contains the parameters for the subframes. */
typedef struct
{
   USIGN16 wIOCRRef;            /*!< This parameter contains the IOCR ID for this subframe data set. */
   USIGN16 wSubFOfs;            /*!< This parameter contains the subframe offset.

                                     Allowed values:
                                     - 0x0000 to 0x059B
                                     - 0x059C to 0xFFFF (reserved)
                                 */
   USIGN32 dwSubFData;          /*!< This parameter contains the subframe data.

                                    Allowed values:
                                    - Bit 0 to 6 (Position)
                                      - 0x00 reserved
                                      - 0x01 to 0x7F: Position of the Subframe
                                    - Bit 7 (reserved)
                                    - Bit 8 to 15 (DataLength)
                                      - 0x00 reserved
                                      - 0x01 to 0xFF: Number of octets of the C_SDU of the subframe
                                    - Bit 16 to 31 (reserved)
                               */
} T_SUBFDATA;

/*! This structure contains the parameters for system redundancy. */
typedef struct
{
   T_GUID  oIRDataUUID;     /*!< This parameter contains the IR Data UUID */
   USIGN16 wNumOfIOCRs;     /*!< This parameter contains the amount of IOCRs with subframe data information

                                 Allowed values:
                                 - 0
                                 - 2
                             */
   T_SUBFDATA *poSubFData;  /*!< Pointer to array with subframe data information for each IOCR. */
} T_IR_INFO_BLOCK;
/* System 4.3 --> */

/*! This structure comprises the input parameter to add one AR. */
typedef struct {
       USIGN16            wARBlockVersion; /*!< Version of the AR Block

                                             High Byte = Block Version High
                                             Low Byte = Block Version Low
                                           */
/*in*/  T_GUID            oARUUID;     /*!< This attribute defines the UUID of the AR entry which is provided
                                            by project planning or the CM Initiator.
                                       */
/*in*/  USIGN16           wARUserID;   /*!< User specific identifier of this AR. */
/*in*/  USIGN16           wARType;     /*!< This attribute defines the type of
                                            the AR entry.

                                            Allowed Values:
                                            IOCAR_SINGLE
                                            IOCAR_CIR (not implemented)
                                            IOCAR_CONTROLLER_REDUNDANT (not implemented)
                                            IOCAR_DEVICE_REDUNDANT (not implemented)
                                            IOSAR
                                            IOCAR_SR
                                       */
/*in*/  PHO_BOOL          bMultipleWrite;
                                       /*!< Flag to select the parameter transfer mode
                                            Allowed Values:
                                            FALSE = Send every parameter set in a single write service
                                            TRUE = Send all parameter sets in a single write service
                                       */
/*in*/  USIGN32           dwConnectTimeout;
                                       /*!< This values defines the timeout in [ms] for a RPC connect response from a device.
                                            Allowed Values: >=0
                                            The value 0 indicates that the controller waits infinite for a response.
                                       */
/*in*/  USIGN32           dwDcpParamDoneTimeout;
                                       /*!< This values defines the timeout in [ms] for a DCP parameterisation. */
/*in*/  USIGN16           wCMInitActTOFact;
                                       /*!< This value specifies the CMInitiatorActivityTimeoutFactor with a time base of 100ms.
                                            Allowed Values:
                                            In case of byDeviceAccess = DA_CONTR_BY_EXP_SUBMOD_BLOCK: 1 to 1000
                                            In case of byDeviceAccess = DA_CONTR_BY_IO_DEVICE_APP: 100 to 1000
                                       */
                                       /* System 4.3 */
/*in*/  PHO_BOOL          bIsAllowModuleDiff;
                                       /*!< This flag selects if module differences should be allowed.
                                            Allowed Values:
                                            PHO_FALSE: Module differences are not allowed and the connection to the device is stopped
                                            PHO_TRUE: Module differences are allowed and the connection to the device continues
                                       */
/*in*/  T_AR_PROPERTIES   *poARProperties;
                                       /*!< Structure contains properties for one AR. */
/*in*/  PHO_BOOL          bSwitchARPassiveOverIOAR;
                                       /*!< This flag sets the mode to set an AR passive after Watchdog/DataHold-Timeout of the consumer.
                                            Allowed Values:
                                            PHO_FALSE: Legacy Mode is used
                                            PHO_TRUE: Fast mode is used
                                       */
/*in*/  USIGN16           wNumberOfIOCRs;
                                       /*!< Number of CRs in this AR */
/*in*/  T_IOCR_PARAM_BLOCK  *poIOCRParamBlock;
                                       /*!< Pointer to a dynamic array which contains structures with input parameters of the IOCRs. */
/*in*/  USIGN16           wNumberOfSubmoduleBlocks;
                                       /*!< Number of Submodule blocks in this AR */
/*in*/  T_SUBMODULE_PARAM_BLOCK *poSubmoduleParamBlock;
                                       /*!< Structure with input parameters of the Submodules. */
/*in*/  USIGN16           wNumberOfAlarmCRs;
                                       /*!< Number of Alarm CRs in this AR
                                            Allowed Value: 1
                                            Currently only one Alarm CR is supported
                                       */
/*in*/  T_ALARM_CR_PARAM_BLOCK  *poAlarmCRParamBlock;
                                       /*!< Pointer to a dynamic array which contains structures with input parameters of the Alarm CRs. */
/*in*/  T_DEVICE_PARAM_BLOCK  *poDeviceParamBlock;
                                       /*!< Structure with parameters of the IO Device. */
/*in*/  USIGN32           dwFSUModeAndTimeout;
                                       /*!< This parameter selects the startup procedure of a device.
                                            Allowed Values:
                                            NORMAL_STARTUP_MODE = The device establishes the AR using the simple procedure
                                            FAST_STARTUP_MODE_TIMEOUT_MIN to FAST_STARTUP_MODE_TIMEOUT_MAX = The device establishes the AR using the fast startup procedure

                                            \note
                                            If the device uses the fast startup procedure, the following record data have to be provided by using the function PN_ArParamAdd():
                                            - ARFSUDataAdjust
                                            - PDInterfaceFSUDataAdjust
                                            - PDPortDataAdjust
                                            - PDPortDataCheck
                                       */
/* <-- System 4.3 */
/*in*/  T_SR_INFO_BLOCK *poSRInfoBlock;/*!< Structure containing the parameters for system redundancy.
                                            Will be transported only in case of IOCAR_SR.
                                       */
/*in*/  T_IR_INFO_BLOCK *poIRInfoBlock;/*!< This structure is for future use and contains the parameters for the IR Info Block.
                                            If this parameter is a null pointer, this block will be omitted within the Connect.req.
                                            It is only valid with PNIO_STARTUP_MODE_ADVANCED in conjunction with IRT communication.
                                       */
/* System 4.3 --> */
} T_PNIO_CMCTL_AR_ADD_PARAM_IN;

/*! This structure comprises the input parameter to add parameters to one AR. */
typedef struct {
/*in*/  USIGN16           wARID;       /*!< Unique identifier of the AR

                                            Allowed Values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN32           dwAPI;       /*!< This attribute address the related API context for an established AR.
                                            Default 0, other reserved for future profiles.
                                            This parameter is only needed if more then one AR to one IO Device exists.
                                            */
/*in*/  USIGN16           wSlotNumber; /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0 to 0x7FFF
                                        */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 1 to 0x7FFF
                                        */
/*in*/  USIGN16           wIndex;      /*!< The range from zero to 0x7FFF shall be used to address user specific record data objects.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF user specific
                                            - 0x8000 to 0xBFFF subslot specific
                                            - 0xC000 to 0xDFFF slot specific
                                            - 0xE000 to 0xEFFF AR specific
                                            - 0xF000 to 0xF7FF API specific
                                            - 0xF800 to 0xFFFF device specific
                                        */
/*in*/  USIGN16           wLengthOfParameters;
                                       /*!< Length of the parameter arry */
/*in*/  USIGN8            *pbyParameter;
                                       /*!< Pointer to the dynamic array which contains the parameter */
} T_PNIO_CMCTL_AR_PARAM_ADD_PARAM_IN;

/*! This structure comprises parameters to find the AR of the station name. */
typedef struct {
/*in*/  USIGN8            byIODeviceStationNameLen;
                                       /*!< This attribute defines the length of the Station Name. */
/*in*/  CHAR              *pcIODeviceStationName;
                                       /*!< This attribute is a pointer to the name of the Station.

                                            This field shall be coded as data type String with 1 to 240 octets.
                                            The following syntax applies:
                                            - 1 or more labels, separated by [.]
                                            - total length is 1-240
                                            - label length is 1-63
                                            - labels consist of [a-z0-9-]
                                            - labels do not start with [-]
                                            - labels do not end with [-]
                                            - the first label does not start with "port-xyz" or "port-xyz-abcde" with a,b,c,d,e, x, y, z = 0...9
                                            - -station-names do not have the form n.n.n.n with n = 0...999
                                            - labels do only start with 'xn-' if RFC 3490 is applied
                                            - Furthermore, the definition of RFC 3490 shall be applied.

                                            EXAMPLE: "device-1.machine-1.plant-1.vendor"

                                            \note
                                            The field pcIODeviceStationName is not terminated by zero.
                                            Each pcIODeviceStationName shall be unique in case a device is equipped with more than one interface module
                                        */
/*in*/  USIGN32           dwAPI;       /*!< This attribute address the related API context for an established AR.
                                            Default 0, other reserved for future profiles.
                                            This parameter is only needed if more then one AR to one IO Device exists.
                                        */
/*in*/  USIGN16           wSlotNumber; /*!< This attribute defines to which slot the object belongs.
                                            This parameter is only needed if more then one AR to one IO Device exists.

                                            Allowed values:
                                            - 0 to 0x7FFF
                                        */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which slot the object belongs.
                                            This parameter is only needed if more then one AR to one IO Device exists.

                                            Allowed values:
                                            - 1 to 0x7FFF
                                        */
} T_STATION_NAME_FIND_PARAM;

/*! This structure comprises the input parameters to find the AR. */
typedef struct {
/*in*/  T_STATION_NAME_FIND_PARAM  *poStationNameFindParam;
                                       /*!< This structure contains the parameters which are needed to find the AR by the station name.
                                            This structure is used for search if the byFindOption is set to FIND_BY_STATION_NAME.
                                        */
/*in*/  USIGN16           wARID;       /*!< Unique identifier of the AR
                                            This attribute is used for search if the byFindOption is set to FIND_BY_ARID.

                                            Allowed Values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN16           wARUserID;   /*!< User specific identifier of this AR.
                                            This attribute is used for search if the byFindOption is set to FIND_BY_AR_USER_ID.
                                        */
} T_AR_READ_PARAM_IN;

/*! This structure comprises the output parameters of the AR. */
typedef struct {
/*in*/  USIGN8            byIODeviceStationNameLen;
                                       /*!< This attribute defines the length of the Station Name. */
/*in*/  CHAR              *pcIODeviceStationName;
                                       /*!< This attribute is a pointer to the name of the Station.

                                            This field shall be coded as data type String with 1 to 240 octets.
                                            The following syntax applies:
                                            - 1 or more labels, separated by [.]
                                            - total length is 1-240
                                            - label length is 1-63
                                            - labels consist of [a-z0-9-]
                                            - labels do not start with [-]
                                            - labels do not end with [-]
                                            - the first label does not start with "port-xyz" or "port-xyz-abcde" with a,b,c,d,e, x, y, z = 0...9
                                            - -station-names do not have the form n.n.n.n with n = 0...999
                                            - labels do only start with 'xn-' if RFC 3490 is applied
                                            - Furthermore, the definition of RFC 3490 shall be applied.

                                            EXAMPLE: "device-1.machine-1.plant-1.vendor"

                                            \note
                                            The field pcIODeviceStationName is not terminated by zero.
                                            Each pcIODeviceStationName shall be unique in case a device is equipped with more than one interface module
                                        */
/*in*/  USIGN16           wARID;       /*!< Unique identifier of the AR

                                            Allowed Values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN16           wARUserID;   /*!< User specific identifier of this AR. */
/*in*/  USIGN16           wARStatus;   /*!< Status of the AR

                                            Allowed Values:
                                            - AR_STATUS_ADDED
                                            - AR_STATUS_STARTED
                                            - AR_STATUS_WAIT_FOR_DCP_HELLO
                                            - AR_STATUS_DNS_STARTED
                                            - AR_STATUS_DCP_IDENTIFY_STARTED
                                            - AR_STATUS_WAIT_FOR_DNS_IND
                                            - AR_STATUS_WAIT_FOR_DCP_IDENTIFY_IND
                                            - AR_STATUS_WAIT_FOR_DCP_IDENTIFY_ALIAS_IND
                                            - AR_STATUS_WAIT_FOR_DCP_SET_NAME_IND
                                            - AR_STATUS_WAIT_FOR_DCP_SET_IP_IND
                                            - AR_STATUS_WAIT_FOR_ARP_CNF
                                            - AR_STATUS_DCP_SET_IP
                                            - AR_STATUS_WAIT_FOR_DCP_SET_IP_CNF
                                            - AR_STATUS_RPC_CONNECT_REQ
                                            - AR_STATUS_WAIT_FOR_RPC_CONNECT_CNF
                                            - AR_STATUS_AR_ACTIVATE
                                            - AR_STATUS_WRITE_PRM_REQ
                                            - AR_STATUS_WAIT_FOR_WRITE_PRM_CNF
                                            - AR_STATUS_PRM_END_REQ
                                            - AR_STATUS_WAIT_FOR_PRM_END_CNF
                                            - AR_STATUS_WAIT_FOR_APP_READY_IND
                                            - AR_STATUS_APP_READY_RES
                                            - AR_STATUS_APP_READY
                                            - AR_STATUS_WAIT_FOR_READ_CNF
                                            - AR_STATUS_WAIT_FOR_WRITE_CNF
                                            - AR_STATUS_RPC_CANCEL
                                            - AR_STATUS_WAIT_FOR_RPC_CANCEL_CNF
                                            - AR_STATUS_RELEASE
                                            - AR_STATUS_WAIT_FOR_RELEASE_CNF
                                            - AR_STATUS_AR_DEACTIVATE
                                            - AR_STATUS_STOP
                                            - AR_STATUS_PROV_ERROR_IND
                                            - AR_STATUS_CONS_ERROR_IND
                                            - AR_STATUS_ALARM_ERROR_IND
                                            */
/*in*/  USIGN16           wARAddInfo;    /*!< This attribute contains additional info to the AR state.

                                              Allowed values:
                                              - AR_INFO_NONE: no additional information
                                              - AR_INFO_CON_ERR_TRANS: transmission error
                                              - AR_INFO_CON_ERR_ADR_DNS: DNS address resolution error
                                              - AR_INFO_CON_ERR_ADR_DCP_NAME: DCP address resolution error (device name)
                                              - AR_INFO_CON_ERR_ADR_DCP_ALIAS: DCP address resolution error (neighbourhood information)
                                              - AR_INFO_CON_ERR_MTP_NAME: multiple station names
                                              - AR_INFO_CON_ERR_MTP_ALIAS: multiple alias names
                                              - AR_INFO_CON_ERR_ MTP_IP_ADDRESS: multiple IP addresses
                                              - AR_INFO_CON_ERR_REFUSED_TO: AR refused by the device (timeout)
                                              - AR_INFO_CON_ERR_REFUSED_NEGRES: AR refused by the device (negative response)
                                              - AR_INFO_CON_ERR_DUPL_NBH_INFO: neighborhood information not unique
                                              - AR_INFO_CON_ERR_NAME_ALREADY_USED: Identify.cnf on alias received, but name of the device is already configured in another AR
                                              - AR_INFO_CON_ERR_NBH_NOT_UNIQUE: Neighborhood not unique. There are more than one peers per port stated in PDPortDataReal
                                                */
/*in*/  USIGN8            byErrorCode;   /*!< Status information during negative confirmed request or timeout.
                                              This parameter is only valid in case of wARAddInfo = AR_INFO_CON_ERR_REFUSED_NEGRES or AR_INFO_CON_ERR_REFUSED_TO
                                          */
/*in*/  USIGN8            byErrorDecode; /*!< This parameter selects one Error scheme for Error Code 1 and 2 according IEC 61158 Type 3 specification.
                                              This parameter is only valid in case of wARAddInfo = AR_INFO_CON_ERR_REFUSED_NEGRES or AR_INFO_CON_ERR_REFUSED_TO
                                          */
/*in*/  USIGN8            byErrorCode1;  /*!< The parameter Error Code 1 specifies the faulty element within the appropriate block request structure.
                                              This parameter is only valid in case of wARAddInfo = AR_INFO_CON_ERR_REFUSED_NEGRES or AR_INFO_CON_ERR_REFUSED_TO
                                          */
/*in*/  USIGN8            byErrorCode2;  /*!< The parameter Error Code 2 specifies the faulty element within the appropriate block request structure.
                                              This parameter is only valid in case of wARAddInfo = AR_INFO_CON_ERR_REFUSED_NEGRES or AR_INFO_CON_ERR_REFUSED_TO
                                          */
/*in*/  T_AR_LIST_WITH_DIAGNOSIS *poARwithStatusList;
                                         /*!< Pointer to the structure containing the list of ARs with diagnosis.
                                              If the option FIND_AR_LIST_WITH_DIAGNOSIS is used, only this parameter is valid within this structure.
                                              The memory is allocated by the caller. Otherwise it contains a Null pointer.
                                          */
} T_AR_READ_PARAM_OUT;

/*! This structure comprises input parameters to read data from one AR. */
typedef struct {
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR. If this attribute is zero then the parameter oTargetARUUID or wARUserID are used to identify the source to read data.

                                            Allowed Values:
                                            - 0x0000: oTargetARUUID or wARUserID is used
                                            - 0x0001 to 0xFFFF: wARID is used
                                        */
/*in*/  USIGN16           wARUserID;
                                       /*!< User specific identifier of this AR.
                                            If this parameter is 0 then oTargetARUUID is used.
                                        */
/*in*/  T_GUID            oTargetARUUID;
                                       /*!< This attribute defines the ARUUID of the AR entry which is provided by project planning or the CM Initiator.
                                            If the wARID and the wARUserID are zero then the parameter oTargetARUUID is used to identify the source to read data. */
/*in*/  USIGN32           dwAPI;
                                       /*!< This attribute addresses the related API context for an established AR.

                                            Allowed values:
                                            Default 0, other reserved for future profiles
                                        */
/*in*/  USIGN16           wSlotNumber;
                                        /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0 to 0x7FFF
                                        */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0 to 0x8FFF
                                        */
/*in*/  USIGN16           wIndex;
                                        /*!< This attribute specfies the data within the requested Slot/Subslot combination.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF user specific
                                            - 0x8000 to 0xBFFF subslot specific
                                            - 0xC000 to 0xDFFF slot specific
                                            - 0xE000 to 0xEFFF AR specific
                                            - 0xF000 to 0xF7FF API specific
                                            - 0xF800 to 0xFFFF device specific
                                        */
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< This parameter is the unique identifier of one read request.
                                            This identifier is used by the application to identify the read confirmation to the right read request.*/
/*in*/  USIGN32           dwDataLength;
                                        /*!< This parameter specifies the number of octets of a data object that has to be read.

                                            Allowed values:
                                            - 0 to 2^32-65
                                        */
} T_READ_PARAM_IN;

/*! This structure contains input parameters to read data via implicit read. */
typedef struct
{
/*in*/  USIGN8            pbyIPAddr[IP_ADDR_SIZE];
                                       /*!< IP address of the device which handles the implicit read request.

                                            Example:
                                            - IP-Address: 192.168.10.1
                                              - pbyIPAddr[0] = 192;
                                              - pbyIPAddr[1] = 168;
                                              - pbyIPAddr[2] = 10;
                                              - pbyIPAddr[3] = 1;
                                        */
/*in*/  USIGN32           dwAPI;
                                       /*!< This attribute addresses the related API context for an established AR.

                                            Allowed values:
                                            Default 0, other reserved for future profiles
                                        */
/*in*/  USIGN16           wSlotNumber;
                                        /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0 to 0x7FFF
                                        */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0 to 0x8FFF
                                        */
/*in*/  USIGN16           wIndex;
                                        /*!< This attribute specfies the data within the requested Slot/Subslot combination.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF user specific
                                            - 0x8000 to 0xBFFF subslot specific
                                            - 0xC000 to 0xDFFF slot specific
                                            - 0xE000 to 0xEFFF AR specific
                                            - 0xF000 to 0xF7FF API specific
                                            - 0xF800 to 0xFFFF device specific
                                        */
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< This parameter is the unique identifier of one read request.
                                            This identifier is used by the application to identify the read confirmation to the right read request.*/
/*in*/  USIGN32           dwDataLength;
                                        /*!< This parameter specifies the number of octets of a data object that has to be read.

                                            Allowed values:
                                            - 0 to 2^32-65
                                        */
/*in*/  USIGN16           wInstanceID;
                                       /*!< Unique identifier of the addressed device instance (taken from GSDML or EPMAP). */
/*in*/  USIGN16           wVendorID;
                                       /*!< Unique vendor identifier of the addressed device (taken from GSDML or EPMAP). */
/*in*/  USIGN16           wDeviceID;
                                       /*!< Unique device identifier of the addressed device (taken from GSDML or EPMAP). */
} T_IMPL_READ_PARAM_IN;

/*! This structure comprises the results of the Read request from one AR. */
typedef struct {
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR

                                            Allowed Values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN16           wARUserID;
                                       /*!< User specific identifier of this AR */
/*in*/  USIGN8            byErrorCode; 
                                       /*!< This parameter selects one Error Code */
/*in*/  USIGN8            byErrorDecode;
                                       /*!< This parameter selects one Error scheme for Error Code 1 and 2 according IEC 61158 Type 3 specification */
/*in*/  USIGN8            byErrorCode1;
                                       /*!< The Error Code 1 assumes one of the following values: 
                                            PNIO_STATUS_NO_ERROR
                                            
                                            PNIORW Errors
                                            Application
                                            - PNIORW_WRITE_ERROR
                                            - PNIORW_MODULE_FAILURE
                                            - PNIORW_BUSY
                                            - PNIORW_VERSION_CONFLICT
                                            - PNIORW_FEATURE_NOT_SUPPORTED
                                            Access
                                            - PNIORW_INVALID_INDEX
                                            - PNIORW_WRITE_LENGTH_ERROR
                                            - PNIORW_INVALID_SLOT_SUBSLOT
                                            - PNIORW_TYPE_CONFLICT
                                            - PNIORW_INVALID_AREA
                                            - PNIORW_STATE_CONFLICT
                                            - PNIORW_ACCESS_DENIED
                                            - PNIORW_INVALID_RANGE
                                            - PNIORW_INVALID_PARAMETER
                                            - PNIORW_INVALID_TYPE
                                            - PNIORW_BACKUP
                                            Resource
                                            - PNIORW_WRITE_CONSTRAIN_CONFLICT
                                            - PNIORW_RESOURCE_BUSY
                                            - PNIORW_RESOURCE_UNAVAILABLE
                                            
                                            PNIO Errors
                                            Connect Parameter Error
                                            - PNIO_FAULTY_ AR_BLOCK_REQ
                                            - PNIO_FAULTY_IOCR_BLOCK_REQ
                                            - PNIO_FAULTY_EXP_SUBMODULE_BLOCK_REQ
                                            - PNIO_FAULTY_ALARM_CR_BLOCK_REQ
                                            - PNIO_FAULTY_PRM_SERVER_LOCK_REQ
                                            IODControl Parameter Error
                                            - PNIO_FAULTY_CONTROL_BLOCK_CONNECT
                                       */

/*in*/  USIGN8            byErrorCode2;
                                       /*!< The parameter Error Code 2 is user specific for byErrorDecode = PNIORW for byErrorDecode = PNIO byErrorCode2 gives the parameter number */
/*in*/  USIGN16           wAddData1;   
                                       /*!< The parameter Add Data is API specific (profile) */
/*in*/  USIGN16           wAddData2;   
                                       /*!< The parameter Add Data is user specific */
/*in*/  USIGN16           wIndex;      
                                       /*!< The range from zero to 0x7FFF shall be used to address user specific record data objects.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF user specific
                                            - 0x8000 to 0xBFFF subslot specific
                                            - 0xC000 to 0xDFFF slot specific
                                            - 0xE000 to 0xEFFF AR specific
                                            - 0xF000 to 0xF7FF API specific
                                            - 0xF800 to 0xFFFF device specific
                                        */
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< Transaction number is the unique identifier of one read request.
                                            This identifier is used by the Application to identify the
                                            read confirmation to the right read request
                                       */
/*in*/  USIGN32           dwDataLength;
                                        /*!< The parameter dwDataLength indicates the number of octets of the data object that has been read.

                                            Allowed values:
                                            - 0 to 2^32-65
                                        */
/*in*/  USIGN8            *pbyData;    
                                        /*!< The parameter pbyData contains the value of the object which has been read and
                                             consists of the number of octets indicated in the length of the response.
                                        */
} T_READ_CONF_PARAM;

/*! This structure comprises the input parameters to write data to one AR. */
typedef struct {
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR. If this attribute is zero then the parameter oTargetARUUID or wARUserID are used to identify the source to read data.

                                            Allowed values:
                                            Value            |   Meaning
                                                            -|-
                                            0x0000           | wARUserID is used
                                            0x0001 to 0xFFFF | wARID is used
                                        */
/*in*/  USIGN16           wARUserID;
                                       /*!< User specific identifier of this AR.
                                            If this parameter is 0 then oTargetARUUID is used.
                                        */
/*in*/  USIGN32           dwAPI;
                                       /*!< This attribute addresses the related API context for an established AR.

                                            Allowed values:
                                            Default 0, other reserved for future profiles
                                        */
/*in*/  USIGN16           wSlotNumber;
                                        /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0 to 0x7FFF
                                        */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0 to 0x8FFF
                                        */
/*in*/  USIGN16           wIndex;
                                        /*!< This attribute specfies the data within the requested Slot/Subslot combination.

                                            Allowed values:
                                            Value            |   Meaning
                                                            -|-
                                            0x0000 to 0x7FFF | User specific
                                            0x8000 to 0xBFFF | Subslot specific
                                            0xC000 to 0xDFFF | Slot specific
                                            0xE000 to 0xEFFF | AR specific
                                            0xF000 to 0xF7FF | API specific
                                            0xF800 to 0xFFFF | Device specific
                                        */
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< This parameter is the unique identifier of one read request.
                                            This identifier is used by the application to identify the read confirmation to the right read request.*/
/*in*/  USIGN32           dwDataLength;
                                        /*!< This parameter specifies the number of octets of the data object that has to be written.

                                            Allowed values:
                                            - 0 to 2^32-65
                                        */
/*in*/  USIGN8            *pbyData;
                                       /*!< This parameter contains the value of the data object which has to be written
                                            and consists of the number of octets indicated in the dwDataLength of the request.
                                            The data lies in byte one behind the other and in big endian byte order.
                                        */
} T_WRITE_PARAM_IN;

/*! This structure comprises the results of the Write request from one AR. */
typedef struct {
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR

                                            Allowed Values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN16           wARUserID;
                                       /*!< User specific identifier of this AR */
/*in*/  USIGN8            byErrorCode; 
                                       /*!< This parameter selects one Error Code */
/*in*/  USIGN8            byErrorDecode;
                                       /*!< This parameter selects one Error scheme for Error Code 1 and 2 according IEC 61158 Type 3 specification */
/*in*/  USIGN8            byErrorCode1;
                                       /*!< The Error Code 1 assumes one of the following values: 
                                            PNIO_STATUS_NO_ERROR
                                            
                                            PNIORW Errors
                                            Application
                                            - PNIORW_WRITE_ERROR
                                            - PNIORW_MODULE_FAILURE
                                            - PNIORW_BUSY
                                            - PNIORW_VERSION_CONFLICT
                                            - PNIORW_FEATURE_NOT_SUPPORTED
                                            Access
                                            - PNIORW_INVALID_INDEX
                                            - PNIORW_WRITE_LENGTH_ERROR
                                            - PNIORW_INVALID_SLOT_SUBSLOT
                                            - PNIORW_TYPE_CONFLICT
                                            - PNIORW_INVALID_AREA
                                            - PNIORW_STATE_CONFLICT
                                            - PNIORW_ACCESS_DENIED
                                            - PNIORW_INVALID_RANGE
                                            - PNIORW_INVALID_PARAMETER
                                            - PNIORW_INVALID_TYPE
                                            - PNIORW_BACKUP
                                            Resource
                                            - PNIORW_WRITE_CONSTRAIN_CONFLICT
                                            - PNIORW_RESOURCE_BUSY
                                            - PNIORW_RESOURCE_UNAVAILABLE
                                            
                                            PNIO Errors
                                            Connect Parameter Error
                                            - PNIO_FAULTY_ AR_BLOCK_REQ
                                            - PNIO_FAULTY_IOCR_BLOCK_REQ
                                            - PNIO_FAULTY_EXP_SUBMODULE_BLOCK_REQ
                                            - PNIO_FAULTY_ALARM_CR_BLOCK_REQ
                                            - PNIO_FAULTY_PRM_SERVER_LOCK_REQ
                                            IODControl Parameter Error
                                            - PNIO_FAULTY_CONTROL_BLOCK_CONNECT
                                       */

/*in*/  USIGN8            byErrorCode2;
                                       /*!< The parameter Error Code 2 is user specific for byErrorDecode = PNIORW for byErrorDecode = PNIO byErrorCode2 gives the parameter number */
/*in*/  USIGN16           wAddData1;   
                                       /*!< The parameter Add Data is API specific (profile) */
/*in*/  USIGN16           wAddData2;   
                                       /*!< The parameter Add Data is user specific */
/*in*/  USIGN16           wIndex;      
                                       /*!< The range from zero to 0x7FFF shall be used to address user specific record data objects.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF user specific
                                            - 0x8000 to 0xBFFF subslot specific
                                            - 0xC000 to 0xDFFF slot specific
                                            - 0xE000 to 0xEFFF AR specific
                                            - 0xF000 to 0xF7FF API specific
                                            - 0xF800 to 0xFFFF device specific
                                        */
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< Transaction number is the unique identifier of one write request.
                                            This identifier is used by the application to identify the
                                            write confirmation to the write request.
                                       */
} T_WRITE_CONF_PARAM;

/*! This structure comprises the properties of one Submodule. */
typedef struct {
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This attribute defines to which slot the object belongs.

                                            Allowed values:
                                            - 0x0001 to 0x7FFF
                                        */
/*in*/  USIGN32           dwSubmoduleIdentNum;
                                       /*!< This attribute defines the identification of the Submodule.

                                            Allowed values:
                                            - 0x00000000 to 0xFFFFFFFF
                                        */
/*in*/  USIGN16           wSubmoduleState;
                                       /*!< The field SubmoduleState is only responded if there is a difference between expected and real configuration data.

                                            Allowed values if the format indicator is set to 1 (MSB = 1):
                                            - SUBMODULE_STATE_ADD_INFO_NONE
                                            - SUBMODULE_STATE_ADD_INFO_TAKEOVER_NOT_ALLOWED
                                            - SUBMODULE_STATE_QUALIFIED_INFO_AVAILABLE
                                            - SUBMODULE_STATE_ MAINTENANCE_REQUIRED_AVAILABLE
                                            - SUBMODULE_STATE_ MAINTENANCE_DEMANDED_AVAILABLE
                                            - SUBMODULE_STATE_ DIAGNOSIS_DATA_AVAILABLE
                                            - SUBMODULE_STATE_AR_INFO_OWN
                                            - SUBMODULE_STATE_AR_INFO_APPLICATION_READY_PENDING
                                            - SUBMODULE_STATE_AR_INFO_SUPERORDINATION_LOCKED
                                            - SUBMODULE_STATE_AR_INFO_LOCKED_BY_IO_CONTROLLER
                                            - SUBMODULE_STATE_AR_INFO_LOCKED_BY_IO_SUPERVISOR
                                            - SUBMODULE_STATE_IDENT_INFO_OK
                                            - SUBMODULE_STATE_IDENT_INFO_SUBSTITUTE
                                            - SUBMODULE_STATE_IDENT_INFO_WRONG
                                            - SUBMODULE_STATE_IDENT_INFO_NO_SUBMODULE

                                            Allowed values if the format indicator is set to 0 (MSB = 0):
                                            - SUBMODULE_STATE_DETAIL_NO_SUBMODULE
                                            - SUBMODULE_STATE_DETAIL_WRONG_SUBMODULE
                                            - SUBMODULE_STATE_DETAIL_LOCKED_BY_IO_CONTROLLER
                                            - SUBMODULE_STATE_DETAIL_APPLICATION_READY_PENDING
                                            - SUBMODULE_STATE_DETAIL_SUBSTITUTE
                                       */
} T_LIST_OF_SUBMODULES;


/*! This structure comprises the properties of one Module. */
typedef struct {
/*in*/  USIGN16           wSlotNumber;
                                        /*!< This attribute defines to which slot the object belongs.

                                             Allowed values:
                                             - 0 to 0x7FFF
                                        */
/*in*/  USIGN32           dwModuleIdentNumber;
                                        /*!< This parameter contains the value of the attribute Module Ident Number. */
/*in*/  USIGN16           wModuleState;
                                        /*!< State of the Module

                                             Allowed Values:
                                             - MODULE_STATE_NO_MODULE = i.e. the module is not plugged
                                             - MODULE_STATE_WRONG_MODULE = i.e. the module ident number is wrong
                                             - MODULE_STATE_PROPER_MODULE = i.e. the module is okay but at least one submodule is locked, wrong or missing
                                             - MODULE_STATE_SUBSTITUTE = i.e. the module is not the same as requested but compatible. The IO device was able to adapt by its own decision.
                                         */
/*in*/  USIGN16           wNumberOfSubmodules;
                                       /*!< This attribute defines the lower limit of the subslot number of the slot. */
/*in*/  T_LIST_OF_SUBMODULES *poListOfSubmodules;
                                       /*!< Pointer to the list which contains the properties of the Submodules. */

} T_LIST_OF_MODULES;


/*! This structure comprises the parameter for one API for values which differ from the requested ones. */
typedef struct {
/*in*/  USIGN32           dwAPI;
                                       /*!< API Identifier */
/*in*/  USIGN16           wNumberOfModules;
                                       /*!< This parameter contains the number of the following modules which differ from the requested configuration.
                                            If there is no difference between expected and really configuration, the wNumberOfModules is zero.
                                       */
/*in*/  T_LIST_OF_MODULES *poListOfModules;
                                       /*!< Pointer to the list of structures which content the properties of the Modules */
} T_RELATED_API_MODULE_DIFF;


/*! This structure comprises the parameter for values which differ from the requested ones. */
typedef struct {
/*in*/  USIGN16 wNumberOfRelatedAPIs;
                                       /*!< Number of related APIs */
/*in*/  T_RELATED_API_MODULE_DIFF *poRelatedAPI;
                                       /*!< Pointer to the list of structures which content parameter of the APIs */
} T_MODULE_DIFF_BLOCK;


/*! This structure comprises the parameter for the Application Ready indication. */
typedef struct {
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR

                                            Allowed Values:
                                            - 0x0000 to 0xFFFF
                                        */
/*in*/  USIGN16           wARUserID;
                                       /*!< User specific identifier of this AR */
/*in*/  USIGN8            byStationNameLen;
                                       /*!< This attribute defines the length of the Station Name */
/*in*/  CHAR              *pcIODeviceStationName;
                                       /*!< This attribute is a pointer to the name of the Station */
/*in*/  USIGN16           wAlarmSeqNum;
                                       /*!< Alarm Sequence Number. */
/*in*/  T_MODULE_DIFF_BLOCK *poModuleDiffBlock;
                                       /*!< The parameter consists of the sub-parameter for values which differ from the requested ones */
/*in*/  USIGN8            byErrorCode; 
                                       /*!< This parameter selects one Error Code */
/*in*/  USIGN8            byErrorDecode;
                                       /*!< This parameter selects one Error scheme for Error Code 1 and 2 according IEC 61158 Type 3 specification */
/*in*/  USIGN8            byErrorCode1;
                                       /*!< The Error Code 1 assumes one of the following values: 
                                            PNIO_STATUS_NO_ERROR
                                            
                                            PNIORW Errors
                                            Application
                                            - PNIORW_WRITE_ERROR
                                            - PNIORW_MODULE_FAILURE
                                            - PNIORW_BUSY
                                            - PNIORW_VERSION_CONFLICT
                                            - PNIORW_FEATURE_NOT_SUPPORTED
                                            Access
                                            - PNIORW_INVALID_INDEX
                                            - PNIORW_WRITE_LENGTH_ERROR
                                            - PNIORW_INVALID_SLOT_SUBSLOT
                                            - PNIORW_TYPE_CONFLICT
                                            - PNIORW_INVALID_AREA
                                            - PNIORW_STATE_CONFLICT
                                            - PNIORW_ACCESS_DENIED
                                            - PNIORW_INVALID_RANGE
                                            - PNIORW_INVALID_PARAMETER
                                            - PNIORW_INVALID_TYPE
                                            - PNIORW_BACKUP
                                            Resource
                                            - PNIORW_WRITE_CONSTRAIN_CONFLICT
                                            - PNIORW_RESOURCE_BUSY
                                            - PNIORW_RESOURCE_UNAVAILABLE
                                            
                                            PNIO Errors
                                            Connect Parameter Error
                                            - PNIO_FAULTY_ AR_BLOCK_REQ
                                            - PNIO_FAULTY_IOCR_BLOCK_REQ
                                            - PNIO_FAULTY_EXP_SUBMODULE_BLOCK_REQ
                                            - PNIO_FAULTY_ALARM_CR_BLOCK_REQ
                                            - PNIO_FAULTY_PRM_SERVER_LOCK_REQ
                                            IODControl Parameter Error
                                            - PNIO_FAULTY_CONTROL_BLOCK_CONNECT
                                       */

/*in*/  USIGN8            byErrorCode2;
                                       /*!< The parameter Error Code 2 is user specific for byErrorDecode = PNIORW for byErrorDecode = PNIO byErrorCode2 gives the parameter number */
/*in*/  USIGN16           wAddData1;   
                                       /*!< The parameter Add Data is API specific (profile) */
/*in*/  USIGN16           wAddData2;   
                                       /*!< The parameter Add Data is user specific */

} T_APP_READY_IND_PARAM;

/*! This structure contains the Alarm Notification Parameter. */
typedef struct {
       USIGN16        wBlockType;
                                       /*!< Type of the Alarm CR Block */
       USIGN16        wBlockVersion;
                                       /*!< Version of the Alarm CR Block

                                            - High Byte: Block Version High
                                            - Low Byte: Block Version Low
                                        */
       USIGN16        wBlockLength;
                                       /*!< Length of the Alarm CR Block */
       USIGN16        wAlarmType;
                                       /*!< Type of the Alarm

                                            Allowed Values:
                                            - ALARM_TYPE_DIAG_APPEARS
                                            - ALARM_TYPE_PROCESS
                                            - ALARM_TYPE_PULL
                                            - ALARM_TYPE_PLUG
                                            - ALARM_TYPE_STATUS
                                            - ALARM_TYPE_UPDATE
                                            - ALARM_TYPE_REDUNDANCY
                                            - ALARM_TYPE_CONTR_BY_SUPERVISOR
                                            - ALARM_TYPE_RELEASED
                                            - ALARM_TYPE_PLUG_WRONG_SUBMODULE
                                            - ALARM_TYPE_RETURN_OF_SUBMODULE
                                            - ALARM_TYPE_DIAG_DISAPPEARS
                                            - ALARM_TYPE_MC_COMM_MISMATCH
                                            - ALARM_TYPE_PORT_DATA_CHANGED_NOTIF
                                            - ALARM_TYPE_SYNC_DATA_CHANGED_NOTIF
                                            - ALARM_TYPE_ISO_MODE_PROBLEM_NOTIF
                                        */
       USIGN32        dwAPI;
                                        /*!< API Identifier */
       USIGN16        wSlotNumber;
                                        /*!< This parameter contains the value of the attribute Slot Number.

                                             Allowed values:
                                             - 0x0001 to 0xFFFF
                                         */
       USIGN16        wSubslotNumber;
                                        /*!< This attribute defines to which slot the object belongs.

                                             Allowed values:
                                             - 0x0001 to 0xFFFF
                                         */
       USIGN32        dwModuleIdentNumber;
                                        /*!< This parameter contains the module ident number.

                                             Allowed values: >0
                                         */
       USIGN32        dwSubmoduleIdentNumber;
                                        /*!< This parameter contains the submodule ident number

                                             Allowed values: >0
                                         */
       USIGN16        wAlarmSpecifier;
                                        /*!< Alarm Specifier is needed to identify the acknowledge by the sender of this alarm. */
       USIGN16        wLengthOfAlarmItem;
                                        /*!< Length of the AlarmItem in bytes */
       USIGN8         *pbyAlarmItem;
                                        /*!< Pointer to AlarmItem.
                                             The structure of the AlarmItem depends on the UserStructureIdentifier (USI).
                                             The first two bytes of the AlarmItem contain the USI in BigEndian format.
                                         */
} T_PNIO_IOAR_ALARM_NOTIFICATION;

/*! This structure comprised the attributs of the Alarm Notification */
typedef struct {
       USIGN16        wAlarmType;
                                       /*!< Type of the Alarm

                                            Allowed Values:
                                            - ALARM_TYPE_DIAG_APPEARS
                                            - ALARM_TYPE_PROCESS
                                            - ALARM_TYPE_PULL
                                            - ALARM_TYPE_PLUG
                                            - ALARM_TYPE_STATUS
                                            - ALARM_TYPE_UPDATE
                                            - ALARM_TYPE_REDUNDANCY
                                            - ALARM_TYPE_CONTR_BY_SUPERVISOR
                                            - ALARM_TYPE_RELEASED
                                            - ALARM_TYPE_PLUG_WRONG_SUBMODULE
                                            - ALARM_TYPE_RETURN_OF_SUBMODULE
                                            - ALARM_TYPE_DIAG_DISAPPEARS
                                            - ALARM_TYPE_MC_COMM_MISMATCH
                                            - ALARM_TYPE_PORT_DATA_CHANGED_NOTIF
                                            - ALARM_TYPE_SYNC_DATA_CHANGED_NOTIF
                                            - ALARM_TYPE_ISO_MODE_PROBLEM_NOTIF
                                        */
       USIGN32        dwAPI;
                                        /*!< API Identifier */
       USIGN16        wSlotNumber;
                                        /*!< This parameter contains the value of the attribute Slot Number.

                                             Allowed values:
                                             - 0x0000 to 0xFFFF
                                         */
       USIGN16        wSubslotNumber;
                                        /*!< This attribute defines to which slot the object belongs.

                                             Allowed values:
                                             - 0x0001 to 0xFFFF
                                         */
       USIGN16        wAlarmSpecifier;
                                        /*!< Set the Alarm Specifier to the value which was received for this alarm. */
       USIGN8         byErrorCode;
                                        /*!< This parameter selects one Error Code

                                             Allowed Values:
                                             - 0x00 = No error
                                             - 0xDA = AlarmAcknowledge
                                         */
       USIGN8         byErrorDecode;
                                        /*!< This parameter selects one Error scheme for Error Code 1 and 2 according IEC 61158 Type 3 specification.

                                             Allowed Value:
                                             - 0x00 = No Error
                                             - 0x81 = PNIO
                                         */
       USIGN8         byErrorCode1;
                                        /*!< The Error Code 1 assumes one of the following values:

                                             - 0x00 = No Error
                                             - 0x3C = AlarmAck Error Codes
                                         */
       USIGN8         byErrorCode2;
                                        /*!< The Error Code 1 assumes one of the following values:

                                                - 0x00 = AlarmType not supported
                                                - 0x01 = Wrong Submodule State
                                                - 0x02 = IOCARSR Backup - Alarm not executed
                                         */
} T_PNIO_IOAR_ALARM_ACK;

/*! This structure comprised the input parameter of one IOAR IOxS list element */
typedef struct
{
/*in*/ USIGN16                       wProvID;
                                        /*!< This parameter contains the unique provider ID for this list element

                                             Allowed values:
                                             - 0x0001 to 0xFFFF
                                         */
/*in*/ USIGN16                       wOffset;
                                        /*!< This parameter defines the offset of the provider state within the ethernet frame of the requested provider. */
/*in*/ USIGN8                        byIOxSDataState;
                                        /*!< This parameter contains the Consumer Status (IOCS) or Provider Status (IOPS) for one Slot/Subslot combination. */
} T_PNIO_IOAR_Element_IOXS_LIST;

/*! This structure contains the data of one current configured subslot of the PROFINET Device. */
typedef struct _CTM_CMDEV_SUBSLOT
{
    USIGN16          wSubslotNumber;
                                       /*!< This attribute contains the number of the current subslot. */
    USIGN16          wInDataLength;
                                       /*!< This attribute contains the input data length of the current subslot. */
    USIGN16          wOutDataLength;
                                       /*!< This attribute contains the output data length of the current subslot. */
    USIGN32          dwSubmoduleId;
                                       /*!< This attribute contains the submodule ID of the current subslot. */
}T_CTM_CMDEV_SUBSLOT;

/*! This structure contains the data of one current configured slot of the PROFINET Device. */
typedef struct _CTM_CMDEV_SLOT
{
    USIGN16             wSlotNumber;
                                       /*!< This attribute contains the number of the current slot. */
    USIGN16             wNumberOfSubslots;
                                       /*!< This attribute contains the amount of subslots assigned to this slot. */
    USIGN32             dwModuleId;
                                       /*!< This attribute contains the module ID of the current slot. */
    T_CTM_CMDEV_SUBSLOT *poSubslotList;
                                       /*!< Pointer to a list of structures which content the currently configurd subslots (submodules) of the current slot. */
}T_CTM_CMDEV_SLOT;

/*! This structure contains the data of the current configuration of the PROFINET Device. */
typedef struct _CTM_CMDEV_CONFIG
{
    USIGN16          wInstanceID;
                                       /*!< Unique identifier of this device instance */
    USIGN16          wARCountMax;
                                       /*!< Maximal number of device ARs which can be established by an external PROFINET controller */
    USIGN16          wMaxSlots;
                                       /*!< Maximum amount of slots (modules) that are supported by the device. */
    USIGN16          wMaxSubslots;
                                       /*!< Maximum amount of subslots (submodules) that are supported by the device. */
    USIGN32          dwInterfaceSubmoduleId;
                                       /*!< This attribute contains the ID of the interface submodule. */
    USIGN32          dwPort1SubmoduleId;
                                       /*!< This attribute contains the ID of the port 1 submodule. */
    USIGN32          dwPort2SubmoduleId;
                                       /*!< This attribute contains the ID of the port 2 submodule.

                                            \note This parameter is only needed in case of the simulation of a 2 port device.
                                        */
    USIGN16          wNumberOfSlots;
                                       /*!< Current amount of slots (modules) as configured by the configuration file. */
    USIGN16          wNumberOfSubslots;
                                       /*!< Current amount of subslots (submodules) as configured by the configuration file. */
    T_CTM_CMDEV_SLOT *poSlotList;
                                       /*!< Pointer to a list of structures which content the currently configurd slots (modules). */
    }T_CTM_CMDEV_CONFIG;

/*! Contains the properties. */
typedef struct
{
  USIGN16  wType;               
                                        /*!< This attribute defines the type of the subslot

                                             Allowed values:
                                             - PNIO_CMDEV_NO_IO
                                             - PNIO_CMDEV_INPUT
                                             - PNIO_CMDEV_OUTPUT
                                             - PNIO_CMDEV_IO
                                         */
  USIGN8   bySharedInput;       
                                        /*!< Allowed values:
                                             - PNIO_CMDEV_IOC: Can be used together with all possible values of SubmoduleProperties.Type
                                             - PNIO_CMDEV_IOC_SHARED: Only the shared IO controller shall use this value. The shared IO controller does not receive alarms and is restricted to read access.
                                             
                                             \note  The value PNIO_CMDEV_IOC_SHARED shall not be used together with wType == PNIO_CMDEV_OUTPUT.
                                                    In this cases the value of this attribute will be ignored.
                                         */
  USIGN8   byRedInpSubmodLength;
                                        /*!< Allowed values:
                                             - PNIO_CMDEV_EXPECTED: Use expected input SubmoduleDataLength for I-CR
                                             - PNIO_CMDEV_ZERO: Reduce input SubmoduleDataLength to zero for I-CR
                                         */
  USIGN8   byRedOutSubmodLength;
                                        /*!< Allowed values:
                                             - PNIO_CMDEV_EXPECTED: Use expected output SubmoduleDatalength for O-CR
                                             - PNIO_CMDEV_ZERO: Reduce output SubmoduleDatalength to zero for O-CR
                                         */
  USIGN8   byDiscardIOXS;       
                                        /*!< Allowed values:
                                             - PNIO_CMDEV_EXPECTED: Use expected output SubmoduleDatalength for O-CR
                                             - PNIO_CMDEV_ZERO: Reduce output SubmoduleDatalength to zero for O-CR
                                         */
} T_CMDEV_SUBMOD_PROP_DESC;

/*! Contains the description of the elements for one submodule. */
typedef struct
{
/*in*/   USIGN8   byElementType;    /*!< Type of element inthe Data Field. Allowed values: CMDEV_DATA, CMDEV_IOPS, CMDEV_IOCS */
/*in*/   USIGN16  wIOCRID;          /*!< Unique identifier of one IOCR. The value is identical to the respective provider or consumer ID. */
/*in*/   USIGN16  wIOCRBufferLength;/*!< Data Length of IOCR Telegram, needed for Lock and Unlock Functions from App */
/*in*/   USIGN16  wSendClockFactor; /*!< Send Interval. The time base is 31,25us. */
/*in*/   USIGN16  wReductionRatio;  /*!< Reduction Ratio. */
/*in*/   USIGN16  wDataDesc;        /*!< Description of the data for this Submodule. Allowed values: CMDEV_DATA_INPUT, CMDEV_DATA_OUTPUT */
/*in*/   USIGN16  wFrameOffset;     /*!< Defines the offset within the frame of this data object. Allowed values: 0 = 1439 */
/*in*/   USIGN16  wSubModEleLength; /*!< Length of this sub module element in byte. Allowed values: 0 = 1439 */
} T_CMDEV_SUBMOD_ELEM_DESC;

/*! Contains description of one submodule. */
typedef struct
{
/*in*/   USIGN16                    wSubslotNumber;      /*!< Slot number. Allowed values: 1 = 0x7FFF */
/*in*/   USIGN32                    dwSubmodIdentNum;    /*!< Identification of the subslot. Allowed Values: 0 = 0xFFFFFFFF */
/*in*/   T_CMDEV_SUBMOD_PROP_DESC   oSubmodProps;        /*!< Properties of this submodule */
/*in*/   USIGN16                    wNumOfElements;      /*!< Number of Elements for this submodule */
/*in*/   T_CMDEV_SUBMOD_ELEM_DESC   *poSubmodElemDesc;   /*!< Pointer to array with wNumOfElements elements; contains structures with detailed descriptions for every element of this submodule. */
} T_CMDEV_SUBMOD_DESC;


/*! Parameter for the related API of submodules. */
typedef struct
{
/*in*/   USIGN32              dwAPI;            /*!< API identifier */
/*in*/   USIGN16              wSlotNumber;      /*!< Slot Number */
/*in*/   USIGN32              dwModIdentNumber; /*!< Module ident number */
/*in*/   USIGN16              wModProps;        /*!< Module properties. Reserved for future use; shall always be set to zero */
/*in*/   USIGN16              wNumOfSubmods;    /*!< Lower limit of the subslot number of the slot. */
/*in*/   T_CMDEV_SUBMOD_DESC  *poSubmodDesc;    /*!< Pointer to array with wNumOfSubmods elements that contains structures with detailed descriptions for Submodules */
} T_CMDEV_RELATED_API_SUBMOD;

/*! Parameter of one expected module. */
typedef struct
{
/*in*/   USIGN16                    wNumOfRelatedAPIs;/*!< Number of related APIs. */
/*in*/   T_CMDEV_RELATED_API_SUBMOD *poRelatedAPI;    /*!< Pointer to array with wNumOfRelatedAPIs elements that contains structures with parameters of the API */
} T_CMDEV_EXP_MOD_REQ;

/*! Parameter taken from the connect service. */
typedef struct
{
/*in*/   USIGN16              wARID;            /*!< Contains the unique local identifier of the AR
                                                     generated by the IO stack.
                                                     Allowed values: 0x0001 - 0xFFFF */
/*in*/   USIGN16              wNumOfProvIDs;    /*!< Contains the number of provider IDs within the AR
                                                     taken from the connect request of an external controller.
                                                     Allowed values: 0x0001 - 0xFFFF */
/*in*/   USIGN16              *pwProvIDList;    /*!< Contains the unique provider ID list of the AR
                                                     taken from the connect request of an external controller.
                                                     The values are identical to the respective IOCR ID.
                                                     Allowed values: 0x0001 - 0xFFFF */
/*in*/   USIGN16              wNumOfConsIDs;    /*!< Contains the number of consumer IDs within the AR
                                                     taken from the connect request of an external controller.
                                                     Allowed values: 0x0001 - 0xFFFF */
/*in*/   USIGN16              *pwConsIDList;    /*!< Contains the unique consumer ID list of the AR
                                                     taken from the connect request of an external controller.
                                                     The values are identical to the respective IOCR ID.
                                                     Allowed values: 0x0001 - 0xFFFF */
/*in*/   USIGN8               pbyCMInitMacAddr[MAC_ADDR_SIZE];
                                                /*!< Contains the MAC address of the initiator */
/*in*/   USIGN8               pbyCMInitIPAddr[IP_ADDR_SIZE];
                                                /*!< Contains the IP address of the initiator */
/*in*/   USIGN16              wCMInitNameLength;
                                                /*!< Contains the Station name length of the initiator */
/*in*/   USIGN8               pbyCMInitName[MAX_STATION_NAME_SIZE];
                                                /*!< Contains the Station name of the initiator */
/*in*/   USIGN16              wNumOfExpMods;    /*!< Contains the number of expected module blocks in this AR. */
/*in*/   T_CMDEV_EXP_MOD_REQ  *poExpModReq;     /*!< Pointer to array with wNumOfExpMods;
                                                     contains structures with parameters to the expected modules. */
/*in*/   USIGN16              wNumOfValidMods;  /*!< Contains the number of valid module blocks in this AR. */
/*in*/   T_CMDEV_EXP_MOD_REQ  *poValidMods;     /*!< Pointer to array with valid module blocks;
                                                     contains structures with parameters of valid modules. */
} T_CMDEV_AR_BLOCK_REQ;

/*! This structure comprises the parameters for the Application Ready Indication */
typedef struct
{
/*in*/  USIGN16           wInstanceID; /*!< Local identifier of the releated IO device instance */
/*in*/  USIGN16           wARID;       /*!< Unique identifier of the AR
                                            Allowed values:
                                            0x0001 to 0xFFFF */
/*in*/  USIGN16           wLocalAlarmReference;
                                       /*!< Unique identifier of the local alarm reference */
/*in*/  USIGN16           wMaxAlarmDataLength;
                                       /*!< This attribute contains the length
                                            of the alarm data.
                                            Allowed value:
                                            200 to 1432 */
/*in*/  T_MODULE_DIFF_BLOCK *poModuleDiffBlock;
                                       /*!< The parameter consists of the sub-parameter
                                            for values which differ from the requested one's.
                                            */
} T_CMDEV_APP_READY_IND_PARAM;

/*! This structure contains the the elements of the PNIO error state. */
typedef struct
{
/*in*/   USIGN8               byErrorCode;
                                        /*!< This parameter contains the PNIO ErrorCode */
/*in*/   USIGN8               byErrorDecode;
                                        /*!< This parameter contains the PNIO ErrorDecode scheme for Error Code 1 and 2 according IEC 61158 Type 3 specification. */
/*in*/   USIGN8               byErrorCode1;
                                        /*!< This parameter contains the PNIO ErrorCode1  */
/*in*/   USIGN8               byErrorCode2;
                                        /*!< This parameter contains the PNIO ErrorCode2 */
/*in*/   USIGN16              wAddData1;
                                        /*!< This parameter contains additional user specific data. */
/*in*/   USIGN16              wAddData2;
                                        /*!< This parameter contains additional user specific data. */
} T_PNIO_ERROR_STATE;

/*! This structure contains the data of an incoming read record request. */
typedef struct
{
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< Transaction number is the unique identifier of this read record service.
                                            This identifier is used by the external PROFINET Controller to identify the corresponding read record confirmation.
                                        */
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR

                                            Allowed values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN32           dwAPI;
                                       /*!< This parameter defines the API. */
/*in*/  USIGN16           wSlotNumber;
                                       /*!< This parameter defines the slot. */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This parameter defines the subslot. */
/*in*/  USIGN16           wIndex;
                                       /*!< This parameter defines the index of the specific record data object.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF
                                        */
/*in*/  USIGN32           dwRecDataLength;
                                       /*!< This parameter indicates the number of octets of the specific record data object.

                                            Allowed values:
                                            - 0 to 2^32-65.
                                        */
} T_REC_READ_IND;

/*! This structure contains the data of an incoming write record request. */
typedef struct
{
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< Transaction number is the unique identifier of this read record service.
                                            This identifier is used by the external PROFINET Controller to identify the corresponding read record confirmation.
                                        */
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR

                                            Allowed values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN32           dwAPI;
                                       /*!< This parameter defines the API. */
/*in*/  USIGN16           wSlotNumber;
                                       /*!< This parameter defines the slot. */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This parameter defines the subslot. */
/*in*/  USIGN16           wIndex;
                                       /*!< This parameter defines the index of the specific record data object.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF
                                        */
/*in*/  USIGN32           dwDataLength;
                                       /*!< This parameter indicates the number of octets of the specific record data object.

                                            Allowed values:
                                            - 0 to 2^32-65.
                                        */
/*in*/  USIGN8            *pbyData;    
                                        /*!< This parameter contains the data of the specific record data object. */
/*in*/  T_PNIO_ERROR_STATE *poErrorState;
                                        /*!< This parameter contains the PNIO error information. */

} T_REC_WRITE_IND;

/*! This structure contains the data of the read record response. */
typedef struct
{
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< Transaction number is the unique identifier of this read record service.
                                            This identifier is used by the external PROFINET Controller to identify
                                            the corresponding write record confirmation. */
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR

                                            Allowed values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN32           dwAPI;
                                       /*!< This parameter defines the API. */
/*in*/  USIGN16           wSlotNumber;
                                       /*!< This parameter defines the slot. */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This parameter defines the subslot. */
/*in*/  USIGN16           wIndex;
                                       /*!< This parameter defines the index of the specific record data object.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF
                                        */
/*in*/  T_PNIO_ERROR_STATE oErrorState;
                                       /*!< This parameter contains the PNIO error information. */

} T_REC_WRITE_RSP;

/*! This structure contains the data of the read record response of the PROFINET Device. */
typedef struct
{
/*in*/  USIGN32           dwTransactionNum;
                                       /*!< Transaction number is the unique identifier of this read record service.
                                            This identifier is used by the external PROFINET Controller to identify
                                            the corresponding read record confirmation. */
/*in*/  USIGN16           wARID;
                                       /*!< Unique identifier of the AR

                                            Allowed values:
                                            - 0x0001 to 0xFFFF
                                        */
/*in*/  USIGN32           dwAPI;
                                       /*!< This parameter defines the API. */
/*in*/  USIGN16           wSlotNumber;
                                       /*!< This parameter defines the slot. */
/*in*/  USIGN16           wSubslotNumber;
                                       /*!< This parameter defines the subslot. */
/*in*/  USIGN16           wIndex;
                                       /*!< This parameter defines the index of the specific record data object.

                                            Allowed values:
                                            - 0x0000 to 0x7FFF
                                        */
/*in*/  T_PNIO_ERROR_STATE oErrorState;
                                       /*!< This parameter contains the PNIO error information. */

/*in*/  USIGN32           dwDataLength;
                                       /*!< This parameter indicates the number of octets of the specific record data object.

                                            Allowed values:
                                            - 0 to 2^32-65
                                        */
/*in*/  USIGN8            *pbyData;
                                       /*!< This parameter is an array with dwDataLength elements which contains the data of the specific record data object. */
} T_REC_READ_RSP;

/*! This structure comprises the parameters for manufacturer specific dcp requests. */
typedef struct
{
    USIGN8  byService;
                                       /*!< This parameter contains the DCP service ID

                                            Allowed values:
                                            - DCP_GET
                                            - DCP_SET
                                        */
    USIGN8  pbyManufacturerOUI[OUI_SIZE];
                                       /*!< This field shall be coded as OctetString[3] with the Organizationally unique identifier (OUI) as defined by IEEE 802. */
    USIGN16 wManufacturerSpecificStringLength;
                                       /*!< This parameter contains the length of the manufacturer specific string. */
    USIGN8  *pbyManufacturerSpecificString;
                                       /*!< Pointer to manufacturer specific string.
                                            This field shall be coded as data type OctetString and shall contain the values of the related option and suboptions.
                                        */
} T_DCP_ARP_MANUFACTURER_INFO;

/*! This structure contains the data of external DCP requests towards the PROFINET stack. */
typedef struct
{
/*in*/  USIGN8  byOption;
                                       /*!< Requested option of the DCP request.

                                            Possible values:
                                            - DCP_DEVICE_PROP_OPTIONS
                                            - DCP_TCP_IP_OPTIONS
                                            - DCP_CONTROL
                                        */
/*in*/  USIGN8  bySuboption;
                                       /*!< Requested suboption of the DCP request.

                                            Possible value for option DCP_DEVICE_PROP_OPTIONS:
                                            - DCP_DEV_PROP_NAME_OF_STATION

                                            Possible value in case of  option DCP_TCP_IP_OPTION:
                                            - DCP_TCP_IP_PARAMETER

                                            Possible values in case of  option DCP_CONTROL:
                                            - DCP_CONTROL_SIGNAL
                                            - DCP_CONTROL_FACTORY_RESET
                                            - DCP_CONTROL_START_TRANS
                                            - DCP_CONTROL_END_TRANS
                                        */
/*in*/  USIGN16 wDcpStatus;
                                       /*!< Requested DCP status (block qualifier) of the DCP request.

                                            Possible values:
                                            - DCP_STORE_VOLATILE
                                            - DCP_STORE_NON_VOLATILE
                                        */
/*in*/  USIGN8  pbyIPAddr[IP_ADDR_SIZE];
                                       /*!< Requested IP address of the DCP request.

                                            \note The IP address is only valid in case of option/suboption set to DCP_TCP_IP_OPTION/DCP_TCP_IP_PARAMETER.
                                        */
/*in*/  USIGN8  pbySubnetMask[IP_ADDR_SIZE];
                                       /*!< Requested subnet mask of the DCP request.

                                            \note The subnet mask is only valid in case of option/suboption set to DCP_TCP_IP_OPTION/DCP_TCP_IP_PARAMETER.
                                        */
/*in*/  USIGN8  pbyStdGateway[IP_ADDR_SIZE];
                                       /*!< Requested standard gateway of the DCP request.

                                            \note The standard gateway is only valid in case of option/suboption set to DCP_TCP_IP_OPTION/DCP_TCP_IP_PARAMETER.
                                        */
/*in*/  USIGN8  byStationNameLength;
                                       /*!< Station name length of the requested station name of the DCP request.

                                            \note The station name length is only valid in case of option/suboption set to DCP_DEVICE_PROP_OPTIONS/DCP_DEV_PROP_NAME_OF_STATION.
                                        */
/*in*/  CHAR    pcStationName[MAX_STATION_NAME_SIZE];
                                       /*!< Requested station name of the DCP request.

                                            \note The station name is only valid in case of option/suboption set to DCP_DEVICE_PROP_OPTIONS / DCP_DEV_PROP_NAME_OF_STATION.
                                        */
/*in*/  T_DCP_ARP_MANUFACTURER_INFO* poManufacturerInfo;
                                       /*!< The Manufacturer specific data. May be NULL if none given. */
} T_DCP_REQUEST;

/*! This structure contains information about the target in case of a DCP request.
    The elements must be filled with regard to service that shall be sent. */
typedef struct
{
   USIGN8      pbyMACAddr[MAC_ADDR_SIZE];
                                       /*!< MAC address of the target. This is a pointer to a buffer with length MAC_ADDR_SIZE */
   USIGN8      pbyIPAddr[IP_ADDR_SIZE];
                                       /*!< IP address of the target. This is a pointer to a buffer with length IP_ADDR_SIZE */
   USIGN8      pbySubnetMask[IP_ADDR_SIZE];
                                       /*!< Subnet mask of the target. This is a pointer to a buffer with length IP_ADDR_SIZE */
   USIGN8      pbyStdGateway[IP_ADDR_SIZE];
                                       /*!< IP address of the StandardGateway. This is a pointer to a buffer with length IP_ADDR_SIZE */
   USIGN16     wBlockQualifier;
                                       /*!< This attribute influences the behaviour of DCP requests

                                            BlockQualifier with options IPOption, DevicePropertiesOption, DHCPOption and ManufacturerSpecificOption:
                                            - Bit 0 = 0:   Store the value of the option/suboption and its attribute "temporary" in the DCP ASE and use the value subsequently.
                                                           The attribute temporary means that the value shall be reset to its default value after a power off / power on cycle.
                                            - Bit 0 = 1:   Store the value of the option/suboption and its attribute "permanent" in the DCP ASE and use the value subsequently.
                                                           The attribute permanent means that the value shall be available after a power off / power on cycle.
                                            - Bit 1 to 15: Should be set to zero by the sender and shall not be checked by the receiver.

                                            BlockQualifier with option ControlOption and suboption SuboptionResetToFactory:
                                            - Bit 0:            Should be set to zero by the sender and shall not be checked by the receiver.
                                            - Bit 1 to 15 = 0:  Reserved
                                            - Bit 1 to 15 = 1:  (Recommended)
                                                                Reset application data
                                                                Reset data which have been stored permanent in submodules and modules to factory values.
                                            - Bit 1 to 15 = 2:  (Mandatory)
                                                                Reset communication parameter
                                                                All parameters active for the interface or the ports and the ARs shall be set to the default values and if permanently stored reset.
                                                                This mode is intended to set the addressed communication interface of a device into a state which almost similar to the "out of the box" state.
                                                                It includes parameters adjusted by SNMP, like sysContact, sysName, and sysLocation from MIB-II.
                                            - Bit 1 to 15 = 3:  (Recommended)
                                                                Reset engineering parameter
                                                                Reset engineering parameters which have been stored permanent in the IOC or IOD to factory values.
                                            - Bit 1 to 15 = 4:  (Recommended)
                                                                Reset all stored data
                                                                Reset all stored data in the IOD or IOC to its factory default values.
                                            - Bit 1 to 15 = 5:  Reserved
                                            - Bit 1 to 15 = 6:  Reserved
                                            - Bit 1 to 15 = 7:  Reserved
                                            - Bit 1 to 15 = 8:  (Recommended)
                                                                Reset all stored data in the IOD or IOC to its factory values.
                                                                This service shall reset the communication parameters of all interfaces of the device and should reset all parameters of the device.
                                                                This mode is intended to set the device into a state which is similar to the "out of the box" state.
                                                                It includes parameters adjusted by SNMP, like sysContact, sysName, and sysLocation from MIB-II.
                                            - Bit 1 to 15 = 9:  (Optional)
                                                                Reset and restore data
                                                                Reset installed software revisions to factory images.
                                            - Bit 1 to 15 = 10: Reserved
                                            - Bit 1 to 15 = 11: Reserved
                                            - Bit 1 to 15 = 12: Reserved
                                            - Bit 1 to 15 = 13: Reserved
                                            - Bit 1 to 15 = 14: Reserved
                                            - Bit 1 to 15 = 15: Reserved

                                             BlockQualifier with option ControlOption and NOT suboption SuboptionResetToFactory:
                                            - Bit 0 to 15: Should be set to zero by the sender and shall not be checked by the receiver.
                                        */
   USIGN8      byNameLength;
                                       /*!< This attribute defines the length of the station name */
   CHAR        pcName[MAX_STATION_NAME_SIZE];
                                       /*!< This attribute is a pointer to the name of the station. Is a buffer with maximum size of MAX_STATION_NAME_SIZE */
} T_DCP_ARP_TARGET_INFO;

/*!  This structure comprises the version information. */
typedef struct {
       USIGN16        wMajor;                  /*!< Current major version number */
       USIGN16        wMinor;                  /*!< Current minor version number */
       USIGN16        wRevision;               /*!< Current revision number      */
       USIGN16        wBuild;                  /*!< Current build number         */
} T_PNIO_VERSION_STRUCT;

/*! This structure contains the parameters of a read request confirmation. */
typedef struct
{
    T_READ_CONF_PARAM *poReadConfParam;        /*!< This attribute receives a pointer to a structure which contains the values of the read request confirmation */
} T_READ_CNF_PARA;

/*! This structure contains the parameters of a write request confirmation. */
typedef struct
{
    T_WRITE_CONF_PARAM *poWriteConfParam;      /*!< This attribute receives a pointer to a structure which contains the values of the write request confirmation */
} T_WRITE_CNF_PARA;

/*! This structure contains the parameters of an abort indication for one AR from the Context Management to the application. */
typedef struct
{
    USIGN16 wARID;                              /*!< This attribute receives the ID of the aborted AR */
    USIGN16 wARUserID;                          /*!< This attribute receives the user specific ID of the aborted AR */
} T_IND_ABORT_PARA;

/*! This structure contains the parameters of an application ready indication from the context management to the application.
    By this service the device indicates to the controller that the application is ready. */
typedef struct
{
    T_APP_READY_IND_PARAM *poAppReadyIndParam;  /*!< This attribute receives a pointer to a structure which contains the values of the Application Ready indication */
} T_IND_APP_READY_PARA;

/*! This structure contains the parameters of a neighbourhood changed indication.
    The context management calls this function if the neighbourhood information has changed
    (i.e. new cabling) to store/overwrite this string in a local database. */
typedef struct
{
    USIGN16 wARUserID;                          /*!< This attribute receives the user specific identifier of the AR with changed alias name */
    USIGN8  byAliasNameLen;                     /*!< This attribute receives the length of the alias name */
    CHAR    *pcAliasName;                       /*!< This attribute receives a pointer to the alias name */
} T_IND_NBH_CHANGED_PARA;

/*! This structure contains the parameters of an ARP request confirmation. */
typedef struct
{
    USIGN8 pbySrcMAC[MAC_ADDR_SIZE];
                                                /*!< This attribute receives the MAC address of the ARP request target.

                                                    Example:
                                                    - MAC-Address: 001122334455
                                                        - pbySrcMAC [0] = 0x00;
                                                        - pbySrcMAC [1] = 0x11;
                                                        - pbySrcMAC [2] = 0x22;
                                                        - pbySrcMAC [3] = 0x33;
                                                        - pbySrcMAC [4] = 0x44;
                                                        - pbySrcMAC [5] = 0x55;
                                                */
    USIGN8 pbySrcIP[IP_ADDR_SIZE];
                                                /*!< This attribute receives the IP address of the ARP request target.

                                                    Example:
                                                    - IP-Address: 192.168.10.1
                                                        - pbySrcIP [0] = 192;
                                                        - pbySrcIP [1] = 168;
                                                        - pbySrcIP [2] = 10;
                                                        - pbySrcIP [3] = 1;
                                                */
} T_ARP_CNF_PARA;

/*! This structure contains the parameters of an DCP request confirmation. */
typedef struct
{
    USIGN32 dwXID;                                
                                                  /*!< This variable contains the transaction identifier which was returned by PN_DcpSend() to identify the corresponding request. */
    USIGN8  byServiceID;                          
                                                  /*!< This variable contains the DCP service number which issued this confirmation.
                                                        
                                                        Possible values:
                                                        - DCP_GET
                                                        - DCP_SET
                                                        - DCP_IDENTIFY
                                                  */
    USIGN8  pbyMACAddr[MAC_ADDR_SIZE];            
                                                /*!< This variable receives the MAC address of the DCP request target.
                                                     It is a pointer to a buffer with size MAC_ADDR_SIZE.

                                                    Example:
                                                    - MAC-Address: 001122334455
                                                        - pbySrcMAC [0] = 0x00;
                                                        - pbySrcMAC [1] = 0x11;
                                                        - pbySrcMAC [2] = 0x22;
                                                        - pbySrcMAC [3] = 0x33;
                                                        - pbySrcMAC [4] = 0x44;
                                                        - pbySrcMAC [5] = 0x55;
                                                */
    USIGN8  pbyIPAddr[IP_ADDR_SIZE];              
                                                /*!< This attribute receives the IP address of the DCP request target.
                                                     It is a pointer to a buffer with size IP_ADDR_SIZE.

                                                    Example:
                                                    - IP-Address: 192.168.10.1
                                                        - pbySrcIP [0] = 192;
                                                        - pbySrcIP [1] = 168;
                                                        - pbySrcIP [2] = 10;
                                                        - pbySrcIP [3] = 1;
                                                */
    USIGN8  pbyGateway[IP_ADDR_SIZE];             
                                                /*!< This attribute receives the standard gateway address of the DCP request target.
                                                     It is a pointer to a buffer with size IP_ADDR_SIZE.

                                                    Example:
                                                    - IP-Address: 192.168.10.1
                                                        - pbySrcIP [0] = 192;
                                                        - pbySrcIP [1] = 168;
                                                        - pbySrcIP [2] = 10;
                                                        - pbySrcIP [3] = 1;
                                                */
    USIGN8  pbyNetworkMask[IP_ADDR_SIZE];         
                                                /*!< This attribute receives the network mask of the DCP request target.
                                                     It is a pointer to a buffer with size IP_ADDR_SIZE.

                                                    Example:
                                                    - IP-Address: 192.168.10.1
                                                        - pbySrcIP [0] = 192;
                                                        - pbySrcIP [1] = 168;
                                                        - pbySrcIP [2] = 10;
                                                        - pbySrcIP [3] = 1;
                                                */
    USIGN8  byStationNameLength;                  
                                                  /*!< This attribute receives the length of the station name. */
    CHAR    pcStationName[MAX_STATION_NAME_SIZE]; 
                                                  /*!< This attribute receives a pointer to a buffer with size MAX_STATION_NAME_SIZE containing the station name. */
    USIGN8  byAliasNameLength;                    
                                                  /*!< This attribute receives the length of the alias name. */
    CHAR    pcAliasName[MAX_STATION_NAME_SIZE];   
                                                  /*!< This attribute receives a pointer to a buffer with size MAX_STATION_NAME_SIZE containing the alias name. */
    USIGN8  byDevTypeLength;                      
                                                  /*!< This attribute receives the actual length of the device type. */
    CHAR    pcDevType[MAX_STATION_NAME_SIZE];     
                                                  /*!< This attribute receives a pointer to a buffer with size DEVICE_TYPE_LEN containing the device type. */
    USIGN16 wVendorID;                            
                                                  /*!< This attribute receives the vendor ID. */
    USIGN16 wDeviceID;                            
                                                  /*!< This attribute receives the device ID. */
    USIGN8  byDeviceRole;                         
                                                  /*!< This attribute receives the device role.
                                                        
                                                        Possible values:
                                                        - Bit 0: The device contains an IO device interface.
                                                        - Bit 1: The device contains an IO controller interface.
                                                        - Bit 2: The device contains multiple IO device interfaces.
                                                        - Bit 3: The device contains an IO supervisor interface.
                                                        - Bit 4 to 73: Reserved
                                                  */
    USIGN8  byResult;                             
                                                  /*!< This variable contains the result of the DCP operation.*/
    T_DCP_ARP_MANUFACTURER_INFO oManufacturerInfo;
                                                  /*!< This structure contains the manufacturer specific data.*/
} T_DCP_CNF_PARA;

/*! This structure contains all parameters of a DCP identify confirmation. */
typedef struct
{
   USIGN8      pbyMACAddr[MAC_ADDR_SIZE];
                                       /*!< MAC address of the responding device. This is a pointer to a buffer with length MAC_ADDR_SIZE */
   USIGN8      pbyIPAddr[IP_ADDR_SIZE];
                                       /*!< IP address of the responding device. This is a pointer to a buffer with length IP_ADDR_SIZE */
   USIGN8      pbyGateway[IP_ADDR_SIZE];
                                       /*!< IP address of the StandardGateway. This is a pointer to a buffer with length IP_ADDR_SIZE */
   USIGN8      pbyNetworkMask[IP_ADDR_SIZE];
                                       /*!< Subnet mask of the responding device. This is a pointer to a buffer with length IP_ADDR_SIZE */
   USIGN8      byNameLength;
                                       /*!< Station name length of the responding device. */
   CHAR        pcName[MAX_STATION_NAME_SIZE];
                                       /*!< Station name of the responding device. */
   USIGN8      byAliasNameLength;
                                       /*!< Alias name length of the responding device. */
   CHAR        pcAliasName[MAX_STATION_NAME_SIZE];
                                       /*!< Alias name of the responding device. */
   USIGN8      byDevTypeLength;
                                       /*!< Length of the device type. */
   CHAR        pcDevType[DEVICE_TYPE_LEN];
                                       /*!< Pointer to a buffer containing the device type. */
   USIGN8      byDeviceRole;
                                       /*!< This attribute contains the device role.
                                                        
                                            Possible values:
                                            - Bit 0: The device contains an IO device interface.
                                            - Bit 1: The device contains an IO controller interface.
                                            - Bit 2: The device contains multiple IO device interfaces.
                                            - Bit 3: The device contains an IO supervisor interface.
                                            - Bit 4 to 73: Reserved
                                        */
   USIGN16     wDeviceID;
                                        /*!< This attribute contains the device ID. */
   USIGN16     wVendorID;
                                        /*!< This attribute contains the vendor ID. */
} T_DCP_CONF_DATA;

/*! This structure contains all parameters of a device ar info indication. */
typedef struct
{
    USIGN16 wDEVInstID;                           /*!< Contains the related device instance ID */
    T_CMDEV_AR_BLOCK_REQ *poArBlockReq;           /*!< Contains the parameter taken from the connect service */
    T_GUID                oARUUID;                /*!< Contains the AR UUID taken from the connect service */
    PHO_BOOL              boSRInfoBlockAvailable; /*!< Indicates, if a system redundancy block is available */
    T_SR_INFO_BLOCK oSRInfoBlock;                 /*!< Contains the system redundancy parameters taken from the connect service */
    PHO_BOOL              boIRInfoBlockAvailable; /*!< Indicates, if a isochronous relay block is available */
    T_IR_INFO_BLOCK oIRInfoBlock;                 /*!< Contains the isochronous relay parameters taken from the connect service */
} T_IND_CMDEV_AR_INFO_PARA;

/*! This structure contains the parameters of an AR check indication.
    By this service the application is informed about the module diff block list and
    is able to modify the states of the modules and submodules in this list.
 */
typedef struct
{
    USIGN16               wDEVInstID;             /*!< This parameter contains the related device instance ID */
    USIGN16               wARID;                  /*!< Unique identifier of the AR */
    T_MODULE_DIFF_BLOCK **ppoModuleDiffBlock;     /*!< Pointer to a pointer with the list of structures which contain parameter of the APIs for modification purposes */
} T_IND_CMDEV_AR_CHECK_PARA;

/*! This structure contains all parameters of a device application ready indication. */
typedef struct
{
    T_CMDEV_APP_READY_IND_PARAM *poAppReadyIndParam;  /*!< Pointer to structure which contains the        */
                                                      /*!< parameters for the Application Ready indication*/
} T_IND_CMDEV_APP_READY_PARA;

/*! This structure contains all parameters of a device abort indication. */
typedef struct
{
    USIGN16 wDEVInstID;                         /*!< Contains the related device instance ID */
    USIGN16 wARID;                              /*!< Contains the unique identifier of the AR */
    T_PNIO_ERROR_STATE oErrorState;             /*!< Contains the elements of the PNIO error state */
} T_IND_CMDEV_ABORT_PARA;

/*! This structure contains all parameters of a AR input data indication. */
typedef struct
{
    USIGN16 wDEVInstID;                         /*!< Contains the related device instance ID */
    USIGN16 wARID;                              /*!< Contains the unique identifier of the AR */
} T_IND_CMDEV_AR_INDATA_PARA;

/*! This structure contains all parameters of a device parameterization begin indication. */
typedef struct
{
    USIGN16 wDEVInstID;                         /*!< Contains the related device instance ID */
    USIGN16 wARID;                              /*!< Contains the unique identifier of the AR */
} T_IND_CMDEV_PRM_BEGIN_PARA;

/*! This structure contains all parameters of a device parameterization end indication. */
typedef struct
{
    USIGN16 wDEVInstID;                         /*!< Contains the related device instance ID */
    USIGN16 wARID;                              /*!< Contains the unique identifier of the AR */
} T_IND_CMDEV_PRM_END_PARA;

/*! This structure contains all parameters of a state indication. */
typedef struct
{
    USIGN16 wDEVInstID;                         /*!< Contains the related device instance ID */
    USIGN16 wARID;                              /*!< Contains the unique identifier of the AR */
    USIGN8 byARState;                           /*!< Contains the AR status */
    T_MODULE_DIFF_BLOCK **ppoModuleDiffBlock;   /*!< Pointer to a pointer with the list of structures which contain parameter of the APIs for modification purposes */
} T_IND_CMDEV_STATE_PARA;

/*! This structure contains the data of the I&M0 response of the Controller or Device. */
typedef struct {
    USIGN8  byVendorIDHigh;
                                       /*!< Vendor ID MSB */
    USIGN8  byVendorIDLow;
                                       /*!< Vendor ID LSB */
    USIGN8  byOrderID[IM0_ORDERID_LEN];
                                       /*!< This parameter contains the complete order number or
                                            at least a relevant part that allows unambiguous
                                            identification of the device or module.
                                            The length of the order ID is 20 octets,
                                            unused octets shall be set "20h" (blank).
                                       */
    USIGN8  bySerialNumber[IM0_SERIALNUMBER_LEN];
                                       /*!< This parameter contains the serial number which is
                                            a unique production number of the device manufacturer.
                                            The length of the serial number is 16 octets,
                                            unused octets shall be set "20h" (blank).
                                       */
    USIGN16 wHardwareRevision;
                                       /*!< This parameter contains the revision number
                                            which characterizes the edition of the hardware.
                                       */
    CHAR    cSWRevisionPrefix;
                                       /*!< This parameter contains the software revision prefix.

                                            Allowed values:
                                            - V = officially released version
                                            - R = Revision
                                            - P = Prototype
                                            - U = Under Test
                                            - T = Test Device
                                       */
    USIGN8  bySWRevisionFunctionalEnhancement;
                                       /*!< This parameter contains the software functional enhancement value.

                                            Allowed values: 0 to 255
                                        */
    USIGN8  bySWRevisionBugfix;
                                       /*!< This parameter contains the software revision bug fix value.

                                            Allowed values: 0 to 255
                                        */
    USIGN8  bySWRevisionInternalChange;
                                       /*!< This parameter contains the software revision internal change value.

                                            Allowed values: 0 to 255
                                        */
    USIGN16 wRevisionCounter;
                                       /*!< This parameter contains the revision counter which marks a change of hardware or of its parameters.

                                            Allowed values: 0 to 65535
                                        */
    USIGN16 wProfileID;
                                       /*!< This parameter contains the value of a special profile of the device or module
                                            which may offer extended information.
                                            The current profile IDs can be found in the appropriate specification.
                                       */
    USIGN16 wProfileSpecificType;
                                       /*!< In case a device or module follows a special profile,
                                            this parameter contains information about the usage of its channels and/or sub devices.
                                       */
    USIGN8  byImVersionMajor;
                                       /*!< This parameter contains the implemented major version number of the I&M functions.
                                       */
    USIGN8  byImVersionMinor;
                                       /*!< This parameter contains the implemented minor version number of the I&M functions.
                                       */
    USIGN16 wImSupported;              /*!< This attribute contains the IM support information
                                            - Bit 0:  IM_Supported.Profil_specific
                                                      This field shall be set related to the profiles.
                                            - Bit 1:  IM_Supported.I&M1
                                                      This field shall be set if the I&M1 record contains data.
                                            - ...
                                            - Bit 15: IM_Supported.I&M15
                                                      This field shall be set if the I&M15 record contains data. */
} T_IM0_DATA;

/*! This structure contains the data of the I&M1 response of the Controller or Device. */
typedef struct {
    USIGN8  byTagFunction[IM1_TAG_FUNCTION_LEN];
                                       /*!< This parameter contains a unique label for each device or module
                                            within a plant which is necessary for the identification of its function or task.
                                            The length of the tag function is 32 octets, unused octets shall be set "20h" (blank).
                                       */
    USIGN8  byTagLocation[IM1_TAG_LOCATION_LEN];
                                       /*!< This parameter contains a unique label for each device or module
                                            within a plant which is necessary for the identification of its location.
                                            The length of the tag location is 22 octets, unused octets shall be set "20h" (blank).
                                       */
} T_IM1_DATA;

/*! This structure contains the data of the I&M2 response of the Controller or Device. */
typedef struct {
    USIGN8  byInstallationDate[IM2_INSTALLATION_DATE_LEN];
                                       /*!< This parameter contains the date of installation or commissioning of a device or module.
                                            The date format has to be according to ISO 8601: YYYY-MM-DD,
                                            where YYYY is the year in the usual Gregorian calendar,
                                            MM is the month of the year between 01 (January) and 12 (December)
                                            and DD is the day of the month between 01 and 31.
                                            Time information shall be added where applicable: hh:mm
                                            where hh is the hour of the day (0..23) and mm the minutes of the hour (0..59).
                                            Time and date shall be separated by a blank character.
                                            The length of the installation date is 16 octets, unused octets shall be set "20h" (blank).

                                            Examples:
                                            - 2011-05-26 14:23
                                            - 2011-05-26
                                       */
} T_IM2_DATA;

/*! This structure contains the data of the I&M3 response of the Controller or Device. */
typedef struct {
    USIGN8  byDescriptor[IM3_DESCRIPTOR_LEN];
                                       /*!< This parameter is a comment field which allows customers to
                                            store any individual additional information and annotation.
                                            The length of the descriptor is 54 octets, unused octets shall be set "20h" (blank).
                                       */
} T_IM3_DATA;

/*! This structure contains the data of the I&M4 response of the Controller or Device. */
typedef struct {
    USIGN8  bySignature[IM4_SIGNATURE_LEN];
                                       /*!< This attribute allows parameterization tools to store a
                                            "security" code as a reference for a particular parameterization session
                                            and audit trail tools to retrieve the code for integrity checks.
                                            The length of the signature is 54 octets, unused octets shall be set "20h" (blank).
                                       */
} T_IM4_DATA;

/*! This structure contains the data of the PDev response of the PROFINET Device. */
typedef struct {
    USIGN16 wMauType;                 /*!< This parameter contains the MAU type of the ethernet interface.

                                           Allowed Values:
                                           - MAUTYPE_10BASET
                                           - MAUTYPE_10BASETXHD
                                           - MAUTYPE_10BASETXFD
                                           - MAUTYPE_10BASEFLHD
                                           - MAUTYPE_10BASEFLFD
                                           - MAUTYPE_100BASETXHD
                                           - MAUTYPE_100BASETXFD
                                           - MAUTYPE_100BASEFXHD
                                           - MAUTYPE_100BASEFXFD
                                           - MAUTYPE_1000BASEXHD
                                           - MAUTYPE_1000BASEXFD
                                           - MAUTYPE_1000BASELXHD
                                           - MAUTYPE_1000BASELXFD
                                           - MAUTYPE_1000BASESXHD
                                           - MAUTYPE_1000BASESXFD
                                           - MAUTYPE_1000BASETHD
                                           - MAUTYPE_1000BASETFD
                                           - MAUTYPE_10GIGBASEFX
                                           - MAUTYPE_100BASELX10
                                           - MAUTYPE_100BASEPXFD
                                       */
    USIGN32 dwMediaType;              /*!< This parameter contains the media type of the ethernet interface.

                                            Allowed Values:
                                            - MEDIATYPE_UNKNOWN
                                            - MEDIATYPE_COPPER
                                            - MEDIATYPE_FIBEROPTIC
                                            - MEDIATYPE_RADIO
                                       */
    USIGN8 byAutoNeg;                 /*!< This parameter indicates the ethernet interface auto negotiation status.

                                            Allowed Values:
                                            - AUTO_NEGOTIATION_SUPPORT
                                            - AUTO_NEGOTIATION_ENABLED
                                       */
    USIGN16 wAutoNegCap;              /*!< This parameter indicates the ethernet interface auto negotiation capabilities

                                            Allowed Values:
                                            - AUTO_NEG_CAP_UNKNOWN
                                            - AUTO_NEG_CAP_B10BASET
                                            - AUTO_NEG_CAP_B10BASETFD
                                            - AUTO_NEG_CAP_B100BASET4
                                            - AUTO_NEG_CAP_B100BASETX
                                            - AUTO_NEG_CAP_B100BASETXFD
                                            - AUTO_NEG_CAP_B100BASET2
                                            - AUTO_NEG_CAP_B100BASET2FD
                                            - AUTO_NEG_CAP_BFDXPAUSE
                                            - AUTO_NEG_CAP_BFDXAPAUSE
                                            - AUTO_NEG_CAP_BFDXSPAUSE
                                            - AUTO_NEG_CAP_BFDXBPAUSE
                                            - AUTO_NEG_CAP_B1000BASEX
                                            - AUTO_NEG_CAP_B1000BASEXFD
                                            - AUTO_NEG_CAP_B1000BASET
                                            - AUTO_NEG_CAP_B1000BASETFD
                                       */
} T_PDEV_DATA;

/*! Contains the elements of the input data object element response. */
typedef struct
{
/*in*/   USIGN8               byIOCS;             /*!< Value of IOCS status byte */
/*in*/   USIGN8               byIOPS;             /*!< Value of IOPS status byte */
/*in*/   USIGN16              wLengthDATA;        /*!< Length of input data */
/*in*/   USIGN8              *pbyDATA;            /*!< Input data */
} T_INPUT_DATA_OBJECT_ELEMENT;

/*! Contains the elements of the output data object element response. */
typedef struct
{
/*in*/   USIGN16              wSubstituteActiveFlag;    /*!< Current substitute status of output values */
/*in*/   USIGN8               byIOCS;                   /*!< Value of IOCS status byte */
/*in*/   USIGN8               byIOPS;                   /*!< Value of IOPS status byte */
/*in*/   USIGN16              wLengthDATA;              /*!< Length of input data */
/*in*/   USIGN8              *pbyDATA;                  /*!< Input data */
/*in*/   USIGN16              wSubstitutionMode;        /*!< Current substitution mode of output values */
} T_OUTPUT_DATA_OBJECT_ELEMENT;

/*! This structure contains the description of a channel entry. The API, Slot and Subslot will be automatically registered if new. */
typedef struct
{
   USIGN16      wSlot;
                                    /*!< Slot number to which the channel is assigned to

                                         Allowed values:
                                         - 0 to 0x7FFF
                                    */
   USIGN16      wSubslot;
                                    /*!< Subslot number to which the channel is assigned to

                                         Allowed values:
                                         - 1 to 0x9FFF
                                    */
   USIGN32      dwModuleID;         /*!< Module Ident Number of this slot */
   USIGN16      wChannelNumber;
                                    /*!< Channel number to be registered for this slot/subslot combination.

                                         Allowed values:
                                         - 0 to 0x7FFF
                                    */
   USIGN32      dwSubmoduleID;      /*!< Submodule Ident Number of this subslot */
} T_PNDIAG_CHANNEL_INFO;

/*! This structure contains the description of a diagnosis entry. */
typedef struct
{
   USIGN16      wChannelProperties; /*!< Properties of this channel */
   USIGN16      wUserStructureIdentifier;
                                    /*!< User structure identifier (USI) describing the diagnosis type

                                        Allowed Values:
                                        - PNDIAG_CHAN_DIAG (0x8000, Channel Diagnosis)
                                        - PNDIAG_EXT_CHAN_DIAG (0x8002, Ext Channel Diagnosis)
                                        - PNDIAG_QUA_CHAN_DIAG (0x8003, Qualified Channel Diagnosis)
                                    */
   USIGN16      wChannelErrorType;  /*!< Error type of the channel. This value is valid for all USI */
   USIGN16      wExtChannelErrorType;
                                    /*!< Extended error type of the channel
                                         This value is valid for USI PNDIAG_EXT_CHAN_DIAG and PNDIAG_QUA_CHAN_DIAG only
                                    */
   USIGN32      dwExtChannelAddValue;
                                    /*!< Additional value in case of extended channel  diagnosis
                                         This value is valid for USI PNDIAG_EXT_CHAN_DIAG and PNDIAG_QUA_CHAN_DIAG only
                                    */
   USIGN32      dwQualifiedChannelQualifier;
                                    /*!< Qualified channel qualifier
                                         This value ist valid for USI PNDIAG_QUA_CHAN_DIAG only.
                                    */
   USIGN8       byRTATrigger;
                                    /*!< With this parameter the client module can choose the RTA handling mode of the channel diagnosis entry. If an alarm handling is triggered (low or high priority), the applicable acknowledge of the remote IOC is given back to the triggered module, here the DiagAgent.

                                        Allowed Values:
                                        - PNDIAG_NO_RTA_TRIGGER: No Alarm handling is triggered, only the Diagnosis ASU will be updated
                                        - PNDIAG_LOW_RTA_TRIGGER: An Alarm handling with low priority is triggered (RTA frame with low priority is sent)
                                        - PNDIAG_HIGH_RTA_TRIGGER: An Alarm handling with high priority is triggered (RTA frame with high priority is sent)
                                    */
} T_PNDIAG_DIAG_INFO;

/*! This structure contains the state of a controller or device instance. */
typedef struct
{
   USIGN16 wInstanceID;             /*!< Contains the unique identifier of the controller or device instance */
   USIGN8 byState;                  /*!< Contains the current diagnosis state.

                                        Allowed values
                                        - PNDIAG_STATE_GOOD: there is no diagnosis entry available
                                        - PNDIAG_STATE_MR: there is in at least one diagnosis entry with the channel property "maintenance required"
                                        - PNDIAG_STATE_MD: there is in at least one diagnosis entry with the channel property "maintenance demanded"
                                        - PNDIAG_STATE_DIAGNOSIS: there is in at least one diagnosis available
                                     */
}T_PNDIAG_STATE_INFO;

/*! This structure contains the parameters of a diagnosis state info indication.
    The diagnosis module calls this function in case the diagnosis state of the controller or of a device instance has changed. */
typedef struct
{
    USIGN8               byNumElements;     /*!< This parameter contains the number of elements within the following list. */
    T_PNDIAG_STATE_INFO *poListOfStates;    /*!< This parameter contains a pointer to a list with diagnosis states of all registered instances. */
} T_IND_PNDIAG_STATE_INFO_PARA;

/*! This is a dummy data structure for the state of active modules. It is only used internally. */
typedef struct t_pmodule
{
    USIGN8  byState;                   /*!< current state of module */

} T_PMODULE;

/*! This structure contains the parameters of an AR diagnosis state info changed indication.
    The dwDiagInfoState bit field holds the event reasons for calling the ARDiagStateChangedInd :
    | diagnosis event                            | Bit in dwDiagInfoState | addition info                                                                                                     |
    |:------------------------------------------:|:----------------------:|:-----------------------------------------------------------------------------------------------------------------:|
    | multiple station names found in network    | bit 0 | szDeviceStationName is filled, oErrorState.byErrorCode1 = CTLDINA, oErrorState.byErrorCode2 = CTLDINA_DCP_MULTIPLE_REALSTATIONNAME |
    | multiple IP addresses found in Network     | bit 1 | pbyIPAddr is filled, oErrorState.byErrorCode1 = CTLDINA, oErrorState.byErrorCode2 = CTLDINA_ARP_MULTIPLE_IP_ADDRESSES              |           
    | AR is deactivated                          | bit 2 |                                                                                                                                    |
    | no neighborhood information is available   | bit 3 |                                                                                                                                    |
    | neighborhood information is not unique     | bit 4 |                                                                                                                                    |
    | connect response error                     | bit 5 | oErrorState is filled with the error code                                                                                          |
*/
typedef struct
{
    USIGN16              wARID;                                         /*!< Unique identifier of the AR */
    USIGN16              wARUserID;                                     /*!< User specific identifier of this AR. */
    USIGN32              dwDiagInfoState;                               /*!< Bit field holding the diagnosis event reason */
    T_PNIO_ERROR_STATE   oErrorState;                                   /*!< Additional Error code depending on the dwDiagInfoState bits */
    CHAR                 szDeviceStationName[MAX_STATION_NAME_LENGTH];  /*!< Stationname if multiple devices with this name were found */
    USIGN8               pbyIPAddr[IP_ADDR_SIZE];                       /*!< IP address if multiple devices with this IP address were found */
}T_AR_DIAG_STATE_INFO;

typedef struct
{
    T_AR_DIAG_STATE_INFO *poARDiagStateInfo; /*!< for details see T_AR_DIAG_STATE_INFO */
}T_IND_AR_DIAG_STATE_CHANGED_PARA;

/* for compatibility reasons, we define some type definitions */
typedef T_DCP_REQUEST T_CMCTL_DCP_REQUEST;
typedef T_DCP_REQUEST T_CMDEV_DCP_REQUEST;

typedef T_REC_READ_RSP T_CMCTL_REC_READ_RSP;
typedef T_REC_READ_IND T_CMDEV_REC_READ_IND;
typedef T_REC_READ_RSP T_CMDEV_REC_READ_RSP;

typedef T_REC_WRITE_RSP T_CMCTL_REC_WRITE_RSP;
typedef T_REC_WRITE_IND T_CMDEV_REC_WRITE_IND;
typedef T_REC_WRITE_RSP T_CMDEV_REC_WRITE_RSP;

typedef T_MODULE_DIFF_BLOCK T_CMDEV_MODULE_DIFF_BLOCK;
typedef T_RELATED_API_MODULE_DIFF T_CMDEV_RELATED_API_MODULE_DIFF;
typedef T_LIST_OF_MODULES T_CMDEV_LIST_OF_MODULES;
typedef T_LIST_OF_SUBMODULES T_CMDEV_LIST_OF_SUBMODULES;

/*****************************************************************************
****  EXTERN FUNCTION DEFINITIONS                                         ****
*****************************************************************************/
/* system functions */
extern FWAPIRET PN_Init (VOID);
extern FWAPIRET PN_DeInit (VOID);
extern FWAPIRET PN_Tick (VOID);
extern FWAPIRET PN_GetStackState (USIGN16 *wState);
extern FWAPIRET PN_GetVersionInfo (T_PNIO_VERSION_STRUCT *poVersionInfo);
extern FWAPIRET CM_SetConfigVar(T_PMODULE*, CHAR*, CHAR*, CHAR*, CHAR*, H_ES);
extern FWAPIRET CM_GetConfigVar(T_PMODULE*, CHAR*, CHAR*, CHAR*, USIGN32, CHAR*, H_ES);
extern FWAPIRET EA_WriteEntry(USIGN8 byModId, USIGN16 wZone, USIGN16 wEntryType, USIGN32 dwFlag, const CHAR *pszTxt, H_ES hEs);
/* controller functions */
extern FWAPIRET PN_AddCtrlInst (VOID);
extern FWAPIRET PN_RemoveCtrlInst (VOID);
extern FWAPIRET PN_ParseXmlFileDebugOutput (PHO_BOOL bSwitch);
extern FWAPIRET PN_ParseXmlFile (CHAR *szXMLFile);
extern FWAPIRET PN_ArAdd (T_PNIO_CMCTL_AR_ADD_PARAM_IN *poArAddParamIn, USIGN16 *pwARID, USIGN16 *pwNumberOfRestARs);
extern FWAPIRET PN_ArParamAdd (T_PNIO_CMCTL_AR_PARAM_ADD_PARAM_IN* poArParam_AddParam);
extern FWAPIRET PN_RemoveAr (USIGN16 wARID);
extern FWAPIRET PN_StartAr (USIGN16 wARID);
extern FWAPIRET PN_StartArEx(USIGN16 wARID, USIGN8 byARStartupMode);
extern FWAPIRET PN_StopAr (USIGN16 wARID);
extern FWAPIRET PN_ArGetDebugOutput (PHO_BOOL bSwitch);
extern FWAPIRET PN_ArGet (USIGN16 wARID, T_PNIO_CMCTL_AR_ADD_PARAM_IN **poArGetOut);
extern FWAPIRET PN_ArGetFree (T_PNIO_CMCTL_AR_ADD_PARAM_IN *poArAddParam);
extern FWAPIRET PN_ArParamGet (USIGN16 wARID, USIGN16 *pwNumberOfAddParams, T_PNIO_CMCTL_AR_PARAM_ADD_PARAM_IN **ppoArParamGetOut);
extern FWAPIRET PN_ArParamGetFree (USIGN16 wNumberOfAddParams, T_PNIO_CMCTL_AR_PARAM_ADD_PARAM_IN *poArParam);
extern FWAPIRET PN_SetProviderStatus (USIGN16 wNumberOfProv, USIGN16 *pwProvIDList, USIGN8 byStatusMode, USIGN16 wProvDataStatus, USIGN16 wProvIOPS, USIGN16 wProvIOCS, USIGN16 wNumberOfElements, T_PNIO_IOAR_Element_IOXS_LIST *poElementIOXSList);
extern FWAPIRET PN_WriteBuf (USIGN16 wProvID, USIGN16 wDataLength, USIGN8 *pbyDataBuffer, USIGN8 byDataStatus);
extern FWAPIRET PN_ReadBuf (USIGN16 wConsID, USIGN16 wDataLength, USIGN8 *pbyDataBuffer, USIGN8 *pbyDataStatus);
extern FWAPIRET PN_ReadArParam (USIGN8 byFindOption, T_AR_READ_PARAM_IN *poARReadParamIn, T_AR_READ_PARAM_OUT *poARReadParamOut);
extern FWAPIRET PN_Read (T_READ_PARAM_IN *poReadParamIn, USIGN16 wTimeout);
extern FWAPIRET PN_ReadImpl(T_IMPL_READ_PARAM_IN *poImplReadParamIn);
extern FWAPIRET PN_Write (T_WRITE_PARAM_IN *poWriteParamIn, USIGN16 wTimeout);
extern FWAPIRET PN_ArParamBegin(USIGN16 wARID);
extern FWAPIRET PN_DcpParameterizationDone(USIGN8 *pbyMACAddr);
/* device functions */
extern FWAPIRET PN_AddDevInst (USIGN16* pwInstanceID, CHAR *szXMLFile);
extern FWAPIRET PN_DevAddAPI(USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wMaxSlots, USIGN16 wMaxSubslots);
extern FWAPIRET PN_DevRemoveAPI(USIGN16 wInstanceID, USIGN32 dwAPI);
extern FWAPIRET PN_DevPlugModule(USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlotNumber, USIGN32 dwModuleID);
extern FWAPIRET PN_DevPullModule(USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlotNumber);
extern FWAPIRET PN_DevPlugSubmodule(USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlotNumber, USIGN16 wSubslotNumber, USIGN32 dwSubmoduleID, USIGN16 wInputDataLength, USIGN16 wOutputDataLength);
extern FWAPIRET PN_DevPullSubmodule(USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlotNumber, USIGN16 wSubslotNumber);
extern FWAPIRET PN_DevUpdateSubmodule(USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlotNumber, USIGN32 dwModuleID, USIGN16 wSubslotNumber, USIGN32 dwSubmoduleID);
extern FWAPIRET PN_DevStart(USIGN16 wInstanceID);
extern FWAPIRET PN_DevStop(USIGN16 wInstanceID);
extern FWAPIRET PN_DevRelease(USIGN16 wInstanceID, USIGN16 wARID);
extern FWAPIRET PN_DevSetApplReady(USIGN16 wInstanceID, USIGN16 wARID, T_CMDEV_MODULE_DIFF_BLOCK *poModuleDiffBlock);
extern FWAPIRET PN_RemoveDevInst (USIGN16 wInstanceID);
extern FWAPIRET PN_DevArGetDebugOutput (USIGN16 wInstanceID, PHO_BOOL bSwitch);
extern FWAPIRET PN_DevArGet(USIGN16 wInstanceID, T_CTM_CMDEV_CONFIG **poArGetOut);
extern FWAPIRET PN_DevArGetFree(T_CTM_CMDEV_CONFIG *poArParam);
extern FWAPIRET PN_DevSetProviderStatus (USIGN16 wInstanceID, USIGN8 byStatusMode, USIGN16 wProvDataStatus, USIGN16 wProvIOPS, USIGN16 wProvIOCS);
extern FWAPIRET PN_WriteDevBuf (USIGN16 wInstanceID, USIGN16 wDataLength, USIGN8 *pbyDataBuffer, USIGN8 byDataStatus);
extern FWAPIRET PN_ReadDevBuf (USIGN16 wInstanceID, USIGN16 wDataLength, USIGN8 *pbyDataBuffer, USIGN8 *pbyDataStatus);
extern FWAPIRET PN_DevSetProviderStatusEx (USIGN16 wInstanceID, USIGN16 wNumberOfProv, USIGN16 *pwProvIDList, USIGN8 byStatusMode, USIGN16 wProvDataStatus, USIGN16 wProvIOPS, USIGN16 wProvIOCS, USIGN16 wNumberOfElements, T_PNIO_IOAR_Element_IOXS_LIST *poElementIOXSList);
extern FWAPIRET PN_WriteDevBufEx (USIGN16 wInstanceID, USIGN16 wProvID, USIGN16 wDataLength, USIGN8 *pbyDataBuffer, USIGN8 byDataStatus);
extern FWAPIRET PN_ReadDevBufEx (USIGN16 wInstanceID, USIGN16 wConsID, USIGN16 wDataLength, USIGN8 *pbyDataBuffer, USIGN8 *pbyDataStatus);
extern FWAPIRET PN_DevReadResponse (USIGN16 wInstanceID, T_REC_READ_RSP *poRecReadRsp);
extern FWAPIRET PN_DevWriteResponse (USIGN16 wInstanceID, T_REC_WRITE_RSP *poRecWriteRsp);
extern FWAPIRET PN_DevSendAlarm (USIGN16 wInstanceID, USIGN16 wAlarmReference, T_PNIO_IOAR_ALARM_NOTIFICATION *poAlarmNotification);
extern FWAPIRET PN_DevSendStatusAlarm(USIGN16 wInstanceID, USIGN16 wAlarmReference);
extern FWAPIRET PN_DevSrlFinalizeAlarmAck(USIGN16 wInstanceID, USIGN16 wARID, USIGN16 wARUserID, USIGN16 wLocalAlarmReference, T_PNIO_IOAR_ALARM_ACK *poAlarmAck);
/* controller and device functions */
extern FWAPIRET PN_DiagRegisterChannel( USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wNumberOfChannels, T_PNDIAG_CHANNEL_INFO *poListOfChannels);
extern FWAPIRET PN_DiagUnregisterChannel( USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wNumberOfChannels, T_PNDIAG_CHANNEL_INFO *poListOfChannels);
extern FWAPIRET PN_DiagChannelAdd( USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlot, USIGN16 wSubslot, USIGN16 wChannelNumber, T_PNDIAG_DIAG_INFO *poDiagInfo);
extern FWAPIRET PN_DiagChannelRemove( USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlot, USIGN16 wSubslot, USIGN16 wChannelNumber, USIGN16 wChannelErrorType, USIGN16 wExtChannelErrorType);
extern FWAPIRET PN_DiagChannelRemoveAll( USIGN16 wInstanceID);
extern FWAPIRET PN_DiagSetArInfo( USIGN16 wInstanceID, USIGN32 dwAPI, USIGN16 wSlot, USIGN16 wSubslot, T_GUID *poGuid, USIGN16 wARID, USIGN16 wLocAlmRef, USIGN16 wMaxAlmDatLength);
extern FWAPIRET PN_AlarmAck (USIGN16 wInstanceID, USIGN16 wLocalAlarmReference, PHO_BOOL bPriorityFlag, T_PNIO_IOAR_ALARM_ACK *poAlarmAck);
/* DCP/ARP functions */
extern FWAPIRET PN_ArpSend (USIGN8 *pbyDstIP);
extern FWAPIRET PN_DcpSend (USIGN8 byServiceID, USIGN8 byOption, USIGN8 bySuboption, USIGN16 wVendorID, USIGN16 wDeviceID, USIGN8 byDeviceRole, T_DCP_ARP_TARGET_INFO *poTargetInfo, USIGN32 *pdwXID);
extern FWAPIRET PN_DcpSendEx (USIGN8 byServiceID, USIGN8 byOption, USIGN8 bySuboption, USIGN16 wVendorID, USIGN16 wDeviceID, USIGN8 byDeviceRole, T_DCP_ARP_TARGET_INFO *poTargetInfo, T_DCP_ARP_MANUFACTURER_INFO *poManufacturerInfo, USIGN32 *pdwXID);
/* callback functions */
extern FWAPIRET PN_InitIndication (VOID (*)(USIGN16, VOID*));
extern FWAPIRET PN_InitAlarming (VOID (*)(USIGN16, USIGN16, USIGN16, USIGN16, PHO_BOOL, VOID*, VOID*));
extern FWAPIRET PN_InitReadConfirmation (VOID (*)(VOID*));
extern FWAPIRET PN_InitWriteConfirmation (VOID (*)(VOID*));
extern FWAPIRET PN_InitDcpResponse (FWAPIRET (*)(T_DCP_REQUEST*));
extern FWAPIRET PN_InitImResponse (FWAPIRET (*)(USIGN16, USIGN16, USIGN16, VOID*));
extern FWAPIRET PN_InitPdevResponse (FWAPIRET (*)(T_PDEV_DATA*));
extern FWAPIRET PN_InitDcpConfirmation (VOID (*)(VOID*));
extern FWAPIRET PN_InitDcpIdentConfirmation (FWAPIRET (*)(T_DCP_CONF_DATA*));
extern FWAPIRET PN_InitArpConfirmation (VOID (*)(VOID*));
extern FWAPIRET PN_InitDevIndication (FWAPIRET (*)(USIGN16, VOID*));
extern FWAPIRET PN_InitDevAlarming (VOID (*)(USIGN16, USIGN16, USIGN16, USIGN16, PHO_BOOL, VOID*, VOID*));
extern FWAPIRET PN_InitDevReadResponse (FWAPIRET (*)(USIGN16, T_REC_READ_IND*));
extern FWAPIRET PN_InitDevWriteResponse (FWAPIRET (*)(USIGN16, T_REC_WRITE_IND*));
extern FWAPIRET PN_InitDevDcpResponse (FWAPIRET (*)(USIGN16, T_DCP_REQUEST*));
extern FWAPIRET PN_InitDevImResponse (FWAPIRET (*)(USIGN16, USIGN16, USIGN16, USIGN16, VOID*, PHO_BOOL));
extern FWAPIRET PN_InitDevPdevResponse (FWAPIRET (*)(USIGN16, T_PDEV_DATA*));
extern FWAPIRET PN_InitDevIoResponse (FWAPIRET (*)(USIGN16, USIGN16, USIGN16, USIGN16, VOID*));
extern FWAPIRET PN_InitDevSrlSendAlarmIndication(FWAPIRET (*alarmFkt)(USIGN16, USIGN16, T_PNIO_IOAR_ALARM_NOTIFICATION*));
/* helper functions */
extern FWAPIRET PN_UpdateIPAddress (USIGN16 wInstanceID, USIGN8 *pbyIPAddr, USIGN8 *pbySubnetMask, USIGN8 *pbyStdGateway);
extern FWAPIRET PN_UpdateMACAddress (USIGN16 wInstanceID, USIGN8 *pbyMACAddr);
extern FWAPIRET PN_UpdateStationName (USIGN16 wInstanceID, CHAR *pcName, USIGN8 byNameLength);
extern FWAPIRET PN_UpdateDeviceIdentification (USIGN16 wInstanceID, USIGN16 wVendorID, USIGN16 wDeviceID, CHAR *pcDeviceType);
extern FWAPIRET PN_SetInterfaceIpAddress (USIGN32 dwNicNo, USIGN8 *pbyIPAddr, USIGN8 *pbySubnetMask, USIGN8 *pbyStdGateway);
extern FWAPIRET PN_GetStringFromXmlFile(CHAR *szXMLFile, CHAR *pszBlock, CHAR *pszVar, CHAR *pszVal);
extern FWAPIRET PN_GetStringFromIniFile(CHAR *szIniFile, CHAR *pszBlock, CHAR *pszVar, CHAR *pszVal);
extern FWAPIRET PN_SetStringToIniFile(CHAR *szIniFile, CHAR *pszBlock, CHAR *pszVar, CHAR *pszVal);
extern FWAPIRET PN_GetSendErrorCount (USIGN32 *dwAmount);
extern FWAPIRET PN_ResetSendErrorCount (VOID);
extern FWAPIRET PN_DevSendStatusAlarm(USIGN16 wInstanceID, USIGN16 wAlarmReference);
extern FWAPIRET PN_StateMachineDebugOutput (PHO_BOOL bSwitch, USIGN16 wNumberOfARs, USIGN16 *pwARIDList, USIGN32 dwOptions);
extern FWAPIRET PN_DevStateMachineDebugOutput (USIGN16 wInstanceID, PHO_BOOL bSwitch, USIGN32 dwOptions);
extern FWAPIRET PN_PopErrorStack(H_ES hEs, USIGN32 *pdwErrorCode, USIGN32 *pdwAddInfo);

#endif
/*****************************************************************************
****  ERROR CONSTANT DEFINITIONS                                          ****
*****************************************************************************/
/*
ERROR CODING
=================

Error Code
~~~~~~~~~~
|31       28|27       24|23                   16|15                                            0|
+-----------+-----------+-----------------------+-----------------------------------------------+
|   class   |   type    |    module id (MID)    |                     code                      |
+-----------+-----------+-----------------------+-----------------------------------------------+

Additional Error Information
~~~~~~~~~~~~~~~~~~~~~~~~~~~~
|31                   24|23                   16|15                                            0|
+-----------------------+-----------------------+-----------------------------------------------+
|           routine/code label (RCLAB)          |                 additional info                 |
+-----------------------------------------------------------------------------------------------+
*/

/* Definitions of Error Code Masks, Positions */
/* ========================================== */
#define E_MONA_CLASS_MASK       0xF0000000  /* class */
#define E_MONA_TYPE_MASK        0x0F000000  /* type */
#define E_MONA_MID_MASK         0x00FF0000  /* module id */
#define E_MONA_CODE_MASK        0x0000FFFF  /* code */

#define E_MONA_CLASS_POS        28          /* class */
#define E_MONA_TYPE_POS         24          /* type */
#define E_MONA_MID_POS          16          /* code id */
#define E_MONA_CODE_POS         0           /* code */

/* Definitions of Additional Info Masks, Positions */
/* ======================================== */
#define E_MONA_RCLAB_MASK       0xFFFF0000  /* routine/code label */
#define E_MONA_ADD_INFO_MASK    0x0000FFFF  /* additional info */

#define E_MONA_RCLAB_POS        16          /* routine/code label */
#define E_MONA_ADD_INFO_POS     0           /* additional info */

/* Definitions for Error Classes */
/* ================================== */
#define E_MONA_CLASS_COP            0x00000000
#define E_MONA_CLASS_OS             0x10000000
#define E_MONA_CLASS_DDI            0x20000000

#define E_MONA_CLASS_STD_BASE       0xA0000000  /* standard class begins from here */
#define E_MONA_CLASS_BOOT_LOADER    0xA0000000
#define E_MONA_CLASS_VEN            0xB0000000
#define E_MONA_CLASS_BASMOD         0xC0000000
#define E_MONA_CLASS_DDMOD          0xD0000000
#define E_MONA_CLASS_API            0xE0000000
#define E_MONA_CLASS_APP            0xF0000000

/* Definitions for Error Types */
/* ================================ */
#define E_MONA_TYPE_NONE            0x00000000
#define E_MONA_TYPE_USER            0x01000000
#define E_MONA_TYPE_WARNING         0x02000000
#define E_MONA_TYPE_RUNTIME         0x03000000

/* Definitions for Error Codes */
/* ================================ */
#define E_MONA_NO_ERROR                     0x0000
#define E_MONA_PASSED                       0x0001

#define E_MONA_PROC_BASE                    0x0400
#define E_MONA_PROC_FATAL_ERROR             (E_MONA_PROC_BASE + 0x01)
#define E_MONA_PROC_STATE_CONFLICT          (E_MONA_PROC_BASE + 0x02)
#define E_MONA_PROC_STATE_UNDEFINED         (E_MONA_PROC_BASE + 0x03)
#define E_MONA_PROC_OUT_OF_SYNC             (E_MONA_PROC_BASE + 0x04)
#define E_MONA_PROC_STACK_OVERFLOW          (E_MONA_PROC_BASE + 0x05)
#define E_MONA_PROC_PLC_STOPPED             (E_MONA_PROC_BASE + 0x06)

#define E_MONA_ACCESS_BASE                  0x0500
#define E_MONA_ACCESS_FAILED                (E_MONA_ACCESS_BASE + 0x01)
#define E_MONA_ACCESS_DENIED                (E_MONA_ACCESS_BASE + 0x02)
#define E_MONA_ACCESS_CONFLICT              (E_MONA_ACCESS_BASE + 0x03)
#define E_MONA_ACCESS_LOCKED                (E_MONA_ACCESS_BASE + 0x04)
#define E_MONA_ACCESS_OUT_OF_RANGE          (E_MONA_ACCESS_BASE + 0x05)
#define E_MONA_ACCESS_LOCKED_BY_APP         (E_MONA_ACCESS_BASE + 0x06)

#define E_MONA_PARAM_BASE                   0x0600
#define E_MONA_PARAM_INVLD                  (E_MONA_PARAM_BASE + 0x01)
#define E_MONA_PARAM_INVLD_HANDLE           (E_MONA_PARAM_BASE + 0x02)
#define E_MONA_PARAM_INVLD_ID               (E_MONA_PARAM_BASE + 0x03)
#define E_MONA_PARAM_INVLD_POINTER          (E_MONA_PARAM_BASE + 0x04)
#define E_MONA_PARAM_INVLD_PATH             (E_MONA_PARAM_BASE + 0x05)
#define E_MONA_PARAM_INVLD_NAME             (E_MONA_PARAM_BASE + 0x06)
#define E_MONA_PARAM_NAME_LENGTH_EXCEED     (E_MONA_PARAM_BASE + 0x07)
#define E_MONA_PARAM_INVLD_OPTION           (E_MONA_PARAM_BASE + 0x08)
#define E_MONA_PARAM_NULL_BUFFER            (E_MONA_PARAM_BASE + 0x09)
#define E_MONA_PARAM_INCONSISTENT           (E_MONA_PARAM_BASE + 0x0A)
#define E_MONA_PARAM_OUT_OF_RANGE           (E_MONA_PARAM_BASE + 0x0B)
#define E_MONA_PARAM_INVLD_USERNAME         (E_MONA_PARAM_BASE + 0x0C)
#define E_MONA_PARAM_INVLD_PASSWORD         (E_MONA_PARAM_BASE + 0x0D)
#define E_MONA_PARAM_INVLD_DIRECTORY        (E_MONA_PARAM_BASE + 0x0E)
#define E_MONA_PARAM_INVLD_ACCESS           (E_MONA_PARAM_BASE + 0x0F)
#define E_MONA_PARAM_INVLD_USER             (E_MONA_PARAM_BASE + 0x10)
#define E_MONA_PARAM_NO_MORE_FILES          (E_MONA_PARAM_BASE + 0x11)

#define E_MONA_MSG_BASE                     0x0700
#define E_MONA_MSG_UNKNOWN_CODE             (E_MONA_MSG_BASE + 0x01)
#define E_MONA_MSG_UNSUPPORTED_CODE         (E_MONA_MSG_BASE + 0x02)
#define E_MONA_MSG_INVLD_PARA_COUNT         (E_MONA_MSG_BASE + 0x03)
#define E_MONA_MSG_INVLD_PARA_BYTE          (E_MONA_MSG_BASE + 0x04)
#define E_MONA_MSG_INVLD_PARA_WORD          (E_MONA_MSG_BASE + 0x05)
#define E_MONA_MSG_INVLD_PARA_LONG          (E_MONA_MSG_BASE + 0x06)
#define E_MONA_MSG_UNKNOWN_SUB_CODE         (E_MONA_MSG_BASE + 0x07)
#define E_MONA_MSG_UNSUPPORTED_SUB_CODE     (E_MONA_MSG_BASE + 0x08)
#define E_MONA_MSG_INVLD_SUB_PARA_COUNT     (E_MONA_MSG_BASE + 0x09)
#define E_MONA_MSG_INVLD_SUB_PARA_BYTE      (E_MONA_MSG_BASE + 0x0A)
#define E_MONA_MSG_INVLD_SUB_PARA_WORD      (E_MONA_MSG_BASE + 0x0B)
#define E_MONA_MSG_INVLD_SUB_PARA_LONG      (E_MONA_MSG_BASE + 0x0C)

#define E_MONA_MSG_INCONS_PARA_COUNTS       (E_MONA_MSG_BASE + 0x0D)
#define E_MONA_MSG_INCONS_LENGTH            (E_MONA_MSG_BASE + 0x0E)
#define E_MONA_MSG_SIZE_EXCEED              (E_MONA_MSG_BASE + 0x0F)
#define E_MONA_MSG_INV_CHECKSUM             (E_MONA_MSG_BASE + 0x10)

#define E_MONA_RESRC_BASE                   0x0800
#define E_MONA_RESRC_EXHAUSTED              (E_MONA_RESRC_BASE + 0x01)
#define E_MONA_RESRC_ALREADY_EXISTS         (E_MONA_RESRC_BASE + 0x02)
#define E_MONA_RESRC_NOT_EXISTS             (E_MONA_RESRC_BASE + 0x03)
#define E_MONA_RESRC_DEFECT                 (E_MONA_RESRC_BASE + 0x04)
#define E_MONA_RESRC_OPERATION_FAILED       (E_MONA_RESRC_BASE + 0x05)
#define E_MONA_RESRC_OPERATION_FATAL        (E_MONA_RESRC_BASE + 0x06)
#define E_MONA_RESRC_WAIT_TIMEOUT           (E_MONA_RESRC_BASE + 0x07)
#define E_MONA_RESRC_WAIT_FAILED            (E_MONA_RESRC_BASE + 0x08)
#define E_MONA_RESRC_EOF                    (E_MONA_RESRC_BASE + 0x09)
#define E_MONA_RESRC_NAME_LENGTH_EXCEED     (E_MONA_RESRC_BASE + 0x0A)
#define E_MONA_RESRC_CANCELED               (E_MONA_RESRC_BASE + 0x0B)
#define E_MONA_RESRC_NO_PDD                 (E_MONA_RESRC_BASE + 0x0C)
#define E_MONA_RESRC_DISC_FULL              (E_MONA_RESRC_BASE + 0x0D)

#define E_MONA_CONN_BASE                    0x0900
#define E_MONA_CONN_EXHAUSTED               (E_MONA_CONN_BASE + 0x01)
#define E_MONA_CONN_LOST                    (E_MONA_CONN_BASE + 0x02)
#define E_MONA_CONN_ABORT                   (E_MONA_CONN_BASE + 0x03)
#define E_MONA_CONN_REC_TIMEOUT             (E_MONA_CONN_BASE + 0x04)
#define E_MONA_CONN_TARGET_NOT_FOUND        (E_MONA_CONN_BASE + 0x05)
#define E_MONA_CONN_OUT_OF_RES              (E_MONA_CONN_BASE + 0x06)

#define E_MONA_INI_BASE                     0x0A00
#define E_MONA_INI_BLOCK_INVLD_NAME         (E_MONA_INI_BASE + 0x01)
#define E_MONA_INI_BLOCK_NOT_EXISTS         (E_MONA_INI_BASE + 0x02)
#define E_MONA_INI_BLOCK_INVLD_SYNTAX       (E_MONA_INI_BASE + 0x03)
#define E_MONA_INI_LINE_LENGTH_EXCEED       (E_MONA_INI_BASE + 0x04)
#define E_MONA_INI_VAR_INVLD_NAME           (E_MONA_INI_BASE + 0x05)
#define E_MONA_INI_VAR_NOT_EXISTS           (E_MONA_INI_BASE + 0x06)
#define E_MONA_INI_VAR_INVLD_SYNTAX         (E_MONA_INI_BASE + 0x07)
#define E_MONA_INI_VAR_INVLD_VALUE          (E_MONA_INI_BASE + 0x08)
#define E_MONA_INI_CF_CARD_NOT_PRESENT      (E_MONA_INI_BASE + 0x09)
#define E_MONA_INI_INVALID_CTRL_TYPE        (E_MONA_INI_BASE + 0x0A)
#define E_MONA_INI_INVALID_CHECKSUM         (E_MONA_INI_BASE + 0x0B)
#define E_MONA_INI_INVALID_IP_CONFIG        (E_MONA_INI_BASE + 0x0C)

#define E_MONA_SECU_BASE                    0x0B00
#define E_MONA_SECU_WRONG_PASSW             (E_MONA_SECU_BASE + 0x01)
#define E_MONA_SECU_NO_PERMISSION           (E_MONA_SECU_BASE + 0x02)

#define E_MONA_DATA_BASE                    0x0C00
#define E_MONA_DATA_BUFFER_OVERFLOW         (E_MONA_DATA_BASE + 0x01)
#define E_MONA_DATA_INVLD_VALUE             (E_MONA_DATA_BASE + 0x02)
#define E_MONA_DATA_INVLD_VALUE_LEN         (E_MONA_DATA_BASE + 0x03)
#define E_MONA_DATA_VALUE_OUT_OF_RANGE      (E_MONA_DATA_BASE + 0x04)
#define E_MONA_DATA_UNSUPPORTED_BASE        (E_MONA_DATA_BASE + 0x05)
#define E_MONA_DATA_UNSUPPORTED_TYPE        (E_MONA_DATA_BASE + 0x06)
#define E_MONA_DATA_INVLD_DIGIT_CHAR        (E_MONA_DATA_BASE + 0x07)
#define E_MONA_DATA_VAR_NOT_EXISTS          (E_MONA_DATA_BASE + 0x08)
#define E_MONA_DATA_INCONSIST_OFFS          (E_MONA_DATA_BASE + 0x09)
#define E_MONA_DATA_TYPE_MISMATCH           (E_MONA_DATA_BASE + 0x0A)
#define E_MONA_DATA_WRONG_SYNTAX            (E_MONA_DATA_BASE + 0x0B)

/*****************************************************************************
****  MODULE ID DEFINITIONS                                               ****
*****************************************************************************/
#define SDDI_MODUL_ID                  0x24
#define SNMP_MODUL_ID                  0x35
#define LLDP_MODUL_ID                  0x36
#define ETC_MODUL_ID                   0x47
#define CTM_MODUL_ID                   0x50
#define PNIO_IOAR_MODUL_ID             0x51
#define PNIO_RPC_MODUL_ID              0x52
#define PNIO_CMCTL_MODUL_ID            0x53
#define PNIO_CMDEV_MODUL_ID            0x55
#define DCP_ARP_MODUL_ID               0x57
#define PN_MODUL_ID                    0x71
#define PNDIAG_MODUL_ID                0x72

#define VE_API_ID                      0x80
#define MH_API_ID                      0x81
#define ES_API_ID                      0x82
#define DDI_API_ID                     0x83
#define CM_API_ID                      0x84
#define SMEM_API_ID                    0x85
#define DH_API_ID                      0x86
#define DEV_API_ID                     0x87
#define FILE_API_ID                    0x88
#define L2_API_ID                      0x8B
#define MSGQ_API_ID                    0x8F
#define PNIO_IOAR_API_ID               0x90
#define PNIO_RPC_API_ID                0x92
#define PNIO_CMCTL_API_ID              0x94
#define PNIO_CMDEV_API_ID              0x99
#define SNMP_API_ID                    0xA0
#define PN_API_ID                      0xA5
#define XHPE_API_ID                    0xAA     /* INtime ethernet dispatcher */


#ifdef __cplusplus
}
#endif /* #ifdef __cplusplus */

#endif /* #ifndef INC_PNLIBMAIN */

